--[[
AnimalDisplayExtension
Zeigt das richtige Symbole im Bau-Menü und beim Tiertrailer an für Zusätzliche Tierarten. In diesem Fall für Ziegen und Hühner.

Displays the correct icons in the build menu and animal trailer for Additional Animal Species. In this case Goats and Chickens.

Copyright (C) braeven
Author: braeven

Date: 27.09.2023
Version: 1.3.0.0

Changelog:
1.0.0.0 @ 13.01.2023 - Initial Release.
1.1.0.0 @ 13.01.2023 - Added Chickens
1.2.0.0 @ 15.09.2023 - Some Cleanup


Important:
Free to use and customize as long as these headers remain.
Please adapt to your own map paths.

Frei zu verwenden und anzupassen, solange diese Kopfzeilen stehen bleiben.
Bitte an eigene Map-Pfade anpassen.
]]

AnimalDisplayExtension = {}
AnimalDisplayExtension.xmlFile = "xml/terraLifePlus_animalDisplayExtension.xml"
AnimalDisplayExtension.iconFileGoat = "$moddir$FS22_TerraLifePlus/gui/buildingMenue_goat.dds"
AnimalDisplayExtension.iconFileHaycow = "$moddir$FS22_TerraLifePlus/gui/buildingMenue_haycow.dds"

function AnimalDisplayExtension:LoadProfiles()
	local xmlGuiFileName = Utils.getFilename(AnimalDisplayExtension.xmlFile, g_currentModDirectory)
	g_gui:loadProfiles(xmlGuiFileName)
	local filepathHaycow = Utils.getFilename(AnimalDisplayExtension.iconFileHaycow, g_currentModDirectory)
	g_gui.profiles.shopListAttributeIconHaycow.values.imageFilename = filepathHaycow
	local filepathGoat = Utils.getFilename(AnimalDisplayExtension.iconFileGoat, g_currentModDirectory)
	g_gui.profiles.shopListAttributeIconGoat.values.imageFilename = filepathGoat
end



function AnimalDisplayExtension.getSpecValueNumberAnimals(storeItem, realItem)
	local data = storeItem.specs.numberAnimals

	if data == nil then
		return nil
	end

	local profile = nil
	local animalTypeIndex = g_currentMission.animalSystem:getTypeIndexByName(data.animalTypeName)

	if animalTypeIndex == AnimalType.COW then
		profile = "shopListAttributeIconCow"
	elseif animalTypeIndex == AnimalType.SHEEP then
		profile = "shopListAttributeIconSheep"
	elseif animalTypeIndex == AnimalType.HORSE then
		profile = "shopListAttributeIconHorse"
	elseif animalTypeIndex == AnimalType.PIG then
		profile = "shopListAttributeIconPig"
	elseif animalTypeIndex == AnimalType.CHICKEN then
		profile = "shopListAttributeIconChicken"
	elseif animalTypeIndex == AnimalType.GOAT then
		profile = "shopListAttributeIconGoat"
	end

	return data.maxNumAnimals, profile
end

PlaceableHusbandryAnimals.getSpecValueNumberAnimals = Utils.overwrittenFunction(PlaceableHusbandryAnimals.getSpecValueNumberAnimals, AnimalDisplayExtension.getSpecValueNumberAnimals)

function AnimalDisplayExtension:addAnimalCategorieShop()
    g_storeManager:addConstructionTab("animals", "goat", g_i18n:getText("TL_animal_shop_goat"), AnimalDisplayExtension.iconFileGoat, GuiUtils.getUVs("0px 0 32px 32px",string.getVectorN("32 32", 2)), "")
end

function AnimalDisplayExtension.loadMapData(xmlFile, missionInfo, baseDirectory)
    AnimalDisplayExtension:addAnimalCategorieShop()
end

StoreManager.loadMapData = Utils.appendedFunction(StoreManager.loadMapData, AnimalDisplayExtension.loadMapData)


PlaceableHotspot.TYPE.GOAT = 20
PlaceableHotspot.TYPE.HAYCOW = 21
PlaceableHotspot.FILE_RESOLUTIONnew = {100, 100}
PlaceableHotspot.UV[PlaceableHotspot.TYPE.GOAT] = GuiUtils.getUVs({0, 0, 80, 80}, PlaceableHotspot.FILE_RESOLUTIONnew)
PlaceableHotspot.UV[PlaceableHotspot.TYPE.HAYCOW] = GuiUtils.getUVs({0, 0, 80, 80}, PlaceableHotspot.FILE_RESOLUTIONnew)

PlaceableHotspot.CATEGORY_MAPPING[PlaceableHotspot.TYPE.GOAT] = MapHotspot.CATEGORY_ANIMAL
PlaceableHotspot.CATEGORY_MAPPING[PlaceableHotspot.TYPE.HAYCOW] = MapHotspot.CATEGORY_ANIMAL
PlaceableHotspot.iconFileGoat = Utils.getFilename("$moddir$FS22_TerraLifePlus/gui/mapHotspot_goat.dds", g_currentModDirectory)
PlaceableHotspot.iconFileHayCow = Utils.getFilename("$moddir$FS22_TerraLifePlus/gui/mapHotspot_hayCow.dds", g_currentModDirectory)

function PlaceableHotspot:setPlaceableType(placeableType)
	self.placeableType = placeableType

	if self.icon ~= nil then
		if placeableType == 20 then
			self.icon:setImage(PlaceableHotspot.iconFileGoat)
		elseif placeableType == 21 then
			self.icon:setImage(PlaceableHotspot.iconFileHayCow)
		end

		self.icon:setUVs(PlaceableHotspot.UV[placeableType])
	end
end


AnimalDisplayExtension:LoadProfiles()


function AnimalDisplayExtension:onLoad(superFunc, savegame)
if self.spec_husbandry ~= nil then
	local xmlFile = self.xmlFile
	local animalTypeName = xmlFile:getValue("placeable.husbandry.animals#type")
	xmlFile:iterate("placeable.triggerMarkers.triggerMarker", function (_, key)
		local xmlFilename = self.xmlFile:getValue(key .. "#filename")
		if xmlFilename ~= nil then
			if xmlFilename == "$data/shared/assets/marker/markerIcons.xml" then
				local id = self.xmlFile:getValue(key .. "#id")
				if id ~= nil then
					if id == "ANIMALS" then
						xmlFile:setString(key .. "#filename", "$moddir$FS22_TerraLifePlus_Assets/markerIcons/markerIcons_TL.xml")
						if animalTypeName == "COW" then
							xmlFile:setString(key .. "#id", "COW")
						elseif animalTypeName == "PIG" then
							xmlFile:setString(key .. "#id", "PIG")
						elseif animalTypeName == "SHEEP" then
							xmlFile:setString(key .. "#id", "SHEEP")
						elseif animalTypeName == "GOAT" then
							xmlFile:setString(key .. "#id", "GOAT")
						elseif animalTypeName == "CHICKEN" then
							xmlFile:setString(key .. "#id", "CHICKEN")
						elseif animalTypeName == "HORSE" then
							xmlFile:setString(key .. "#id", "HORSE")
						elseif animalTypeName == "HAYCOW" then
							xmlFile:setString(key .. "#id", "HAYCOW")
						end
					end
				end
			end
		end
	end)
end

superFunc(self, savegame)
end

PlaceableTriggerMarkers.onLoad = Utils.overwrittenFunction(PlaceableTriggerMarkers.onLoad, AnimalDisplayExtension.onLoad)