-- Farming Agency Multi Item Purchase - Shop Extension
-- Allows purchasing of different Pallet-Types, Configurations and FillLevels on one Shop-Screen
-- by [F/A] Braeven
-- Version 1.0 - 10.09.2023

-- Only for Use with TerraLife, It is not allowed to use this script without permission from Braeven / Farming Agency

FAShopExtension = {}


function FAShopExtension:getBuyPrice(superFunc, storeItem, configurations, saleItem)
	local price = storeItem.price

	if saleItem ~= nil then
		price = saleItem.price
	end

	local upgradePrice = 0

	if configurations ~= nil then
		for name, id in pairs(configurations) do
			local configs = storeItem.configurations[name]

			if configs ~= nil then
				local hasBought = saleItem ~= nil and saleItem.boughtConfigurations[name] ~= nil and saleItem.boughtConfigurations[name][id]
				if not hasBought then
					if name == "fAPalletFillLevels" then
						if price == 98765432 then
							price = (2 - configs[id].surcharge) * (configs[id].amount / 1000)
						else
							price = price * (2 - configs[id].surcharge) * (configs[id].amount / 1000)
						end
					elseif name == "fAPalletAmount" then
						if price == 98765432 then
							price = configs[id].discount * id
						else
							price = price * (configs[id].discount * id)
						end
					elseif name == "fAPalletFillTypes" then
						if price == 98765432 then
							fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(configs[id].fillType)
							price = g_currentMission.economyManager:getPricePerLiter(fillTypeIndex) * 1000
						else
							fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(configs[id].fillType)
							price = price * g_currentMission.economyManager:getPricePerLiter(fillTypeIndex) * 1000
						end
					else
						price = price + configs[id].price
						upgradePrice = upgradePrice * configs[id].price
					end
				end
			end
		end
	end

	return price, upgradePrice
end

EconomyManager.getBuyPrice = Utils.overwrittenFunction(EconomyManager.getBuyPrice, FAShopExtension.getBuyPrice)

--Preisanzeige an der Configuration
function FAShopExtension:setConfigPrice(superFunc, configName, configIndex, priceTextElement, vehicle)
	local configItems = self.storeItem.configurations[configName]
	local price = configItems[configIndex].price
	local text = ""

	if vehicle ~= nil then
		if ConfigurationUtil.hasBoughtConfiguration(vehicle, configName, configIndex) then
			price = 0
		end
	elseif self.saleItem ~= nil and ConfigurationUtil.hasBoughtConfiguration(self.saleItem, configName, configIndex) then
		price = 0
	end
	
	local text = "+" .. self.l10n:formatMoney(price)
	if configName == "fAPalletFillTypes" then
		fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(configItems[configIndex].fillType)
		price = g_currentMission.economyManager:getPricePerLiter(fillTypeIndex) * 1000
		configItems[configIndex].price = price
		text = self.l10n:formatMoney(price) .. " " .. g_i18n:getText("TL_FAMultiItemPurchase_PricePer1000L")
	elseif configName == "fAPalletFillLevels" then
		local amount = math.floor(((1 - configItems[configIndex].surcharge) * 100))
		text = " +" .. amount .. g_i18n:getText("TL_FAMultiItemPurchase_Surcharge")
		if amount < 0 or amount == 0 then
			text = amount .. g_i18n:getText("TL_FAMultiItemPurchase_Bonus")
		end
	elseif configName == "fAPalletAmount" then
		local amount = math.floor(((1 - configItems[configIndex].discount) * 100))
		text = amount .. g_i18n:getText("TL_FAMultiItemPurchase_Discount")
		if amount > 0 then
			text = " -" .. amount .. g_i18n:getText("TL_FAMultiItemPurchase_Discount")
		end
	end
	priceTextElement:setText(text)
	priceTextElement:setVisible(true)
end

ShopConfigScreen.setConfigPrice = Utils.overwrittenFunction(ShopConfigScreen.setConfigPrice, FAShopExtension.setConfigPrice)


--Zusätzliche Daten auslesen und in der Configuration hinterlegen
function FAShopExtension.getConfigurationsFromXML(xmlFile, superfunc, key, baseDir, customEnvironment, isMod, storeItem)
	local configurations, defaultConfigurationIds = superfunc(xmlFile, key, baseDir, customEnvironment, isMod, storeItem)

	if configurations ~= nil then
		if configurations["fAPalletFillTypes"] ~= nil then

			local baseKey = key .. ".fAPalletFillTypesConfigurations"
			for n = 1, #configurations["fAPalletFillTypes"] do
				local configKey = string.format(baseKey .. ".fAPalletFillTypesConfiguration(%d)", n-1)
				local fillTypeName = xmlFile:getString(configKey .. "#fillType", "Manure")
				configurations["fAPalletFillTypes"][n].fillType = fillTypeName
			end

			local baseKey = key .. ".fAPalletFillLevelsConfigurations"
			for n = 1, #configurations["fAPalletFillLevels"] do
				local configKey = string.format(baseKey .. ".fAPalletFillLevelsConfiguration(%d)", n-1)
				local surcharge = xmlFile:getFloat(configKey .. "#surcharge", 1)
				local amount = xmlFile:getFloat(configKey .. "#amount", 1)
				if surcharge <= 0 then
					surcharge = 1
				end
				configurations["fAPalletFillLevels"][n].surcharge = surcharge
				configurations["fAPalletFillLevels"][n].amount = amount
			end

			local baseKey = key .. ".fAPalletAmountConfigurations"
			for n = 1, #configurations["fAPalletAmount"] do
				local configKey = string.format(baseKey .. ".fAPalletAmountConfiguration(%d)", n-1)
				local discount = xmlFile:getFloat(configKey .. "#discount", 1)
				if discount <= 0 then
					discount = 1
				end
				configurations["fAPalletAmount"][n].discount = discount
			end

		end
	end
	return configurations, defaultConfigurationIds
end

StoreItemUtil.getConfigurationsFromXML = Utils.overwrittenFunction(StoreItemUtil.getConfigurationsFromXML, FAShopExtension.getConfigurationsFromXML)


function FAShopExtension:generateRandomVehicle(superFunc)
	local vehicle = superFunc()
	if vehicle ~= nil and vehicle.price < 10000000 then
		return vehicle
	end
end

VehicleSaleSystem.generateRandomVehicle = Utils.overwrittenFunction(VehicleSaleSystem.generateRandomVehicle, FAShopExtension.generateRandomVehicle)