FarmlandInfoLoader = {}

local FarmlandInfoLoader_mt = Class(FarmlandInfoLoader, AbstractManager)

function FarmlandInfoLoader.new(customMt)
	local self = AbstractManager.new(customMt or FarmlandInfoLoader_mt)
    self.farmlandInfos = {}

	return self
end

function FarmlandInfoLoader:loadMapData(xmlFile)
	FarmlandManager:superClass().loadMapData(self)

    XMLUtil.loadDataFromMapXML(xmlFile, "farmlandInfos", g_currentMission.baseDirectory, self, self.loadFarmlandInfoData)
end

function FarmlandInfoLoader:loadFarmlandInfoData(xmlFile)
    if xmlFile == nil then
        return
    end

    local i = 0

    while true do
		local key = string.format("farmlandInfos.info(%d)", i)

		if not hasXMLProperty(xmlFile, key) then
			break
		end

        local info = {
            text = getXMLString(xmlFile, key .. "#text"),
            scale = getXMLFloat(xmlFile, key .. "#scale"),
            bold = getXMLBool(xmlFile, key .. "#bold"),
            showOnMiniMap = getXMLBool(xmlFile, key .. "#showOnMinimap")
        }

        local filename = getXMLString(xmlFile, key .. "#filename")
        if filename ~= nil then
            info.filename = Utils.getFilename(filename, g_currentMission.baseDirectory)
            info.height = getXMLFloat(xmlFile, key .. "#height")
            info.width = getXMLFloat(xmlFile, key .. "#width")
            info.rotation = getXMLFloat(xmlFile, key .. "#rotation")
        end

        info.worldPositionX, info.worldPositionZ = string.getVector(getXMLString(xmlFile, key.."#worldPosition"))
        info.color = string.getVectorN(getXMLString(xmlFile, key.."#color"), 4)

        table.insert(self.farmlandInfos, info)

		i = i + 1
    end

    self:addMapHotspot()
end

function FarmlandInfoLoader:addMapHotspot()
    for _, info in pairs(self.farmlandInfos) do
        local mapHotspot = InfoTextHotspot.new()
        mapHotspot:setInfo(info)
        g_currentMission:addMapHotspot(mapHotspot)
    end
end

g_farmlandInfoLoader = FarmlandInfoLoader.new()