MapInfoLoader = {}

local MapInfoLoader_mt = Class(MapInfoLoader, AbstractManager)

function MapInfoLoader.new(customMt)
	local self = AbstractManager.new(customMt or MapInfoLoader_mt)
    self.mapInfos = {}

	return self
end

function MapInfoLoader:loadMapData(xmlFile)
	MapInfoLoader:superClass().loadMapData(self)

    XMLUtil.loadDataFromMapXML(xmlFile, "mapInfos", g_currentMission.baseDirectory, self, self.loadMapInfoData)
end

function MapInfoLoader:loadMapInfoData(xmlFile)
    if xmlFile == nil then
        return
    end

    local i = 0

    while true do
		local key = string.format("mapInfos.info(%d)", i)

		if not hasXMLProperty(xmlFile, key) then
			break
		end

        local info = {
            text = getXMLString(xmlFile, key .. "#text"),
            hotspotType = "CATEGORY_"..getXMLString(xmlFile, key .. "#type"),
            scale = getXMLFloat(xmlFile, key .. "#scale"),
            bold = getXMLBool(xmlFile, key .. "#bold"),
            showOnMiniMap = getXMLBool(xmlFile, key .. "#showOnMinimap")
        }

        local filename = getXMLString(xmlFile, key .. "#filename")
        if filename ~= nil then
            info.filename = Utils.getFilename(filename, g_currentMission.baseDirectory)
            info.height = getXMLFloat(xmlFile, key .. "#height")
            info.width = getXMLFloat(xmlFile, key .. "#width")
            info.rotation = getXMLFloat(xmlFile, key .. "#rotation")
            local uvs = getXMLString(xmlFile, key.."#uvs")
            if uvs ~= nil then
                info.uvs = string.getVectorN(uvs, 4)
                info.uvResolution = string.getVectorN(getXMLString(xmlFile, key.."#uvResolution"), 2)
            end
        end

        info.worldPositionX, info.worldPositionZ = string.getVector(getXMLString(xmlFile, key.."#worldPosition"))
        info.color = string.getVectorN(getXMLString(xmlFile, key.."#color"), 4)

        table.insert(self.mapInfos, info)

		i = i + 1
    end

    self:addMapHotspot()
end

function MapInfoLoader:addMapHotspot()
    for _, info in pairs(self.mapInfos) do
        local mapHotspot = InfoTextHotspot.new()
        mapHotspot:setInfo(info)
        g_currentMission:addMapHotspot(mapHotspot)
    end
end

g_mapInfoLoader = MapInfoLoader.new()