TLExtendedSprayerExtension = {}

local ExtendedSprayer = _G['FS22_precisionFarming'].ExtendedSprayer
local ExtendedSprayerHUDExtension = _G['FS22_precisionFarming'].ExtendedSprayerHUDExtension

function TLExtendedSprayerExtension:getCurrentSprayerMode()
	local sprayer, fillUnitIndex = ExtendedSprayer.getFillTypeSourceVehicle(self)
	local fillType = sprayer:getFillUnitLastValidFillType(fillUnitIndex)

	if fillType == FillType.LIME or fillType == FillType.CARBOLIME or fillType == FillType.LIQUIDCARBOLIME  then
		return true, false
	end

	if fillType == FillType.FERTILIZER or fillType == FillType.LIQUIDFERTILIZER or fillType == FillType.MANURE or fillType == FillType.LIQUIDFERTILIZER or fillType == FillType.LIQUIDMANURE or fillType == FillType.DIGESTATE or fillType == FillType.COMPOST then
		return false, true
	end

	if fillType == FillType.HERBICIDE then
		return false, false
	end

	if self:getIsAIActive() then
		return false, true
	end

	return false, false
end

ExtendedSprayer.getCurrentSprayerMode = Utils.overwrittenFunction(ExtendedSprayer.getCurrentSprayerMode, TLExtendedSprayerExtension.getCurrentSprayerMode)

--Statistiken
function TLExtendedSprayerExtension:onEndWorkAreaProcessing(dt, hasProcessed)
	local spec = self.spec_extendedSprayer
	local specSprayer = self.spec_sprayer

	if self.isServer and specSprayer.workAreaParameters.isActive then
		local sprayVehicle = specSprayer.workAreaParameters.sprayVehicle
		local usage = specSprayer.workAreaParameters.usage
		local fillType = specSprayer.workAreaParameters.sprayFillType

		if (sprayVehicle ~= nil or self:getIsAIActive()) and self:getIsTurnedOn() then
			local usageRegular = spec.lastRegularUsage
			local farmlandStatistics, _, farmlandId = self:getPFStatisticInfo()

			if farmlandStatistics ~= nil and farmlandId ~= nil then
				if fillType == FillType.LIME or fillType == FillType.CARBOLIME or fillType == FillType.LIQUIDCARBOLIME then
					farmlandStatistics:updateStatistic(farmlandId, "usedLime", usage)
					farmlandStatistics:updateStatistic(farmlandId, "usedLimeRegular", usageRegular)
				elseif fillType == FillType.FERTILIZER then
					farmlandStatistics:updateStatistic(farmlandId, "usedMineralFertilizer", usage)
					farmlandStatistics:updateStatistic(farmlandId, "usedMineralFertilizerRegular", usageRegular)
				elseif fillType == FillType.LIQUIDFERTILIZER then
					farmlandStatistics:updateStatistic(farmlandId, "usedLiquidFertilizer", usage)
					farmlandStatistics:updateStatistic(farmlandId, "usedLiquidFertilizerRegular", usageRegular)
				elseif fillType == FillType.MANURE or fillType == FillType.COMPOST then
					farmlandStatistics:updateStatistic(farmlandId, "usedManure", usage)
					farmlandStatistics:updateStatistic(farmlandId, "usedManureRegular", usageRegular)
				elseif fillType == FillType.LIQUIDMANURE or fillType == FillType.DIGESTATE then
					farmlandStatistics:updateStatistic(farmlandId, "usedLiquidManure", usage)
					farmlandStatistics:updateStatistic(farmlandId, "usedLiquidManureRegular", usageRegular)
				end
			end
		end
	end
end

ExtendedSprayer.onEndWorkAreaProcessing = Utils.overwrittenFunction(ExtendedSprayer.onEndWorkAreaProcessing, TLExtendedSprayerExtension.onEndWorkAreaProcessing)

function TLExtendedSprayerExtension:onUpdate(superFunc, dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local spec = self.spec_extendedSprayer

	if self.isServer then
		if g_time - spec.lastAreaChangeTime > 500 then
			spec.lastGroundUpdateDistance = spec.lastGroundUpdateDistance + self.lastMovedDistance

			if spec.groundUpdateDistance < spec.lastGroundUpdateDistance then
				spec.lastGroundUpdateDistance = 0
				local workArea = self:getWorkAreaByIndex(1)

				if workArea ~= nil then
					local x, y, z = nil
					local lx, _, _ = localToLocal(workArea.start, self.rootNode, 0, 0, 0)

					if math.abs(lx) < 0.5 then
						x, y, z = getWorldTranslation(workArea.start)
					else
						local x1, y1, z1 = getWorldTranslation(workArea.start)
						local x2, y2, z2 = getWorldTranslation(workArea.width)
						z = (z1 + z2) * 0.5
						y = (y1 + y2) * 0.5
						x = (x1 + x2) * 0.5
					end

					local isOnField, _ = FSDensityMapUtil.getFieldDataAtWorldPosition(x, 0, z)

					if isOnField then
						local sprayer, fillUnitIndex = ExtendedSprayer.getFillTypeSourceVehicle(self)
						local fillType = sprayer:getFillUnitLastValidFillType(fillUnitIndex)

						if fillType == FillType.UNKNOWN then
							fillType = sprayer:getFillUnitFirstSupportedFillType(fillUnitIndex)
						end

						if fillType == FillType.LIME or fillType == FillType.CARBOLIME or fillType == LIQUIDCARBOLIME then
							local pHLevel = spec.pHMap:getLevelAtWorldPos(x, z)
							local pHOptimal = 0
							local soilTypeIndex = spec.soilMap:getTypeIndexAtWorldPos(x, z)

							if soilTypeIndex > 0 then
								pHOptimal = spec.pHMap:getOptimalPHValueForSoilTypeIndex(soilTypeIndex)
							end

							spec.phChangeBuffer:add(0)
							spec.phActualBuffer:add(pHLevel, true)
							spec.phTargetBuffer:add(pHOptimal, true)

							spec.lastTouchedSoilTypeReal = soilTypeIndex

							if spec.lastTouchedSoilType == 0 then
								spec.lastTouchedSoilType = soilTypeIndex
							end
						else
							local forcedFruitType = nil

							if self.spec_sowingMachine ~= nil then
								forcedFruitType = self.spec_sowingMachine.workAreaParameters.seedsFruitType
							end

							local nLevel = spec.nitrogenMap:getLevelAtWorldPos(x, z)
							local nTarget, soilTypeIndex, fruitTypeIndex = spec.nitrogenMap:getTargetLevelAtWorldPos(x, z, nil, forcedFruitType, fillType, nLevel, spec.nApplyAutoModeFruitRequirementDefaultIndex)

							spec.nChangeBuffer:add(0)
							spec.nActualBuffer:add(nLevel, true, true)
							spec.nTargetBuffer:add(nTarget, true, true)
							self:setSprayAmountAutoFruitTypeIndex(fruitTypeIndex)

							spec.lastTouchedSoilTypeReal = soilTypeIndex

							if spec.lastTouchedSoilType == 0 then
								spec.lastTouchedSoilType = soilTypeIndex
							end
						end
					else
						spec.phChangeBuffer:reset()
						spec.phActualBuffer:reset()
						spec.phTargetBuffer:reset()
						spec.nChangeBuffer:reset()
						spec.nActualBuffer:reset()
						spec.nTargetBuffer:reset()

						spec.lastTouchedSoilType = 0
						spec.lastTouchedSoilTypeReal = 0
						spec.lastLitersPerHectar = 0
						spec.lastNitrogenProportion = 0

						self:raiseDirtyFlags(spec.usageValuesDirtyFlag)
					end
				end
			else
				spec.phActualBuffer:add(nil, true)
				spec.phTargetBuffer:add(nil, true)
				spec.nActualBuffer:add(nil, true)
				spec.nTargetBuffer:add(nil, true)
			end
		elseif self:getIsTurnedOn() then
			spec.phActualBuffer:add()
			spec.phTargetBuffer:add()
			spec.nActualBuffer:add()
			spec.nTargetBuffer:add()

			spec.lastGroundUpdateDistance = spec.groundUpdateDistance * 0.5
		else
			spec.lastGroundUpdateDistance = spec.groundUpdateDistance * 0.5
		end
	end
end

ExtendedSprayer.onUpdate = Utils.overwrittenFunction(ExtendedSprayer.onUpdate, TLExtendedSprayerExtension.onUpdate)


local function formatDecimalNumber(n)
	if math.floor(n) == n then
		return string.format("%.1f", n)
	else
		return string.format("%s", n)
	end
end

local function renderDoubleText(x, y, textSize, text, maxWidth)
	if maxWidth ~= nil then
		while maxWidth < getTextWidth(textSize, text) do
			textSize = textSize * 0.98
		end
	end

	setTextColor(0, 0, 0, 1)
	renderText(x, y - 0.0015, textSize, text)
	setTextColor(1, 1, 1, 1)
	renderText(x, y, textSize, text)
end

TLExtendedSprayerHUDExtensionExtension = {}

function TLExtendedSprayerHUDExtensionExtension:draw(superFunc, leftPosX, rightPosX, posY)
	if not self:canDraw() then
		return
	end

	local upperPosY = posY + (self.displayHeight or 0) + (self.additionalDisplayHeight or 0)
	local vehicle = self.realVehicle
	local spec = vehicle.spec_extendedSprayer
	local headline = self.texts.headline_ph_lime
	local applicationRate = 0
	local applicationRateReal = 0
	local applicationRateStr = "%.2f t/ha"
	local changeBarText = ""
	local minValue = 0
	local maxValue = 0
	self.hasValidValues = false
	local soilTypeName = ""

	if spec.lastTouchedSoilType ~= 0 and self.soilMap ~= nil then
		local soilType = self.soilMap:getSoilTypeByIndex(spec.lastTouchedSoilType)

		if soilType ~= nil then
			soilTypeName = soilType.name
		end
	end

	local hasLimeLoaded = false
	local fillTypeDesc = nil
	local sourceVehicle, fillUnitIndex = self:getFillTypeSourceVehicle(self.vehicle)
	local sprayFillType = sourceVehicle:getFillUnitFillType(fillUnitIndex)
	fillTypeDesc = g_fillTypeManager:getFillTypeByIndex(sprayFillType)
	local massPerLiter = fillTypeDesc.massPerLiter / FillTypeManager.MASS_SCALE

	if sprayFillType == FillType.LIME then
		hasLimeLoaded = true
	elseif sprayFillType == FillType.CARBOLIME then
		hasLimeLoaded = true
		headline = g_i18n:getText("TL_headline_ph_carboLime")
	elseif sprayFillType == FillType.LIQUIDCARBOLIME then
		hasLimeLoaded = true
		headline = g_i18n:getText("TL_headline_ph_liquidCarboLime")
	end

	local descriptionText = ""
	local stepResolution = nil
	local enableZeroTargetFlag = false

	if hasLimeLoaded then
		self.gradientInactive:setUVs(self.isColorBlindMode and self.colorBlindGradientUVsInactive or self.pHGradientUVsInactive)
		self.gradient:setUVs(self.isColorBlindMode and self.colorBlindGradientUVs or self.pHGradientUVs)

		local pHChanged = 0
		applicationRate = spec.lastLitersPerHectar * massPerLiter

		if not spec.sprayAmountAutoMode then
			local requiredLitersPerHa = self.pHMap:getLimeUsageByStateChange(spec.sprayAmountManual)
			pHChanged = self.pHMap:getPhValueFromChangedStates(spec.sprayAmountManual)
			applicationRate = requiredLitersPerHa * massPerLiter

			if pHChanged > 0 then
				changeBarText = string.format("pH +%s", formatDecimalNumber(pHChanged))
			end
		end

		local pHActualInt = spec.phActualBuffer:get()
		local pHTargetInt = spec.phTargetBuffer:get()
		local pHActual = self.pHMap:getPhValueFromInternalValue(pHActualInt)
		local pHTarget = self.pHMap:getPhValueFromInternalValue(pHTargetInt)

		if pHActualInt ~= 0 and pHTargetInt ~= 0 then
			self.actualValue = pHActual
			self.setValue = pHActual + pHChanged
			self.targetValue = pHTarget

			if spec.sprayAmountAutoMode then
				pHChanged = self.targetValue - self.actualValue

				if pHChanged > 0 then
					changeBarText = string.format("pH +%s", formatDecimalNumber(pHChanged))
				end

				self.setValue = self.targetValue
			end

			self.actualValueStr = "pH %.3f"

			if soilTypeName ~= "" then
				if spec.sprayAmountAutoMode then
					descriptionText = string.format(self.texts.description_limeAuto, soilTypeName, formatDecimalNumber(pHTarget))
				else
					descriptionText = string.format(self.texts.description_limeManual, soilTypeName, formatDecimalNumber(pHTarget))
				end
			end

			self.hasValidValues = true
		end

		if self.pHMap ~= nil then
			minValue, maxValue = self.pHMap:getMinMaxValue()
		end

		stepResolution = spec.pHMap:getPhValueFromChangedStates(1)
	else
		self.gradientInactive:setUVs(self.isColorBlindMode and self.colorBlindGradientUVsInactive or self.nGradientUVsInactive)
		self.gradient:setUVs(self.isColorBlindMode and self.colorBlindGradientUVs or self.nGradientUVs)

		local litersPerHectar = spec.lastLitersPerHectar
		local nitrogenChanged = 0

		if not spec.sprayAmountAutoMode then
			litersPerHectar = self.nitrogenMap:getFertilizerUsageByStateChange(spec.sprayAmountManual, sprayFillType)
			nitrogenChanged = self.nitrogenMap:getNitrogenFromChangedStates(spec.sprayAmountManual)

			if nitrogenChanged > 0 then
				changeBarText = string.format("+%dkg N/ha", nitrogenChanged)
			end
		end

		if spec.isSolidFertilizerSprayer then
			headline = self.texts.headline_n_solidFertilizer
			applicationRateStr = "%d kg/ha"
			applicationRate = litersPerHectar * massPerLiter * 1000
			if sprayFillType == FillType.COMPOST then
				headline = g_i18n:getText("TL_headline_n_compost")
			end
		elseif spec.isLiquidFertilizerSprayer then
			headline = self.texts.headline_n_liquidFertilizer
			applicationRateStr = "%d l/ha"
			applicationRate = litersPerHectar
			if sprayFillType == FillType.LIQUIDCARBOLIME then
				headline = g_i18n:getText("TL_headline_ph_liquidCarboLime")
			end
		elseif spec.isSlurryTanker then
			headline = self.texts.headline_n_slurryTanker
			applicationRateStr = "%.1f m³/ha"
			applicationRate = litersPerHectar / 1000
			if sprayFillType == FillType.LIQUIDCARBOLIME then
				headline = g_i18n:getText("TL_headline_ph_liquidCarboLime")
			end
			if spec.sprayAmountAutoMode and soilTypeName ~= "" then
				descriptionText = string.format(self.texts.description_slurryAuto, soilTypeName)
			end
		elseif spec.isManureSpreader then
			headline = self.texts.headline_n_manureSpreader
			applicationRateStr = "%.1f t/ha"
			applicationRate = litersPerHectar * massPerLiter

			if spec.sprayAmountAutoMode and soilTypeName ~= "" then
				descriptionText = string.format(self.texts.description_manureAuto, soilTypeName)
			end
		end

		local nActualInt = spec.nActualBuffer:get()
		local nTargetInt = spec.nTargetBuffer:get()
		local nActual = self.nitrogenMap:getNitrogenValueFromInternalValue(nActualInt)
		local nTarget = self.nitrogenMap:getNitrogenValueFromInternalValue(nTargetInt)

		if nActualInt > 0 and nTargetInt > 0 then
			self.actualValue = nActual
			self.setValue = nActual + nitrogenChanged
			self.targetValue = nTarget

			if spec.sprayAmountAutoMode then
				nitrogenChanged = self.targetValue - self.actualValue

				if nitrogenChanged > 0 then
					changeBarText = string.format("+%dkg N/ha", nitrogenChanged)
				end

				self.setValue = self.targetValue
			end

			self.actualValueStr = "%dkg N/ha"
			local forcedFruitType = nil

			if vehicle.spec_sowingMachine ~= nil then
				forcedFruitType = vehicle.spec_sowingMachine.workAreaParameters.seedsFruitType
			end

			local fillType = g_fillTypeManager:getFillTypeByIndex(g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(forcedFruitType or spec.nApplyAutoModeFruitType))

			if fillType ~= nil and fillType ~= FillType.UNKNOWN and soilTypeName ~= "" then
				if nTarget > 0 then
					if spec.sprayAmountAutoMode then
						descriptionText = string.format(self.texts.description_fertilizerAutoFruit, fillType.title, soilTypeName)
					else
						descriptionText = string.format(self.texts.description_fertilizerManualFruit, fillType.title, soilTypeName)
					end
				else
					descriptionText = self.texts.description_noFertilizerRequired
					enableZeroTargetFlag = true
				end
			end

			if descriptionText == "" and soilTypeName ~= "" then
				if spec.sprayAmountAutoMode then
					descriptionText = string.format(self.texts.description_fertilizerAutoNoFruit, soilTypeName)

					if self.nitrogenMap ~= nil then
						local fruitTypeIndex = self.nitrogenMap:getFruitTypeIndexByFruitRequirementIndex(spec.nApplyAutoModeFruitRequirementDefaultIndex)

						if fruitTypeIndex ~= nil then
							local _fillType = g_fillTypeManager:getFillTypeByIndex(g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(fruitTypeIndex))

							if _fillType ~= nil then
								descriptionText = string.format(self.texts.description_fertilizerAutoNoFruitDefault, _fillType.title, soilTypeName)
							end
						end
					end
				else
					descriptionText = string.format(self.texts.description_fertilizerManualNoFruit, soilTypeName)
				end
			end

			self.hasValidValues = true
		end

		if self.nitrogenMap ~= nil then
			minValue, maxValue = self.nitrogenMap:getMinMaxValue()
			local nAmount = spec.lastNitrogenProportion

			if nAmount == 0 then
				nAmount = self.nitrogenMap:getNitrogenAmountFromFillType(sprayFillType)
			end

			if spec.isSlurryTanker then
				local str = " (~%skgN/m³)"

				if sourceVehicle:getIsUsingExactNitrogenAmount() then
					str = " (%skgN/m³)"
				end

				applicationRateStr = applicationRateStr .. string.format(str, MathUtil.round(nAmount * 1000, 1))
			else
				applicationRateStr = applicationRateStr .. string.format(" (%s%%%%N)", MathUtil.round(nAmount * 100, 1))
			end

			stepResolution = self.nitrogenMap:getNitrogenFromChangedStates(1)
		end
	end

	if spec.sprayAmountAutoMode then
		applicationRateStr = applicationRateStr .. string.format(" (%s)", self.texts.automaticShort)
		soilTypeName = ""
	end

	self.actualPos = math.min((self.actualValue - minValue) / (maxValue - minValue), 1)
	self.setValuePos = math.min((self.setValue - minValue) / (maxValue - minValue), 1)
	self.targetPos = math.min((self.targetValue - minValue) / (maxValue - minValue), 1)
	local centerX = leftPosX + (rightPosX - leftPosX) * 0.5

	setTextColor(unpack(self.uiTextColor))
	setTextBold(true)
	setTextAlignment(RenderText.ALIGN_CENTER)
	renderDoubleText(centerX, upperPosY - self.textHeightHeadline * 1.1, self.textHeightHeadline, headline)
	setTextBold(false)

	local gradientPosX = centerX - self.gradientInactive.width * 0.5 + self.gradientPosX
	local gradientPosY = upperPosY + self.gradientPosY

	if not self.hasValidValues then
		gradientPosY = gradientPosY + self.actualBar.height - self.gradientInactive.height + self.textHeight
	end

	self.gradientInactive:setPosition(gradientPosX, gradientPosY)
	self.gradientInactive:render()

	local gradientVisibilePos = 0

	if self.hasValidValues then
		gradientVisibilePos = self.actualPos
	end

	self.gradient:setPosition(gradientPosX, gradientPosY)
	self.gradient:setDimension(gradientVisibilePos * self.gradientInactive.width)

	self.gradient.uvs[5] = self.gradientInactive.uvs[1] + (self.gradientInactive.uvs[5] - self.gradientInactive.uvs[1]) * gradientVisibilePos
	self.gradient.uvs[7] = self.gradientInactive.uvs[3] + (self.gradientInactive.uvs[7] - self.gradientInactive.uvs[3]) * gradientVisibilePos

	self.gradient:setUVs(self.gradient.uvs)
	self.gradient:render()

	local labelMin, labelMax = nil

	if hasLimeLoaded then
		labelMin = string.format("pH\n%s", minValue)
		labelMax = string.format("pH\n%s", maxValue)
	else
		labelMin = string.format("%s\nkg/ha", minValue)
		labelMax = string.format("%s\nkg/ha", maxValue)
	end

	local widthDiff = (rightPosX - leftPosX - self.gradientInactive.width) * 0.425

	renderDoubleText(gradientPosX - widthDiff, gradientPosY + self.gradientInactive.height * 0.85, self.gradientInactive.height * 1.3, labelMin)
	renderDoubleText(gradientPosX + self.gradientInactive.width + widthDiff, gradientPosY + self.gradientInactive.height * 0.85, self.gradientInactive.height * 1.3, labelMax)

	local additionalChangeLineHeight = 0
	local changeBarRendered = false

	if self.hasValidValues then
		local targetBarX, targetBarY = nil
		local showFlag = self.targetPos ~= 0 or enableZeroTargetFlag

		if showFlag then
			targetBarX = gradientPosX + self.gradientInactive.width * self.targetPos - self.targetBar.width * 0.5
			targetBarY = gradientPosY

			self.targetBar:setPosition(targetBarX, targetBarY)
			self.targetBar:render()
			self.targetFlag:setPosition(targetBarX, targetBarY + self.targetBar.height)
			self.targetFlag:render()
		end

		local actualBarText = nil
		local actualBarTextOffset = self.actualBar.height + self.textHeight * 1.1
		local actualBarSkipFlagCollisionCheck = false

		if self.actualPos ~= self.targetPos then
			actualBarText = string.format(self.texts.actualValue, string.format(self.actualValueStr, self.actualValue))
		elseif (spec.sprayAmountAutoMode or self.targetPos == self.setValuePos) and self.targetPos ~= 0 then
			actualBarText = string.format(self.texts.targetReached, string.format(self.actualValueStr, self.actualValue))
			actualBarTextOffset = -self.textHeight * 0.7
			actualBarSkipFlagCollisionCheck = true
			changeBarRendered = true
		end

		if actualBarText ~= nil then
			local actualBarX = gradientPosX + self.gradientInactive.width * self.actualPos - self.actualBar.width * 0.5
			local actualBarY = gradientPosY + (self.gradientInactive.height - self.actualBar.height) * 0.5

			self.actualBar:setPosition(actualBarX, actualBarY)
			self.actualBar:render()

			local actualTextWidth = getTextWidth(self.textHeight * 0.7, actualBarText)
			actualBarX = math.max(math.min(actualBarX, rightPosX - actualTextWidth * 0.5), leftPosX + actualTextWidth * 0.5)

			if not actualBarSkipFlagCollisionCheck and showFlag then
				local rightTextBorder = actualBarX + actualTextWidth * 0.5

				if targetBarX < rightTextBorder and rightTextBorder < targetBarX + self.targetFlag.width * 0.5 then
					actualBarX = targetBarX - actualTextWidth * 0.5 - self.pixelSizeX
				end

				local leftTextBorder = actualBarX - actualTextWidth * 0.5

				if targetBarX < leftTextBorder and leftTextBorder < targetBarX + self.targetFlag.width * 0.5 or leftTextBorder < targetBarX and targetBarX < rightTextBorder then
					actualBarX = targetBarX + self.targetFlag.width + self.pixelSizeX + actualTextWidth * 0.5
				end
			end

			renderDoubleText(actualBarX, actualBarY + actualBarTextOffset, self.textHeight * 0.7, actualBarText)
		end

		if self.actualPos < self.setValuePos then
			local goodColor = ExtendedSprayerHUDExtension.COLOR.SET_VALUE_BAR_GOOD
			local badColor = ExtendedSprayerHUDExtension.COLOR.SET_VALUE_BAR_BAD
			local difference = math.min(math.abs(self.setValue - self.targetValue) / stepResolution / 3, 1)
			local differenceInv = 1 - difference
			local r = difference * badColor[1] + differenceInv * goodColor[1]
			local g = difference * badColor[2] + differenceInv * goodColor[2]
			local b = difference * badColor[3] + differenceInv * goodColor[3]
			local a = 1
			local setValueBarX = gradientPosX + self.gradientInactive.width * self.actualPos
			local setValueBarY = gradientPosY - self.gradientInactive.height - self.setValueBar.height

			self.setValueBar:setPosition(setValueBarX, setValueBarY)
			self.setValueBar:setDimension(self.gradientInactive.width * (math.min(self.setValuePos, 1) - self.actualPos))
			self.setValueBar:setColor(r, g, b, a)
			self.setValueBar:render()

			local setBarTextX = setValueBarX + self.setValueBar.width * 0.5
			local setBarTextY = setValueBarY + self.setValueBar.height * 0.2
			local setTextWidth = getTextWidth(self.setValueBar.height * 0.9, changeBarText)

			if setTextWidth > self.setValueBar.width * 0.95 then
				setBarTextY = setValueBarY - self.setValueBar.height
				additionalChangeLineHeight = self.setValueBar.height
			end

			renderDoubleText(setBarTextX, setBarTextY, self.setValueBar.height * 0.9, changeBarText)

			changeBarRendered = true
		end
	else
		descriptionText = self.texts.invalidValues
	end

	if descriptionText ~= "" and self.additionalDisplayHeight ~= 0 then
		setTextAlignment(RenderText.ALIGN_CENTER)
		renderDoubleText(centerX, posY + self.footerOffset + self.additionalTextHeightOffset - self.textHeight * 0.2, self.textHeight * 0.85, descriptionText, self.footerSeparationBar.width)
	end

	self.footerSeparationBar:setPosition(centerX - self.footerSeparationBar.width * 0.5, posY + self.footerOffset + self.textHeight * 1.2)
	self.footerSeparationBar:render()
	setTextAlignment(RenderText.ALIGN_LEFT)
	renderDoubleText(leftPosX, posY + self.footerOffset, self.textHeight, self.texts.applicationRate .. " " .. string.format(applicationRateStr, applicationRate, applicationRateReal))

	if soilTypeName ~= "" then
		setTextAlignment(RenderText.ALIGN_RIGHT)
		renderDoubleText(rightPosX, posY + self.footerOffset, self.textHeight, string.format(self.texts.soilType, soilTypeName))
	end

	self.additionalDisplayHeight = additionalChangeLineHeight

	if descriptionText ~= "" then
		self.additionalDisplayHeight = self.additionalDisplayHeight + self.additionalTextHeightOffset
	end

	if not self.hasValidValues then
		self.additionalDisplayHeight = self.additionalDisplayHeight - self.invalidHeightOffset
	elseif not changeBarRendered then
		self.additionalDisplayHeight = self.additionalDisplayHeight - self.noSetBarHeightOffset
	end

	return posY
end

ExtendedSprayerHUDExtension.draw = Utils.overwrittenFunction(ExtendedSprayerHUDExtension.draw, TLExtendedSprayerHUDExtensionExtension.draw)