
-- override this to prevent spotpray to revert our special weed usage
TLExtendedWeedControlExtension = {}

function TLExtendedWeedControlExtension.updateHerbicideArea(startWorldX, superFunc, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, groundType)
-- print("test")
	local numPixels = 0
	local totalNumPixels = 0
	local weedSystem = g_currentMission.weedSystem

	if weedSystem:getMapHasWeed() then
		local functionData = FSDensityMapUtil.functionCache.updateHerbicideArea

		if functionData == nil then
			local terrainRootNode = g_currentMission.terrainRootNode
			local fieldGroundSystem = g_currentMission.fieldGroundSystem
			local groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.GROUND_TYPE)
			local sprayTypeMapId, sprayTypeFirstChannel, sprayTypeNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.SPRAY_TYPE)
			local weedMapId, weedFirstChannel, weedNumChannels = weedSystem:getDensityMapData()
			local firstSowableValue, lastSowableValue = fieldGroundSystem:getSowableRange()
			local replacementData = weedSystem:getHerbicideReplacements()
			functionData = {
				replacements = replacementData.weed.replacements,
				weedModifier = DensityMapModifier.new(weedMapId, weedFirstChannel, weedNumChannels, terrainRootNode),
				sprayModifier = DensityMapModifier.new(sprayTypeMapId, sprayTypeFirstChannel, sprayTypeNumChannels, terrainRootNode),
				groundFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels)
			}

			functionData.groundFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, firstSowableValue, lastSowableValue)

			-- to disable the stuff from ExtendedWeedControl we need to rename this. No other way is possible because extended there as inline function
			functionData.tlWeedFilters = {}

			for sourceState, _ in pairs(replacementData.weed.replacements) do
				local weedFilter = DensityMapFilter.new(weedMapId, weedFirstChannel, weedNumChannels, terrainRootNode)

				weedFilter:setValueCompareParams(DensityValueCompareType.EQUAL, sourceState)

				functionData.tlWeedFilters[sourceState] = weedFilter
			end

			functionData.fruitFilters = {}

			for index, desc in pairs(g_fruitTypeManager:getFruitTypes()) do
				if desc.terrainDataPlaneId ~= nil then
					local fruitFilter = DensityMapFilter.new(desc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels)
					functionData.fruitFilters[index] = fruitFilter
				end
			end

			if replacementData.custom ~= nil then
				functionData.multiModifierCustom = DensityMapMultiModifier.new()
				local fruitModifier, sourceStateFilter = nil

				for _, data in ipairs(replacementData.custom) do
					local desc = data.fruitType

					if desc.terrainDataPlaneId ~= nil then
						if fruitModifier == nil then
							fruitModifier = DensityMapModifier.new(desc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels, terrainRootNode)
						else
							fruitModifier:resetDensityMapAndChannels(desc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels, terrainRootNode)
						end

						if sourceStateFilter == nil then
							sourceStateFilter = DensityMapFilter.new(desc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels, terrainRootNode)
						else
							sourceStateFilter:resetDensityMapAndChannels(desc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels, terrainRootNode)
						end

						for sourceState, targetState in pairs(data.replacements) do
							sourceStateFilter:setValueCompareParams(DensityValueCompareType.EQUAL, sourceState)
							functionData.multiModifierCustom:addExecuteSet(targetState, fruitModifier, sourceStateFilter)
						end
					end
				end
			end

			FSDensityMapUtil.functionCache.updateHerbicideArea = functionData
		end

		local weedModifier = functionData.weedModifier
		local tlWeedFilters = functionData.tlWeedFilters
		local fruitFilters = functionData.fruitFilters
		local sprayModifier = functionData.sprayModifier
		local groundFilter = functionData.groundFilter
		local multiModifierCustom = functionData.multiModifierCustom

		weedModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)
		sprayModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)

		for sourceState, targetState in pairs(functionData.replacements) do
			local weedFilter = tlWeedFilters[sourceState]

			for index, fruitFilter in pairs(fruitFilters) do
				local desc = g_fruitTypeManager:getFruitTypeByIndex(index)

				fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, 1, desc.minHarvestingGrowthState - 1)
				sprayModifier:executeSet(groundType, fruitFilter, weedFilter)

				local _, numP, totalNumP = weedModifier:executeSet(targetState, fruitFilter, weedFilter)
				numPixels = numPixels + numP
				totalNumPixels = totalNumPixels + totalNumP

				-- Change note: In base game, you can still spray weeds between haulm topping and harvesting. We don't want this behavior for TL+.
				--              Additionally, `cutState + 1` is a very unreliable way for detecting the ready-to-harvest state, since it would include mulched states for other fruit, for example.
				--[[
				fruitFilter:setValueCompareParams(DensityValueCompareType.EQUAL, desc.cutState + 1)
				sprayModifier:executeSet(groundType, fruitFilter, weedFilter)

				_, numP, totalNumP = weedModifier:executeSet(targetState, fruitFilter, weedFilter)
				numPixels = numPixels + numP
				totalNumPixels = totalNumPixels + totalNumP
				]]--

				if desc.destruction ~= nil and desc.destruction.state ~= desc.cutState + 1 then
					fruitFilter:setValueCompareParams(DensityValueCompareType.EQUAL, desc.destruction.state)
					sprayModifier:executeSet(groundType, fruitFilter, weedFilter)

					_, numP, totalNumP = weedModifier:executeSet(targetState, fruitFilter, weedFilter)
					numPixels = numPixels + numP
					totalNumPixels = totalNumPixels + totalNumP
				end
			end

			sprayModifier:executeSet(groundType, groundFilter, weedFilter)

			local _, numP, totalNumP = weedModifier:executeSet(targetState, groundFilter, weedFilter)
			numPixels = numPixels + numP
			totalNumPixels = totalNumPixels + totalNumP
		end

		if multiModifierCustom ~= nil then
			multiModifierCustom:updateParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)
			multiModifierCustom:execute(false)
		end
	end

	return numPixels, totalNumPixels
end

FSDensityMapUtil.updateHerbicideArea = Utils.overwrittenFunction(FSDensityMapUtil.updateHerbicideArea, TLExtendedWeedControlExtension.updateHerbicideArea)