TLInGameMenuExtension = {
	MOD_NAME = g_currentModName,
	MOD_DIR = g_currentModDirectory,
	GUI_ELEMENTS_SIZE = {
		1024,
		1024
	},
	GUI_FRAME_REF_XML = g_currentModDirectory .. "gui/IngameMenuTLAnimalFrameReference.xml"
}
local TLInGameMenuExtension_mt = Class(TLInGameMenuExtension)

function TLInGameMenuExtension.new(customMt)
	local self = setmetatable({}, customMt or TLInGameMenuExtension_mt)

	return self
end

function TLInGameMenuExtension.sortPrecisionFarmingPage()
	local inGameMenu = g_gui.screenControllers[InGameMenu]
	
	local orderId = nil
	
	-- find original animal dialog
	for i = 1, #inGameMenu.pagingElement.elements do
		local child = inGameMenu.pagingElement.elements[i]

		if child == inGameMenu.pageFinances then
			orderId = i+1

			break
		end
	end
	
	if orderId == nil then
		return;
	end

	for i = 1, #inGameMenu.pagingElement.elements do
		local child = inGameMenu.pagingElement.elements[i]

		if child == inGameMenu.pageTLAnimals then
			table.remove(inGameMenu.pagingElement.elements, i)
			table.insert(inGameMenu.pagingElement.elements, orderId, child)

			break
		end
	end

	for i = 1, #inGameMenu.pagingElement.pages do
		local child = inGameMenu.pagingElement.pages[i]

		if child.element == inGameMenu.pageTLAnimals then
			table.remove(inGameMenu.pagingElement.pages, i)
			table.insert(inGameMenu.pagingElement.pages, orderId, child)

			break
		end
	end

	-- remove original animal dialog
	for i = 1, #inGameMenu.pagingElement.elements do
		local child = inGameMenu.pagingElement.elements[i]

		if child == inGameMenu.pageAnimals then
			table.remove(inGameMenu.pagingElement.elements, i)

			break
		end
	end

	for i = 1, #inGameMenu.pagingElement.pages do
		local child = inGameMenu.pagingElement.pages[i]

		if child.element == inGameMenu.pageAnimals then
			table.remove(inGameMenu.pagingElement.pages, i)

			break
		end
	end
end

function TLInGameMenuExtension.sortPrecisionFarmingPageFrame()
	local inGameMenu = g_gui.screenControllers[InGameMenu]
	
	local orderId = nil
	
	-- find original animal dialog
	for i = 1, #inGameMenu.pagingElement.elements do
		local child = inGameMenu.pagingElement.elements[i]

		if child == inGameMenu.pageFinances then
			orderId = i+1

			break
		end
	end
	
	if orderId == nil then
		return;
	end

	for i = 1, #inGameMenu.pageFrames do
		local child = inGameMenu.pageFrames[i]

		if child == inGameMenu.pageTLAnimals then
			table.remove(inGameMenu.pageFrames, i)
			table.insert(inGameMenu.pageFrames, orderId, child)

			break
		end
	end

	-- remove original animal dialog
	for i = 1, #inGameMenu.pageFrames do
		local child = inGameMenu.pageFrames[i]

		if child == inGameMenu.pageAnimals then
			table.remove(inGameMenu.pageFrames, i)

			break
		end
	end

	inGameMenu:rebuildTabList()
end

function TLInGameMenuExtension:overwriteGameFunctions(pfModule)
	local inGameMenu = g_gui.screenControllers[InGameMenu]
	local xmlFile = loadXMLFile("Temp", TLInGameMenuExtension.GUI_FRAME_REF_XML)

	if xmlFile ~= nil and xmlFile ~= 0 then
		for k, v in pairs({
			"pageTLAnimals"
		}) do
			inGameMenu.controlIDs[v] = nil
		end

		inGameMenu:registerControls({
			"pageTLAnimals"
		})
		g_gui:loadGuiRec(xmlFile, "inGameMenuTLAnimalsFrameReference", inGameMenu.pagingElement, self)
		inGameMenu:exposeControlsAsFields("pageTLAnimals")
		TLInGameMenuExtension.sortPrecisionFarmingPage()
		inGameMenu.pagingElement:updateAbsolutePosition()
		inGameMenu.pagingElement:updatePageMapping()
		delete(xmlFile)
	end
	
	self.inGameMenuTLAnimalsFrame = g_gui:resolveFrameReference(inGameMenu.pageTLAnimals)
	local inGameMenuTLAnimalsFrame = self.inGameMenuTLAnimalsFrame

	TLInGameMenuExtension.overwriteGameFunction(InGameMenu, "setPlayerFarm", function (superFunc, self, farm)
		inGameMenuTLAnimalsFrame:setPlayerFarm(farm)
		superFunc(self, farm)
	end)
	TLInGameMenuExtension.overwriteGameFunction(InGameMenu, "onLoadMapFinished", function (superFunc, self)
		-- inGameMenuTLAnimalsFrame:onLoadMapFinished()
		TLInGameMenuExtension.sortPrecisionFarmingPage()
		TLInGameMenuExtension.sortPrecisionFarmingPageFrame()
		superFunc(self)
	end)
	inGameMenu:registerPage(inGameMenu.pageTLAnimals, 19, function ()
		return true
	end)
	local iconFileName = Utils.getFilename('menuIconDEUKA.dds', TLInGameMenuExtension.MOD_DIR .. "gui/")
	inGameMenu:addPageTab(inGameMenu.pageTLAnimals, iconFileName, GuiUtils.getUVs({0,0,1024,1024}))
	inGameMenu.pageTLAnimals:applyScreenAlignment()
	inGameMenu.pageTLAnimals:updateAbsolutePosition()
	inGameMenu.pageTLAnimals:onGuiSetupFinished()
	TLInGameMenuExtension.sortPrecisionFarmingPageFrame()
	self.inGameMenuTLAnimalsFrame:initialize(inGameMenu.clickBackCallback)
end

function TLInGameMenuExtension.overwriteGameFunction(object, funcName, newFunc)
	if object == nil then
		Logging.error("Failed to overwrite '%s'", funcName)
		printCallstack()

		return
	end

	local oldFunc = object[funcName]

	if oldFunc ~= nil then
		object[funcName] = function (...)
			return newFunc(oldFunc, ...)
		end
	end
end

if g_gui ~= nil then
	local xmlGuiFileName = Utils.getFilename("gui/guiProfiles.xml", g_currentModDirectory)
	g_gui:loadProfiles(xmlGuiFileName)
	
	local frameController = TerraLifeInGameMenuAnimalsFrame.new(nil, g_messageCenter, g_i18n, g_fillTypeManager)

	local filename = Utils.getFilename("gui/TerraLifeInGameMenuAnimalsFrame.xml", g_currentModDirectory)
	g_gui:loadGui(filename, "TLAnimalsFrame", frameController, true)
end

-- neuer Animal Frames dann auch laden
local extension = TLInGameMenuExtension.new()
extension:overwriteGameFunctions(self)