TLMowerExtension = {}

function TLMowerExtension.getFillTypeLiterPerSqm(fruitTypeIndex, windrowFillTypeIndex)
	local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex)
	if fruitDesc.windrowLiterPerSqm ~= nil and fruitDesc.windrowLiterPerSqm ~= 0 then
		return fruitDesc.windrowLiterPerSqm
	else
		-- If the fruit type doesn't have liters per sqm defined, the output fill type might have (this is used for e.g. grass missions)
		local fillType = g_fillTypeManager.fillTypes[windrowFillTypeIndex]
		if fillType.windrowLiterPerSqm ~= nil then
			return fillType.windrowLiterPerSqm
		else
			return 1 -- Fallback, should in theory only happen if fruit types are configured incorrectly
		end
	end
end

function TLMowerExtension:processMowerArea(superFunc, workArea, dt)
	local spec = self.spec_mower
	local xs, _, zs = getWorldTranslation(workArea.start)
	local xw, _, zw = getWorldTranslation(workArea.width)
	local xh, _, zh = getWorldTranslation(workArea.height)

	if self:getLastSpeed() > 1 then
		spec.isWorking = true
		spec.stoneLastState = FSDensityMapUtil.getStoneArea(xs, zs, xw, zw, xh, zh)
	else
		spec.stoneLastState = 0
	end

	local limitToField = self:getIsAIActive()

	for inputFruitType, converterData in pairs(spec.fruitTypeConverters) do
		local changedArea, totalArea, sprayFactor, plowFactor, limeFactor, weedFactor, stubbleFactor, rollerFactor, _, growthState, _ = FSDensityMapUtil.updateMowerArea(inputFruitType, xs, zs, xw, zw, xh, zh, limitToField)

		if changedArea > 0 then
			local multiplier = g_currentMission:getHarvestScaleMultiplier(inputFruitType, sprayFactor, plowFactor, limeFactor, weedFactor, stubbleFactor, rollerFactor)
			changedArea = changedArea * multiplier
			local pixelToSqm = g_currentMission:getFruitPixelsToSqm()
			local sqm = changedArea * pixelToSqm

			-- FIX: Instead of finding the fruit type through the windrow fill tpye, access the already known input fruit type directly and use its windrowLiterPerSqm setting.
			-- Otherwise, the lookup would find the last fruit type which produces this windrow type, and might use the windrowLiterPerSqm value of the wrong fruit type.
			local litersToDrop = sqm * TLMowerExtension.getFillTypeLiterPerSqm(inputFruitType, converterData.fillTypeIndex)
			--local litersToDrop = sqm * g_fruitTypeManager:getFillTypeLiterPerSqm(converterData.fillTypeIndex, 1)

			workArea.lastPickupLiters = litersToDrop
			workArea.pickedUpLiters = litersToDrop
			local dropArea = self:getDropArea(workArea)

			if dropArea ~= nil then
				dropArea.litersToDrop = dropArea.litersToDrop + litersToDrop
				dropArea.fillType = converterData.fillTypeIndex
				dropArea.workAreaIndex = workArea.index

				if dropArea.fillType == FillType.GRASS_WINDROW then
					local lsx, lsy, lsz, lex, ley, lez, radius = DensityMapHeightUtil.getLineByArea(workArea.start, workArea.width, workArea.height, true)
					local pickup = nil
					pickup, workArea.lineOffset = DensityMapHeightUtil.tipToGroundAroundLine(self, -math.huge, FillType.DRYGRASS_WINDROW, lsx, lsy, lsz, lex, ley, lez, radius, nil, workArea.lineOffset or 0, false, nil, false)
					dropArea.litersToDrop = dropArea.litersToDrop - pickup
				end

				dropArea.litersToDrop = math.min(dropArea.litersToDrop, 1000)
			elseif spec.fillUnitIndex ~= nil and self.isServer then
				self:addFillUnitFillLevel(self:getOwnerFarmId(), spec.fillUnitIndex, litersToDrop, converterData.fillTypeIndex, ToolType.UNDEFINED)
			end

			spec.workAreaParameters.lastInputFruitType = inputFruitType
			spec.workAreaParameters.lastInputGrowthState = growthState
			spec.workAreaParameters.lastCutTime = g_time
			spec.workAreaParameters.lastChangedArea = spec.workAreaParameters.lastChangedArea + changedArea
			spec.workAreaParameters.lastStatsArea = spec.workAreaParameters.lastStatsArea + changedArea
			spec.workAreaParameters.lastTotalArea = spec.workAreaParameters.lastTotalArea + totalArea
			spec.workAreaParameters.lastUsedAreas = spec.workAreaParameters.lastUsedAreas + 1
		end
	end

	spec.workAreaParameters.lastUsedAreasSum = spec.workAreaParameters.lastUsedAreasSum + 1

	return spec.workAreaParameters.lastChangedArea, spec.workAreaParameters.lastTotalArea
end

Mower.processMowerArea = Utils.overwrittenFunction(Mower.processMowerArea, TLMowerExtension.processMowerArea)