

local WeedSpotSpray = _G['FS22_precisionFarming'].WeedSpotSpray

TLWeedSpotSprayExtension = {}

function TLWeedSpotSprayExtension:onLoad(superFunc, savegame)
	superFunc(self, savegame);
	
	local spec = self.spec_weedSpotSpray

	local weedSystem = g_currentMission.weedSystem
	local replacementData = weedSystem:getHerbicideReplacements()
	
	spec.weedDetectionStatesFruits = {}
	
	if replacementData.weed ~= nil then
		for _, data in ipairs(replacementData.custom) do
			spec.weedDetectionStatesFruits[data.fruitType.index] = {}
			for sourceState, targetState in pairs(data.replacements) do
				if targetState ~= 0 then
					spec.weedDetectionStatesFruits[data.fruitType.index][sourceState] = true
				end
			end
		end
	end
	-- TerraLiveExtension.DebugTable("weedDetectionStatesFruits", spec.weedDetectionStatesFruits, 2)
end

WeedSpotSpray.onLoad = Utils.overwrittenFunction(WeedSpotSpray.onLoad, TLWeedSpotSprayExtension.onLoad)

function TLWeedSpotSprayExtension:onUpdate(superFunc, dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local spec = self.spec_weedSpotSpray

	if spec.isAvailable then
		local sprayFillType = self.spec_sprayer.workAreaParameters.sprayFillType

		if spec.isEnabled then
			if self:getIsTurnedOn() then
				for i = 1, WeedSpotSpray.NOZZLE_UPDATES_PER_FRAME do
					local nozzleNode = spec.nozzleNodes[spec.currentUpdateIndex]

					if sprayFillType == FillType.HERBICIDE then
						local x, y, z = localToWorld(nozzleNode.node, 0, 0, nozzleNode.zOffset)
						local densityBits = getDensityAtWorldPos(spec.weedMapId, x, y, z)
						local weedState = bitAND(bitShiftRight(densityBits, spec.weedFirstChannel), 2^spec.weedNumChannels - 1)

						if spec.weedDetectionStates[weedState] then
							nozzleNode.lastActiveTime = g_time + spec.nozzleUpdateFrameDelay * dt * 1.5
							spec.effectsDirty = true
						end
						
						-- entension for non weed sprayFillType
						local fruitTypeIndex, growthState = FSDensityMapUtil.getFruitTypeIndexAtWorldPos(x, z)
						if spec.weedDetectionStatesFruits[fruitTypeIndex] ~= nil then
							if spec.weedDetectionStatesFruits[fruitTypeIndex][growthState] then
								nozzleNode.lastActiveTime = g_time + spec.nozzleUpdateFrameDelay * dt * 1.5
								spec.effectsDirty = true
							end
						end
						
					else
						local x, y, z = localToWorld(nozzleNode.node, 0, 0, nozzleNode.zOffset * 2)
						local densityBits = getDensityAtWorldPos(spec.sprayTypeMapId, x, y, z)
						local sprayType = bitAND(bitShiftRight(densityBits, spec.sprayTypeFirstChannel), 2^spec.sprayTypeNumChannels - 1)
						local densityBitsGround = getDensityAtWorldPos(spec.groundTypeMapId, x, y, z)
						local groundType = bitAND(bitShiftRight(densityBitsGround, spec.groundTypeFirstChannel), 2^spec.groundTypeNumChannels - 1)

						if groundType ~= 0 and sprayType ~= FieldSprayType.FERTILIZER then
							nozzleNode.lastActiveTime = g_time + spec.nozzleUpdateFrameDelay * dt * 1.5
							spec.effectsDirty = true
						end
					end

					spec.currentUpdateIndex = spec.currentUpdateIndex + 1

					if spec.currentUpdateIndex > #spec.nozzleNodes then
						spec.currentUpdateIndex = 1
					end
				end
			end

			self:updateNozzleEffects(dt, false)
		else
			self:updateNozzleEffects(dt, true, sprayFillType == FillType.UNKNOWN)
		end
	end
end

WeedSpotSpray.onUpdate = Utils.overwrittenFunction(WeedSpotSpray.onUpdate, TLWeedSpotSprayExtension.onUpdate)