TerraLifeInGameMenuAnimalsFrame = {}
local TerraLifeInGameMenuAnimalsFrame_mt = Class(TerraLifeInGameMenuAnimalsFrame, TabbedMenuFrameElement)
TerraLifeInGameMenuAnimalsFrame.CONTROLS = {
	REPRODUCTION_RATE_TEXT = "animalReproductionRateText",
	CLEANLINESS_ELEMENT = "cleanlinessElement",
	ANIMAL_TYPE_NAME_TEXT = "animalDetailTypeNameText",
	ANIMAL_VALUE_TEXT = "animalProductValue",
	AGE_TEXT = "animalAgeText",
	HEALTH_BAR_VALUE_TEXT = "healthValueText",
	CONDITION_VALUE_TEXTS = "conditionValue",
	FITNESS_ELEMENT = "fitnessElement",
	FOOD_VALUE_TEXTS = "foodValue",
	DETAIL_OUTPUT_BOX = "detailOutputBox",
	CONDITION_ROWS = "conditionRow",
	INFO_BARS = "infoStatusBar",
	REPRODUCTION_BAR = "reproductionStatusBar",
	NO_HUSBANDRIES_TEXT = "noHusbandriesBox",
	CLEANLINESS_BAR_VALUE_TEXT = "cleanlinessValueText",
	FITNESS_BAR_VALUE_TEXT = "fitnessValueText",
	REQUIREMENTS_LAYOUT = "requirementsLayout",
	FOOD_ROWS = "foodRow",
	INFO_ROWS = "infoRow",
	FOOD_BARS = "foodStatusBar",
	FITNESS_BAR = "fitnessStatusBar",
	DESCRIPTION_TEXT = "detailDescriptionText",
	LIVESTOCK_ATTRIBUTES_LAYOUT = "livestockAttributesLayout",
	ANIMAL_TYPE_VALUE_TEXT = "animalDetailTypeValueText",
	ANIMAL_TYPE_IMAGE = "animalDetailTypeImage",
	NO_ANIMALS_TEXT = "noAnimalsBox",
	ANIMAL_LIST = "list",
	FOOD_HEADER = "foodHeader",
	REPRODUCTION_ELEMENT = "reproductionElement",
	CONDITIONS_HEADER = "conditionsHeader",
	ANIMAL_BOX_TEMPLATE = "animalTemplate",
	TIME_UNTIL_REPRODUCTION_TEXT = "animalTimeTillNextAnimalText",
	TOTAL_FOOD_BAR = "foodRowTotalStatusBar",
	DETAILS_BOX = "detailsBox",
	TOTAL_FOOD_VALUE = "foodRowTotalValue",
	DETAIL_INPUT_BOX = "detailInputBox",
	CONDITION_BARS = "conditionStatusBar",
	CONDITION_LABEL_TEXTS = "conditionLabel",
	DETAIL_DESCRIPTION_BOX = "detailDescriptionBox",
	INFO_VALUE_TEXTS = "infoValue",
	HEALTH_BAR = "healthStatusBar",
	CLEANLINESS_BAR = "cleanlinessStatusBar",
	INFO_LABEL_TEXTS = "infoLabel",
	FOOD_LABEL_TEXTS = "foodLabel",
	REPRODUCTION_BAR_VALUE_TEXT = "reproductionValueText",
	ANIMAL_LIST_BOX = "animalsListBox",
	ANIMALS_CONTAINER = "animalsContainer",
	FOOD_LIST = "foodList",
	MONTH_SINCE_LAST_BIRTH_ROW = "monthSinceLastBirthRow",
	MONTH_SINCE_LAST_BIRTH_TEXT = "monthSinceLastBirthText",
	INSEMINTION_INFO_ROW = "inseminationInfoRow",
	INSEMINTION_INFO_TEXTS = "inseminationInfoText"
}
TerraLifeInGameMenuAnimalsFrame.UPDATE_INTERVAL = 5000
TerraLifeInGameMenuAnimalsFrame.HORSE_TYPE = "HORSE"
TerraLifeInGameMenuAnimalsFrame.CHICKEN_TYPE = "CHICKEN"
TerraLifeInGameMenuAnimalsFrame.ANIMAL_PRODUCT_FILL_TYPES = {
	EGG = "EGG",
	MILK = "MILK",
	WOOL = "WOOL"
}
TerraLifeInGameMenuAnimalsFrame.MAX_ANIMAL_NAME_LENGTH = 16

function TerraLifeInGameMenuAnimalsFrame.new(subclass_mt, messageCenter, l10n, fillTypeManager)
	-- print(string.format("TerraLifeInGameMenuAnimalsFrame:new(%s,%s,%s,%s)",subclass_mt, messageCenter, l10n, fillTypeManager));
	local self = TabbedMenuFrameElement.new(nil, subclass_mt or TerraLifeInGameMenuAnimalsFrame_mt)

	self:registerControls(TerraLifeInGameMenuAnimalsFrame.CONTROLS)

	self.messageCenter = messageCenter
	self.l10n = l10n
	self.fillTypeManager = fillTypeManager
	self.sortedHusbandries = {}
	self.foodInfos = {}
	self.selectedClusterIsHorse = false
	self.selectedCluster = nil
	self.selectedHusbandry = nil
	self.animalDataUpdateTime = TerraLifeInGameMenuAnimalsFrame.UPDATE_INTERVAL
	self.hasCustomMenuButtons = true
	self.renameButtonInfo = {}
	self.ejectColostrumMenuButton = nil

	return self
end

function TerraLifeInGameMenuAnimalsFrame.createFromExistingGui(gui, guiName)
	local newGui = TerraLifeInGameMenuAnimalsFrame.new(nil, gui.messageCenter, gui.l10n, gui.fillTypeManager)

	g_gui.frames[gui.name].target:delete()
	g_gui.frames[gui.name]:delete()
	g_gui:loadGui(gui.xmlFilename, guiName, newGui, true)

	return newGui
end

function TerraLifeInGameMenuAnimalsFrame:copyAttributes(src)
	-- print(string.format("TerraLifeInGameMenuAnimalsFrame:copyAttributes(%s,%s,%s)", src.messageCenter, src.l10n, src.fillTypeManager));
	TerraLifeInGameMenuAnimalsFrame:superClass().copyAttributes(self, src)

	self.messageCenter = src.messageCenter
	self.l10n = src.l10n
	self.fillTypeManager = src.fillTypeManager
end

function TerraLifeInGameMenuAnimalsFrame:onGuiSetupFinished()
	-- print("TerraLifeInGameMenuAnimalsFrame:onGuiSetupFinished()");
	TerraLifeInGameMenuAnimalsFrame:superClass().onGuiSetupFinished(self)
	self.list:setDataSource(self)
	self.list:setDelegate(self)
	self.foodList:setDataSource(self)
	self.foodList:setDelegate(self)
end

function TerraLifeInGameMenuAnimalsFrame:onFrameOpen()
	-- print("TerraLifeInGameMenuAnimalsFrame:onFrameOpen()");
	TerraLifeInGameMenuAnimalsFrame:superClass().onFrameOpen(self)
	self.messageCenter:subscribe(MessageType.HUSBANDRY_ANIMALS_CHANGED, self.onAnimalDataChanged, self)
	self.messageCenter:subscribe(MessageType.HUSBANDRY_SYSTEM_ADDED_PLACEABLE, self.updateHusbandries, self)
	self.messageCenter:subscribe(MessageType.HUSBANDRY_SYSTEM_REMOVED_PLACEABLE, self.updateHusbandries, self)
	self:updateHusbandries()
	self:updateMenuButtons()
	self:setSoundSuppressed(true)
	FocusManager:setFocus(self.list)
	self:setSoundSuppressed(false)
end

function TerraLifeInGameMenuAnimalsFrame:onFrameClose()
	-- print("TerraLifeInGameMenuAnimalsFrame:onFrameClose()")
	self.messageCenter:unsubscribe(MessageType.HUSBANDRY_ANIMALS_CHANGED, self)
	self.messageCenter:unsubscribe(MessageType.HUSBANDRY_SYSTEM_ADDED_PLACEABLE, self)
	self.messageCenter:unsubscribe(MessageType.HUSBANDRY_SYSTEM_REMOVED_PLACEABLE, self)
	TerraLifeInGameMenuAnimalsFrame:superClass().onFrameClose(self)
end

function TerraLifeInGameMenuAnimalsFrame:initialize()
	-- print("TerraLifeInGameMenuAnimalsFrame:initialize()")
	self.renameButtonInfo = {
		inputAction = InputAction.MENU_ACTIVATE,
		text = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.BUTTON_RENAME),
		callback = function ()
			self:onButtonRename()
		end
	}
	self.hotspotButtonInfo = {
		inputAction = InputAction.MENU_CANCEL,
		text = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.BUTTON_HOTSPOT),
		callback = function ()
			self:onButtonHotspot()
		end
	}
end

function TerraLifeInGameMenuAnimalsFrame:onAnimalDataChanged()
	self:updateAnimalData()
end

function TerraLifeInGameMenuAnimalsFrame:setPlayerFarm(playerFarm)
	self.playerFarm = playerFarm

	self:updateAnimalData()
end

function TerraLifeInGameMenuAnimalsFrame:update(dt)
	TerraLifeInGameMenuAnimalsFrame:superClass().update(self, dt)

	if self.selectedHusbandry ~= nil then
		self.animalDataUpdateTime = self.animalDataUpdateTime - dt

		if self.animalDataUpdateTime < 0 then
			self:updateAnimalData()
			self:displayCluster(self.selectedCluster, self.selectedHusbandry)

			self.animalDataUpdateTime = TerraLifeInGameMenuAnimalsFrame.UPDATE_INTERVAL
		end
	end
end

local function sortHusbandries(a, b)
	local aX, aZ, bX, bZ = nil
	local aTypeIndex = a:getAnimalTypeIndex()
	local bTypeIndex = b:getAnimalTypeIndex()

	if aTypeIndex ~= bTypeIndex then
		if aTypeIndex == AnimalType.HORSE then
			return false
		elseif bTypeIndex == AnimalType.HORSE then
			return true
		end
	end

	local aHotspot, bHotspot = nil

	if a.getHotspot ~= nil then
		aHotspot = a:getHotspot(1)
	end

	if b.getHotspot ~= nil then
		bHotspot = b:getHotspot(1)
	end

	aX, aZ = aHotspot:getWorldPosition()
	bX, bZ = bHotspot:getWorldPosition()

	if aX == bX then
		return aZ < bZ
	end

	return aX < bX
end

function TerraLifeInGameMenuAnimalsFrame:updateHusbandries()
	self.sortedHusbandries = g_currentMission.husbandrySystem:getPlaceablesByFarm(self.playerFarm.farmId)

	table.sort(self.sortedHusbandries, sortHusbandries)
	self:updateAnimalData()
end

function TerraLifeInGameMenuAnimalsFrame:updateAnimalData()
	local hasHusbandries = #self.sortedHusbandries > 0
	local hasAnimals = false

	for _, husbandry in ipairs(self.sortedHusbandries) do
		if husbandry:getNumOfClusters() > 0 then
			hasAnimals = true

			break
		end
	end

	self.animalsListBox:setVisible(hasHusbandries)
	self.detailsBox:setVisible(hasAnimals)
	self.noHusbandriesBox:setVisible(not hasHusbandries)
	self.noAnimalsBox:setVisible(hasHusbandries and not hasAnimals)
	self.list:reloadData()
	self.foodList:reloadData()
end

function TerraLifeInGameMenuAnimalsFrame:getMainElementSize()
	return self.animalsContainer.size
end

function TerraLifeInGameMenuAnimalsFrame:getMainElementPosition()
	return self.animalsContainer.absPosition
end

function TerraLifeInGameMenuAnimalsFrame:setStatusBarValue(statusBarElement, value, invertedBar, disabled, color)
	local profile = "ingameMenuAnimalsSmallStatusBarLow"
	local testValue = value

	if invertedBar then
		testValue = 1 - value
	end

	if TerraLifeInGameMenuAnimalsFrame.STATUS_BAR_MEDIUM < testValue and testValue <= TerraLifeInGameMenuAnimalsFrame.STATUS_BAR_HIGH then
		profile = "ingameMenuAnimalsSmallStatusBarMedium"
	elseif TerraLifeInGameMenuAnimalsFrame.STATUS_BAR_HIGH < testValue then
		profile = "ingameMenuAnimalsSmallStatusBar"
	end

	if color ~= nil then
		if color == "red" then
			profile = "ingameMenuAnimalsSmallStatusBarLow"
		elseif color == "orange" then
			profile = "ingameMenuAnimalsSmallStatusBarMedium"
		else
			profile = "ingameMenuAnimalsSmallStatusBar"
		end
	end

	statusBarElement:applyProfile(profile)

	local fullWidth = statusBarElement.parent.size[1] - statusBarElement.margin[1] * 2
	local minSize = 0

	if statusBarElement.startSize ~= nil then
		minSize = statusBarElement.startSize[1] + statusBarElement.endSize[1]
	end

	statusBarElement:setSize(math.max(minSize, fullWidth * math.min(1, value)), nil)

	disabled = Utils.getNoNil(disabled, false)

	statusBarElement:setDisabled(disabled)
end

function TerraLifeInGameMenuAnimalsFrame:updateConditionDisplay(husbandry)
	local infos = husbandry:getConditionInfos()

	for index, row in ipairs(self.conditionRow) do
		local info = infos[index]

		row:setVisible(info ~= nil)

		if info ~= nil then
			local valueText = info.valueText or self.l10n:formatVolume(info.value, 0, info.customUnitText)

			self.conditionLabel[index]:setText(info.title)
			self.conditionValue[index]:setText(valueText)
			self:setStatusBarValue(self.conditionStatusBar[index], info.ratio, info.invertedBar)
		end
	end
end

function TerraLifeInGameMenuAnimalsFrame:updateFoodDisplay(cluster, husbandry)
	self.foodInfos = husbandry:getFoodInfos(cluster)
	local infos = self.foodInfos;
	local totalCapacity = 0
	local totalValue = 0
	
	for index, info in ipairs(infos) do
		totalCapacity = info.capacity
		totalValue = totalValue + info.value
	end

	local totalValueText = self.l10n:formatVolume(totalValue, 0)
	local totalRatio = 0

	if totalCapacity > 0 then
		totalRatio = totalValue / totalCapacity
	end

	totalCapacity = self.l10n:formatVolume(totalCapacity, 0)
	-- hier mal nur die gesamtmenge an fütter für diesen cluster
	-- self.foodRowTotalValue:setText(totalValueText.."/"..totalCapacity.."/"..totalFillLevelText)
	self.foodRowTotalValue:setText(totalValueText)
	self:setStatusBarValue(self.foodRowTotalStatusBar, totalRatio, false)
	self.foodHeader:setText(string.format("%s (%s)", g_i18n:getText("info_fillLevel"), g_i18n:getText("animals_foodMixEffectiveness")))
end

function TerraLifeInGameMenuAnimalsFrame:displayCluster(cluster, husbandry)
	if not g_currentMission.isRunning then
		return
	end
	
	local subTypeIndex = cluster:getSubTypeIndex()
	local age = cluster:getAge()
	local visual = g_currentMission.animalSystem:getVisualByAge(subTypeIndex, age)

	if visual ~= nil then
		local name = visual.store.name

		if cluster.getName ~= nil then
			name = cluster:getName()
		end

		self.animalDetailTypeNameText:setText(name)
		self.animalDetailTypeImage:setImageFilename(visual.store.imageFilename)

		local value = cluster:getSellPrice()
		local priceText = self.l10n:formatMoney(value, 0, true, true)

		self.animalDetailTypeValueText:setText(priceText)

		local ageText = self.l10n:formatNumMonth(age)

		self.animalAgeText:setText(ageText)

        local monthSinceLastBirth = cluster:getMonthSinceLastBirth()
        self.monthSinceLastBirthRow:setVisible(monthSinceLastBirth ~= nil)
        if monthSinceLastBirth ~= nil then
            local monthSinceLastBirthText = self.l10n:formatNumMonth(monthSinceLastBirth)
            self.monthSinceLastBirthText:setText(monthSinceLastBirthText)
        end

        if not cluster:getSupportsReproduction() then
            self.inseminationInfoText:setText(g_i18n:getText("TL_ui_canNotBeInseminated"))
        elseif cluster:getIsInseminated() and cluster:getReproductionFactor() == 0 then
            self.inseminationInfoText:setText(g_i18n:getText("TL_ui_alreadyInseminated"))
        elseif not cluster:canBeInseminated() then
            self.inseminationInfoText:setText(g_i18n:getText("TL_ui_canNotBeInseminatedYet"))
        else
            self.inseminationInfoText:setText(g_i18n:getText("TL_ui_canBeInseminated"))
        end

		local infos = husbandry:getAnimalInfos(cluster)

		for index, row in ipairs(self.infoRow) do
			local info = infos[index]

			row:setVisible(info ~= nil)

			if info ~= nil then
				local valueText = info.valueText or self.l10n:formatVolume(info.value, 0, info.customUnitText)

				self.infoLabel[index]:setText(info.title)
				self.infoValue[index]:setText(valueText)
				self:setStatusBarValue(self.infoStatusBar[index], info.ratio, info.invertedBar, info.disabled)
			end
		end

		local animalDescriptionText = husbandry:getAnimalDescription(cluster)
	end

	self:updateConditionDisplay(husbandry)
	self:updateFoodDisplay(cluster, husbandry)
end

function TerraLifeInGameMenuAnimalsFrame:updateMenuButtons()
	self.menuButtonInfo = {
		{
			inputAction = InputAction.MENU_BACK
		}
	}

	if self.selectedClusterIsHorse then
		table.insert(self.menuButtonInfo, self.renameButtonInfo)
	end

	if self.selectedHusbandry ~= nil then
		local hotspot = nil

		if self.selectedHusbandry.getHotspot ~= nil then
			hotspot = self.selectedHusbandry:getHotspot(1)
		end

		if hotspot ~= nil then
			if hotspot == g_currentMission.currentMapTargetHotspot then
				self.hotspotButtonInfo.text = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.REMOVE_MARKER)
			else
				self.hotspotButtonInfo.text = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.SET_MARKER)
			end

			table.insert(self.menuButtonInfo, self.hotspotButtonInfo)
		end

		if self.selectedHusbandry.spec_husbandryFood ~= nil and #self.selectedHusbandry.spec_husbandryFood.ejectAreas ~= 0 then
			table.insert(self.menuButtonInfo, {
				profile = "buttonOk",
				inputAction = InputAction.MENU_EXTRA_2,
				text = g_i18n:getText("TL_eject_food"),
				callback = function()
					self.selectedHusbandry:ejectFood()
				end
			})
		elseif self.selectedHusbandry.spec_husbandryStraw ~= nil then
			table.insert(self.menuButtonInfo, {
				profile = "buttonOk",
				inputAction = InputAction.MENU_EXTRA_2,
				text = g_i18n:getText("TL_eject_food"),
				callback = function()
					self.selectedHusbandry:ejectFood("manure")
				end
			})
		else
			table.insert(self.menuButtonInfo, {
				profile = "buttonOk",
				inputAction = InputAction.TOGGLE_COVER,
				text = g_i18n:getText("TL_sell_feedingTrough"),
				callback = function()
					self.selectedHusbandry:ejectFood("sell")
				end
			})
		end

        if self.selectedCluster ~= nil and self.selectedCluster:canBeInseminated() then
            table.insert(self.menuButtonInfo, {
				profile = "buttonOk",
				inputAction = InputAction.MENU_ACCEPT,
				text = g_i18n:getText("TL_inseminate"),
				callback = function()
                    self:onButtonInsemination()
				end
			})
        end

		if self.selectedHusbandry.spec_husbandryColostrum ~= nil and self.selectedHusbandry.spec_husbandryColostrum.supportColostrum then
            local hasColostrumStorage = self.selectedHusbandry.spec_husbandry ~= nil and self.selectedHusbandry.spec_husbandry.loadingStation ~= nil and self.selectedHusbandry.spec_husbandry.loadingStation:getIsFillTypeSupported(FillType.COLOSTRUM)
            self.ejectColostrumMenuButton = {
				profile = "buttonOk",
				inputAction = InputAction.MENU_ACTIVATE,
				text = g_i18n:getText("TL_eject_colostrum"),
				callback = function()
					g_gui:showEjectColostrumDialog({
                        callback = self.ejectColostrumDialogCallback,
                        target = self,
                        hasColostrumStorage = hasColostrumStorage
                    })
				end
			}

			self.ejectColostrumMenuButton.disabled = self.selectedHusbandry:getFillTypeLevel(FillType.COLOSTRUM) == 0
			table.insert(self.menuButtonInfo, self.ejectColostrumMenuButton)
		end
	end

	self:setMenuButtonInfoDirty()
end

function TerraLifeInGameMenuAnimalsFrame:ejectColostrumDialogCallback(selectedOption, success)
    if not success then
        return
    end

    if selectedOption == EjectColostrumDialog.OPTIONS.EJECT_COLOSTRUM_TO_STORAGE then
        self.selectedHusbandry:ejectColostrumToStorage()
    elseif selectedOption == EjectColostrumDialog.OPTIONS.EJECT_COLOSTRUM_TO_SLURRY then
        self.selectedHusbandry:ejectColostrumToSlurry()
    else
        Logging.error("TerraLifeInGameMenuAnimalsFrame: Option "..selectedOption.." not implemented")
        return
    end

    self:updateConditionDisplay(self.selectedHusbandry)
    self:updateMenuButtons()
end

function TerraLifeInGameMenuAnimalsFrame:inseminateCluster(numAnimals, hasConfirmed)
    if hasConfirmed and numAnimals > 0 and self.selectedCluster ~= nil then
        self.selectedCluster:inseminate(numAnimals, true)
    end
end

function TerraLifeInGameMenuAnimalsFrame:renameCurrentHorse(newName, hasConfirmed)
	if hasConfirmed and self.selectedClusterIsHorse then
		self.selectedHusbandry:renameAnimal(self.selectedCluster.id, newName)
	end
end

function TerraLifeInGameMenuAnimalsFrame:getFoodDescription(animalTypeIndex)
	-- print(string.format("TerraLifeInGameMenuAnimalsFrame:getFoodDescription(%s)", animalTypeIndex));
	local animalFood = g_currentMission.animalFoodSystem:getAnimalFood(animalTypeIndex)
	local consumptionType = animalFood.consumptionType
	local foodGroups = animalFood.groups
	local foodDescription = nil

	if consumptionType == AnimalFoodSystem.FOOD_CONSUME_TYPE_PARALLEL then
		foodDescription = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.FOOD_DESCRIPTION_PARALLEL)
	else
		foodDescription = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.FOOD_DESCRIPTION_SERIAL)
	end

	return foodDescription
end

function TerraLifeInGameMenuAnimalsFrame:getNumberOfSections(list)
	if list == self.list then
		return #self.sortedHusbandries
	elseif list == self.foodList then
		return 1;
	end
end

function TerraLifeInGameMenuAnimalsFrame:getTitleForSectionHeader(list, section)
	if list == self.list then
		local husbandry = self.sortedHusbandries[section]
        local husbandryTitle = string.format(g_i18n:getText("TL_husbandry_title_withNumOfAnimals"), husbandry:getName(), husbandry:getNumOfAnimals(), husbandry:getMaxNumOfAnimals())
		return husbandryTitle
	elseif list == self.foodList then
		return "hiddenTitle";
	end
end

function TerraLifeInGameMenuAnimalsFrame:getNumberOfItemsInSection(list, section)
	if list == self.list then
		local husbandry = self.sortedHusbandries[section]

		return husbandry:getNumOfClusters()
	elseif list == self.foodList then
		return #self.foodInfos
	end
end

function TerraLifeInGameMenuAnimalsFrame:populateCellForItemInSection(list, section, index, cell)
		
	if list == self.list then
		local husbandry = self.sortedHusbandries[section]
		local cluster = husbandry:getCluster(index)
		local subTypeIndex = cluster:getSubTypeIndex()
		local age = cluster:getAge()
		local visual = g_currentMission.animalSystem:getVisualByAge(subTypeIndex, age)
		local subType = g_currentMission.animalSystem:getSubTypeByIndex(subTypeIndex)

		if visual ~= nil then
			local name = visual.store.name

			if cluster.getName ~= nil then
				name = cluster:getName()
			end

			local price = cluster:getSellPrice()
			local priceText = self.l10n:formatMoney(price, 0, true, true)

			cell:getAttribute("priceValue"):setText(priceText)
			cell:getAttribute("count"):setValue(cluster.numAnimals)
			cell:getAttribute("count"):setVisible(subType.typeIndex ~= AnimalType.HORSE)
			cell:getAttribute("name"):setText(name)
			cell:getAttribute("typeIcon"):setImageFilename(visual.store.imageFilename)
		end
	elseif list == self.foodList then
		local foodInfo = self.foodInfos[index]
		local valueText = self.l10n:formatVolume(foodInfo.value, 0)
		
		cell.elements[1].elements[1]:setText(foodInfo.title)
		cell.elements[1].elements[2]:setText(valueText)
		local color = "orange"
		if foodInfo.percents > 0 then
			if foodInfo.value > 1 then
				color = "green"
			else
				color = "red"
			end
		end
		self:setStatusBarValue(cell.elements[2].elements[1], foodInfo.ratio, foodInfo.invertedBar, nil, color)
	end
end

function TerraLifeInGameMenuAnimalsFrame:onButtonInsemination()
	local title = self.l10n:getText("TL_prompt_inseminate")
	local imePromptText = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.BUTTON_CANCEL)
	local confirmText = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.BUTTON_CONFIRM)

    g_gui:showNumPickerDialog({
		target = self,
		callback = self.inseminateCluster,
		title = title,
		imePrompt = imePromptText,
		confirmText = confirmText,
        maxNumAnimals = self.selectedCluster:getNumAnimals()
	})
end

function TerraLifeInGameMenuAnimalsFrame:onButtonRename()
	local promptText = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.PROMPT_RENAME)
	local imePromptText = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.IME_PROMPT_RENAME)
	local confirmText = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.BUTTON_CONFIRM)
	local activateInputText = self.l10n:getText(TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL.BUTTON_RENAME)

	g_gui:showTextInputDialog({
		target = self,
		callback = self.renameCurrentHorse,
		defaultText = self.selectedCluster:getName(),
		dialogPrompt = promptText,
		imePrompt = imePromptText,
		confirmText = confirmText,
		maxCharacters = TerraLifeInGameMenuAnimalsFrame.MAX_ANIMAL_NAME_LENGTH,
		activateInputText = activateInputText
	})
end

function TerraLifeInGameMenuAnimalsFrame:onButtonHotspot()
	local husbandry = self.selectedHusbandry
	local hotspot = nil

	if self.selectedHusbandry.getHotspot ~= nil then
		hotspot = self.selectedHusbandry:getHotspot(1)
	end

	if husbandry and hotspot ~= nil then
		if hotspot == g_currentMission.currentMapTargetHotspot then
			g_currentMission:setMapTargetHotspot()
		else
			g_currentMission:setMapTargetHotspot(hotspot)
		end

		self:updateMenuButtons()
	end
end

function TerraLifeInGameMenuAnimalsFrame:onListSelectionChanged(list, section, index)
	if list == self.list then
		local husbandry = self.sortedHusbandries[section]
		local cluster = husbandry:getCluster(index)
		local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster:getSubTypeIndex())
		self.selectedHusbandry = husbandry
		local isHorse = subType.typeIndex == AnimalType.HORSE
		self.selectedClusterIsHorse = isHorse
		self.selectedCluster = cluster

		self:displayCluster(self.selectedCluster, husbandry)
		self.livestockAttributesLayout:invalidateLayout()
		self:updateMenuButtons()
		
		self.foodInfos = husbandry:getFoodInfos(cluster)
		self.foodList:reloadData()
	elseif list == self.foodList then
		-- local description = self.l10n:getText("TL_youCanFeed") .. ": ";
		
		local foodInfo = self.foodInfos[index];
		-- local first = true;
		
		-- TerraLiveExtension.DebugTable("foodInfo", foodInfo)
		
		-- for fillTypeIndex, _  in pairs(foodInfo.fillTypes) do
			-- local fillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex);
			-- if string.find(fillType.name:upper(), "SLIDER") then
				-- goto skip
			-- end
			
			-- if first then
				-- first = false;
			-- else
				-- description = description .. ", ";
			-- end
			
			-- description = description .. fillType.title;
			
			-- ::skip::
		-- end
		
		
		
		-- self.detailDescriptionText:setText(description)
		self.detailDescriptionText:setText(foodInfo.info)
	end
end

TerraLifeInGameMenuAnimalsFrame.FILL_TYPE_SEPARATOR = " / "
TerraLifeInGameMenuAnimalsFrame.L10N_SYMBOL = {
	CLEANLINESS = "statistic_cleanliness",
	BUTTON_CONFIRM = "button_confirm",
	BUTTON_CANCEL = "button_cancel",
	REMOVE_MARKER = "action_untag",
	FOOD_DESCRIPTION_PARALLEL = "animals_foodMixDescriptionParallel",
	FOOD_DESCRIPTION_SERIAL = "animals_foodMixDescriptionSerial",
	IME_PROMPT_RENAME = "ui_horseName",
	BUTTON_HOTSPOT = "button_showOnMap",
	FOOD_MIX_QUANITITY = "animals_foodMixQuantity",
	WATER = "statistic_water",
	PROMPT_RENAME = "ui_enterHorseName",
	HORSE_FITNESS = "ui_horseFitness",
	BUTTON_RENAME = "button_rename",
	SET_MARKER = "action_tag",
	STRAW = "statistic_strawStorage"
}
TerraLifeInGameMenuAnimalsFrame.STATUS_BAR_HIGH = 0.66
TerraLifeInGameMenuAnimalsFrame.STATUS_BAR_MEDIUM = 0.33
