-- overriding animal data via XML file 

-- by modelleicher (Farming Agency)
-- reworked for TerraLife Plus by [F/A] Braeven 17.06.2023
-- can now load different food files from the main mod [F/A]Braeven 17.07.2023
-- bugfixes, some rework and more error-messages [F/A]Braeven 17.09.2023
-- rewritten to work with multiple animal subtypes files [F/A]Braeven 23.09.2023

AnimalSystem.SEND_NUM_BITS = 5

animalDataOverride = {}

-- auslesen der max werte pro tier für Trog Anpassung
animalDataOverride.TLCorrectHubandriesCapacities = {}

local modDirectory = g_currentModDirectory

-- add XML Schema
g_xmlManager:addCreateSchemaFunction(function ()
	animalDataOverride.xmlSchema = XMLSchema.new("animalDataOverride")
	local schema = animalDataOverride.xmlSchema
	
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?)#type", "Type of Animal this Dataset overrides", nil)	 
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?)#subType", "Sub-Type Animal for this Dataset", "COW_SWISS_BROWN")			
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?).inputs#overrideType", "The type of override, 'add' will only add/replace for this input-type while 'replace' will replace all the inputs with the new ones.", nil)
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?).outputs#overrideType", "The type of override, 'add' will only add/replace for this output-type while 'replace' will replace all the outputs with the new ones.", nil)
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?).inputs.input(?)#type", "Input-Type most likely Filltype", "STRAW")
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?).outputs.output(?)#type", "Output-Type most likely Filltype", "MILK")
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?).inputs.input(?).key(?)#ageMonth", "Age-Month for this particular animation-key element", "0")
	schema:register(XMLValueType.INT, "animalDataOverride.animalData(?).inputs.input(?).key(?)#value", "Value this particular animation-key element", "0")
	schema:register(XMLValueType.STRING, "animalDataOverride.animalData(?).outputs.output(?).key(?)#ageMonth", "Age-Month for this particular animation-key element", "0")
	schema:register(XMLValueType.INT, "animalDataOverride.animalData(?).outputs.output(?).key(?)#value", "Value this particular animation-key element", "0")
end)

function animalDataOverride.loadAnimalData(self, superFunc, xmlFile, baseDir)
	local function loadAFAOverwrite(path)
		print("TerraLife Plus: Loading AFA-File: "..path)
		local xml = XMLFile.load("animalDataOverride", path , animalDataOverride.xmlSchema)

		xml:iterate("animalDataOverride.animalData", function (animalDataIndex, curPath)
			local animalTypeName = xml:getValue(curPath.."#type")
			-- TerraLiveExtension.DebugText("animalTypeName: %s", animalTypeName)
		
			-- Troganpassung
			if animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName] == nil then
				animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName] = {};
			end
			
			-- check if AnimalType exists
			if animalTypeName == nil then
				Logging.xmlError(xml, "TerraLife Plus: Animal-Type is NIL for Animal-Data: "..animalDataIndex..". Loading AnimalData was skipped.")
			elseif self.nameToType[animalTypeName:upper()] == nil then
				Logging.xmlError(xml, "TerraLife Plus: Animal-Type does not exist for "..animalTypeName..". Loading AnimalData was skipped.")
			else
				-- check if AnimalSubType exists
				local subTypeName = xml:getValue(curPath.."#subType")
				-- TerraLiveExtension.DebugText("subTypeName: %s", subTypeName)
				if subTypeName == nil then
					Logging.xmlError(xml, "TerraLife Plus: SubAnimal-Type is NIL for Animal-Data: "..animalDataIndex..". Loading AnimalData was skipped.")
				elseif self.nameToSubType[subTypeName:upper()] == nil then
					Logging.xmlError(xml, "TerraLife Plus: SubAnimal-Type does not exist for: "..subTypeName..". Loading AnimalData was skipped.")
				else
					local subType = self.nameToSubType[subTypeName:upper()]

					-- go througn inputs
					local overrideTypeInput = xml:getValue(curPath..".inputs#overrideType")
					if overrideTypeInput ~= nil then
						if string.upper(overrideTypeInput) == "REPLACE" then
							subType.input = {}
						end

						xml:iterate(curPath..".inputs.input", function (inputIndex, inputPath)
							local inputType = xml:getValue(inputPath.."#type")
							if inputType == nil then
								Logging.xmlError(xml, "TerraLife Plus: inputType is NIL for Animal-Data: "..animalDataIndex..", Animal Type "..animalTypeName..", SubType "..subTypeName..". Loading AnimalData was skipped.")
							else	 
								subType.input[inputType] = self:loadAnimCurve(xml, inputPath)
								
								-- Troganpassung
								inputType = inputType:upper()
								if animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][inputType] == nil then
									animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][inputType] = 0;
								end
								xml:iterate(inputPath .. ".key", function (_, keyInputsKeys)
									local inputValue = xml:getValue(keyInputsKeys .. "#value");
									-- Logging.info("inputValue: %s", inputValue);
									if inputValue >= animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][inputType] then
										animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][inputType] = inputValue;
									end
								end)
							end
						end)
					end
					-- go through outputs
					local overrideTypeOutput = xml:getValue(curPath..".outputs#overrideType")
					-- TerraLiveExtension.DebugText("overrideTypeOutput: %s", overrideTypeOutput)
					if overrideTypeOutput ~= nil then
						if string.upper(overrideTypeOutput) == "REPLACE" then
							subType.output = {}
						end

						xml:iterate(curPath..".outputs.output", function (outputIndex, outputPath)
							local outputType = xml:getValue(outputPath.."#type")
							-- TerraLiveExtension.DebugText("outputType: %s", outputType)
							if outputType == nil then
								Logging.xmlError(xml, "TerraLife Plus: outputType is NIL for Animal-Data: "..animalDataIndex..", Animal Type "..animalTypeName..", SubType "..subTypeName..". Loading AnimalData was skipped.")
							else
								subType.output[outputType] = self:loadAnimCurve(xml, outputPath)
								
								
								-- Troganpassung
								outputType = outputType:upper()
								if animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][outputType] == nil then
									animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][outputType] = 0;
								end
								xml:iterate(outputPath .. ".key", function (_, keyInputsKeys)
									local outputValue = xml:getValue(keyInputsKeys .. "#value");
									-- Logging.info("outputValue: %s", outputValue);
									if outputValue >= animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][outputType] then
										animalDataOverride.TLCorrectHubandriesCapacities[animalTypeName][outputType] = outputValue;
									end
								end)
							end
						end)
					end
					-- TerraLiveExtension.DebugTable("subType", subType)
				end
			end
		end)
	end

	xmlFile:iterate("animals.animal", function (_, key)
		if #self.types >= 2^AnimalSystem.SEND_NUM_BITS - 1 then
			Logging.xmlWarning(xmlFile, "Maximum number of supported animal types reached. Ignoring remaining types")

			return false
		end

		local typeName = xmlFile:getString(key .. "#type")

		if typeName == nil then
			Logging.xmlError(xmlFile, "Missing animal type. '%s'", key)

			return false
		end

		typeName = typeName:upper()

		local configFilename = xmlFile:getString(key .. ".configFilename")

		if configFilename == nil then
			Logging.xmlError(xmlFile, "Missing config file for animal type '%s'. '%s'", typeName, key)

			return false
		end

		local clusterClassName = xmlFile:getString(key .. "#clusterClass")

		if clusterClassName == nil then
			Logging.xmlError(xmlFile, "Missing animal clusterClass for '%s'!", key)

			return false
		end

		local statsBreedingName = xmlFile:getString(key .. "#statsBreeding")
		local modEnvMeta = getmetatable(_G)
		local env = modEnvMeta.__index
		
		if not env.ClassUtil.getIsValidClassName(clusterClassName) then
			Logging.xmlError(xmlFile, "Invalid animal clusterClass name '%s' for '%s'!", tostring(clusterClassName), key)

			return false
		end

		if env.ClassUtil.getClassObject(clusterClassName) == nil then
			Logging.xmlError(xmlFile, "Unknown animal clusterClass '%s' for '%s'!", tostring(clusterClassName), key)

			return false
		end

		local animalType = {
			name = typeName,
			typeIndex = #self.types + 1,
			configFilename = configFilename,
			clusterClass = env.ClassUtil.getClassObject(clusterClassName),
			statsBreedingName = statsBreedingName,
			subTypes = {}
		}

		if self.nameToType[typeName] ~= nil then
		  animalType = self.nameToType[typeName]
		end

		self:loadAnimalConfig(animalType, baseDirectory)

		if self:loadSubTypes(animalType, xmlFile, key, baseDirectory) then
			if self.nameToType[typeName] == nil then
				table.insert(self.types, animalType)
			end

			self.nameToType[typeName] = animalType
			self.nameToTypeIndex[typeName] = animalType.typeIndex
			self.typeIndexToName[animalType.typeIndex] = typeName
		end
	end)

	local mod = g_modManager:getModByName("FS22_TerraLifePlus_animalFoodAdditions")
	local own = g_modManager:getModByName("FS22_TerraLifePlus_animalFoodAdditions_own")
	local AFA = false
	local ownAFA = false
	
	if mod ~= nil then
		AFA = g_modIsLoaded["FS22_TerraLifePlus_animalFoodAdditions"]
	end
	if own ~= nil then
		ownAFA = g_modIsLoaded["FS22_TerraLifePlus_animalFoodAdditions_own"]
	end

	if ownAFA == true and fileExists(own.modDir .. AnimalSystem.currentSubType) then
		loadAFAOverwrite(own.modDir .. AnimalSystem.currentSubType)
	elseif AFA == true and fileExists(mod.modDir .. AnimalSystem.currentSubType) then
		loadAFAOverwrite(mod.modDir .. AnimalSystem.currentSubType)
	else
		print("TerraLife Plus Error: AnimalDataOverride could not be loaded: " ..mod.modDir..AnimalSystem.currentSubType)
	end
	
	-- TerraLiveExtension.DebugTable("TLCorrectHubandriesCapacities", animalDataOverride.TLCorrectHubandriesCapacities)
end

AnimalSystem.loadAnimals = Utils.overwrittenFunction(AnimalSystem.loadAnimals, animalDataOverride.loadAnimalData)