AnimalSystemExtension = {}

function AnimalSystemExtension:loadAnimalConfig(superFunc, animalType, baseDirectory)
    local returnValue = superFunc(self, animalType, baseDirectory)

	local configFilename = Utils.getFilename(animalType.configFilename, baseDirectory)
	local xmlFile = XMLFile.load("animalsConfig", configFilename)

	if xmlFile == nil then
		return false
	end

	xmlFile:iterate("animalHusbandry.animals.animal", function (index, key)
        local animal = animalType.animals[index]

		local audio = {
			eatSoundGroup = xmlFile:getString(key .. ".audio#eatSoundGroup"),
			yellSoundGroup = xmlFile:getString(key .. ".audio#yellSoundGroup"),
			walkSoundGroup = xmlFile:getString(key .. ".audio#walkSoundGroup"),
		}

		animal.audio = audio
	end)

    animalType.sound = {}

    xmlFile:iterate("animalHusbandry.sound.soundGroup", function (_, key)
		local soundGroup = {
			name = xmlFile:getString(key .. "#name"),
			volume = xmlFile:getFloat(key .. "#volume"),
			indoorVolume = xmlFile:getFloat(key .. "#indoorVolume"),
			range = xmlFile:getInt(key .. "#range"),
			innerRange = xmlFile:getFloat(key .. "#innerRange"),
            samples = {}
		}

        xmlFile:iterate(key .. ".sample", function (_, key)
            local sample = {
                filename = Utils.getFilename(xmlFile:getString(key .. "#filename"), baseDirectory)
            }

            table.insert(soundGroup.samples, sample)
        end)

		table.insert(animalType.sound, soundGroup)
	end)


	xmlFile:delete()

    return returnValue
end

AnimalSystem.loadAnimalConfig = Utils.overwrittenFunction(AnimalSystem.loadAnimalConfig, AnimalSystemExtension.loadAnimalConfig)

function AnimalSystemExtension:loadVisualData(superFunc, animalType, xmlFile, key, baseDirectory)
    local returnValue = superFunc(self, animalType, xmlFile, key, baseDirectory)

	local visualAnimalIndex = xmlFile:getInt(key .. "#visualAnimalIndex")
	local animal = animalType.animals[visualAnimalIndex]
    returnValue.audio = animal.audio

    return returnValue
end

AnimalSystem.loadVisualData = Utils.overwrittenFunction(AnimalSystem.loadVisualData, AnimalSystemExtension.loadVisualData)

function AnimalSystem:getSound(animalTypeIndex)
    local animalType = self:getTypeByIndex(animalTypeIndex)
	local sound = animalType.sound

	return sound
end