TLAnimalFoodSystem = {}
local TLAnimalFoodSystem_mt = Class(TLAnimalFoodSystem)

g_xmlManager:addInitSchemaFunction(function ()
	local schema = AnimalFoodSystem.xmlSchema

	schema:register(XMLValueType.STRING, "animalFood.animals.animal(?)#animalSubType", "Animal sub type name")
	schema:register(XMLValueType.INT, "animalFood.animals.animal(?)#minAge", "min age for this feed grouping")
	schema:register(XMLValueType.FLOAT, "animalFood.animals.animal(?).foodGroup(?)#milkBonus", "Food group milk bonus when added")
	schema:register(XMLValueType.FLOAT, "animalFood.animals.animal(?).foodGroup(?)#fatteningBonus", "Food group fattening bonus when added")
end)

function TLAnimalFoodSystem.new(self, superFunc, mission, customMt)
	-- print(string.format("TLAnimalFoodSystem.new(%s, %s, %s, %s)", self, superFunc, mission, customMt));
	
	local self = superFunc(self, mission, customMt or TLAnimalFoodSystem_mt)
	
	return self
end

function TLAnimalFoodSystem:loadAnimalFood(superFunc, xmlFile)
	-- TerraLiveExtension.DebugText("TLAnimalFoodSystem.loadAnimalFood(%s, %s)", superFunc, xmlFile);
	
	xmlFile:iterate("animalFood.animals.animal", function (_, key)
		local animalTypeName = xmlFile:getValue(key .. "#animalType")
		local animalTypeIndex = self.mission.animalSystem:getTypeIndexByName(animalTypeName)
		
		local animalSubType = xmlFile:getValue(key .. "#animalSubType")
		local animalSubTypeIndex = nil;
		if animalSubType ~= nil then
			animalSubTypeIndex = self.mission.animalSystem:getSubTypeIndexByName(animalSubType);
		end
		
		local minAge = xmlFile:getValue(key .. "#minAge") or 0

		if animalTypeIndex ~= nil then
			local animalFood = {}

			if self:loadAnimalFoodData(animalFood, xmlFile, key) then
				animalFood.index = #self.animalFood + 1
				animalFood.animalTypeIndex = animalTypeIndex
				animalFood.animalSubTypeIndex = animalSubTypeIndex
				animalFood.minAge = minAge

				table.insert(self.animalFood, animalFood)

				self.indexToAnimalFood[animalFood.index] = animalFood
				self.animalTypeIndexToFood[animalTypeIndex] = animalFood
			end
		else
			Logging.xmlWarning(xmlFile, "Animal type '%s' not defined for foodgroup '%s'", animalTypeName, key)
		end
	end)
	
	-- TerraLiveExtension.DebugTable("animalFood1", self.animalFood, 3)

	for _, animalFood in pairs(self.animalFood) do
		if animalFood.consumptionType == AnimalFoodSystem.FOOD_CONSUME_TYPE_PARALLEL then
			local sumWeigths = 0
			local eatWeights = 0

			for _, foodGroup in pairs(animalFood.groups) do
				sumWeigths = sumWeigths + foodGroup.productionWeight
				eatWeights = eatWeights + foodGroup.eatWeight
			end

			for _, foodGroup in pairs(animalFood.groups) do
				if sumWeigths > 0 then
					foodGroup.productionWeight = foodGroup.productionWeight / sumWeigths
				end

				if eatWeights > 0 then
					foodGroup.eatWeight = foodGroup.eatWeight / eatWeights
				end
			end
		end
	end
	
	-- TerraLiveExtension.DebugTable("animalFood2", self.animalFood, 3)

	return true
end

function TLAnimalFoodSystem:getAnimalFood(superFunc, animalTypeIndex, cluster)
	-- print(string.format("TLAnimalFoodSystem:getAnimalFood(%s, %s, %s)", superFunc, animalTypeIndex, cluster));

	-- hier alle gruppen für den animalTypeIndex zusammenfassen
	-- den ersten gefundenen nehme ich als basis

	local newAnimalFood = nil
	local foodGroupAge = {}
	local foodGroupData = {}

	-- wenn kein cluster vorhanden, komplett übertragen
	-- wenn cluster vorhanden, nur relevante Daten übertragen
	for _, animalFood in pairs(self.animalFood) do
		if animalFood.animalTypeIndex == animalTypeIndex then
			if cluster == nil or (cluster.subTypeIndex == animalFood.animalSubTypeIndex) then
				if newAnimalFood == nil then
					newAnimalFood = {};
					newAnimalFood.animalTypeIndex = animalFood.animalTypeIndex;
					newAnimalFood.groups = {};
					newAnimalFood.consumptionType = animalFood.consumptionType
				end

				-- zusammenbauen
				for _, foodGroup in pairs(animalFood.groups) do
					local newFoodGroup = {
						title = foodGroup.title,
						info = foodGroup.info,
						productionWeight = 0,
						eatWeight = 0,
						milkBonus = 0,
						fatteningBonus = 0,
						fillTypes = {}
					}
					for _, fillTypeIndex in pairs(foodGroup.fillTypes) do
						table.insert(newFoodGroup.fillTypes, fillTypeIndex)
					end

					if cluster == nil then
						table.insert(newAnimalFood.groups, newFoodGroup)
					elseif foodGroupAge[foodGroup.title] == nil then
						foodGroupAge[foodGroup.title] = animalFood.minAge
						foodGroupData[foodGroup.title] = newFoodGroup
					elseif animalFood.minAge <= cluster.age and animalFood.minAge > foodGroupAge[foodGroup.title] then
						--Daten ersetzen wenn besser passender Datensatz vorhanden ist
						foodGroupAge[foodGroup.title] = animalFood.minAge
						foodGroupData[foodGroup.title] = newFoodGroup
					end
				end
			end
		end
	end

	if cluster ~= nil then
		for _, foodGroup in pairs(foodGroupData) do
			table.insert(newAnimalFood.groups, foodGroup)
		end
	end

	-- welches food für den aktuellen cluster?
	local bestFitAnimalFood = nil;
	local fallBackAnimalFood = nil;
	if cluster ~= nil then
		local animalSubTypeIndex = cluster.subTypeIndex;
		local age = cluster.age;
		for _, animalFood in pairs(self.animalFood) do
			if animalFood.animalTypeIndex == animalTypeIndex then

				if animalFood.animalSubTypeIndex ~= nil and animalFood.animalSubTypeIndex == animalSubTypeIndex then
					if animalFood.minAge <= age then
						if bestFitAnimalFood == nil then
							bestFitAnimalFood = animalFood;
						else
							if animalFood.minAge > bestFitAnimalFood.minAge then
								bestFitAnimalFood = animalFood;
							end
						end
					end
				end

				if animalFood.animalSubTypeIndex == nil then
					fallBackAnimalFood = animalFood;
				end
			end
		end
	end

	if bestFitAnimalFood == nil then
		bestFitAnimalFood = fallBackAnimalFood;
	end

	if bestFitAnimalFood ~= nil then
		for _, foodGroupBest in pairs(bestFitAnimalFood.groups) do
			for _, foodGroupNew in pairs(newAnimalFood.groups) do
				if foodGroupBest.title == foodGroupNew.title then
					foodGroupNew.productionWeight = foodGroupBest.productionWeight;
					foodGroupNew.eatWeight = foodGroupBest.eatWeight;
					foodGroupNew.milkBonus = foodGroupBest.milkBonus;
					foodGroupNew.fatteningBonus = foodGroupBest.fatteningBonus;
				end
			end
		end
	end

	return newAnimalFood;
end

function TLAnimalFoodSystem:loadAnimalFoodData(superFunc, animalFood, xmlFile, key)
	-- print(string.format("TLAnimalFoodSystem:loadAnimalFoodData(%s, %s, %s, %s)", superFunc, animalFood, xmlFile, key));
	
	local consumptionType = AnimalFoodSystem.FOOD_CONSUME_TYPE_SERIAL
	local consumptionTypeName = xmlFile:getValue(key .. "#consumptionType", "SERIAL")

	if consumptionTypeName:upper() == "PARALLEL" then
		consumptionType = AnimalFoodSystem.FOOD_CONSUME_TYPE_PARALLEL
	end

	local groups = {}
	local usedFillTypes = {}

	xmlFile:iterate(key .. ".foodGroup", function (_, foodGroupKey)
		local title = xmlFile:getValue(foodGroupKey .. "#title")

		if title == nil then
			Logging.xmlError(xmlFile, "Missing title for animal food group '%s'", foodGroupKey)

			return false
		end

		local foodGroup = {
			title = g_i18n:convertText(title, self.customEnvironment),
			info = g_i18n:convertText(title .. "_info", self.customEnvironment),
			productionWeight = MathUtil.round(xmlFile:getValue(foodGroupKey .. "#productionWeight", 0), 3),
			eatWeight = MathUtil.round(xmlFile:getValue(foodGroupKey .. "#eatWeight", 1), 3),
			milkBonus = xmlFile:getValue(foodGroupKey .. "#milkBonus"),
			fatteningBonus = xmlFile:getValue(foodGroupKey .. "#fatteningBonus"),
			fillTypes = {}
		}

		if self:getFillTypesFromXML(foodGroup.fillTypes, usedFillTypes, xmlFile, foodGroupKey .. "#fillTypes") then
			table.insert(groups, foodGroup)
		end
	end)

	animalFood.groups = groups
	animalFood.consumptionType = consumptionType
	
	--TerraLiveExtension.DebugTable("animalFood", animalFood)
	
	return true
end

function TLAnimalFoodSystem:consumeFood(superFunc, animalTypeIndex, amountToConsume, fillLevels, consumedFood, cluster)
	-- print(string.format("TLAnimalFoodSystem:consumeFood(%s, %s, %s, %s, %s, %s)", superFunc, animalTypeIndex, amountToConsume, fillLevels, consumedFood, cluster));
	local animalFood = self:getAnimalFood(animalTypeIndex, cluster);
	
	if animalFood ~= nil then
		if animalFood.consumptionType == AnimalFoodSystem.FOOD_CONSUME_TYPE_SERIAL then
			return self:consumeFoodSerially(amountToConsume, animalFood.groups, fillLevels, consumedFood)
		elseif animalFood.consumptionType == AnimalFoodSystem.FOOD_CONSUME_TYPE_PARALLEL then
			return self:consumeFoodParallelly(amountToConsume, animalFood.groups, fillLevels, consumedFood)
		end
	end

	return 0, nil
end

function TLAnimalFoodSystem:consumeFoodSerially(superFunc, amount, foodGroups, fillLevels, consumedFood)
	-- print(string.format("TLAnimalFoodSystem:consumeFoodSerially(%s, %s, %s, %s, %s)", superFunc, amount, foodGroups, fillLevels, consumedFood));
	local productionWeight = 0
	local milkBonus = nil
	local fatteningBonus = nil
	local totalAmountToConsume = amount

	if totalAmountToConsume > 0 then
		for _, foodGroup in ipairs(foodGroups) do
			if foodGroup.eatWeight > 0 then
				local oldAmount = amount
				amount = self:consumeFoodGroup(foodGroup, amount, fillLevels, consumedFood)
				local deltaProdWeight = (oldAmount - amount) / totalAmountToConsume * foodGroup.productionWeight
				productionWeight = productionWeight + deltaProdWeight
				
				if foodGroup.milkBonus ~= nil then
					if (oldAmount - amount) ~= 0 then
						-- wenn in der group milch bonus war und es wurde verbrauch festgelegt, dann schon mal den milchfaktor für die rückgabe setzen
						milkBonus = foodGroup.milkBonus;
					else
						milkBonus = foodGroup.milkBonus * -1;
					end
				end
				
				if foodGroup.milkBonus ~= nil then
					if (oldAmount - amount) ~= 0 then
						-- wenn in der group mast bonus war und es wurde verbrauch festgelegt, dann schon mal den mast faktor für die rückgabe setzen
						fatteningBonus = foodGroup.fatteningBonus;
					else
						fatteningBonus = foodGroup.fatteningBonus * -1;
					end
				end
			end
		end
	end
	
	return productionWeight, milkBonus, fatteningBonus
end

function TLAnimalFoodSystem:consumeFoodParallelly(superFunc, amount, foodGroups, fillLevels, consumedFood)
	-- print(string.format("TLAnimalFoodSystem:consumeFoodParallelly(%s, %s, %s, %s, %s)", superFunc, amount, foodGroups, fillLevels, consumedFood));
	local productionWeight = 0
	local milkBonus = nil
	local fatteningBonus = nil

	if amount > 0 then
		for _, foodGroup in pairs(foodGroups) do
			if foodGroup.eatWeight > 0 then
				local totalFillLevelInGroup = self:getTotalFillLevelInGroup(foodGroup, fillLevels)
				local foodGroupConsume = amount * foodGroup.eatWeight
				local consumeFood = math.min(totalFillLevelInGroup, foodGroupConsume)
				local ret = self:consumeFoodGroup(foodGroup, consumeFood, fillLevels, consumedFood)
				local foodFactor = (consumeFood - ret) / foodGroupConsume
				productionWeight = productionWeight + foodFactor * foodGroup.productionWeight
				
				if foodGroup.milkBonus ~= nil then
					if (consumeFood - ret) ~= 0 then
						-- wenn in der group milch bonus war und es wurde verbrauch festgelegt, dann schon mal den milchfaktor für die rückgabe setzen
						milkBonus = foodGroup.milkBonus;
					else
						milkBonus = foodGroup.milkBonus * -1;
					end
				end
				
				if foodGroup.fatteningBonus ~= nil then
					if (consumeFood - ret) ~= 0 then
						-- wenn in der group mast bonus war und es wurde verbrauch festgelegt, dann schon mal den mast faktor für die rückgabe setzen
						fatteningBonus = foodGroup.fatteningBonus;
					else
						-- beim Mast Bonus sinkt der viel langsamer
						fatteningBonus = foodGroup.fatteningBonus * -0.5;
					end
				end
			end
		end
	end
	-- TerraLiveExtension.DebugTable("foodGroups", foodGroups)
	-- print(string.format("TLAnimalFoodSystem:consumeFoodParallelly returns: %s, %s", productionWeight, milkBonus));

	return productionWeight, milkBonus, fatteningBonus
end


function TLAnimalFoodSystem:loadMixture(superFunc, mixture, xmlFile, key)
	local ingredients = {}
	local usedFillTypes = {}

	xmlFile:iterate(key .. ".ingredient", function (_, ingredientKey)
		local ingredient = {
			fillTypes = {},
			weight = MathUtil.round(xmlFile:getValue(ingredientKey .. "#weight", 0), 3)
		}

		if self:getFillTypesFromXML(ingredient.fillTypes, usedFillTypes, xmlFile, ingredientKey .. "#fillTypes") then
			table.insert(ingredients, ingredient)
		end
	end)

	mixture.ingredients = ingredients

	return true
end

-- austausch des alten gegen das neue
AnimalFoodSystem.new = Utils.overwrittenFunction(AnimalFoodSystem.new, TLAnimalFoodSystem.new)
AnimalFoodSystem.loadAnimalFood = Utils.overwrittenFunction(AnimalFoodSystem.loadAnimalFood, TLAnimalFoodSystem.loadAnimalFood)
AnimalFoodSystem.getAnimalFood = Utils.overwrittenFunction(AnimalFoodSystem.getAnimalFood, TLAnimalFoodSystem.getAnimalFood)
AnimalFoodSystem.loadAnimalFoodData = Utils.overwrittenFunction(AnimalFoodSystem.loadAnimalFoodData, TLAnimalFoodSystem.loadAnimalFoodData)
AnimalFoodSystem.consumeFood = Utils.overwrittenFunction(AnimalFoodSystem.consumeFood, TLAnimalFoodSystem.consumeFood)
AnimalFoodSystem.consumeFoodSerially = Utils.overwrittenFunction(AnimalFoodSystem.consumeFoodSerially, TLAnimalFoodSystem.consumeFoodSerially)
AnimalFoodSystem.consumeFoodParallelly = Utils.overwrittenFunction(AnimalFoodSystem.consumeFoodParallelly, TLAnimalFoodSystem.consumeFoodParallelly)
AnimalFoodSystem.loadMixture = Utils.overwrittenFunction(AnimalFoodSystem.loadMixture, TLAnimalFoodSystem.loadMixture)
