-- foodFactor initialisieren und productionFactor auch syncen für die Anzeige im Dialog

AnimalCluster.NUM_BITS_AGE = 9

AnimalClusterExtension = {}

AnimalClusterExtension.MAX_AGE = math.pow(2, AnimalCluster.NUM_BITS_AGE) - 1
AnimalClusterExtension.DIEING_BY_AGE_PERCENTAGE = 10

function AnimalClusterExtension.new(self, superFunc, customMt)
	--print(string.format("AnimalClusterExtension.new(%s, %s, %s)", self, superFunc, customMt));
	local self = superFunc(self, customMt)
	
	self.foodFactor = 0
	self.productionFactor = 0
	self.milkBonus = 0
	self.fatteningBonus = 0
    self.monthSinceLastBirth = nil
    self.isInseminated = false
	
	return self
end
AnimalCluster.new = Utils.overwrittenFunction(AnimalCluster.new, AnimalClusterExtension.new)

function AnimalClusterExtension:readStream(superFunc, streamId, connection)
	--print(string.format("AnimalClusterExtension.readStream(%s, %s, %s)", superFunc, streamId, connection));
	superFunc(self, streamId, connection)
	self.productionFactor = streamReadUInt8(streamId) / 255
	self.milkBonus = streamReadUInt8(streamId) / 255
	self.fatteningBonus = streamReadUInt8(streamId) / 255
    self.isInseminated = streamReadBool(streamId)

    if streamReadBool(streamId) then
        self.monthSinceLastBirth = streamReadUIntN(streamId, AnimalCluster.NUM_BITS_AGE)
    end
end
AnimalCluster.readStream = Utils.overwrittenFunction(AnimalCluster.readStream, AnimalClusterExtension.readStream)

function AnimalClusterExtension:writeStream(superFunc, streamId, connection)
	--print(string.format("AnimalClusterExtension.writeStream(%s, %s, %s)", superFunc, streamId, connection));
	superFunc(self, streamId, connection)
	streamWriteUInt8(streamId, MathUtil.round(self.productionFactor * 255))
	streamWriteUInt8(streamId, MathUtil.round(self.milkBonus * 255))
	streamWriteUInt8(streamId, MathUtil.round(self.fatteningBonus * 255))
    streamWriteBool(streamId, self.isInseminated)

    local hasMonthSinceLastBirth = self.monthSinceLastBirth ~= nil
    streamWriteBool(streamId, hasMonthSinceLastBirth)
    if hasMonthSinceLastBirth then
        streamWriteUIntN(streamId, math.floor(self.monthSinceLastBirth), AnimalCluster.NUM_BITS_AGE)
    end
end
AnimalCluster.writeStream = Utils.overwrittenFunction(AnimalCluster.writeStream, AnimalClusterExtension.writeStream)

function AnimalClusterExtension:readUpdateStream(superFunc, streamId, connection)
	--print(string.format("AnimalClusterExtension.readUpdateStream(%s, %s, %s)", superFunc, streamId, connection));
	superFunc(self, streamId, connection)
	self.productionFactor = streamReadUInt8(streamId) / 100
	self.milkBonus = streamReadUInt8(streamId) / 100
	self.fatteningBonus = streamReadUInt8(streamId) / 100
    self.isInseminated = streamReadBool(streamId)

	if streamReadBool(streamId) then
        self.monthSinceLastBirth = streamReadUIntN(streamId, AnimalCluster.NUM_BITS_AGE)
    end
end
AnimalCluster.readUpdateStream = Utils.overwrittenFunction(AnimalCluster.readUpdateStream, AnimalClusterExtension.readUpdateStream)

function AnimalClusterExtension:writeUpdateStream(superFunc, streamId, connection)
	--print(string.format("AnimalClusterExtension.writeUpdateStream(%s, %s, %s)", superFunc, streamId, connection));
	superFunc(self, streamId, connection)
	streamWriteUInt8(streamId, MathUtil.round(self.productionFactor * 100))
	streamWriteUInt8(streamId, MathUtil.round(self.milkBonus * 100))
	streamWriteUInt8(streamId, MathUtil.round(self.fatteningBonus * 100))
    streamWriteBool(streamId, self.isInseminated)

    local hasMonthSinceLastBirth = self.monthSinceLastBirth ~= nil
    streamWriteBool(streamId, hasMonthSinceLastBirth)
    if hasMonthSinceLastBirth then
        streamWriteUIntN(streamId, math.floor(self.monthSinceLastBirth), AnimalCluster.NUM_BITS_AGE)
    end
end
AnimalCluster.writeUpdateStream = Utils.overwrittenFunction(AnimalCluster.writeUpdateStream, AnimalClusterExtension.writeUpdateStream)

-- productionFactor pro cluster muss auch gespeichert werden
function AnimalClusterExtension:saveToXMLFile(superFunc, xmlFile, key, usedModNames)
	--print(string.format("AnimalClusterExtension.saveToXMLFile(%s, %s, %s, %s)", superFunc, xmlFile, key, usedModNames));
	superFunc(self, xmlFile, key, usedModNames)
	xmlFile:setFloat(key .. "#productionFactor", self.productionFactor)
	xmlFile:setFloat(key .. "#milkBonus", self.milkBonus)
	xmlFile:setFloat(key .. "#fatteningBonus", self.fatteningBonus)
    xmlFile:setBool(key .. "#isInseminated", self.isInseminated)

    if self.monthSinceLastBirth ~= nil then
        xmlFile:setInt(key .. "#monthSinceLastBirth", self.monthSinceLastBirth)
    end
end
AnimalCluster.saveToXMLFile = Utils.overwrittenFunction(AnimalCluster.saveToXMLFile, AnimalClusterExtension.saveToXMLFile)

function AnimalClusterExtension:loadFromXMLFile(superFunc, xmlFile, key)
	--print(string.format("AnimalClusterExtension.loadFromXMLFile(%s, %s, %s)", superFunc, xmlFile, key));
	superFunc(self, xmlFile, key)

	self.age = MathUtil.clamp(xmlFile:getInt(key .. "#age", self.age), 0, AnimalClusterExtension.MAX_AGE)
	self.productionFactor = xmlFile:getFloat(key .. "#productionFactor", self.productionFactor, 0)
	self.milkBonus = xmlFile:getFloat(key .. "#milkBonus", self.milkBonus, 0)
	self.fatteningBonus = xmlFile:getFloat(key .. "#fatteningBonus", self.fatteningBonus, 0)
    self.isInseminated = xmlFile:getBool(key .. "#isInseminated", self.isInseminated, false)
    self.monthSinceLastBirth = xmlFile:getInt(key .. "#monthSinceLastBirth")

	return true
end
AnimalCluster.loadFromXMLFile = Utils.overwrittenFunction(AnimalCluster.loadFromXMLFile, AnimalClusterExtension.loadFromXMLFile)

function AnimalClusterExtension:addInfos(superFunc, infos)
	local healthFactor = self:getHealthFactor()
	self.infoHealth.value = healthFactor
	self.infoHealth.ratio = healthFactor
	self.infoHealth.valueText = string.format("%d %%", g_i18n:formatNumber(healthFactor * 100, 0))

	table.insert(infos, self.infoHealth)
	
	local animalTypeIndex = g_currentMission.animalSystem:getTypeIndexBySubTypeIndex(self:getSubTypeIndex())
	if animalTypeIndex ~= AnimalType.HORSE and animalTypeIndex ~= AnimalType.PIG then
		local clusterProductionTable = {
			text = "",
			title = g_i18n:getText("statistic_productivity"),
			value = self.productionFactor,
			ratio = self.productionFactor,
			valueText = string.format("%d %%", g_i18n:formatNumber(self.productionFactor * 100, 0))
		}
		table.insert(infos, clusterProductionTable)
	end
	
	if self.milkBonus ~= nil and self.milkBonus ~= 0 then
		local clusterProductionTable = {
			text = "",
			title = g_i18n:getText("statistic_milkProductivity"),
			value = self.milkBonus,
			ratio = self.milkBonus/0.3,
			valueText = string.format("%d %%", g_i18n:formatNumber(self.milkBonus * 100, 0))
		}
		table.insert(infos, clusterProductionTable)
	end
	
	if self.fatteningBonus ~= nil and self.fatteningBonus ~= 0 then
		local clusterProductionTable = {
			text = "",
			title = g_i18n:getText("statistic_fatteningBonus"),
			value = self.fatteningBonus,
			ratio = self.fatteningBonus/0.3,
			valueText = string.format("%d %%", g_i18n:formatNumber(self.fatteningBonus * 100, 0))
		}
		table.insert(infos, clusterProductionTable)
	end
	
	if self:getSupportsReproduction() then
		local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())

		if self.age < subType.reproductionMinAgeMonth then
			local minAgeFactor = MathUtil.clamp(self:getAge() / subType.reproductionMinAgeMonth, 0, 1)
			self.infoReproductionMinAge.value = minAgeFactor
			self.infoReproductionMinAge.ratio = minAgeFactor
			self.infoReproductionMinAge.valueText = string.format("%d %%", g_i18n:formatNumber(minAgeFactor * 100, 0))

			table.insert(infos, self.infoReproductionMinAge)
		else
			local reproductionFactor = self:getReproductionFactor()
			self.infoReproduction.value = reproductionFactor
			self.infoReproduction.ratio = reproductionFactor
			self.infoReproduction.valueText = string.format("%d %%", g_i18n:formatNumber(reproductionFactor * 100, 0))
			self.infoReproduction.disabled = not self:getCanReproduce()

			table.insert(infos, self.infoReproduction)
		end
	end
end
AnimalCluster.addInfos = Utils.overwrittenFunction(AnimalCluster.addInfos, AnimalClusterExtension.addInfos)

function AnimalClusterExtension:getSellPrice(superFunc)
	local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
	local sellPrice = subType.sellPrice:get(self.age)
	local healthFactor = self:getHealthFactor()
	
	if self.fatteningBonus ~= nil and self.fatteningBonus ~= 0 then
		sellPrice = sellPrice * (1 + self.fatteningBonus);
	end

	return sellPrice * 0.4 + sellPrice * 0.6 * healthFactor
end
AnimalCluster.getSellPrice = Utils.overwrittenFunction(AnimalCluster.getSellPrice, AnimalClusterExtension.getSellPrice)

function AnimalClusterExtension:getHash(superFunc)
	-- Das original System von Giants benutzt die Werte + 100 und somit immer 900 mögliche Werte obwohl nur 100 benötigt werden
	-- neues system bin hexadezimalen kann eine stelle pro wert sparen
	-- mit dem neuen System das bei 16 startet (Hex 10) können bis zu 239 Werte genutzt werden mit weniger Stellen
	-- Die stellen dürfen nicht zu lang werden, sonst können sie nicht als index einer Liste benutzt werden.
	-- Milch und Mast Bonus sind ja bonuswerte die von 0 bis 0.3 gehen können. diese werden mit 100 nultipliziert gerundet verwendet.
	-- Hierdurch kann es vorkommen, dass durch das rauf und runter dann cluster die nah zusammen liegen zusammengeführt werden, was aber auch vor Hex system so war.
	-- Heumilch Block Stunden sind 14*24 = 336, deshalb startet der wert bei 256 (Hex 100) und somit können 3839 werte gespeichert werden.
	-- TerraLiveExtension.DebugText("age: %s health: %s reproduction: %s subTypeIndex: %s milkBonus: %s fatteningBonus: %s", self.age, self.health, self.reproduction, self.subTypeIndex, math.floor(self.milkBonus * 100), math.floor(self.fatteningBonus * 100))
    local insemination = 0
    if self.isInseminated then
        insemination = 1
    end

    local monthSinceLastBirth = self.monthSinceLastBirth
    if monthSinceLastBirth == nil then
        monthSinceLastBirth = -1
    end

	local newHash = string.format("%X%X%X%X%X%X%X%X", self.age + 16, self.health + 16, self.reproduction + 16, self.subTypeIndex + 16, math.floor(self.milkBonus * 100) + 16, math.floor(self.fatteningBonus * 100) + 16, monthSinceLastBirth + 16, insemination + 16)
	-- TerraLiveExtension.DebugText("NewHash: %s", newHash)
	return newHash
end

AnimalCluster.getHash = Utils.overwrittenFunction(AnimalCluster.getHash, AnimalClusterExtension.getHash)

function AnimalClusterExtension:clone(superFunc)
	local returnValue = superFunc(self)

	returnValue.foodFactor = self.foodFactor
	returnValue.productionFactor = self.productionFactor
	returnValue.milkBonus = self.milkBonus
	returnValue.fatteningBonus = self.fatteningBonus
    returnValue.isInseminated = self.isInseminated
    returnValue.monthSinceLastBirth = self.monthSinceLastBirth

    return returnValue
end

AnimalCluster.clone = Utils.overwrittenFunction(AnimalCluster.clone, AnimalClusterExtension.clone)

function AnimalClusterExtension:onPeriodChanged(superFunc)
	-- TerraLiveExtension.DebugText("AnimalClusterExtension:onPeriodChanged")
    if self.clusterSystem.owner ~= nil and self.clusterSystem.owner:getOwnerFarmId() ~= 0 then
        self:changeMonthSinceLastBirth(1)
        self:removeOldAnimals()
    end

	superFunc(self)
end

AnimalCluster.onPeriodChanged = Utils.overwrittenFunction(AnimalCluster.onPeriodChanged, AnimalClusterExtension.onPeriodChanged)

function AnimalClusterExtension:getAgeFactor(superFunc)
	return MathUtil.clamp(self.age / AnimalClusterExtension.MAX_AGE, 0, 1)
end

AnimalCluster.getAgeFactor = Utils.overwrittenFunction(AnimalCluster.getAgeFactor, AnimalClusterExtension.getAgeFactor)

function AnimalClusterExtension:changeAge(superFunc, delta)
	local old = self.age
	self.age = MathUtil.clamp(math.floor(self.age + delta), 0, AnimalClusterExtension.MAX_AGE)
	-- TerraLiveExtension.DebugText("changeAge %s %s %s", delta, old, self.age)
	if math.abs(self.age - old) > 0 then
		self:setDirty()
	end
end

AnimalCluster.changeAge = Utils.overwrittenFunction(AnimalCluster.changeAge, AnimalClusterExtension.changeAge)

function AnimalClusterExtension:updateReproduction(superFunc)
    local numOfNewAnimals = 0

    if self:getCanReproduce() then
		local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
		local reproductionDelta = self:getReproductionDelta(subType.reproductionDurationMonth)

        self:inseminateByMaleAnimals()
        self:calculateFailedInseminations(subType)

		if reproductionDelta > 0 and self.isInseminated then
			self:changeReproduction(reproductionDelta)

			if self.reproduction >= 100 then
				self.reproduction = 0
                self.monthSinceLastBirth = 0
                self.isInseminated = false

				self:setDirty()

				numOfNewAnimals = self.numAnimals
			end
		end
	end

    if numOfNewAnimals == 0 then
        return numOfNewAnimals
    end

    local reproductionValues = self:getReproductionValues()
    local specificNumberOfNewAnimals = self:getSpecificNumberOfNewAnimals(reproductionValues, numOfNewAnimals)

	return specificNumberOfNewAnimals
end

AnimalCluster.updateReproduction = Utils.overwrittenFunction(AnimalCluster.updateReproduction, AnimalClusterExtension.updateReproduction)

function AnimalClusterExtension:getCanReproduce(superFunc)
	local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())

	if subType.supportsReproduction then
		return subType.reproductionMinAgeMonth <= self.age
	end

	return false
end

AnimalCluster.getCanReproduce = Utils.overwrittenFunction(AnimalCluster.getCanReproduce, AnimalClusterExtension.getCanReproduce)

--------------------------------------------------- New AnimalCluster functions ---------------------------------------------------

function AnimalCluster:removeOldAnimals()
    local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())

    if self:getAge() > subType.maxAge then
        local diedAnimals = 0
        for i=1, self:getNumAnimals(), 1 do
            local randomValue = math.random(100)

            if randomValue <= AnimalClusterExtension.DIEING_BY_AGE_PERCENTAGE then
                diedAnimals = diedAnimals + 1
            end
        end

        if diedAnimals > 0 then
            local msg = string.format(g_i18n:getText("TL_husbandry_dec_by_age_single"), self.clusterSystem.owner:getName(), diedAnimals)
            if diedAnimals > 1 then
                msg = string.format(g_i18n:getText("TL_husbandry_dec_by_age_multiple"), self.clusterSystem.owner:getName(), diedAnimals)
            end
            g_client:getServerConnection():sendEvent(InGameNotification.new(InGameNotification.TYPE.CRITICAL, msg, self.clusterSystem.owner:getOwnerFarmId()))
            self:changeNumAnimals(-diedAnimals)
        end
    end
end

function AnimalCluster:getReproductionValues()
    local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
    local reproductionValues = {}

    for _, reproductionValue in pairs(subType.reproductionProbabilities) do
        for i=1, reproductionValue.probability, 1 do
            table.insert(reproductionValues, reproductionValue.numOfAnimals)
        end
    end

    return reproductionValues
end

function AnimalCluster:getSpecificNumberOfNewAnimals(reproductionValues, numOfNewAnimals)
    local specificNumberOfNewAnimals = 0

    for i=1, numOfNewAnimals, 1 do
        local randomValue = math.random(#reproductionValues)
        local reproductionNumber = reproductionValues[randomValue]
        specificNumberOfNewAnimals = specificNumberOfNewAnimals + reproductionNumber
    end

	return specificNumberOfNewAnimals
end

function AnimalCluster:getMaleSubTypeIndex()
	local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
    local maleSubTypeName = subType.malePart

    if maleSubTypeName == nil then
        Logging.warning("SubType " .. subType.name .. " has no male part")
        return nil
    end

    local maleSubType = g_currentMission.animalSystem:getSubTypeByName(maleSubTypeName)
    return maleSubType.subTypeIndex
end

function AnimalCluster:changeMonthSinceLastBirth(delta)
    if self.monthSinceLastBirth == nil then
        return
    end

    self.monthSinceLastBirth = math.floor(self.monthSinceLastBirth + delta)
	self:setDirty()
end

function AnimalCluster:getMonthSinceLastBirth()
    return self.monthSinceLastBirth
end

function AnimalCluster:canBeInseminated()
	local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())

    if not subType.supportsReproduction or subType.reproductionDurationMonth == nil or subType.reproductionMinAgeMonth == nil or self.isInseminated then
        return false
    end

    local neededAge = subType.reproductionMinAgeMonth
    return neededAge <= self:getAge()
end

function AnimalCluster:inseminate(numAnimals, mustBePaid)
    if self.clusterSystem.isServer then
        if numAnimals == self:getNumAnimals() then
            self.isInseminated = true
            self:setDirty()
        else
            local inseminatedCluster = self:clone()
            inseminatedCluster.isInseminated = true
            inseminatedCluster:changeNumAnimals(numAnimals)
            self:changeNumAnimals(-numAnimals)
            self.clusterSystem:addPendingAddCluster(inseminatedCluster)
        end

        if mustBePaid then
            local farmId = self.clusterSystem.owner:getOwnerFarmId()
            local farm = g_farmManager:getFarmById(farmId)
            if farm ~= nil then
                local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
                local inseminationCost = subType.inseminationCost * numAnimals
                farm:changeBalance(-inseminationCost, MoneyType.ANIMAL_UPKEEP)
                g_currentMission:addMoneyChange(-inseminationCost, farmId, MoneyType.ANIMAL_UPKEEP, true)
            end
        end
	else
		g_client:getServerConnection():sendEvent(InseminationEvent.new(self.clusterSystem.owner, self.id, numAnimals, mustBePaid))
    end
end

function AnimalCluster:calculateFailedInseminations(subType)
    if self.reproduction ~= 0 or self.monthSinceLastBirth == nil then
        return
    end

    local numOfFailedInseminations = 0
    local inseminationProbability = 100

    for _, inseminationValue in pairs(subType.inseminationProbabilities) do
        if inseminationValue.monthSinceLastBirth == self.monthSinceLastBirth then
            inseminationProbability = inseminationValue.probability
            break
        end
    end

    for i=1, self:getNumAnimals(), 1 do
        local randomValue = math.random(100)
        if randomValue > inseminationProbability then
            numOfFailedInseminations = numOfFailedInseminations + 1
        end
    end

    if numOfFailedInseminations == self:getNumAnimals() then
        self.isInseminated = false
    elseif numOfFailedInseminations > 0 then
        local failedCluster = self:clone()
        failedCluster.isInseminated = false
        failedCluster:changeNumAnimals(numOfFailedInseminations)
        self:changeNumAnimals(-numOfFailedInseminations)
        self.clusterSystem:addPendingAddCluster(failedCluster)
    end
end

function AnimalCluster:getIsInseminated()
    return self.isInseminated
end

function AnimalCluster:getMonthSinceLastBirth()
    return self.monthSinceLastBirth
end

function AnimalCluster:inseminateByMaleAnimals()
    if not self:canBeInseminated() then
        return
    end

	local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
    local malePart = subType.malePart

    if malePart == nil then
        return
    end

    local hasMalePart = false
    for _, cluster in pairs(self.clusterSystem:getClusters()) do
        local clusterSubType = g_currentMission.animalSystem:getSubTypeByIndex(cluster:getSubTypeIndex())
        if clusterSubType.name == malePart then
            if clusterSubType.reproductionMinAgeMonth <= cluster:getAge() then
                hasMalePart = true
                break
            end
        end
    end

    if hasMalePart then
        self:inseminate(self:getNumAnimals(), false)
    end
end

function AnimalCluster:getLactationFoodFactor()
    local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
    local foodFactor = 1.0
    for _, lactationValue in pairs(subType.lactation) do
        if lactationValue.month == self:getMonthSinceLastBirth() then
            foodFactor = lactationValue.food
            break
        end
    end
    return foodFactor
end

function AnimalCluster:getLactationMilkFactor()
    local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())

    local milkFactor = 0.0

    if #subType.lactation == 0 then
        milkFactor = 1.0
    end

    for _, lactationValue in pairs(subType.lactation) do
        if lactationValue.month == self:getMonthSinceLastBirth() then
            milkFactor = lactationValue.milk
            break
        end
    end
    return milkFactor
end

function AnimalCluster:getMaxColostrumProduceAge()
    local subType = g_currentMission.animalSystem:getSubTypeByIndex(self:getSubTypeIndex())
    return subType.maxColostrumProduceAge
end

function AnimalCluster:produceColostrum()
    local maxColostrumProduceAge = self:getMaxColostrumProduceAge()
    if maxColostrumProduceAge ~= nil and self:getMonthSinceLastBirth() ~= nil then
        return self:getMonthSinceLastBirth() <= maxColostrumProduceAge
    else
        return false
    end
end