PlaceableHusbandryColostrum = {
    AnimalsSupportColostrum = {
        COW = true,
        HAYCOW = true
    }
}

function PlaceableHusbandryColostrum.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(PlaceableHusbandryAnimals, specializations)
end

function PlaceableHusbandryColostrum.registerFunctions(placeableType)
	SpecializationUtil.registerFunction(placeableType, "ejectColostrumToStorage", PlaceableHusbandryColostrum.ejectColostrumToStorage)
	SpecializationUtil.registerFunction(placeableType, "ejectColostrumToSlurry", PlaceableHusbandryColostrum.ejectColostrumToSlurry)
end

function PlaceableHusbandryColostrum.registerOverwrittenFunctions(placeableType)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateInfo", PlaceableHusbandryColostrum.updateInfo)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateOutput", PlaceableHusbandryColostrum.updateOutput)
    SpecializationUtil.registerOverwrittenFunction(placeableType, "getConditionInfos", PlaceableHusbandryColostrum.getConditionInfos)
end


function PlaceableHusbandryColostrum.registerEventListeners(placeableType)
	SpecializationUtil.registerEventListener(placeableType, "onLoad", PlaceableHusbandryColostrum)
end

function PlaceableHusbandryColostrum:onLoad(savegame)
    local spec = {}
    self.spec_husbandryColostrum = spec
    local animalTypeName = self.xmlFile:getValue("placeable.husbandry.animals#type")

	if PlaceableHusbandryColostrum.AnimalsSupportColostrum[animalTypeName] then
        self.spec_husbandryColostrum.supportColostrum = true
    else
        self.spec_husbandryColostrum.supportColostrum = false
    end
end

function PlaceableHusbandryColostrum:updateOutput(superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor)
    if not self.spec_husbandryColostrum.supportColostrum then
        superFunc(self, cluster, foodFactor, productionFactor, clusterProductionFactor)
        return
    end

    local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex);

	local litersPerHour = 0;
	if subType ~= nil then
		local milk = subType.output.milk

		if milk ~= nil and cluster.reproduction < 80 and cluster.monthSinceLastBirth ~= nil then
			local age = cluster:getAge()
			local litersPerAnimals = milk:get(age)
			local litersPerDay = litersPerAnimals * cluster:getNumAnimals()
			local lactationFactor = cluster:getLactationMilkFactor()
			litersPerHour = litersPerHour + litersPerDay / 24
			litersPerHour = litersPerHour * lactationFactor
			if cluster.milkBonus ~= nil then
				local milkBonus = cluster.milkBonus + 1
				litersPerHour = litersPerHour * milkBonus
			end
		end
	end
		
	if self.isServer and litersPerHour > 0 then
		local liters = productionFactor * clusterProductionFactor * litersPerHour * g_currentMission.environment.timeAdjustment

		if cluster:produceColostrum() then
			self:addFood(self:getOwnerFarmId(), liters, FillType.COLOSTRUM, nil)
		end
	end

    superFunc(self, cluster, foodFactor, productionFactor, clusterProductionFactor)
end

function PlaceableHusbandryColostrum:updateInfo(superFunc, infoTable)
	superFunc(self, infoTable)

    if self.spec_husbandryColostrum.supportColostrum and self:getHusbandryIsFillTypeSupported(FillType.COLOSTRUM) then
        local fillLevel = self:getHusbandryFillLevel(FillType.COLOSTRUM)
        local info = {
            title = g_i18n:getText("TL_fillType_colostrum"),
            text = string.format("%d l", fillLevel)
        }

        table.insert(infoTable, info)
    end
end

function PlaceableHusbandryColostrum:getConditionInfos(superFunc)
	local infos = superFunc(self)

		if PlaceableHusbandryColostrum.AnimalsSupportColostrum[self.spec_husbandryAnimals.animalType.name] == nil then
				return infos
		end

    if self.spec_husbandry.loadingStation ~= nil and not self.spec_husbandry.loadingStation:getIsFillTypeSupported(FillType.COLOSTRUM) then

        return infos
    end

    local info = {}
    local colostrumFillType = g_fillTypeManager:getFillTypeByIndex(FillType.COLOSTRUM)
    info.title = colostrumFillType.title
    info.value = self:getHusbandryFillLevel(FillType.COLOSTRUM)
    local capacity = self:getHusbandryCapacity(FillType.COLOSTRUM)
	info.valueText = g_i18n:formatNumber(info.value, 0) .. " / " .. g_i18n:formatVolume(capacity, 0);
    local ratio = 0

    if capacity > 0 then
        ratio = info.value / capacity
    end

    info.ratio = MathUtil.clamp(ratio, 0, 1)
    info.invertedBar = true
    table.insert(infos, info)

	return infos
end

function PlaceableHusbandryColostrum:ejectColostrumToStorage()
    if not self.isServer then
		g_client:getServerConnection():sendEvent(EjectColostrumEvent.new(self, EjectColostrumEvent.EJECT_TO.STORAGE))
	else
        local fillLevel = self:getFillTypeLevel(FillType.COLOSTRUM)
		local maxFreeLevel = self:getHusbandryFreeCapacity(FillType.COLOSTRUM, self:getOwnerFarmId())
		local level = math.min(fillLevel, maxFreeLevel)
        self:removeFood(level, FillType.COLOSTRUM)
        self:addHusbandryFillLevelFromTool(self:getOwnerFarmId(), level, FillType.COLOSTRUM, nil, nil, nil)
    end
end

function PlaceableHusbandryColostrum:ejectColostrumToSlurry()
    if not self.isServer then
		g_client:getServerConnection():sendEvent(EjectColostrumEvent.new(self, EjectColostrumEvent.EJECT_TO.SLURRY))
	else
        local fillLevel = self:getFillTypeLevel(FillType.COLOSTRUM)

        if self.spec_husbandryLiquidManure ~= nil then
            local maxFreeLevel = self:getHusbandryFreeCapacity(FillType.LIQUIDMANURE, self:getOwnerFarmId())
            local level = math.min(fillLevel, maxFreeLevel)
            self:removeFood(level, FillType.COLOSTRUM)
            self:addHusbandryFillLevelFromTool(self:getOwnerFarmId(), level, FillType.LIQUIDMANURE, nil, nil, nil)
        else
            self:removeFood(fillLevel, FillType.COLOSTRUM)
        end
    end
end