source(g_currentModDirectory .. "scripts/animals/events/EjectFoodEvent.lua")

PlaceableHusbandryFoodExtension = {}

function PlaceableHusbandryFoodExtension:getFoodInfos(superFunc2, superFunc, cluster)
	-- print(string.format("PlaceableHusbandryFoodExtension:getFoodInfos(%s, %s, %s)", superFunc2, superFunc, cluster));
	
	local foodInfos = superFunc(self)
	local spec = self.spec_husbandryFood
	local animalFood = g_currentMission.animalFoodSystem:getAnimalFood(spec.animalTypeIndex, cluster)

	if animalFood ~= nil then
		local function compfoodGroup(w1,w2)
			return w1.productionWeight > w2.productionWeight
		end
		table.sort(animalFood.groups,compfoodGroup)
		
		-- TerraLiveExtension.DebugTable("animalFood.groups", animalFood.groups)

		for _, foodGroup in pairs(animalFood.groups) do
			local title = foodGroup.title
			local fillLevel = 0
			local capacity = spec.capacity
			local fillTypes = {}

			for _, fillTypeIndex in pairs(foodGroup.fillTypes) do
				if spec.fillLevels[fillTypeIndex] ~= nil then
					fillLevel = fillLevel + spec.fillLevels[fillTypeIndex]
				end
				fillTypes[fillTypeIndex] = true;
			end

			local number = foodGroup.productionWeight * 100
			local percents = math.floor(number)
			local rest = number - percents
			if rest > 0 then
				rest = MathUtil.round(rest * 10)
				percents = percents.."."..rest
			end
			
			local info = {
				title = title.."("..percents.."%)",
				info = foodGroup.info,
				value = fillLevel,
				capacity = capacity,
				ratio = 0,
				percents = number,
				fillTypes = fillTypes
			}

			if capacity > 0 then
				info.ratio = fillLevel / capacity
			end

			table.insert(foodInfos, info)
		end
	end

	return foodInfos
end

function PlaceableHusbandryFoodExtension:updateFeeding(superFunc2, superFunc)
	-- TerraLiveExtension.DebugText("PlaceableHusbandryFoodExtension:updateFeeding(%s, %s)", superFunc2, superFunc);
	-- TerraLiveExtension.DebugTable("self", self);
	
	local factor = superFunc(self)
	-- TerraLiveExtension.DebugText("PlaceableHusbandryFoodExtension superFunc returns '%s'", factor);
	
	local spec = self.spec_husbandryFood
	if self.isServer and spec.animalTypeIndex ~= nil then
		-- cluster holen und einzeln berechnen und die rückgabe als mix machen aus superfunc und durchschnitt der cluster
		local clusters = self.spec_husbandryAnimals.clusterSystem:getClusters()
		local numClusters = 0;
		local clusterFactorSum = 0;
		
		for _, cluster in ipairs(clusters) do
		
			-- liter selbst berechnen, damit cluster abhängig
			-- wird in PlaceableHusbandryFood:onHusbandryAnimalsUpdate(clusters) für alle cluster gesetzt
			local litersPerHour = 0
			local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex)
			if subType ~= nil then
				local food = subType.input.food

				if food ~= nil then
					local age = cluster:getAge()
					local litersPerAnimal = food:get(age)
					local litersPerDay = litersPerAnimal * cluster:getNumAnimals()
					local lactationFactor = cluster:getLactationFoodFactor()
					litersPerHour = litersPerHour + litersPerDay / 24
					litersPerHour = litersPerHour * lactationFactor
				end
			end
		
			local consumedFood = {}
			local milkBonus = nil
			local fatteningBonus = nil
			cluster.foodFactor, milkBonus, fatteningBonus = g_currentMission.animalFoodSystem:consumeFood(spec.animalTypeIndex, litersPerHour * g_currentMission.environment.timeAdjustment, spec.fillLevels, consumedFood, cluster)
			numClusters = numClusters + 1;
			clusterFactorSum = clusterFactorSum + cluster.foodFactor
			
			-- print(string.format("milkBonus: '%s'", milkBonus));
			-- print(string.format("cluster.milkBonus old: '%s'", cluster.milkBonus));
			if milkBonus ~= nil then
				-- wenn ein milkBonus zurück gekommen ist, dann hier auf den faktor vom cluster addieren
				cluster.milkBonus = MathUtil.clamp(cluster.milkBonus + milkBonus, 0, 0.3);
				-- print(string.format("cluster.milkBonus new: '%s'", cluster.milkBonus));
			end
			
			-- print(string.format("fatteningBonus: '%s'", fatteningBonus));
			-- print(string.format("cluster.fatteningBonus old: '%s'", cluster.fatteningBonus));
			if fatteningBonus ~= nil then
				-- wenn ein fatteningBonus zurück gekommen ist, dann hier auf den faktor vom cluster addieren
				-- aber das ganze nicht absolut sonder als % wert vom maxFactor. Also ein + von 1% erhöht den Factor von 0 auf 0,003, wenn 0,3 max ist.
				local maxFactor = 0.3;
				cluster.fatteningBonus = MathUtil.clamp(cluster.fatteningBonus + (fatteningBonus * maxFactor), 0, maxFactor);
				-- print(string.format("cluster.fatteningBonus new: '%s'", cluster.fatteningBonus));
			end
	
			-- local amountToTip = 0;
			for fillTypeIndex, delta in pairs(consumedFood) do
				self:removeFood(delta, fillTypeIndex)
				-- amountToTip = amountToTip + delta;
			end
			
				
			-- test zum auswerfen von futter, eventuell für verunrinigung nutzen?
			-- local fillTypeIndexToTip = g_fillTypeManager:getFillTypeIndexByName("COMPOSTRAW")
			-- print("amountToTip:" .. amountToTip)
			-- for _, ejectArea in pairs(spec.ejectAreas) do
				-- local lsx, lsy, lsz, lex, ley, lez, radius = DensityMapHeightUtil.getLineByArea(ejectArea.start, ejectArea.width, ejectArea.height, false)
				-- local dropped, lineOffset = DensityMapHeightUtil.tipToGroundAroundLine(nil, amountToTip, fillTypeIndexToTip, lsx, lsy, lsz, lex, ley, lez, radius, radius, 0, false, nil, nil, true)
				-- print("dropped:" .. dropped)
				-- amountToTip = math.max(amountToTip - dropped, 0)

				-- if amountToTip <= 0 then
					-- break
				-- end
			-- end
		end
		
	end
	-- TerraLiveExtension.DebugText("PlaceableHusbandryFoodExtension:updateFeeding returns '%s'", factor);
	return factor
end

function PlaceableHusbandryFoodExtension.registerXMLPaths(schema, superFunc, basePath)
	-- print(string.format("PlaceableHusbandryFoodExtension:registerXMLPaths(%s, %s, %s)", schema, superFunc, basePath));
	superFunc(schema, basePath)
	
	schema:setXMLSpecializationType("Husbandry")

	basePath = basePath .. ".husbandry.food.feedingTroughEjects.feedingTroughEject(?)"
	
	-- areas for eject
	schema:register(XMLValueType.NODE_INDEX, basePath .. "#startNode", "Eject area start node")
	schema:register(XMLValueType.NODE_INDEX, basePath .. "#widthNode", "Eject area width node")
	schema:register(XMLValueType.NODE_INDEX, basePath .. "#heightNode", "Eject area height node")
	schema:setXMLSpecializationType()
end

function PlaceableHusbandryFoodExtension:onLoad(superFunc, savegame)
	superFunc(self, savegame)
	
	local spec = self.spec_husbandryFood
	spec.ejectAreas = {}
	
	self.xmlFile:iterate("placeable.husbandry.food.feedingTroughEjects.feedingTroughEject", function (_, key)
		local ejectAreaStart = self.xmlFile:getValue(key .. "#startNode", nil, self.components, self.i3dMappings)
		local ejectAreaWidth = self.xmlFile:getValue(key .. "#widthNode", nil, self.components, self.i3dMappings)
		local ejectAreaHeight = self.xmlFile:getValue(key .. "#heightNode", nil, self.components, self.i3dMappings)

		if ejectAreaStart == nil then
			Logging.xmlError(self.xmlFile, "Missing clear eject start node for feedingTroughEject")

			return false
		end

		if ejectAreaWidth == nil then
			Logging.xmlError(self.xmlFile, "Missing clear eject width node for feedingTroughEject")

			return false
		end

		if ejectAreaHeight == nil then
			Logging.xmlError(self.xmlFile, "Missing clear eject height node for feedingTroughEject")

			return false
		end

		table.insert(spec.ejectAreas, {
			start = ejectAreaStart,
			width = ejectAreaWidth,
			height = ejectAreaHeight
		})
	end)
	
end

function PlaceableHusbandryFoodExtension.registerFunctions(placeableType, superFunc)
	superFunc(placeableType)
	SpecializationUtil.registerFunction(placeableType, "getFillTypeLevel", PlaceableHusbandryFood.getFillTypeLevel)
	SpecializationUtil.registerFunction(placeableType, "ejectFood", PlaceableHusbandryFood.ejectFood)
end

PlaceableHusbandryFood.getFoodInfos = Utils.overwrittenFunction(PlaceableHusbandryFood.getFoodInfos, PlaceableHusbandryFoodExtension.getFoodInfos)
PlaceableHusbandryFood.updateFeeding = Utils.overwrittenFunction(PlaceableHusbandryFood.updateFeeding, PlaceableHusbandryFoodExtension.updateFeeding)
PlaceableHusbandryFood.registerXMLPaths = Utils.overwrittenFunction(PlaceableHusbandryFood.registerXMLPaths, PlaceableHusbandryFoodExtension.registerXMLPaths)
PlaceableHusbandryFood.onLoad = Utils.overwrittenFunction(PlaceableHusbandryFood.onLoad, PlaceableHusbandryFoodExtension.onLoad)
PlaceableHusbandryFood.registerFunctions = Utils.overwrittenFunction(PlaceableHusbandryFood.registerFunctions, PlaceableHusbandryFoodExtension.registerFunctions)

function PlaceableHusbandryFood:ejectFood(target)
	if not self.isServer then
		g_client:getServerConnection():sendEvent(EjectFoodEvent.new(self, target))
	else
		local spec = self.spec_husbandryFood;
		
		-- test zum auswerfen von futter
		local amountToTip = self:getTotalFood()
		local totalAmountBefore = amountToTip;
		local droppedTotal = 0;
		
		-- hier noch ermitteln, ob es nur ein Filltype ist
		local fillTypeIndexToTip;
		local amountOfFillLevels = 0;
		for fillTypeIndex, fillLevel in pairs(spec.fillLevels) do
			if fillLevel > 0 then
				amountOfFillLevels = amountOfFillLevels + 1;
				fillTypeIndexToTip = fillTypeIndex;
			end
		end
		
		if amountOfFillLevels ~= 1 then
			fillTypeIndexToTip = g_fillTypeManager:getFillTypeIndexByName("COMPOSTRAW");
		end
		
		if #spec.ejectAreas ~= 0 then
			for _, ejectArea in pairs(spec.ejectAreas) do
				local lsx, lsy, lsz, lex, ley, lez, radius = DensityMapHeightUtil.getLineByArea(ejectArea.start, ejectArea.width, ejectArea.height, false)
				local dropped, lineOffset = DensityMapHeightUtil.tipToGroundAroundLine(nil, amountToTip, fillTypeIndexToTip, lsx, lsy, lsz, lex, ley, lez, radius, radius, 0, false, nil, nil, true)
				amountToTip = math.max(amountToTip - dropped, 0);
				droppedTotal = droppedTotal + dropped;

				if amountToTip <= 0 then
					break;
				end
			end
		end
		
		if target == "manure" then
			-- suche mithaufen zum spawnen
			local dropped = self:addHusbandryFillLevelFromTool(self:getOwnerFarmId(), amountToTip, FillType.MANURE, nil, nil, nil)
			amountToTip = math.max(amountToTip - dropped, 0);
			droppedTotal = droppedTotal + dropped;
		end
		
		if target == "sell" then
			-- Verkaufen zum aktuelle preis
			local lowestSellPrice = math.huge

			for _, unloadingStation in pairs(g_currentMission.storageSystem:getUnloadingStations()) do
				if unloadingStation.owningPlaceable ~= nil and unloadingStation.isSellingPoint and unloadingStation.acceptedFillTypes[FillType.MANURE] then
					local price = unloadingStation:getEffectiveFillTypePrice(FillType.MANURE)

					if price > 0 then
						lowestSellPrice = math.min(lowestSellPrice, price)
					end
				end
			end

			if lowestSellPrice == math.huge then
				lowestSellPrice = 0.5
			end
			
			local price = amountToTip * lowestSellPrice * PlaceableSilo.PRICE_SELL_FACTOR
			g_currentMission:addMoney(price, self:getOwnerFarmId(), MoneyType.HARVEST_INCOME, true, true)
			droppedTotal = droppedTotal + amountToTip;
		end

		-- abziehen der Menge aus den Trögen
		if droppedTotal > 0 then
			local amountToRemove = droppedTotal;
			local percentPerFillType = amountToRemove / totalAmountBefore;
			for fillTypeIndex, fillLevel in pairs(spec.fillLevels) do
				if fillLevel > 0 then
					local removed = self:removeFood(fillLevel * percentPerFillType, fillTypeIndex)
					amountToRemove = math.max(amountToRemove - removed, 0);
					if amountToRemove <= 0 then
						break;
					end
				end
			end
		end
	end
end

function PlaceableHusbandryFood:getFillTypeLevel(fillTypeIndex)
    local spec = self.spec_husbandryFood

	if spec.supportedFillTypes[fillTypeIndex] == nil then
		return 0
	end

    return spec.fillLevels[fillTypeIndex]
end