PlaceableHusbandryGoatMilk = {}

source("dataS/scripts/animals/husbandry/objects/MilkingRobot.lua")

function PlaceableHusbandryGoatMilk.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(PlaceableHusbandry, specializations)
end

function PlaceableHusbandryGoatMilk.registerFunctions(placeableType)
	SpecializationUtil.registerFunction(placeableType, "onMilkingRobotLoaded", PlaceableHusbandryGoatMilk.onMilkingRobotLoaded)
end

function PlaceableHusbandryGoatMilk.registerOverwrittenFunctions(placeableType)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateOutput", PlaceableHusbandryGoatMilk.updateOutput)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateProduction", PlaceableHusbandryGoatMilk.updateProduction)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateInfo", PlaceableHusbandryGoatMilk.updateInfo)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "getConditionInfos", PlaceableHusbandryGoatMilk.getConditionInfos)
end

function PlaceableHusbandryGoatMilk.registerEventListeners(placeableType)
	SpecializationUtil.registerEventListener(placeableType, "onLoad", PlaceableHusbandryGoatMilk)
	SpecializationUtil.registerEventListener(placeableType, "onDelete", PlaceableHusbandryGoatMilk)
	SpecializationUtil.registerEventListener(placeableType, "onFinalizePlacement", PlaceableHusbandryGoatMilk)
	SpecializationUtil.registerEventListener(placeableType, "onHusbandryAnimalsUpdate", PlaceableHusbandryGoatMilk)
	SpecializationUtil.registerEventListener(placeableType, "onReadStream", PlaceableHusbandryGoatMilk)
	SpecializationUtil.registerEventListener(placeableType, "onWriteStream", PlaceableHusbandryGoatMilk)
	SpecializationUtil.registerEventListener(placeableType, "onReadUpdateStream", PlaceableHusbandryGoatMilk)
	SpecializationUtil.registerEventListener(placeableType, "onWriteUpdateStream", PlaceableHusbandryGoatMilk)
end

function PlaceableHusbandryGoatMilk.registerXMLPaths(schema, basePath)
	schema:setXMLSpecializationType("Husbandry")

	basePath = basePath .. ".husbandry.goatMilk"
	schema:setXMLSpecializationType()
end

function PlaceableHusbandryGoatMilk.registerSavegameXMLPaths(schema, basePath)
	schema:setXMLSpecializationType("Husbandry")
	schema:register(XMLValueType.FLOAT, basePath .. "#pendingLiters", "Pending liters")
	schema:setXMLSpecializationType()
end

function PlaceableHusbandryGoatMilk:loadFromXMLFile(xmlFile, key)
	local spec = self.spec_husbandryGoatMilk
	spec.pendingLiters = xmlFile:getValue(key .. "#pendingLiters", spec.pendingLiters)
end

function PlaceableHusbandryGoatMilk:saveToXMLFile(xmlFile, key, usedModNames)
	local spec = self.spec_husbandryGoatMilk

	xmlFile:setValue(key .. "#pendingLiters", spec.pendingLiters)
end

function PlaceableHusbandryGoatMilk:onReadStream(streamId, connection)
	local spec = self.spec_husbandryGoatMilk
	spec.pendingLiters = streamReadFloat32(streamId)
end

function PlaceableHusbandryGoatMilk:onWriteStream(streamId, connection)
	local spec = self.spec_husbandryGoatMilk

	streamWriteFloat32(streamId, spec.pendingLiters)
end

function PlaceableHusbandryGoatMilk:onReadUpdateStream(streamId, timestamp, connection)
	if connection:getIsServer() and streamReadBool(streamId) then
		local spec = self.spec_husbandryGoatMilk
		spec.pendingLiters = streamReadFloat32(streamId)
	end
end

function PlaceableHusbandryGoatMilk:onWriteUpdateStream(streamId, connection, dirtyMask)
	if not connection:getIsServer() then
		local spec = self.spec_husbandryGoatMilk

		if streamWriteBool(streamId, bitAND(dirtyMask, spec.dirtyFlag) ~= 0) then
			streamWriteFloat32(streamId, spec.pendingLiters)
		end
	end
end

function PlaceableHusbandryGoatMilk:onLoad(savegame)
	self.spec_husbandryGoatMilk = {}
	local spec = self.spec_husbandryGoatMilk
	spec.litersPerHour = 0
	spec.pendingLiters = 0
	spec.fillType = FillType.GOATMILK
	spec.info = {
		text = "",
		title = g_i18n:getText("TL_fillType_goatMilk")
	}
	spec.milkingRobots = {}
	spec.husbandry = nil

	self.xmlFile:iterate("placeable.husbandry.milk.milkingRobots.milkingRobot", function (_, key)
		local filename = Utils.getFilename(self.xmlFile:getValue(key .. "#filename", nil), self.baseDirectory)

		if filename == nil then
			Logging.xmlWarning(self.xmlFile, "Milkingrobot filename missing for '%s'", key)

			return
		end

		local className = self.xmlFile:getValue(key .. "#class", "")
		local modEnvMeta = getmetatable(_G)
		local env = modEnvMeta.__index
		local class = env.ClassUtil.getClassObject(className)

		if class == nil then
			Logging.xmlWarning(self.xmlFile, "Milkingrobot class '%s' not defined for '%s'", className, key)

			return
		end

		local linkNode = self.xmlFile:getValue(key .. "#linkNode", nil, self.components, self.i3dMappings)

		if linkNode == nil then
			Logging.xmlWarning(self.xmlFile, "Milkingrobot linkNode not defined for '%s'", key)

			return
		end

		local args = {
			loadingTask = self:createLoadingTask(self)
		}
		local robot = class.new(self, self.baseDirectory)

		if robot:load(linkNode, filename, self.onMilkingRobotLoaded, self, args) then
			table.insert(spec.milkingRobots, robot)
		else
			self:finishLoadingTask(args.loadingTask)
			robot:delete()
		end
	end)
	spec.dirtyFlag = self:getNextDirtyFlag()
end

function PlaceableHusbandryGoatMilk:onMilkingRobotLoaded(robot, args)
	self:finishLoadingTask(args.loadingTask)
end

function PlaceableHusbandryGoatMilk:onDelete()
    local spec = self.spec_husbandryGoatMilk

	if spec ~= nil and spec.milkingRobots ~= nil then
		for _, robot in ipairs(spec.milkingRobots) do
			robot:delete()
		end

		spec.milkingRobots = {}
	end
end

function PlaceableHusbandryGoatMilk:onFinalizePlacement()
   local spec = self.spec_husbandryGoatMilk

	for _, robot in ipairs(spec.milkingRobots) do
		robot:finalizePlacement()
	end
end

function PlaceableHusbandryGoatMilk:updateOutput(superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor)
    local spec = self.spec_husbandryGoatMilk

	local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex);

	local litersPerHour = 0;
	if subType ~= nil then
		local milk = subType.output.milk;

		if milk ~= nil and cluster.reproduction < 80 and cluster.monthSinceLastBirth ~= nil then
			local age = cluster:getAge();
			local litersPerAnimals = milk:get(age);
			local litersPerDay = litersPerAnimals * cluster:getNumAnimals();
			local lactationFactor = cluster:getLactationMilkFactor()
			litersPerHour = litersPerHour + litersPerDay / 24
			litersPerHour = litersPerHour * lactationFactor
			if cluster.milkBonus ~= nil then
				local milkBonus = cluster.milkBonus + 1;
				litersPerHour = litersPerHour * milkBonus;
			end
		end
	end

	if self.isServer and litersPerHour > 0 then
		local liters = productionFactor * clusterProductionFactor * litersPerHour * g_currentMission.environment.timeAdjustment;
		spec.pendingLiters = spec.pendingLiters + liters
		
		if spec.pendingLiters > 3000 then
			spec.pendingLiters  = 3000
		end

		self:raiseDirtyFlags(spec.dirtyFlag)
	end

	superFunc(self, cluster, foodFactor, productionFactor, clusterProductionFactor)
end

function PlaceableHusbandryGoatMilk:updateProduction(superFunc, foodFactor)
	local spec = self.spec_husbandryGoatMilk
	local factor = superFunc(self, foodFactor)

	return factor
end

function PlaceableHusbandryGoatMilk:onHusbandryAnimalsUpdate(clusters)
    local spec = self.spec_husbandryGoatMilk
	spec.litersPerHour = 0

	for _, cluster in ipairs(clusters) do
		local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex)

		if subType ~= nil then
			local milk = subType.output.milk

			if milk ~= nil and cluster.reproduction < 80 and cluster.monthSinceLastBirth ~= nil then
				local age = cluster:getAge()
				local litersPerAnimals = milk:get(age)
				local litersPerDay = litersPerAnimals * cluster:getNumAnimals()
				local lactationFactor = cluster:getLactationMilkFactor()
				spec.litersPerHour = spec.litersPerHour + litersPerDay / 24
				spec.litersPerHour = spec.litersPerHour * lactationFactor
			end
		end
	end
end

function PlaceableHusbandryGoatMilk:updateInfo(superFunc, infoTable)
    local spec = self.spec_husbandryGoatMilk

	superFunc(self, infoTable)

	local fillLevel = spec.pendingLiters
	spec.info.text = string.format("%d l", fillLevel)

	table.insert(infoTable, spec.info)
end

function PlaceableHusbandryGoatMilk:getConditionInfos(superFunc)
    local spec = self.spec_husbandryGoatMilk
	local infos = superFunc(self)
	local info = {}
	local fillType = g_fillTypeManager:getFillTypeByIndex(spec.fillType)
	info.title = fillType.title
	info.value = spec.pendingLiters
	local capacity = 3000
	info.valueText = g_i18n:formatNumber(info.value, 0) .. " / " .. g_i18n:formatVolume(capacity, 0);
	local ratio = info.value / capacity

	info.ratio = MathUtil.clamp(ratio, 0, 1)
	info.invertedBar = true

	table.insert(infos, info)

	return infos
end