PlaceableHusbandryHayMilk = {
	ROBOT_LOADING_FIX_ENABLED = true
}

source("dataS/scripts/animals/husbandry/objects/MilkingRobot.lua")

function PlaceableHusbandryHayMilk.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(PlaceableHusbandry, specializations)
end

function PlaceableHusbandryHayMilk.registerFunctions(placeableType)
	SpecializationUtil.registerFunction(placeableType, "onMilkingRobotLoaded", PlaceableHusbandryHayMilk.onMilkingRobotLoaded)
end

function PlaceableHusbandryHayMilk.registerOverwrittenFunctions(placeableType)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateOutput", PlaceableHusbandryHayMilk.updateOutput)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateProduction", PlaceableHusbandryHayMilk.updateProduction)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "updateInfo", PlaceableHusbandryHayMilk.updateInfo)
	SpecializationUtil.registerOverwrittenFunction(placeableType, "getConditionInfos", PlaceableHusbandryHayMilk.getConditionInfos)
end

function PlaceableHusbandryHayMilk.registerEventListeners(placeableType)
	SpecializationUtil.registerEventListener(placeableType, "onLoad", PlaceableHusbandryHayMilk)
	SpecializationUtil.registerEventListener(placeableType, "onDelete", PlaceableHusbandryHayMilk)
	SpecializationUtil.registerEventListener(placeableType, "onFinalizePlacement", PlaceableHusbandryHayMilk)
	SpecializationUtil.registerEventListener(placeableType, "onHusbandryAnimalsUpdate", PlaceableHusbandryHayMilk)
end

function PlaceableHusbandryHayMilk.registerXMLPaths(schema, basePath)
	schema:setXMLSpecializationType("Husbandry")

	basePath = basePath .. ".husbandry.milk"

	schema:register(XMLValueType.NODE_INDEX, basePath .. ".milkingRobots.milkingRobot(?)#linkNode", "Milkingrobot link node")
	schema:register(XMLValueType.STRING, basePath .. ".milkingRobots.milkingRobot(?)#class", "Milkingrobot class name")
	schema:register(XMLValueType.STRING, basePath .. ".milkingRobots.milkingRobot(?)#filename", "Milkingrobot config file")
	schema:setXMLSpecializationType()
end

function PlaceableHusbandryHayMilk:onLoad(savegame)
--TerraLiveExtension.DebugTable("self", self)
	if self.spec_husbandryHayMilk == nil then self.spec_husbandryHayMilk = {} end
	local spec = self.spec_husbandryHayMilk
	spec.litersPerHour = 0
	spec.fillType = FillType.HAYMILK
	spec.info = {
		text = "",
		title = g_i18n:getText("TL_fillType_hayMilk")
	}
	spec.milkingRobots = {}
	spec.husbandry = nil

	self.xmlFile:iterate("placeable.husbandry.milk.milkingRobots.milkingRobot", function (_, key)
		local filename = Utils.getFilename(self.xmlFile:getValue(key .. "#filename", nil), self.baseDirectory)

		if filename == nil then
			Logging.xmlWarning(self.xmlFile, "Milkingrobot filename missing for '%s'", key)

			return
		end

		local className = self.xmlFile:getValue(key .. "#class", "")
		local modEnvMeta = getmetatable(_G)
		local env = modEnvMeta.__index
		local class = env.ClassUtil.getClassObject(className)

		if class == nil then
			Logging.xmlWarning(self.xmlFile, "Milkingrobot class '%s' not defined for '%s'", className, key)

			return
		end

		local linkNode = self.xmlFile:getValue(key .. "#linkNode", nil, self.components, self.i3dMappings)

		if linkNode == nil then
			Logging.xmlWarning(self.xmlFile, "Milkingrobot linkNode not defined for '%s'", key)

			return
		end

		local args = {
			loadingTask = self:createLoadingTask(self)
		}
		local robot = class.new(self, self.baseDirectory)

		if robot:load(linkNode, filename, self.onMilkingRobotLoaded, self, args) then
			table.insert(spec.milkingRobots, robot)
		else
			self:finishLoadingTask(args.loadingTask)
			robot:delete()
		end
	end)
end

function PlaceableHusbandryHayMilk:onMilkingRobotLoaded(robot, args)
	self:finishLoadingTask(args.loadingTask)
end

function PlaceableHusbandryHayMilk:onDelete()
	local spec = self.spec_husbandryHayMilk

	if spec ~= nil and spec.milkingRobots ~= nil then
		for _, robot in ipairs(spec.milkingRobots) do
			robot:delete()
		end

		spec.milkingRobots = {}
	end
end

function PlaceableHusbandryHayMilk:onFinalizePlacement()
	local spec = self.spec_husbandryHayMilk

	if not self:getHusbandryIsFillTypeSupported(spec.fillType) then
		Logging.warning("Missing filltype 'haymilk' in husbandry storage!")
	end

	for _, robot in ipairs(spec.milkingRobots) do
		robot:finalizePlacement()
	end
end

function PlaceableHusbandryHayMilk:updateOutput(superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor)
	--print(string.format("PlaceableHusbandryMilkExtension:updateOutput(%s, %s, %s, %s, %s, %s)", superFunc2, superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor));
	local spec = self.spec_husbandryHayMilk;

	local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex);

	-- liter selbst berechnen, damit cluster abhängig
	-- wird in PlaceableHusbandryMilk:onHusbandryAnimalsUpdate(clusters) für alle cluster gesetzt
	local litersPerHour = 0;
	if subType ~= nil then
		local milk = subType.output.milk;

		if milk ~= nil and cluster.reproduction < 80 and cluster.monthSinceLastBirth ~= nil then
			local age = cluster:getAge();
			local litersPerAnimals = milk:get(age);
			local litersPerDay = litersPerAnimals * cluster:getNumAnimals();
			local lactationFactor = cluster:getLactationMilkFactor()
			litersPerHour = litersPerHour + litersPerDay / 24
			litersPerHour = litersPerHour * lactationFactor
			if cluster.milkBonus ~= nil then
				local milkBonus = cluster.milkBonus + 1;
				litersPerHour = litersPerHour * milkBonus;
			end
		end
	end
		
	if self.isServer and litersPerHour > 0 then
		local liters = productionFactor * clusterProductionFactor * litersPerHour * g_currentMission.environment.timeAdjustment;

		if not cluster:produceColostrum() then
			self:addHusbandryFillLevelFromTool(self:getOwnerFarmId(), liters, spec.fillType, nil, nil, nil)
		end
	end

	superFunc(self, cluster, foodFactor, productionFactor, clusterProductionFactor)
end



function PlaceableHusbandryHayMilk:updateProduction(superFunc, foodFactor)
	local spec = self.spec_husbandryHayMilk
	local factor = superFunc(self, foodFactor)
	local freeCapacity = self:getHusbandryFreeCapacity(spec.fillType)

	if freeCapacity <= 0 then
		factor = 0
	end

	return factor
end

function PlaceableHusbandryHayMilk:onHusbandryAnimalsUpdate(clusters)
	local spec = self.spec_husbandryHayMilk
	spec.litersPerHour = 0

	for _, cluster in ipairs(clusters) do
		local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex)

		if subType ~= nil then
			local milk = subType.output.milk

			if milk ~= nil and cluster.reproduction < 80 and cluster.monthSinceLastBirth ~= nil then
				local age = cluster:getAge()
				local litersPerAnimals = milk:get(age)
				local litersPerDay = litersPerAnimals * cluster:getNumAnimals()
				local lactationFactor = cluster:getLactationMilkFactor()
				spec.litersPerHour = spec.litersPerHour + litersPerDay / 24
				spec.litersPerHour = spec.litersPerHour * lactationFactor
			end
		end
	end
end

function PlaceableHusbandryHayMilk:updateInfo(superFunc, infoTable)
	local spec = self.spec_husbandryHayMilk

	superFunc(self, infoTable)

	local fillLevel = self:getHusbandryFillLevel(spec.fillType)
	spec.info.text = string.format("%d l", fillLevel)

	table.insert(infoTable, spec.info)
end

function PlaceableHusbandryHayMilk:getConditionInfos(superFunc)
	local spec = self.spec_husbandryHayMilk
	local infos = superFunc(self)
	local info = {}
	local fillType = g_fillTypeManager:getFillTypeByIndex(spec.fillType)
	info.title = fillType.title
	info.value = self:getHusbandryFillLevel(spec.fillType)
	local capacity = self:getHusbandryCapacity(spec.fillType)
	info.valueText = g_i18n:formatNumber(info.value, 0) .. " / " .. g_i18n:formatVolume(capacity, 0);
	local ratio = 0

	if capacity > 0 then
		ratio = info.value / capacity
	end

	info.ratio = MathUtil.clamp(ratio, 0, 1)
	info.invertedBar = true

	table.insert(infos, info)

	return infos
end