PlaceableHusbandryMilkExtension = {}

function PlaceableHusbandryMilkExtension:updateOutput(superFunc2, superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor)
	--print(string.format("PlaceableHusbandryMilkExtension:updateOutput(%s, %s, %s, %s, %s, %s)", superFunc2, superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor));
	local spec = self.spec_husbandryMilk;

	local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex);

	-- liter selbst berechnen, damit cluster abhängig
	-- wird in PlaceableHusbandryMilk:onHusbandryAnimalsUpdate(clusters) für alle cluster gesetzt
	local litersPerHour = 0;
	if subType ~= nil then
		local milk = subType.output.milk

		if milk ~= nil and cluster.reproduction < 80 and cluster.monthSinceLastBirth ~= nil then
			local age = cluster:getAge()
			local litersPerAnimals = milk:get(age)
			local litersPerDay = litersPerAnimals * cluster:getNumAnimals()
			local lactationFactor = cluster:getLactationMilkFactor()
			litersPerHour = litersPerHour + litersPerDay / 24
			litersPerHour = litersPerHour * lactationFactor
			if cluster.milkBonus ~= nil then
				local milkBonus = cluster.milkBonus + 1
				litersPerHour = litersPerHour * milkBonus
			end
		end
	end
		
	if self.isServer and litersPerHour > 0 then
		local liters = productionFactor * clusterProductionFactor * litersPerHour * g_currentMission.environment.timeAdjustment

		if not cluster:produceColostrum() then
			self:addHusbandryFillLevelFromTool(self:getOwnerFarmId(), liters, spec.fillType, nil, nil, nil)
		end
	end

	superFunc(self, cluster, foodFactor, productionFactor, clusterProductionFactor)
end

PlaceableHusbandryMilk.updateOutput = Utils.overwrittenFunction(PlaceableHusbandryMilk.updateOutput, PlaceableHusbandryMilkExtension.updateOutput)


function PlaceableHusbandryMilkExtension:getConditionInfos(superFunc2, superFunc)
	local spec = self.spec_husbandryMilk
	local infos = superFunc(self)
	local info = {}
	local fillType = g_fillTypeManager:getFillTypeByIndex(spec.fillType)
	info.title = fillType.title
	info.value = self:getHusbandryFillLevel(spec.fillType)
	local capacity = self:getHusbandryCapacity(spec.fillType)
	info.valueText = g_i18n:formatNumber(info.value, 0) .. " / " .. g_i18n:formatVolume(capacity, 0);
	local ratio = 0

	if capacity > 0 then
		ratio = info.value / capacity
	end

	info.ratio = MathUtil.clamp(ratio, 0, 1)
	info.invertedBar = true

	table.insert(infos, info)

	return infos
end
PlaceableHusbandryMilk.getConditionInfos = Utils.overwrittenFunction(PlaceableHusbandryMilk.getConditionInfos, PlaceableHusbandryMilkExtension.getConditionInfos)