PlaceableHusbandryStrawExtension = {}
function PlaceableHusbandryStrawExtension:updateOutput(superFunc2, superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor)
	--print(string.format("PlaceableHusbandryStrawExtension:updateOutput(%s, %s, %s, %s, %s, %s)", superFunc2, superFunc, cluster, foodFactor, productionFactor, clusterProductionFactor));
	if self.isServer then
		local spec = self.spec_husbandryStraw
		
		-- liter selbst berechnen, damit cluster abhängig
		-- wird in PlaceableHusbandryStraw:onHusbandryAnimalsUpdate(clusters) für alle cluster gesetzt
		local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex)
		local inputLitersPerHour = 0
		local outputLitersPerHour = 0
		if subType ~= nil then
			local straw = subType.input.straw

			if straw ~= nil then
				local age = cluster:getAge()
				local litersPerAnimal = straw:get(age)
				local litersPerDay = litersPerAnimal * cluster:getNumAnimals()
				inputLitersPerHour = inputLitersPerHour + litersPerDay / 24
			end

			local manure = subType.output.manure

			if manure ~= nil then
				local age = cluster:getAge()
				local litersPerAnimal = manure:get(age)
				local litersPerDay = litersPerAnimal * cluster:getNumAnimals()
				outputLitersPerHour = outputLitersPerHour + litersPerDay / 24
			end
		end

		if inputLitersPerHour > 0 then
			local amount = inputLitersPerHour * g_currentMission.environment.timeAdjustment
			local delta = amount - self:removeHusbandryFillLevel(self:getOwnerFarmId(), amount, spec.inputFillType)

			if outputLitersPerHour > 0 and delta > 0 then
				local liters = foodFactor * outputLitersPerHour * delta / amount * g_currentMission.environment.timeAdjustment

				if liters > 0 then
					self:addHusbandryFillLevelFromTool(self:getOwnerFarmId(), liters, spec.outputFillType, nil, nil, nil)
				end
			end

			self:updateStrawPlane()
		end
	end

	superFunc(self, cluster, foodFactor, productionFactor, clusterProductionFactor)
end
PlaceableHusbandryStraw.updateOutput = Utils.overwrittenFunction(PlaceableHusbandryStraw.updateOutput, PlaceableHusbandryStrawExtension.updateOutput)


--FillTypeName von Stroh gegen Einstreu austauschen
function PlaceableHusbandryStrawExtension:getConditionInfos(superFunc)
	local infos = superFunc(self)
	local spec = self.spec_husbandryStraw
	local info = {}
	local fillType = g_fillTypeManager:getFillTypeByIndex(spec.inputFillType)
	info.title = g_i18n:getText("TL_litter")
	info.value = self:getHusbandryFillLevel(spec.inputFillType)
	local capacity = self:getHusbandryCapacity(spec.inputFillType)
	info.valueText = g_i18n:formatNumber(info.value, 0) .. " / " .. g_i18n:formatVolume(capacity, 0);
	local ratio = 0

	if capacity > 0 then
		ratio = info.value / capacity
	end

	info.ratio = MathUtil.clamp(ratio, 0, 1)
	info.invertedBar = false

	table.insert(infos, info)

	return infos
end
PlaceableHusbandryStraw.getConditionInfos = PlaceableHusbandryStrawExtension.getConditionInfos