--Reworks a lot of Mission Related Stuff

TLBaleMissionExtension = {}

function TLBaleMissionExtension:getData()
	if self.sellPointId ~= nil then
		self:tryToResolveSellPoint()
	end

	local l10nString = "TL_fieldJob_desc_baling_hay"
	local fill = FillType.GRASS
	if self.fillType == FillType.DRYGRASS_WINDROW then
	elseif self.fillType == FillType.DRYCLOVERGRASS_WINDROW then
		fill = FillType.CLOVERGRASS
	elseif self.fillType == FillType.DRYHORSEGRASS_WINDROW then
		fill = FillType.HORSEGRASS
	elseif self.fillType == FillType.DRYFIELDGRASS_WINDROW then
		fill = FillType.FIELDGRASS
	elseif self.fillType == FillType.DRYHERBGRASS_WINDROW then
		fill = FillType.HERBGRASS
	elseif self.fillType == FillType.DRYALFALFA_WINDROW then
		fill = FillType.ALFALFA
	elseif self.fillType == FillType.DRYCLOVER_WINDROW then
		fill = FillType.CLOVER
	elseif self.fillType == FillType.GRASS_FERMENTED then
		l10nString = "TL_fieldJob_desc_baling_silage"
	elseif self.fillType == FillType.CLOVERGRASS_FERMENTED then
		l10nString = "TL_fieldJob_desc_baling_silage"
		fill = FillType.CLOVERGRASS
	elseif self.fillType == FillType.HORSEGRASS_FERMENTED then
		l10nString = "TL_fieldJob_desc_baling_silage"
		fill = FillType.HORSEGRASS
	elseif self.fillType == FillType.FIELDGRASS_FERMENTED then
		l10nString = "TL_fieldJob_desc_baling_silage"
		fill = FillType.FIELDGRASS
	elseif self.fillType == FillType.HERBGRASS_FERMENTED then
		l10nString = "TL_fieldJob_desc_baling_silage"
		fill = FillType.HERBGRASS
	elseif self.fillType == FillType.ALFALFA_FERMENTED then
		l10nString = "TL_fieldJob_desc_baling_silage"
		fill = FillType.ALFALFA
	elseif self.fillType == FillType.CLOVER_FERMENTED then
		l10nString = "TL_fieldJob_desc_baling_silage"
		fill = FillType.CLOVER
	end

	local fillType = g_fillTypeManager:getFillTypeTitleByIndex(fill)
	return {
		location = string.format(g_i18n:getText("fieldJob_number"), self.field.fieldId),
		jobType = g_i18n:getText("fieldJob_jobType_baling"),
		action = g_i18n:getText("fieldJob_desc_action_baling"),
		description = string.format(g_i18n:getText(l10nString), fillType, self.field.fieldId, self.sellPoint:getName())
	}
end

function TLBaleMissionExtension:init(superFunc, field, sprayFactor, fieldSpraySet, fieldPlowFactor, fieldState, growthState, limeFactor, weedFactor, weedState, stubbleFactor, rollerFactor, args)
	if math.random() < BaleMission.SILAGE_VARIANT_CHANCE then
		if field.fruitType == FruitType.GRASS then
			self.fillType = FillType.GRASS_FERMENTED
		elseif field.fruitType == FruitType.CLOVERGRASS then
			self.fillType = FillType.CLOVERGRASS_FERMENTED
		elseif field.fruitType == FruitType.MEADOW then
			self.fillType = FillType.GRASS_FERMENTED
		elseif field.fruitType == FruitType.HORSEGRASS then
			self.fillType = FillType.HORSEGRASS_FERMENTED
		elseif field.fruitType == FruitType.FIELDGRASS then
			self.fillType = FillType.FIELDGRASS_FERMENTED
		elseif field.fruitType == FruitType.HERBGRASS then
			self.fillType = FillType.HERBGRASS_FERMENTED
		elseif field.fruitType == FruitType.ALFALFA then
			self.fillType = FillType.ALFALFA_FERMENTED
		elseif field.fruitType == FruitType.CLOVER then
			self.fillType = FillType.CLOVER_FERMENTED
		end
		self.workAreaTypes[WorkAreaType.TEDDER] = false
	else
		if field.fruitType == FruitType.GRASS then
			self.fillType = FillType.DRYGRASS_WINDROW
		elseif field.fruitType == FruitType.CLOVERGRASS then
			self.fillType = FillType.DRYCLOVERGRASS_WINDROW
		elseif field.fruitType == FruitType.MEADOW then
			self.fillType = FillType.DRYGRASS_WINDROW
		elseif field.fruitType == FruitType.HORSEGRASS then
			self.fillType = FillType.DRYHORSEGRASS_WINDROW
		elseif field.fruitType == FruitType.FIELDGRASS then
			self.fillType = FillType.DRYFIELDGRASS_WINDROW
		elseif field.fruitType == FruitType.HERBGRASS then
			self.fillType = FillType.DRYHERBGRASS_WINDROW
		elseif field.fruitType == FruitType.ALFALFA then
			self.fillType = FillType.DRYALFALFA_WINDROW
		elseif field.fruitType == FruitType.CLOVER then
			self.fillType = FillType.DRYCLOVER_WINDROW
		end
	end

		local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(field.fruitType)
		self.completionModifier = DensityMapModifier.new(fruitDesc.terrainDataPlaneId, fruitDesc.startStateChannel, fruitDesc.numStateChannels, self.mission.terrainRootNode)
		self.completionFilter = DensityMapFilter.new(self.completionModifier)

		self.completionFilter:setValueCompareParams(DensityValueCompareType.EQUAL, fruitDesc.cutState)

	self:updateRewardPerHa()

	if not BaleMission:superClass().init(self, field, sprayFactor, fieldSpraySet, fieldPlowFactor, fieldState, growthState, limeFactor, weedFactor, weedState, stubbleFactor, rollerFactor, args) then
		return false
	end

	self.depositedLiters = 0
	self.expectedLiters = self:roundToWholeBales(self:getMaxCutLiters())
	local highestPrice = 0

	for _, unloadingStation in pairs(self.mission.storageSystem:getUnloadingStations()) do
		if unloadingStation.owningPlaceable ~= nil and unloadingStation:isa(SellingStation) and unloadingStation.allowMissions and unloadingStation.acceptedFillTypes[self.fillType] == true then
			local price = unloadingStation:getEffectiveFillTypePrice(self.fillType)

			if highestPrice < price then
				highestPrice = price
				self.sellPoint = unloadingStation
			end
		end
	end

	if self.sellPoint == nil then
		return false
	end

	self:updateRewardPerHa()

	self.reward = self:calculateReward()

	return true
end

function TLBaleMissionExtension.canRunOnField(field, sprayFactor, fieldSpraySet, fieldPlowFactor, limeFactor, maxWeedState, stubbleFactor, rollerFactor)
	local fruitType = field.fruitType
	--local fruitType = FruitType.GRASS
	local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(fruitType)
	local maxGrowthState = FieldUtil.getMaxHarvestState(field, fruitType)
	local environment = g_currentMission.environment

	if environment ~= nil and environment.currentSeason == Environment.SEASON.WINTER then
		return false
	end

	if maxGrowthState ~= nil and maxGrowthState >= (fruitDesc.minHarvestingGrowthState + 2) and maxGrowthState <= fruitDesc.maxHarvestingGrowthState then
		return true, FieldManager.FIELDSTATE_GROWING, maxGrowthState
	end

	return false
end

function TLBaleMissionExtension:getVehicleVariant(superFunc)
	if self.fillType == FillType.DRYGRASS_WINDROW or self.fillType == FillType.DRYCLOVERGRASS_WINDROW or self.fillType == FillType.DRYHORSEGRASS_WINDROW or self.fillType == FillType.DRYFIELDGRASS_WINDROW or self.fillType == FillType.DRYHERBGRASS_WINDROW or self.fillType == FillType.DRYALFALFA_WINDROW or self.fillType == FillType.DRYCLOVER_WINDROW then
		return "HAY"
	else
		return "SILAGE"
	end
end

function TLBaleMissionExtension:loadFromXMLFile(superFunc, xmlFile, key)
	if not BaleMission:superClass().loadFromXMLFile(self, xmlFile, key) then
		return false
	end

	local baleKey = key .. ".bale(0)"
	local sellPointPlaceableId = getXMLInt(xmlFile, baleKey .. "#sellPointPlaceableId")
	local unloadingStationIndex = getXMLInt(xmlFile, baleKey .. "#unloadingStationIndex")

	if sellPointPlaceableId == nil then
		Logging.xmlError(xmlFile, "no sellPointPlaceable id given at '%s'", baleKey)

		return false
	end

	if unloadingStationIndex == nil then
		Logging.xmlError(xmlFile, "no unloadting station index given at '%s'", baleKey)

		return false
	end

	local placeable = g_currentMission.placeableSystem:getPlaceableBySavegameId(sellPointPlaceableId)

	if placeable == nil then
		Logging.xmlError(xmlFile, "selling station placeable with id '%d' not available at '%s'", sellPointPlaceableId, baleKey)

		return false
	end

	local unloadingStation = g_currentMission.storageSystem:getPlaceableUnloadingStation(placeable, unloadingStationIndex)

	if unloadingStation == nil then
		Logging.xmlError(xmlFile, "Unable to retrieve unloadingStation %d for placeable %s at '%s'", unloadingStationIndex, placeable.configFileName, baleKey)

		return false
	end

	self.sellPoint = unloadingStation
	self.sellPoint.missions[self] = self
	local fillTypeName = getXMLString(xmlFile, baleKey .. "#fillTypeName")
	self.fillType = g_fillTypeManager:getFillTypeIndexByName(fillTypeName)

	self:updateRewardPerHa()

	self.depositedLiters = getXMLFloat(xmlFile, baleKey .. "#depositedLiters")
	self.expectedLiters = self:roundToWholeBales(self:getMaxCutLiters())
	local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(self.field.fruitType)
	self.completionModifier = DensityMapModifier.new(fruitDesc.terrainDataPlaneId, fruitDesc.startStateChannel, fruitDesc.numStateChannels, self.mission.terrainRootNode)
	self.completionFilter = DensityMapFilter.new(self.completionModifier)

	self.completionFilter:setValueCompareParams(DensityValueCompareType.EQUAL, fruitDesc.cutState)
	if self.fillType == FillType.SILAGE then
		self.workAreaTypes[WorkAreaType.TEDDER] = false
	end

	return true
end

BaleMission.getData = Utils.overwrittenFunction(BaleMission.getData, TLBaleMissionExtension.getData)
BaleMission.init = Utils.overwrittenFunction(BaleMission.init, TLBaleMissionExtension.init)
BaleMission.canRunOnField = Utils.overwrittenFunction(BaleMission.canRunOnField, TLBaleMissionExtension.canRunOnField)
BaleMission.getVehicleVariant = Utils.overwrittenFunction(BaleMission.getVehicleVariant, TLBaleMissionExtension.getVehicleVariant)
BaleMission.loadFromXMLFile = Utils.overwrittenFunction(BaleMission.loadFromXMLFile, TLBaleMissionExtension.loadFromXMLFile)


TLHarvestMissionExtension = {}

function TLHarvestMissionExtension:getVehicleVariant(superFunc)
	local fruitType = self.field.fruitType
	if fruitType == FruitType.SUNFLOWER or fruitType == FruitType.MAIZE then
		return "MAIZE"
	elseif fruitType == FruitType.SUGARBEET then
		return "SUGARBEET"
	elseif fruitType == FruitType.FODDERBEET then
		return "FODDERBEET"
	elseif fruitType == FruitType.POTATO then
		return "POTATO"
	elseif fruitType == FruitType.STARCHPOTATO then
		return "STARCHPOTATO"
	elseif fruitType == FruitType.COTTON then
		return "COTTON"
	elseif fruitType == FruitType.SUGARCANE then
		return "SUGARCANE"
	elseif fruitType == FruitType.PARSNIP then
		return "PARSNIP"
	elseif fruitType == FruitType.BEETROOT then
		return "BEETROOT"
	elseif fruitType == FruitType.ONION then
		return "ONION"
	elseif fruitType == FruitType.CARROT then
		return "CARROT"
	elseif fruitType == FruitType.FODDERCARROT then
		return "FODDERCARROT"
	else
		--WHEAT, WINTERWHEAT, BARLEY, WINTERBARLEY, OAT, TRITICALE, SPELT, RYE, SORGHUM, CANOLA, SOYBEAN
		return "GRAIN"
	end
end

function TLHarvestMissionExtension:updateRewardPerHa()
	if self.fillType == FillType.SUGARCANE or self.fillType == FillType.POTATO or self.fillType == FillType.SUGARBEET or self.fillType == FillType.CARROT or self.fillType == FillType.FODDERBEET or self.fillType == FillType.STARCHPOTATO or self.fillType == FillType.FODDERCARROT or self.fillType == FillType.ONION or self.fillType == FillType.PARSNIP or self.fillType == FillType.BEETROOT then
		self.rewardPerHa = HarvestMission.REWARD_PER_HA_SMALL
	else
		self.rewardPerHa = HarvestMission.REWARD_PER_HA_WIDE
	end
end

HarvestMission.getVehicleVariant = Utils.overwrittenFunction(HarvestMission.getVehicleVariant, TLHarvestMissionExtension.getVehicleVariant)
HarvestMission.updateRewardPerHa = Utils.overwrittenFunction(HarvestMission.updateRewardPerHa, TLHarvestMissionExtension.updateRewardPerHa)


TLFieldManagerExtension = {}

function TLFieldManagerExtension:generatePlannedFruitForField(superFunc, field)
	if field.fieldGrassMission then
		return g_fieldManager.grassTypes[math.random(1, #g_fieldManager.grassTypes)]
	else
		return self.availableFruitTypeIndices[math.random(1, self.fruitTypesCount)]
	end
end

--Replace grass-fields with the other grass-types
function TLFieldManagerExtension:loadMapData(superFunc, xmlFile)
	superFunc(self, xmlFile)
	--Grass-Varianten aus map.xml laden
	local mapXML = XMLFile.wrap(xmlFile)
	local grasses = string.split(mapXML:getString("map.terraLife.fieldJobs.grasses#types", "GRASS CLOVERGRASS MEADOW HORSEGRASS FIELDGRASS HERBGRASS ALFALFA CLOVER"), " ")
	local i = 1
	g_fieldManager.grassTypes = {}
	for _, grass in pairs(grasses) do
		local desc = g_fruitTypeManager:getFruitTypeByName(grass)
		if desc ~= nil then
			g_fieldManager.grassTypes[i] = desc.index
			i = i + 1
		end
	end

--Felder Updaten, damit nicht nur Gras wächst
	g_asyncTaskManager:addSubtask(function ()
	local mission = g_currentMission
	if not mission.missionInfo.isValid and g_server ~= nil then
		for _, field in pairs(self.fields) do
			if field.fieldGrassMission then
				local fruitIndex = g_fieldManager.grassTypes[math.random(1, #g_fieldManager.grassTypes)]
				field.plannedFruit = fruitIndex
				local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(fruitIndex)
				local fieldState = FieldManager.FIELDSTATE_GROWING
				local plowState = nil

				if not mission.missionInfo.plowingRequiredEnabled then
					plowState = self.plowLevelMaxValue
				else
					plowState = math.random(0, self.plowLevelMaxValue)
				end

				local sprayLevel = math.random(0, self.sprayLevelMaxValue)
				local limeState = math.random(0, self.limeLevelMaxValue)
				local weedValue = 0
				local growthState = g_currentMission.growthSystem:getRandomInitialState(fruitIndex)

				if growthState == nil and fruitIndex == FruitType.GRASS then
					growthState = 2
				end

				if growthState ~= nil then
					if fruitDesc.plantsWeed then
						if growthState > 4 then
							weedValue = math.random(3, 9)
						else
							weedValue = math.random(1, 7)
						end
					end

					if growthState == fruitDesc.cutState then
						fieldState = FieldManager.FIELDSTATE_HARVESTED
					end
				else
					fieldState = math.random() < 0.5 and FieldManager.FIELDSTATE_CULTIVATED or FieldManager.FIELDSTATE_PLOWED

					if fieldState == FieldManager.FIELDSTATE_PLOWED then
						plowState = self.plowLevelMaxValue
					end

					fruitIndex = 0
				end

				for i = 1, table.getn(field.maxFieldStatusPartitions) do
					g_fieldManager:setFieldPartitionStatus(field, field.maxFieldStatusPartitions, i, fruitIndex, fieldState, growthState, sprayLevel, false, plowState, weedValue, limeState)
				end
			end
		end
	end
	end)
end

FieldManager.generatePlannedFruitForField = Utils.overwrittenFunction(FieldManager.generatePlannedFruitForField, TLFieldManagerExtension.generatePlannedFruitForField)
FieldManager.loadMapData = Utils.overwrittenFunction(FieldManager.loadMapData, TLFieldManagerExtension.loadMapData)


TLMissionManagerExtension = {}

function TLMissionManagerExtension:generateNewFieldMission(superFunc, field)
	if not field.fieldMissionAllowed then
		return nil
	end

	if field.currentMission ~= nil or not field:getIsAIActive() then
		return nil
	end

	local category = MissionManager.CATEGORY_FIELD

	if field.fruitType == FruitType.GRASS or field.fruitType == FruitType.CLOVERGRASS or field.fruitType == FruitType.MEADOW or field.fruitType == FruitType.HORSEGRASS or field.fruitType == FruitType.FIELDGRASS or field.fruitType == FruitType.HERBGRASS or field.fruitType == FruitType.ALFALFA or field.fruitType == FruitType.CLOVER then
		category = MissionManager.CATEGORY_GRASS_FIELD
	end

	local fieldSpraySet, sprayFactor, fieldPlowFactor, limeFactor, weedFactor, maxWeedState, stubbleFactor, rollerFactor = self:getFieldData(field)

	for _, missionType in ipairs(self.missionTypes) do
		if missionType.category == category then
			local canRun, fieldState, growthState, weedState, args = missionType.classObject.canRunOnField(field, sprayFactor, fieldSpraySet, fieldPlowFactor, limeFactor, maxWeedState, stubbleFactor, rollerFactor)

			if canRun then
				local mission = missionType.classObject.new(true, g_client ~= nil)
				mission.type = missionType

				if mission:init(field, sprayFactor, fieldSpraySet, fieldPlowFactor, fieldState, growthState, limeFactor, weedFactor, weedState, stubbleFactor, rollerFactor, args) then
					self:assignGenerationTime(mission)

					return mission
				else
					mission:delete()
				end
			end
		end
	end

	return nil
end

MissionManager.generateNewFieldMission = Utils.overwrittenFunction(MissionManager.generateNewFieldMission, TLMissionManagerExtension.generateNewFieldMission)


TLSprayMissionExtension = {}

function TLSprayMissionExtension:getNewWeedState()
	local weedState = self.weedState
	local replacements = self.mission.weedSystem:getHerbicideReplacements().weed.replacements

	-- here corrected to not need to give all weed states in the source and to speed up reading
	if replacements[weedState] ~= nil and replacements[weedState] ~= 0 then
		return replacements[weedState];
	end

	return weedState
end

SprayMission.getNewWeedState = Utils.overwrittenFunction(SprayMission.getNewWeedState, TLSprayMissionExtension.getNewWeedState)


function AbstractFieldMission:getMaxCutLiters()
	local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(self.field.fruitType)
	if fruitDesc ~= nil then
		local multiplier = g_currentMission:getHarvestScaleMultiplier(self.fruitType, self.sprayFactor, self.fieldPlowFactor, self.limeFactor, self.weedFactor, self.stubbleFactor, self.rollerFactor)
		local area = self.field.fieldArea * multiplier
		local literPerSqm = fruitDesc.literPerSqm

		if fruitDesc.hasWindrow then
			literPerSqm = math.min(literPerSqm, fruitDesc.windrowLiterPerSqm)
		end

		return literPerSqm * area * 10000
	else
		return 0
	end
end


TLSowMissionExtension = {}

function TLSowMissionExtension:getVehicleVariant()
	local fruitType = self.fruitType

	if fruitType == FruitType.SUNFLOWER or fruitType == FruitType.MAIZE then
		return "MAIZE"
	elseif fruitType == FruitType.SUGARBEET then
		return "SUGARBEET"
	elseif fruitType == FruitType.POTATO then
		return "POTATO"
	elseif fruitType == FruitType.COTTON then
		return "COTTON"
	elseif fruitType == FruitType.SUGARCANE then
		return "SUGARCANE"
	elseif fruitType == FruitType.CARROT then
		return "CARROT"
	elseif fruitType == FruitType.PARSNIP then
		return "PARSNIP"
	elseif fruitType == FruitType.BEETROOT then
		return "BEETROOT"
	-- here our fruits
	elseif fruitType == FruitType.STARCHPOTATO then
		return "STARCHPOTATO"
	elseif fruitType == FruitType.SILAGEMAIZE then
		return "SILAGEMAIZE"
	elseif fruitType == FruitType.FODDERBEET then
		return "FODDERBEET"
	elseif fruitType == FruitType.FODDERCARROT then
		return "FODDERCARROT"
	elseif fruitType == FruitType.ONION then
		return "ONION"
	elseif fruitType == FruitType.SILAGESORGHUM then
		return "SILAGESORGHUM"
	else
		return "GRAIN"
	end
end

SowMission.getVehicleVariant = Utils.overwrittenFunction(SowMission.getVehicleVariant, TLSowMissionExtension.getVehicleVariant)

function TLSowMissionExtension:getData()
	local seedName = "SEEDS_"..g_fruitTypeManager:getFruitTypeByIndex(self.fruitType).name

	return {
		location = string.format(g_i18n:getText("fieldJob_number"), self.field.fieldId),
		jobType = g_i18n:getText("fieldJob_jobType_sowing"),
		action = g_i18n:getText("fieldJob_desc_action_sowing"),
		description = string.format(g_i18n:getText("fieldJob_desc_sowing"), self.field.fieldId, g_fruitTypeManager:getFillTypeByFruitTypeIndex(self.fruitType).title),
		extraText = string.format(g_i18n:getText("fieldJob_desc_fillTheUnit"), g_fillTypeManager:getFillTypeByName(seedName).title)
	}
end

SowMission.getData = Utils.overwrittenFunction(SowMission.getData, TLSowMissionExtension.getData)

function TLSowMissionExtension:updateReimbursement(superFunc)
	--TerraLiveExtension.DebugText("----------------------------")
	--TerraLiveExtension.DebugText("Updating reimbursement for field %s", self.field and self.field.fieldId or "(nil)")
	local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(self.fruitType)
	local seedName = "SEEDS_" .. fruitDesc.name
	local seedType = g_fillTypeManager:getFillTypeByName(seedName)
	if seedType ~= nil then
		--TerraLiveExtension.DebugText("Seed type is valid: %s", seedType.name)
		--TerraLiveExtension.DebugText("Cost per liter: %.2f", self.mission.economyManager:getCostPerLiter(seedType.index))
		--TerraLiveExtension.DebugText("Seed usage per sqm: %.5f", fruitDesc.seedUsagePerSqm)
		self.reimbursementPerHa = fruitDesc.seedUsagePerSqm * 10000 * self.mission.economyManager:getCostPerLiter(seedType.index)
		--TerraLiveExtension.DebugText("Reimbursement per ha is %.2f", self.reimbursementPerHa)
	else
		-- Fallback: Use universal seed for the calculation
		--TerraLiveExtension.DebugText("Fallback")
		superFunc(self)
	end
	--TerraLiveExtension.DebugText("----------------------------")
end
SowMission.updateReimbursement = Utils.overwrittenFunction(SowMission.updateReimbursement, TLSowMissionExtension.updateReimbursement)
