--[[
Adds ability to select a fruitType for a machine, makes changing between different fruitTypes possible, for example potatoes and starchpotatoes

Only for Use with TerraLife Plus, It is not allowed to use this script without permission from Braeven / Farming Agency

by: [F/A]Braeven
date: 16.11.2023
Version 1.0 Alpha

Changelog
Version 1.0 - Initial Release
]]

fruitPreparerExtension = {}

function fruitPreparerExtension.initSpecialization(superFunc)
	g_configurationManager:addConfigurationType("fruitPreparer", g_i18n:getText("TL_configuration_fruitType"), "fruitPreparer", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
	
	local schema = Vehicle.xmlSchema

	schema:setXMLSpecializationType("fruitPreparer")
	schema:register(XMLValueType.STRING, "vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(?)#fruitType", "pallet filename")
end

FruitPreparer.initSpecialization = Utils.appendedFunction(FruitPreparer.initSpecialization, fruitPreparerExtension.initSpecialization)



function fruitPreparerExtension:onLoad(superFunc, savegame)
	if not self.xmlFile:hasProperty("vehicle.fruitPreparer.fruitPreparerConfigurations") then
		local check = self.xmlFile:getString("vehicle.fruitPreparer#fruitType")
		if check ~= nil then
			check = string.upper(check)
			if check == "POTATO" then
				self.xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(0)#fruitType", "POTATO")
				self.xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(0)#name", "$l10n_TL_fillType_potato")
				self.xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(1)#fruitType", "STARCHPOTATO")
				self.xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(1)#name", "$l10n_TL_fillType_starchPotato")
				if string.find(self.xmlFile.filename, "FS22_GrimmePack") or string.find(self.xmlFile.filename, "ks754.xml") then
					self.xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(2)#fruitType", "ONION")
					self.xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(2)#name", "$l10n_TL_fillType_onion")
				end
			end
		end
	end
	local spec = self.spec_fruitPreparer
	if self.xmlFile:hasProperty("vehicle.fruitPreparer.fruitPreparerConfigurations") then	
		self.xmlFile:iterate("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration", function(i, baseKey)
		local fruitType = self.xmlFile:getValue(baseKey.."#fruitType")
		local desc = g_fruitTypeManager:getFruitTypeByName(fruitType)
		if desc ~= nil then
			if self.configurations.fruitPreparer == i then
				spec.fruitType = desc.index

					self:setAIFruitRequirements(desc.index, desc.minPreparingGrowthState, desc.maxPreparingGrowthState)

					local aiUsePreparedState = self.xmlFile:getValue("vehicle.fruitPreparer#aiUsePreparedState", self.spec_cutter ~= nil)

					if aiUsePreparedState then
						self:addAIFruitRequirement(desc.index, desc.preparedGrowthState, desc.preparedGrowthState)
					end
				end
		else
			Logging.xmlWarning(self.xmlFile, "Unable to find fruitType '%s' in fruitPreparer", fruitType)
		end
		end)
	end
end

FruitPreparer.onLoad = Utils.appendedFunction(FruitPreparer.onLoad, fruitPreparerExtension.onLoad)



function fruitPreparerExtension.getConfigurationsFromXML(xmlFile, superFunc, key, baseDir, customEnvironment, isMod, storeItem)
	if xmlFile:hasProperty("vehicle.fruitPreparer") then
		if not xmlFile:hasProperty("vehicle.fruitPreparer.fruitPreparerConfigurations") then
			local check = xmlFile:getString("vehicle.fruitPreparer#fruitType")
			if check ~= nil then
				check = string.upper(check)
				if check == "POTATO" then
					xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(0)#fruitType", "POTATO")
					xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(0)#name", "$l10n_TL_fillType_potato")
					xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(1)#fruitType", "STARCHPOTATO")
					xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(1)#name", "$l10n_TL_fillType_starchPotato")
					if string.find(xmlFile.filename, "FS22_GrimmePack") or string.find(xmlFile.filename, "ks754.xml") then
						xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(2)#fruitType", "ONION")
						xmlFile:setString("vehicle.fruitPreparer.fruitPreparerConfigurations.fruitPreparerConfiguration(2)#name", "$l10n_TL_fillType_onion")
					end
				end
			end
		end
	end
	if xmlFile:hasProperty("vehicle.cutter") then
		if not xmlFile:hasProperty("vehicle.cutter.cutterConfigurations") then
			local check = xmlFile:getString("vehicle.cutter#fruitTypeCategories")
			if check ~= nil then
				check = string.upper(check)
				if check == "TOPLIFTINGHARVESTER" then
					xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(0)#fruitType", "CARROT")
					xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(0)#name", "$l10n_TL_fillType_carrot")
					xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(1)#fruitType", "BEETROOT")
					xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(1)#name", "$l10n_TL_fillType_beetRoot")
					xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(2)#fruitType", "PARSNIP")
					xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(2)#name", "$l10n_TL_fillType_parsnip")
					if string.find(xmlFile.filename, "FS22_GrimmePack") or string.find(xmlFile.filename, "ks754.xml") then
						xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(3)#fruitType", "ONION")
						xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(3)#name", "$l10n_TL_fillType_onion")
					end
				end
			end
		end
	end
	return superFunc(xmlFile, key, baseDir, customEnvironment, isMod, storeItem)
end

StoreItemUtil.getConfigurationsFromXML = Utils.overwrittenFunction(StoreItemUtil.getConfigurationsFromXML, fruitPreparerExtension.getConfigurationsFromXML)



function fruitPreparerExtension.onLoadCutter(self, superFunc, savegame)
	local check = "NIL"
	if self.xmlFile:hasProperty("vehicle.fruitPreparer") then
		check = string.upper(self.xmlFile:getString("vehicle.fruitPreparer#fruitType", "NIL"))
	else
		check = string.upper(self.xmlFile:getString("vehicle.cutter#fruitTypes", "NIL"))
	end
	if check == "POTATO" then
		--Zusätzliches Test für "Spezielle" Erntegeräte
		local test = self.xmlFile:getString("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(0).fillUnits.fillUnit(0)#fillTypes", nil)
		if test ~= nil then
			self.xmlFile:setString("vehicle.cutter#fruitTypes", "POTATO STARCHPOTATO")
			if string.find(self.xmlFile.filename, "FS22_GrimmePack") then
				self.xmlFile:setString("vehicle.cutter#fruitTypes", "POTATO STARCHPOTATO ONION")
			end
		end
	end

	return superFunc(self, savegame)
end

Cutter.onLoad = Utils.overwrittenFunction(Cutter.onLoad, fruitPreparerExtension.onLoadCutter)



function fruitPreparerExtension.onLoadFillUnit(self, superFunc, savegame)
	local check = "NIL"
	if self.xmlFile:hasProperty("vehicle.fruitPreparer") then
		check = string.upper(self.xmlFile:getString("vehicle.fruitPreparer#fruitType", "NIL"))
	elseif self.xmlFile:hasProperty("vehicle.cutter") then
		check = string.upper(self.xmlFile:getString("vehicle.cutter#fruitTypes", "NIL"))
	end
	if check == "POTATO" then
		--Zusätzliches Test für "Spezielle" Erntegeräte
		local test = self.xmlFile:getString("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(0).fillUnits.fillUnit(0)#fillTypes", nil)
		if test ~= nil then
			self.xmlFile:setString("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(0).fillUnits.fillUnit(0)#fillTypes", "POTATO STARCHPOTATO")
			if string.find(self.xmlFile.filename, "FS22_GrimmePack") then
				self.xmlFile:setString("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(0).fillUnits.fillUnit(0)#fillTypes", "POTATO STARCHPOTATO ONION")
			end
		end
	end

	return superFunc(self, savegame)
end

FillUnit.onLoad = Utils.overwrittenFunction(FillUnit.onLoad, fruitPreparerExtension.onLoadFillUnit)


function fruitPreparerExtension.initSpecializationCutter(superFunc)
	g_configurationManager:addConfigurationType("cutter", g_i18n:getText("TL_configuration_fruitType"), "cutter", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
	
	local schema = Vehicle.xmlSchema

	schema:setXMLSpecializationType("Cutter")
	schema:register(XMLValueType.STRING, "vehicle.cutter.cutterConfigurations.cutterConfiguration(?)#fruitType", "pallet filename")
end

Cutter.initSpecialization = Utils.appendedFunction(Cutter.initSpecialization, fruitPreparerExtension.initSpecializationCutter)



function fruitPreparerExtension:onLoadCutter(superFunc, savegame)
	if not self.xmlFile:hasProperty("vehicle.cutter.cutterConfigurations") then
		local check = self.xmlFile:getString("vehicle.cutter#fruitTypeCategories")
		if check ~= nil then
			check = string.upper(check)
			if check == "TOPLIFTINGHARVESTER" then
				self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(0)#fruitType", "CARROT")
				self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(0)#name", "$l10n_TL_fillType_carrot")
				self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(1)#fruitType", "BEETROOT")
				self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(1)#name", "$l10n_TL_fillType_beetRoot")
				self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(2)#fruitType", "PARSNIP")
				self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(2)#name", "$l10n_TL_fillType_parsnip")
				if string.find(self.xmlFile.filename, "FS22_GrimmePack") then
					self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(3)#fruitType", "ONION")
					self.xmlFile:setString("vehicle.cutter.cutterConfigurations.cutterConfiguration(3)#name", "$l10n_TL_fillType_onion")
				end
			end
		end
	end
	local spec = self.spec_cutter
	if self.xmlFile:hasProperty("vehicle.cutter.cutterConfigurations") then	
		self.xmlFile:iterate("vehicle.cutter.cutterConfigurations.cutterConfiguration", function(i, baseKey)
		local fruitType = self.xmlFile:getValue(baseKey.."#fruitType")
		local desc = g_fruitTypeManager:getFruitTypeByName(fruitType)
		if desc ~= nil then
			if self.configurations.cutter == i then
				spec.fruitTypes = {}
				table.insert(spec.fruitTypes, desc.index)
				local cutHeight = g_fruitTypeManager:getCutHeightByFruitTypeIndex(desc.index, spec.allowsForageGrowthState)
				self:setCutterCutHeight(cutHeight)

				spec.fillTypes = {}
				if spec.fruitTypeConverters[desc.index] ~= nil then
					table.insert(spec.fillTypes, spec.fruitTypeConverters[desc.index].fillTypeIndex)
				else
					local fillType = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(desc.index)

					if fillType ~= nil then
						table.insert(spec.fillTypes, fillType)
					end
				end
			end
		else
			Logging.xmlWarning(self.xmlFile, "Unable to find fruitType '%s' in cutter", fruitType)
		end
		end)
	end
end

Cutter.onLoad = Utils.appendedFunction(Cutter.onLoad, fruitPreparerExtension.onLoadCutter)