InGameMenuMapFrameExtension = {
    iconsFilename = "$moddir$FS22_TerraLifePlus_Assets/hud/ingameMapInfoIcons.png",
    Categories = {
        Street = {
            name = "CATEGORY_STREET",
            profile = "ingameMenuMapFilterButtonIconMapInfoStreet",
            profileInactive = "ingameMenuMapFilterButtonIconMapInfoStreetInactive",
            textKey = "TL_mapInfo_filter_street"
        },
        City = {
            name = "CATEGORY_CITY",
            profile = "ingameMenuMapFilterButtonIconMapInfoCity",
            profileInactive = "ingameMenuMapFilterButtonIconMapInfoCityInactive",
            textKey = "TL_mapInfo_filter_city"
        },
        Plot = {
            name = "CATEGORY_PLOT",
            profile = "ingameMenuMapFilterButtonIconMapInfoPlot",
            profileInactive = "ingameMenuMapFilterButtonIconMapInfoPlotInactive",
            textKey = "TL_mapInfo_filter_plot"
        },
        Farm = {
            name = "CATEGORY_FARM",
            profile = "ingameMenuMapFilterButtonIconMapInfoFarm",
            profileInactive = "ingameMenuMapFilterButtonIconMapInfoFarmInactive",
            textKey = "TL_mapInfo_filter_farm"
        }
    }
}

function InGameMenuMapFrameExtension.onFrameOpen(self, superFunc)
    local filename = Utils.getFilename(InGameMenuMapFrameExtension.iconsFilename, g_currentModDirectory)

    if self.filterButtonStreet == nil then
        local category = InGameMenuMapFrameExtension.Categories.Street
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[true][MapHotspot[category.name]] = category.profile
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[false][MapHotspot[category.name]] = category.profileInactive
        self.filterButtonStreet = self.filterButtonOther:clone()
        self.filterButtonStreet.onClickCallback = self.onFilterButtonStreetClicked
        self.filterButtonStreet.elements[1]:setImageFilename(filename)
        self.filterButtonStreet.elements[1]:applyProfile(category.profile)
        self.filterButtonStreet.elements[2]:setText(self.l10n:getText(category.textKey))
        self.mapOverviewHotspotBox:addElement(self.filterButtonStreet)
        self:setFilterIconState(self.filterButtonStreet, MapHotspot[category.name])
    end

    if self.filterButtonCity == nil then
        local category = InGameMenuMapFrameExtension.Categories.City
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[true][MapHotspot[category.name]] = category.profile
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[false][MapHotspot[category.name]] = category.profileInactive
        self.filterButtonCity = self.filterButtonOther:clone()
        self.filterButtonCity.onClickCallback = self.onFilterButtonCityClicked
        self.filterButtonCity.elements[1]:setImageFilename(filename)
        self.filterButtonCity.elements[1]:applyProfile(category.profile)
        self.filterButtonCity.elements[2]:setText(self.l10n:getText(category.textKey))
        self.mapOverviewHotspotBox:addElement(self.filterButtonCity)
        self:setFilterIconState(self.filterButtonCity, MapHotspot[category.name])
    end

    if self.filterButtonPlot == nil then
        local category = InGameMenuMapFrameExtension.Categories.Plot
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[true][MapHotspot[category.name]] = category.profile
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[false][MapHotspot[category.name]] = category.profileInactive
        self.filterButtonPlot = self.filterButtonOther:clone()
        self.filterButtonPlot.onClickCallback = self.onFilterButtonPlotClicked
        self.filterButtonPlot.elements[1]:setImageFilename(filename)
        self.filterButtonPlot.elements[1]:applyProfile(category.profile)
        self.filterButtonPlot.elements[2]:setText(self.l10n:getText(category.textKey))
        self.mapOverviewHotspotBox:addElement(self.filterButtonPlot)
        self:setFilterIconState(self.filterButtonPlot, MapHotspot[category.name])
    end

    if self.filterButtonFarm == nil then
        local category = InGameMenuMapFrameExtension.Categories.Farm
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[true][MapHotspot[category.name]] = category.profile
        InGameMenuMapFrame.HOTSPOT_FILTER_ICON_PROFILE[false][MapHotspot[category.name]] = category.profileInactive
        self.filterButtonFarm = self.filterButtonOther:clone()
        self.filterButtonFarm.onClickCallback = self.onFilterButtonFarmClicked
        self.filterButtonFarm.elements[1]:setImageFilename(filename)
        self.filterButtonFarm.elements[1]:applyProfile(category.profile)
        self.filterButtonFarm.elements[2]:setText(self.l10n:getText(category.textKey))
        self.mapOverviewHotspotBox:addElement(self.filterButtonFarm)
        self:setFilterIconState(self.filterButtonFarm, MapHotspot[category.name])
    end

    superFunc(self)
end

InGameMenuMapFrame.onFrameOpen = Utils.overwrittenFunction(InGameMenuMapFrame.onFrameOpen, InGameMenuMapFrameExtension.onFrameOpen)

function InGameMenuMapFrameExtension:buildFieldMapOverlay(superFunc, overlay)

    local mission = g_currentMission
    local fieldGroundSystem = mission.fieldGroundSystem
    local grassType = fieldGroundSystem:getFieldGroundValue(FieldGroundType.GRASS)
    local grassCutType = fieldGroundSystem:getFieldGroundValue(FieldGroundType.GRASS_CUT)
    local groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels = mission.fieldGroundSystem:getDensityMapData(FieldDensityMap.GROUND_TYPE)

    if groundTypeMapId ~= nil then
        local color = self.fieldColor
        local grassColor = self.grassFieldColor

        for i = 1, bitShiftLeft(1, groundTypeNumChannels) - 1 do
            if i == grassType or i == grassCutType then
                setDensityMapVisualizationOverlayStateColor(overlay, groundTypeMapId, 0, 0, groundTypeFirstChannel, groundTypeNumChannels, i, grassColor[1], grassColor[2], grassColor[3])
            else
                setDensityMapVisualizationOverlayStateColor(overlay, groundTypeMapId, 0, 0, groundTypeFirstChannel, groundTypeNumChannels, i, color[1], color[2], color[3])
            end
        end
    end
end
MapOverlayGenerator.buildFieldMapOverlay = Utils.overwrittenFunction(MapOverlayGenerator.buildFieldMapOverlay, InGameMenuMapFrameExtension.buildFieldMapOverlay)

function InGameMenuMapFrame:onFilterButtonStreetClicked(element)
	self.ingameMapBase:setHotspotFilter(MapHotspot.CATEGORY_STREET, not self.ingameMapBase.filter[MapHotspot.CATEGORY_STREET])
	self:setFilterIconState(element, MapHotspot.CATEGORY_STREET)
	self:selectFirstHotspot()
end

function InGameMenuMapFrame:onFilterButtonCityClicked(element)
	self.ingameMapBase:setHotspotFilter(MapHotspot.CATEGORY_CITY, not self.ingameMapBase.filter[MapHotspot.CATEGORY_CITY])
	self:setFilterIconState(element, MapHotspot.CATEGORY_CITY)
	self:selectFirstHotspot()
end

function InGameMenuMapFrame:onFilterButtonPlotClicked(element)
	self.ingameMapBase:setHotspotFilter(MapHotspot.CATEGORY_PLOT, not self.ingameMapBase.filter[MapHotspot.CATEGORY_PLOT])
	self:setFilterIconState(element, MapHotspot.CATEGORY_PLOT)
	self:selectFirstHotspot()
end

function InGameMenuMapFrame:onFilterButtonFarmClicked(element)
	self.ingameMapBase:setHotspotFilter(MapHotspot.CATEGORY_FARM, not self.ingameMapBase.filter[MapHotspot.CATEGORY_FARM])
	self:setFilterIconState(element, MapHotspot.CATEGORY_FARM)
	self:selectFirstHotspot()
end

function InGameMenuMapFrameExtension.onClickMap(self, superFunc, element, worldX, worldZ)
    superFunc(self, element, worldX, worldZ)
    self.fieldInfoBoxPosition = nil
    self.fieldInfoContextBox = nil

    if self.mode == InGameMenuMapFrame.MODE_OVERVIEW and self.currentHotspot == nil then
        local fruitTypeIndex, growthState = FSDensityMapUtil.getFruitTypeIndexAtWorldPos(worldX, worldZ)
        if fruitTypeIndex ~= nil then
            local fillType = g_fruitTypeManager:getFillTypeByFruitTypeIndex(fruitTypeIndex)

            local objectX = (worldX + self.ingameMapBase.worldCenterOffsetX) / self.ingameMapBase.worldSizeX * self.ingameMapBase.mapExtensionScaleFactor + self.ingameMapBase.mapExtensionOffsetX
            local objectZ = (worldZ + self.ingameMapBase.worldCenterOffsetZ) / self.ingameMapBase.worldSizeZ * self.ingameMapBase.mapExtensionScaleFactor + self.ingameMapBase.mapExtensionOffsetZ

            local x, y, z, visible = self.ingameMapBase.layout:getMapObjectPosition(objectX, objectZ, 0, 0, 0, true)

            InGameMenuMapUtil.showFruitInfoBox(self.contextBox, x, y, fillType.title)

            self.fieldInfoContextBox = self.contextBox
            self.fieldInfoBoxPosition = {
                x = worldX,
                z = worldZ
            }
        end
    end
end

InGameMenuMapFrame.onClickMap = Utils.overwrittenFunction(InGameMenuMapFrame.onClickMap, InGameMenuMapFrameExtension.onClickMap)

function InGameMenuMapFrameExtension.update(self, superFunc, dt)
	superFunc(self, dt)

    if self.fieldInfoContextBox ~= nil and self.fieldInfoBoxPosition ~= nil then
        local objectX = (self.fieldInfoBoxPosition.x + self.ingameMapBase.worldCenterOffsetX) / self.ingameMapBase.worldSizeX * self.ingameMapBase.mapExtensionScaleFactor + self.ingameMapBase.mapExtensionOffsetX
        local objectZ = (self.fieldInfoBoxPosition.z + self.ingameMapBase.worldCenterOffsetZ) / self.ingameMapBase.worldSizeZ * self.ingameMapBase.mapExtensionScaleFactor + self.ingameMapBase.mapExtensionOffsetZ

        local x, y, yRot, visible = self.ingameMapBase.layout:getMapObjectPosition(objectX, objectZ, 0, 0, 0, true)
        InGameMenuMapUtil.updateFieldInfoContextBoxPosition(self.fieldInfoContextBox, x, y)
    end
end

InGameMenuMapFrame.update = Utils.overwrittenFunction(InGameMenuMapFrame.update, InGameMenuMapFrameExtension.update)

function InGameMenuMapUtil.showFruitInfoBox(contextBox, x, y, description)
	if contextBox ~= nil then
		contextBox:setVisible(true)

		contextBox:applyProfile("ingameMenuMapContextBoxSmall")
		contextBox:setAbsolutePosition(x, y)

        local image = contextBox:getDescendantByName("image")
		image:setVisible(false)

		local text = contextBox:getDescendantByName("text")

		text:setText(description)

		local farmElem = contextBox:getDescendantByName("farm")
        farmElem:setText("")

        local cornerProfile = InGameMenuMapUtil.CONTEXT_BOX_CORNER_PROFILE[InGameMenuMapUtil.CONTEXT_BOX_ORIENTATION.TOP_RIGHT]
		local corner = contextBox:getDescendantByName("corner")

		corner:applyProfile(cornerProfile)
	end
end


function InGameMenuMapUtil.updateFieldInfoContextBoxPosition(contextBox, x, y)
	if contextBox ~= nil and contextBox:getIsVisible() then
		contextBox:setAbsolutePosition(x, y)
	end
end
