EjectColostrumDialog = {
	CONTROLS = {
		DIALOG_TITLE = "dialogTitleElement",
		NO_BUTTON = "noButton",
		YES_BUTTON = "yesButton"
	},
    OPTIONS = {
        EJECT_COLOSTRUM_TO_STORAGE = 1,
        EJECT_COLOSTRUM_TO_SLURRY = 0
    }
}
local EjectColostrumDialog_mt = Class(EjectColostrumDialog, YesNoDialog)

function EjectColostrumDialog.new(target, custom_mt)
	local self = DialogElement.new(target, custom_mt or EjectColostrumDialog_mt)
	self.isBackAllowed = false
	self.inputDelay = 250
	self.optionElements = {}
    self.selectedOption = EjectColostrumDialog.OPTIONS.EJECT_COLOSTRUM_TO_STORAGE

	self:registerControls(EjectColostrumDialog.CONTROLS)

	return self
end

function EjectColostrumDialog:onOpen()
	EjectColostrumDialog:superClass().onOpen(self)

	self.inputDelay = self.time + 250

    self.dialogTitleElement:setText(g_i18n:getText("TL_ejectColostrumDialogTitle"))
end

function EjectColostrumDialog:setCallback(callbackFunc, target, hasColostrumStorage)
	self.callbackFunc = callbackFunc
	self.target = target
    self.hasColostrumStorage = hasColostrumStorage

    if self.hasColostrumStorage then
        self:updateSelection(EjectColostrumDialog.OPTIONS.EJECT_COLOSTRUM_TO_STORAGE)
    else
        self:updateSelection(EjectColostrumDialog.OPTIONS.EJECT_COLOSTRUM_TO_SLURRY)
    end

	for element, amount in pairs(self.optionElements) do
		element.elements[3]:setText(self:selectedText())
	end
end

function EjectColostrumDialog:onClickActivate()
	if self.inputDelay < self.time then
		self:sendCallback(true)

		return false
	end

	return true
end

function EjectColostrumDialog:selectedText()
    if self.selectedOption == EjectColostrumDialog.OPTIONS.EJECT_COLOSTRUM_TO_STORAGE then
        return g_i18n:getText("TL_ejectColostrumToStorage")
    elseif self.selectedOption == EjectColostrumDialog.OPTIONS.EJECT_COLOSTRUM_TO_SLURRY then
        return g_i18n:getText("TL_ejectColostrumToSlurry")
    else
        Logging.error("EjectColostrumDialog: selected option with index "..self.selectedOption.." is not implemented")
        return ""
    end
end

function EjectColostrumDialog:onClickBack()
	if self.inputDelay < self.time then
		self:sendCallback(false)

		return false
	else
		return true
	end
end

function EjectColostrumDialog:sendCallback(success)
	if self.inputDelay < self.time then
		self:close()

		if self.callbackFunc ~= nil then
			if self.target ~= nil then
				self.callbackFunc(self.target, self.selectedOption, success)
			else
				self.callbackFunc(self.selectedOption, success)
			end
		end
	end
end

function EjectColostrumDialog:onClickLeft(element)
	local currentOption = (self.selectedOption - 1) % self:numOptions()

	self:updateSelection(currentOption)
end

function EjectColostrumDialog:onClickRight(element)
	local currentOption = (self.selectedOption + 1) % self:numOptions()

	self:updateSelection(currentOption)
end

function EjectColostrumDialog:updateSelection(currentOption)
    self.selectedOption = currentOption

	for element, amount in pairs(self.optionElements) do
		element.elements[3]:setText(self:selectedText())
    end
end

function EjectColostrumDialog:onCreateScroller(element)
	self.optionElements[element] = self:numOptions()
end

function EjectColostrumDialog:numOptions()
    if self.hasColostrumStorage then
        return 2
    else
        return 1
    end
end