NumberPickerDialog = {
	CONTROLS = {
		DIALOG_TITLE = "dialogTitleElement",
		NO_BUTTON = "noButton",
		YES_BUTTON = "yesButton"
	}
}
local NumberPickerDialog_mt = Class(NumberPickerDialog, YesNoDialog)

function NumberPickerDialog.new(target, custom_mt)
	local self = DialogElement.new(target, custom_mt or NumberPickerDialog_mt)
	self.isBackAllowed = false
	self.inputDelay = 250
	self.numAnimals = 1
    self.maxNumAnimals = 1
	self.optionElements = {}

	self:registerControls(NumberPickerDialog.CONTROLS)

	return self
end

function NumberPickerDialog:onOpen()
	NumberPickerDialog:superClass().onOpen(self)

	self.inputDelay = self.time + 250

	for element, amount in pairs(self.optionElements) do
		element.elements[3]:setText(string.format(g_i18n:getText("TL_currentValue_maxValue"), self.numAnimals, self.maxNumAnimals))
	end

	self:updateAmount(0)
end

function NumberPickerDialog:setMaxNumAnimals(maxNumAnimals)
    self.numAnimals = maxNumAnimals
    self.maxNumAnimals = maxNumAnimals
	self:updateAmount(0)
end

function NumberPickerDialog:setCallback(callbackFunc, target)
	self.callbackFunc = callbackFunc
	self.target = target
end

function NumberPickerDialog:onClickActivate()
	if self.inputDelay < self.time then
		self:sendCallback(true)

		return false
	end

	return true
end

function NumberPickerDialog:onClickBack()
	if self.inputDelay < self.time then
		self:sendCallback(false)

		return false
	else
		return true
	end
end

function NumberPickerDialog:sendCallback(success)
	if self.inputDelay < self.time then
		self:close()

		if self.callbackFunc ~= nil then
			if self.target ~= nil then
				self.callbackFunc(self.target, self.numAnimals, success)
			else
				self.callbackFunc(self.numAnimals, success)
			end
		end
	end
end

function NumberPickerDialog:onClickLeft(element)
	local amount = -1 * self.optionElements[element.parent]

	self:updateAmount(amount)
end

function NumberPickerDialog:onClickRight(element)
	local amount = self.optionElements[element.parent]

	self:updateAmount(amount)
end

function NumberPickerDialog:updateAmount(diff)
    if self.numAnimals + diff > self.maxNumAnimals then
        self.numAnimals = 1
    elseif self.numAnimals + diff < 1 then
        self.numAnimals = self.maxNumAnimals
    else
        self.numAnimals = math.min(math.max(self.numAnimals + diff, 1), self.maxNumAnimals)
    end

	for element, amount in pairs(self.optionElements) do
		element.elements[3]:setText(string.format(g_i18n:getText("TL_currentValue_maxValue"), self.numAnimals, self.maxNumAnimals))
    end
end

function NumberPickerDialog:onCreateScroller(element)
	self.optionElements[element] = self.maxNumAnimals
end