FillLevelsDisplayExtensions = {}
FillLevelsDisplayExtensions.firstRun = true;

function FillLevelsDisplayExtensions:draw(superFunc)
	-- überschrieben zum einfügen zusätzlichen Zeile mit dem FillTypeText
	if FillLevelsDisplayExtensions.firstRun then
		-- damit die scales beim ersten mal korrigiert werden
		self:storeScaledValues()
		FillLevelsDisplayExtensions.firstRun = false;
	end
	
	FillLevelsDisplay:superClass().draw(self)

	if self:getVisible() then
		local baseX, baseY = self:getPosition()
		local width = self:getWidth()

		for i = 1, #self.fillLevelTextBuffer do
			local fillLevelText = self.fillLevelTextBuffer[i]
			local posX = baseX + width + self.fillLevelTextOffsetX
			local posY = baseY + (i - 1) * (self.frameHeight + self.frameOffsetY)

			if i == 1 then
				posX = posX + self.firstFillTypeOffset
			end

			setTextColor(unpack(FillLevelsDisplay.COLOR.FILL_LEVEL_TEXT))
			setTextBold(false)
			setTextAlignment(RenderText.ALIGN_RIGHT)
			renderText(posX, posY + self.fillLevelTextOffsetY, self.fillLevelTextSize, fillLevelText)
			
			if self.fillTypeTextBuffer[i] ~= nil then
				renderText(posX + self.fillTypeTextOffsetX, posY + self.fillTypeTextOffsetY, self.fillLevelTextSize, self.fillTypeTextBuffer[i])
			end
		end
	end
end


function FillLevelsDisplayExtensions:updateFillLevelFrames(superFunc)
	-- überschrieben zum einfügen der Kapazität und formatieren der werte mit punkten
	local _, yOffset = self:getPosition()
	local isFirst = true

	for i = 1, #self.fillLevelBuffer do
		local fillLevelInformation = self.fillLevelBuffer[i]

		if fillLevelInformation.capacity > 0 or fillLevelInformation.fillLevel > 0 then
			local value = 0

			if fillLevelInformation.capacity > 0 then
				value = fillLevelInformation.fillLevel / fillLevelInformation.capacity
			end

			local frame = self.fillTypeFrames[fillLevelInformation.fillType]

			frame:setVisible(true)

			local fillBar = self.fillTypeLevelBars[fillLevelInformation.fillType]

			fillBar:setValue(value)

			local baseX = self:getPosition()

			if isFirst then
				baseX = baseX + self.firstFillTypeOffset
			end

			frame:setPosition(baseX, yOffset)

			self.weightFrames[fillLevelInformation.fillType]:setVisible(fillLevelInformation.maxReached)

			local fillTypeName, unitShort = nil

			if fillLevelInformation.fillType ~= FillType.UNKNOWN then
				local fillTypeDesc = g_fillTypeManager:getFillTypeByIndex(fillLevelInformation.fillType)
				fillTypeName = fillTypeDesc.title
				unitShort = fillTypeDesc.unitShort
			end

			local formattedCapacity = 0
			if fillLevelInformation.capacity ~= math.huge then
				formattedCapacity = g_i18n:formatVolume(fillLevelInformation.capacity, 0)
			end
			local precision = fillLevelInformation.precision or 0
			local formattedNumber = g_i18n:formatVolume(fillLevelInformation.fillLevel, precision, "")
			local fillText = string.format("%s/ %s (%d%%)", formattedNumber, formattedCapacity, math.floor(100 * value))
			self.fillLevelTextBuffer[#self.fillLevelTextBuffer + 1] = fillText

			if fillTypeName ~= nil then
				self.fillTypeTextBuffer[#self.fillLevelTextBuffer] = fillTypeName
			end

			yOffset = yOffset + self.frameHeight + self.frameOffsetY
			isFirst = false
		end
	end
end


FillLevelsDisplayExtensionsLoader = {}
function FillLevelsDisplayExtensionsLoader:loadMap(name)
	-- Loader zum verhindern, dass die basis funktionen üebrschrieben werden, wenn der VehicleFruitHud im Spiel ist
	if g_modIsLoaded["VehicleFruitHud"] then
		return;
	end
	
	-- überschreiben der Funktionen für die zustätzlichen werte
	FillLevelsDisplay.draw = Utils.overwrittenFunction(FillLevelsDisplay.draw, FillLevelsDisplayExtensions.draw)
	FillLevelsDisplay.updateFillLevelFrames = Utils.overwrittenFunction(FillLevelsDisplay.updateFillLevelFrames, FillLevelsDisplayExtensions.updateFillLevelFrames)

	-- überschreiben der Positionen welche durch die base skripte verwendet werden, alle andere bleiben unverändert
	FillLevelsDisplay.SIZE.BACKGROUND = { 180, 55 }		
	FillLevelsDisplay.SIZE.FILL_TYPE_FRAME = { 180, 55 }

	FillLevelsDisplay.POSITION.FILL_TYPE_TEXT = { -40, 18 }
	FillLevelsDisplay.POSITION.FILL_LEVEL_TEXT = { 0, -15 }
	FillLevelsDisplay.POSITION.BACKGROUND = { 370, 8 }
end

addModEventListener(FillLevelsDisplayExtensionsLoader)