InfoTextHotspot = {}
local InfoTextHotspot_mt = Class(InfoTextHotspot, MapHotspot)

function InfoTextHotspot.new(customMt)
	local self = MapHotspot.new(customMt or InfoTextHotspot_mt)

	return self
end

function InfoTextHotspot:getCategory()
	return MapHotspot[self.hotspotType]
end

function InfoTextHotspot:getWidth()
    if self.icon ~= nil then
        return self.icon.width
    else
        return getTextWidth(self.textSize * self.scale, self.name)
    end
end

function InfoTextHotspot:getHeight()
    if self.icon ~= nil then
        return self.icon.height
    else
        return self.textSize * self.scale
    end
end

function InfoTextHotspot:setInfo(info)
    self.hotspotType = info.hotspotType
    self.name = info.text
    local _, textSize = getNormalizedScreenValues(0, 22)
	self.textSize = textSize
	self.worldX = info.worldPositionX
	self.worldZ = info.worldPositionZ
    self.scale = info.scale or self.scale
    self.color = info.color or self.color
    self.bold = info.bold or false
    self.showOnMiniMap = info.showOnMiniMap or false
    if info.filename then
        self.width, self.height = getNormalizedScreenValues(info.width, info.height)
        self.icon = Overlay.new(info.filename, 0, 0, self.width, self.height)
        self.rotation = info.rotation or 0
        self.uvs = info.uvs
        self.uvResolution = info.uvResolution
        self.uv = GuiUtils.getUVs(self.uvs, self.uvResolution)
    end
end

function InfoTextHotspot:render(x, y, rot, small)
    if not self.showOnMiniMap and small then
        return
    end

    self.lastRenderedIcon = self.icon

	if self.icon ~= nil then
		self.icon:setPosition(x, y)
		self.icon:setRotation(self.rotation, self.icon.width * 0.5, self.icon.height * 0.5)
        if self.uv ~= nil then
            self.icon:setUVs(self.uv)
        end
		self.icon:setColor(nil, nil, nil, self.isBlinking and self:getCanBlink() and IngameMap.alpha or 1)
		self.icon:render()
    elseif self.name ~= "" then
        local shadowOffsetX = 1 / g_screenWidth
        local shadowOffsetY = -1 / g_screenHeight
        local r, g, b, a = unpack(self.color)
        local alpha = 1

        if self.isBlinking then
            alpha = IngameMap.alpha
        end

        setTextBold(self.bold)
        setTextAlignment(RenderText.ALIGN_LEFT)
        setTextWrapWidth(0)
        setTextColor(0, 0, 0, alpha)
        renderText(x + shadowOffsetX, y + shadowOffsetY, self.textSize * self.scale, self.name)
        setTextColor(r, g, b, a * alpha)
        renderText(x, y, self.textSize * self.scale, self.name)
    end
end