-- load animalFood from terraLifePlus_animalFood_x.xml via basegame function
-- by modelleicher (Farming Agency)
-- reworked for TerraLife to be able to use multiple Files by [F/A] Braeven
-- added possibility to use converters by [F/A] Braeven 20.08.2023

loadAnimalFood = {}
AnimalFoodSystem.loadedRecipes = false

local modDirectory = g_currentModDirectory


function loadAnimalFood.loadAnimalOld(self, superFunc, xmlFile)
	g_currentMission.animalFoodSystem.xmlFile = xmlFile
end
AnimalFoodSystem.loadMapData = Utils.overwrittenFunction(AnimalFoodSystem.loadMapData, loadAnimalFood.loadAnimalOld)


function loadAnimalFood.loadAnimalFoodFromXML(self, superFunc, xmlFile)
	local xmlFile = g_currentMission.animalFoodSystem.xmlFile
	local mod = g_modManager:getModByName("FS22_TerraLifePlus_animalFoodAdditions")
	local own = g_modManager:getModByName("FS22_TerraLifePlus_animalFoodAdditions_own")
	local ownAFA = false

	if own ~= nil then
		ownAFA = g_modIsLoaded["FS22_TerraLifePlus_animalFoodAdditions_own"]
	end
	-- reset all basegame tables
	g_currentMission.animalFoodSystem.animalFood = {}
	g_currentMission.animalFoodSystem.indexToAnimalFood = {}
	g_currentMission.animalFoodSystem.animalTypeIndexToFood = {}
	g_currentMission.animalFoodSystem.mixtures = {}
	g_currentMission.animalFoodSystem.recipes = {}
	g_currentMission.animalFoodSystem.recipeFillTypeIndexToRecipe = {}
	g_currentMission.animalFoodSystem.animalMixtures = {}
	g_currentMission.animalFoodSystem.mixtureFillTypeIndexToMixture = {}
	g_currentMission.animalFoodSystem.converters = {}

	local schema = AnimalFoodSystem.xmlSchema
	
	schema:register(XMLValueType.STRING, "animalFood.converters.converter(?)#animalType", "Type of Animal this Dataset overrides", nil)
	schema:register(XMLValueType.STRING, "animalFood.converters.converter(?)#inputFillType", "Type of Animal this Dataset overrides", nil)
	schema:register(XMLValueType.STRING, "animalFood.converters.converter(?)#outputFillType", "Type of Animal this Dataset overrides", nil)
	
	local function loadConverters(xmlFile)
		xmlFile:iterate("animalFood.converters.converter", function (_, key)
			local converter = {}
			local animalTypeName = xmlFile:getValue(key .. "#animalType")
			local animalTypeIndex = g_currentMission.animalFoodSystem.mission.animalSystem:getTypeIndexByName(animalTypeName)
			if animalTypeIndex==nil then
				Logging.xmlError(xmlFile, "TerraLife Plus: AnimalType '%s' is invalid for '%s', skipped adding converter.", animalTypeName, key)
			else
				if g_currentMission.animalFoodSystem.converters[animalTypeIndex] ~= nil then
					converter = g_currentMission.animalFoodSystem.converters[animalTypeIndex]
				end

				local inputFillType = xmlFile:getValue(key .. "#inputFillType")
				local inputFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(inputFillType)
				local outputFillType = xmlFile:getValue(key .. "#outputFillType")
				local outputFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(outputFillType)
				
				if inputFillTypeIndex == nil then
					Logging.xmlError(xmlFile, "TerraLife Plus: inputFillType '%s' is invalid for '%s', skipped adding converter.", inputFillType, key)
				else
					if outputFillTypeIndex == nil then
						Logging.xmlError(xmlFile, "TerraLife Plus: outputFillType '%s' is invalid for '%s', skipped adding converter.", outputFillType, key)
					else
						converter[inputFillTypeIndex] = outputFillTypeIndex
						g_currentMission.animalFoodSystem.converters[animalTypeIndex] = converter			
					end
				end
			end
		end)
	end
	
	local function loadAnimalFood(path)
		print("TerraLife Plus: Loading AFA-File: "..path)
		local animalFoodXml = XMLFile.load("animalFood", path, AnimalFoodSystem.xmlSchema)	
		if animalFoodXml == nil then
			print("TerraLife Plus Warning: Can't load "..path..", no food data will be loaded for this animal!")
		else
			-- load all stuff again but ours this time
			g_currentMission.animalFoodSystem:loadAnimalFood(animalFoodXml)
			g_currentMission.animalFoodSystem:loadMixtures(animalFoodXml)
			g_currentMission.animalFoodSystem:loadRecipes(animalFoodXml)
			loadConverters(animalFoodXml)
			-- delete xml
			animalFoodXml:delete()

		end
	end

	if AnimalFoodSystem.loadedRecipes == false then
			AnimalFoodSystem.loadedRecipes = true
			if ownAFA == true and fileExists(own.modDir.."recipes.xml") then
				loadAnimalFood(own.modDir.."recipes.xml")
			else
				loadAnimalFood(mod.modDir.."recipes.xml")
			end
			if ownAFA == true and fileExists(own.modDir.."cow/mixtures_cow.xml") then
				loadAnimalFood(own.modDir.."cow/mixtures_cow.xml")
			else
				loadAnimalFood(mod.modDir.."cow/mixtures_cow.xml")
			end
			if g_currentMission.animalSystem:getTypeIndexByName("HAYCOW") ~= nil then
				if ownAFA == true and fileExists(own.modDir.."cow/mixtures_haycow.xml") then
					loadAnimalFood(own.modDir.."cow/mixtures_haycow.xml")
				else
					loadAnimalFood(mod.modDir.."cow/mixtures_haycow.xml")
				end
			end
			if ownAFA == true and fileExists(own.modDir.."sheep/mixtures_sheep.xml") then
				loadAnimalFood(own.modDir.."sheep/mixtures_sheep.xml")
			else
				loadAnimalFood(mod.modDir.."sheep/mixtures_sheep.xml")
			end
			if ownAFA == true and fileExists(own.modDir.."goat/mixtures_goat.xml") then
				loadAnimalFood(own.modDir.."goat/mixtures_goat.xml")
			else
				loadAnimalFood(mod.modDir.."goat/mixtures_goat.xml")
			end
			if ownAFA == true and fileExists(own.modDir.."horse/mixtures_horse.xml") then
				loadAnimalFood(own.modDir.."horse/mixtures_horse.xml")
			else
				loadAnimalFood(mod.modDir.."horse/mixtures_horse.xml")
			end
			if ownAFA == true and fileExists(own.modDir.."pig/mixtures_pig.xml") then
				loadAnimalFood(own.modDir.."pig/mixtures_pig.xml")
			else
				loadAnimalFood(mod.modDir.."pig/mixtures_pig.xml")
			end
			if ownAFA == true and fileExists(own.modDir.."chicken/mixtures_chicken.xml") then
				loadAnimalFood(own.modDir.."chicken/mixtures_chicken.xml")
			else
				loadAnimalFood(mod.modDir.."chicken/mixtures_chicken.xml")
			end
	end

	local mapXML = XMLFile.wrap(xmlFile)
	mapXML:iterate("map.terraLife.animals.animalSubType", function(typeIndex, typeKey)
		local filename = mapXML:getString(typeKey .. "#filename")
		if filename == nil or filename == "" then
			Logging.xmlInfo(mapXML, "TerraLife Plus: No filename for entry..".. typeKey)
		else
			filename = Utils.getFilename(filename, baseDirectory)
			local modMapName, _ = Utils.getModNameAndBaseDirectory(filename)
			self.customEnvironment = modMapName
			local xmlFileAnimals = XMLFile.load("animals", filename)

			if xmlFileAnimals == nil then
				Logging.xmlInfo(mapXML, "TerraLife Plus: Animal file ".. filename .." does not exist.")
			else
				local num = 1
				local animalSubType = {}
				for i in string.gmatch(filename, "([^/]+)") do
					animalSubType[num] = i
					num = num + 1
				end

				if ownAFA == true and fileExists(own.modDir..animalSubType[num-2].."/animalFood_"..animalSubType[num-1]) then
					animalSubType = own.modDir..animalSubType[num-2].."/animalFood_"..animalSubType[num-1]
				else
					animalSubType = mod.modDir..animalSubType[num-2].."/animalFood_"..animalSubType[num-1]
				end				
				
				loadAnimalFood(animalSubType)
				xmlFileAnimals:delete()
			end
		end
	end)

	mapXML:delete()
end

FoliageSystem.loadMapData = Utils.appendedFunction(FoliageSystem.loadMapData, loadAnimalFood.loadAnimalFoodFromXML)

-- add disableSelection bool to recipes that shouldn't be selected in the mixerWagonRecipeChanger
function loadAnimalFood.loadRecipe(self, superFunc, recipe, xmlFile, key)
	--Lokale Funktion um SH-Overwrite zu verhindern
	local function getFillTypesFromXML_TL(fillTypes, usedFillTypes, xmlFile, key)
		local fillTypeNameStr = xmlFile:getValue(key)

		if fillTypeNameStr == nil then
			Logging.xmlError(xmlFile, "Missing fillTypes for ingredient '%s'", key)

			return false
		end

		local fillTypeNames = string.split(fillTypeNameStr, " ")

		for i = #fillTypeNames, 1, -1 do
			local fillTypeName = fillTypeNames[i]
			if fillTypeName ~= "" then --Doppelte oder Überflüssige Leerzeichen ausfiltern
				local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeName)

				if fillTypeIndex ~= nil then
					if usedFillTypes[fillTypeIndex] == nil then
						table.addElement(fillTypes, fillTypeIndex)
					else
						Logging.xmlWarning(xmlFile, "FillType '%s' already used in other ingredient", fillTypeName)
					end
				else
					Logging.xmlWarning(xmlFile, "FillType '%s' not defined. Ignoring it", fillTypeName)
				end
			end
		end

		if #fillTypes == 0 then
			Logging.xmlError(xmlFile, "No fillTypes defined - '%s'", key)

			return false
		end

		return true
	end

	local ingredients = {}
	local sumRatios = 0
	local usedFillTypes = {}
	local fillTypeCategoryIndex = g_fillTypeManager:addFillTypeCategory("MIXERWAGON")
	local recipeFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(xmlFile:getValue(key .. "#fillType"))
	g_fillTypeManager:addFillTypeToCategory(recipeFillTypeIndex, fillTypeCategoryIndex)
	
	xmlFile:iterate(key .. ".ingredient", function (_, ingredientKey)
		local ingredient = {
			name = xmlFile:getValue(ingredientKey .. "#name"),
			title = g_i18n:convertText(xmlFile:getValue(ingredientKey .. "#title"), self.customEnvironment),
			minPercentage = xmlFile:getValue(ingredientKey .. "#minPercentage", 0) / 100,
			maxPercentage = xmlFile:getValue(ingredientKey .. "#maxPercentage", 75) / 100
		}
		ingredient.ratio = ingredient.maxPercentage - ingredient.minPercentage
		ingredient.fillTypes = {}
		sumRatios = sumRatios + ingredient.ratio

		if getFillTypesFromXML_TL(ingredient.fillTypes, usedFillTypes, xmlFile, ingredientKey .. "#fillTypes") then
			table.insert(ingredients, ingredient)
			for _, fillTypeIndex in ipairs(ingredient.fillTypes) do
				g_fillTypeManager:addFillTypeToCategory(fillTypeIndex, fillTypeCategoryIndex)
			end
		end
	end)

	for _, ingredient in ipairs(ingredients) do
		ingredient.ratio = ingredient.ratio / sumRatios
	end

	if #ingredients == 0 then
		Logging.xmlWarning(xmlFile, "No ingredients defined for recipe '%s'", key)

		return false
	end

	local disableSelection = xmlFile:getBool(key .. "#disableSelection", false)
	if disableSelection then
		recipe.disableSelection = true;
	end;
	
	recipe.ingredients = ingredients

	return true
end

AnimalFoodSystem.loadRecipe = Utils.overwrittenFunction(AnimalFoodSystem.loadRecipe, loadAnimalFood.loadRecipe)


function AnimalFoodSystem:getConvertersByAnimalTypeIndex(animalTypeIndex)
	return self.converters[animalTypeIndex]
end

--Converter laden und hinterlegen
function loadAnimalFood:onPostLoad()
	local spec = self.spec_husbandryFood
	spec.converters = g_currentMission.animalFoodSystem:getConvertersByAnimalTypeIndex(spec.animalTypeIndex)
	if spec.converters ~= nil then
		for fillType, _ in pairs(spec.converters) do
			spec.supportedFillTypes[fillType] = true
		end
	end
end

PlaceableHusbandryFood.onPostLoad = Utils.appendedFunction(PlaceableHusbandryFood.onPostLoad, loadAnimalFood.onPostLoad)

function loadAnimalFood.updateRequiredFoodIfNecessary(husbandryFood)
	local spec = husbandryFood.spec_husbandryFood

	--LastFeeding hinterlegen, falls nicht vorhanden
	if husbandryFood.lastFeeding == nil then
		husbandryFood.lastFeeding = {}
		husbandryFood.lastFeeding.allowedFillTypes = {}
		husbandryFood.lastFeeding.numClusters = 0
		husbandryFood.lastFeeding.day = -1
		husbandryFood.lastFeeding.sumEatWeight = 0
	end

	--Kontrollieren ob es die Anzahl Cluster sich verändert hat oder die Fütterung veraltet ist
	if husbandryFood.lastFeeding.numClusters ~= husbandryFood:getNumOfClusters() or husbandryFood.lastFeeding.day ~= g_currentMission.environment.currentMonotonicDay then
		--Alte Liste löschen
		husbandryFood.lastFeeding.allowedFillTypes = {}
		husbandryFood.lastFeeding.sumEatWeight = 0
		--Cluster auslesen, Fütterungsdaten auslesen und FillType-Liste erstellen
		local clusters = husbandryFood.spec_husbandryAnimals.clusterSystem:getClusters()
		for _, cluster in ipairs(clusters) do

			-- zur richtigen berechnung und verteilung muss auch die Menge einbezogen werden
			-- deshalb direkt umstellen auf liter pro stunde als vergleichswert
			local litersPerHour = 0
			local subType = g_currentMission.animalSystem:getSubTypeByIndex(cluster.subTypeIndex)
			if subType ~= nil then
				local food = subType.input.food

				if food ~= nil then
					local age = cluster:getAge()
					local litersPerAnimal = food:get(age)
					local litersPerDay = litersPerAnimal * cluster:getNumAnimals()
					local lactationFactor = cluster:getLactationFoodFactor()
					litersPerHour = litersPerHour + litersPerDay / 24
					litersPerHour = litersPerHour * lactationFactor
				end
			end

			local animalFood = g_currentMission.animalFoodSystem:getAnimalFood(spec.animalTypeIndex, cluster);
			-- TerraLiveExtension.DebugTable("animalFood", animalFood)
			for _, foodGroup in pairs(animalFood.groups) do
				if foodGroup.eatWeight > 0 then
					--Brauchen nur den ersten Filltype Eintragen, Alias Slider
					-- der hier benutzte newEatWeight schließt dabei laktation und alles mit ein, so dass es auch im ganzen funktioniert mit der Verteilung von mix sachen auf benötigte slider
					local newEatWeight = foodGroup.eatWeight * litersPerHour;
					if husbandryFood.lastFeeding.allowedFillTypes[foodGroup.fillTypes[1]] == nil then
						husbandryFood.lastFeeding.allowedFillTypes[foodGroup.fillTypes[1]] = 0
					end
					husbandryFood.lastFeeding.allowedFillTypes[foodGroup.fillTypes[1]] = husbandryFood.lastFeeding.allowedFillTypes[foodGroup.fillTypes[1]] + newEatWeight
					husbandryFood.lastFeeding.sumEatWeight = husbandryFood.lastFeeding.sumEatWeight + newEatWeight
				end
			end
		end
		husbandryFood.lastFeeding.numClusters = husbandryFood:getNumOfClusters()
		husbandryFood.lastFeeding.day = g_currentMission.environment.currentMonotonicDay
		-- TerraLiveExtension.DebugTable("lastFeeding", husbandryFood.lastFeeding)
	end
end

PlaceableHusbandryFood.registerEventListeners = Utils.appendedFunction(PlaceableHusbandryFood.registerEventListeners, function(placeableType)
	SpecializationUtil.registerEventListener(placeableType, "onPeriodChanged", PlaceableHusbandryFood)
end)

function PlaceableHusbandryFood:onPeriodChanged()
	-- Force a recalculation of the food. This is necessary since the day has already changed and the food has already been calculated before the period change event was sent
	if self.lastFeeding ~= nil then
		self.lastFeeding.day = -1
	end
	loadAnimalFood.updateRequiredFoodIfNecessary(self)
end

function loadAnimalFood:addFood(superFunc, farmId, deltaFillLevel, fillTypeIndex, fillPositionData, toolType, extraAttributes, isGrazing)
	local spec = self.spec_husbandryFood
	if spec.supportedFillTypes[fillTypeIndex] == nil then
		return 0
	end

	--Sollte ein Converter vorhanden sein, anwenden
	if spec.converters ~= nil then
		if spec.converters[fillTypeIndex] ~= nil then
			fillTypeIndex = spec.converters[fillTypeIndex]
		end
	end

	loadAnimalFood.updateRequiredFoodIfNecessary(self)

	--Sollte der FillType in der Liste fehlen nicht annehmen
	if self.lastFeeding.allowedFillTypes[fillTypeIndex] == nil and extraAttributes ~= nil then
		if extraAttributes.mixtureTL ~= nil then
			return 0
		end
	end

	local SH = g_modIsLoaded["FS22_strawHarvestPack"]
	if SH == true then
		if fillTypeIndex == FillType.STRAW_PELLETS or fillTypeIndex == FillType.HAY_PELLETS then
			--SH Multiplikator zurück setzen
			deltaFillLevel = deltaFillLevel * 4
		end
	end
	
	local mixture = g_currentMission.animalFoodSystem:getAnimalTypeMixture(spec.animalTypeIndex, fillTypeIndex)
	if mixture ~= nil then
		local filled = 0
		local maxDelta = math.min(deltaFillLevel, self:getFreeFoodCapacity(fillTypeIndex))
		
		-- TerraLiveExtension.DebugText("mixture ~= nil maxDelta %s fillTypeIndex %s", maxDelta, fillTypeIndex);
		
		for _, ingredient in ipairs(mixture.ingredients) do
			local delta = maxDelta * ingredient.weight
			
			-- TerraLiveExtension.DebugText("ingredient fillTypes[1] %s weight %s", ingredient.fillTypes[1], ingredient.weight);
			
			if isGrazing ~= nil and isGrazing then
			
				-- Anpassung damit das auch so rausgenommen wird wie im stall gebraucht bei einer TL Mixture
				-- Die Menge wird also anhand der AnimalFood ins verhältniss gesetzt
				-- was nicht in der foodgroup ist fällt vorher schon raus.
				-- Achtung: Egal welche % in der mixture stehen, es wird immer korrekt gefüttert, wenn ein slider da ist
				-- Deshalb wird diese Verteilung hier nur gemacht, wenn es Grazing ist
				-- ohne diese umverteilung hier wird genau so verteilt wie es im mix eingetragen ist ungeachtet dessen was im stall steht
				if self.lastFeeding.allowedFillTypes[ingredient.fillTypes[1]] ~= nil then
					-- TerraLiveExtension.DebugText("old delta %s", delta);
					delta = maxDelta * (self.lastFeeding.allowedFillTypes[ingredient.fillTypes[1]] / self.lastFeeding.sumEatWeight);
					-- TerraLiveExtension.DebugText("new delta %s", delta);
					-- TerraLiveExtension.DebugTable("self.lastFeeding", self.lastFeeding, 3);
				end
			end
			
			local ingredientFillType = ingredient.fillTypes[1]
			if extraAttributes == nil then
				extraAttributes = {}
			end
			extraAttributes.mixtureTL = true
			local filledDelta = self:addFood(farmId, delta, ingredientFillType, fillPositionData, toolType, extraAttributes)
			filled = filled + filledDelta
		end

		if filled > 0 then
			self:updateFillPlanes(fillTypeIndex)
		end

		return filled
	end

	local freeCapacity = self:getFreeFoodCapacity(fillTypeIndex)

	if freeCapacity == 0 then
		return 0
	end

	deltaFillLevel = math.min(freeCapacity, deltaFillLevel)

	if spec.dynamicFoodPlane ~= nil then
		if fillPositionData ~= nil then
			local data = fillPositionData
			local x0, y0, z0 = getWorldTranslation(data.node)
			local d1x, d1y, d1z = localDirectionToWorld(data.node, data.width, 0, 0)
			local d2x, d2y, d2z = localDirectionToWorld(data.node, 0, 0, data.length)

			if VehicleDebug.state == VehicleDebug.DEBUG then
				drawDebugLine(x0, y0, z0, 1, 0, 0, x0 + d1x, y0 + d1y, z0 + d1z, 1, 0, 0)
				drawDebugLine(x0, y0, z0, 0, 0, 1, x0 + d2x, y0 + d2y, z0 + d2z, 0, 0, 1)
				drawDebugPoint(x0, y0, z0, 1, 1, 1, 1)
				drawDebugPoint(x0 + d1x, y0 + d1y, z0 + d1z, 1, 0, 0, 1)
				drawDebugPoint(x0 + d2x, y0 + d2y, z0 + d2z, 0, 0, 1, 1)
			end

			x0 = x0 - (d1x + d2x) / 2
			y0 = y0 - (d1y + d2y) / 2
			z0 = z0 - (d1z + d2z) / 2

			fillPlaneAdd(spec.dynamicFoodPlane, deltaFillLevel, x0, y0, z0, d1x, d1y, d1z, d2x, d2y, d2z)

			if self.isServer and FillVolume.SEND_PRECISION < math.abs(x0 - spec.lastPositionInfoSent[1]) or FillVolume.SEND_PRECISION < math.abs(z0 - spec.lastPositionInfoSent[2]) then
				spec.lastPositionInfoSent[1] = x0
				spec.lastPositionInfoSent[2] = z0

				self:raiseDirtyFlags(spec.dirtyFlagPosition)
			end
		else
			local x, y, z = localToWorld(spec.dynamicFoodPlane, 0, 0, 0)
			local d1x, d1y, d1z = localDirectionToWorld(spec.dynamicFoodPlane, 0.1, 0, 0)
			local d2x, d2y, d2z = localDirectionToWorld(spec.dynamicFoodPlane, 0, 0, 0.1)

			if not self.isServer and spec.lastPositionInfo[1] ~= 0 and spec.lastPositionInfo[2] ~= 0 then
				x, y, z = localToWorld(spec.dynamicFoodPlane, spec.lastPositionInfo[1], 0, spec.lastPositionInfo[2])
			end

			local steps = MathUtil.clamp(math.floor(deltaFillLevel / 400), 1, 25)

			for _ = 1, steps do
				fillPlaneAdd(spec.dynamicFoodPlane, deltaFillLevel / steps, x, y, z, d1x, d1y, d1z, d2x, d2y, d2z)
			end
		end
	end

	if self.isServer then
		self:raiseDirtyFlags(spec.dirtyFlagFillLevel)
	end

	spec.fillLevels[fillTypeIndex] = spec.fillLevels[fillTypeIndex] + deltaFillLevel

	self:updateFillPlanes(fillTypeIndex)
	self:updateFoodPlaces()

	return deltaFillLevel
end

PlaceableHusbandryFood.addFood = Utils.overwrittenFunction(PlaceableHusbandryFood.addFood, loadAnimalFood.addFood)

function loadAnimalFood:loadMixtures(superFunc, xmlFile)
	xmlFile:iterate("animalFood.mixtures.mixture", function (_, key)
		local mixtureFillTypeName = xmlFile:getValue(key .. "#fillType")

		if mixtureFillTypeName == nil then
			Logging.xmlError(xmlFile, "Missing fillType for food mixture '%s'", key)

			return false
		end

		local mixtureFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(mixtureFillTypeName)

		if mixtureFillTypeIndex == nil then
			Logging.xmlError(xmlFile, "FillType '%s' not defined for food mixture '%s'", mixtureFillTypeName, key)

			return false
		end

		-- if self.mixtureFillTypeIndexToMixture[mixtureFillTypeIndex] ~= nil then
			-- Logging.xmlError(xmlFile, "FillType '%s' already defined for mixture '%s'", mixtureFillTypeName, key)

			-- return false
		-- end

		local animalTypeName = xmlFile:getValue(key .. "#animalType")

		if animalTypeName == nil then
			Logging.xmlError(xmlFile, "Missing animal type for food mixture '%s'", key)

			return false
		end

		local animalTypeIndex = self.mission.animalSystem:getTypeIndexByName(animalTypeName)

		if animalTypeIndex == nil then
			Logging.xmlError(xmlFile, "Animal type '%s' not defined for food mixture '%s'", animalTypeName, key)

			return false
		end

		local mixture = {}

		if self:loadMixture(mixture, xmlFile, key) then
			mixture.index = #self.mixtures + 1

			table.insert(self.mixtures, mixture)

			self.animalMixtures[animalTypeIndex] = self.animalMixtures[animalTypeIndex] or {}

			table.insert(self.animalMixtures[animalTypeIndex], mixtureFillTypeIndex)

			--Mixturen AnimalsTypes Zuordnen
			if self.animalTypesIndexToMixtures == nil then
				self.animalTypesIndexToMixtures = {}
			end
			if self.animalTypesIndexToMixtures[animalTypeIndex] == nil then
				self.animalTypesIndexToMixtures[animalTypeIndex] = {}
			end
			self.animalTypesIndexToMixtures[animalTypeIndex][mixtureFillTypeIndex] = mixture
			--animal[mixtureFillTypeIndex] = mixture
			
			--wird nicht mehr benutzt, bleibt für Kompat erstmal drin
			self.mixtureFillTypeIndexToMixture[mixtureFillTypeIndex] = mixture
		end
	end)

	for _, mixture in pairs(self.mixtures) do
		local sumWeigths = 0

		for _, ingredient in pairs(mixture.ingredients) do
			sumWeigths = sumWeigths + ingredient.weight
		end

		if sumWeigths > 0 then
			for _, ingredient in pairs(mixture.ingredients) do
				ingredient.weight = ingredient.weight / sumWeigths
			end
		end
	end
	
	-- TerraLiveExtension.DebugTable("loadMixtures result", self.mixtures, 3)
	
	return true
end

AnimalFoodSystem.loadMixtures = Utils.overwrittenFunction(AnimalFoodSystem.loadMixtures, loadAnimalFood.loadMixtures)

function AnimalFoodSystem:getAnimalTypeMixture(animalTypeIndex, fillTypeIndex)
	local animalType = self.animalTypesIndexToMixtures[animalTypeIndex]
	if animalType == nil then
		return nil
	end
	return animalType[fillTypeIndex]
end



function loadAnimalFood:updateFillPlanes(superFunc, fillTypeIndex)
	local spec = self.spec_husbandryFood
	if spec.foodPlane ~= nil then
		local fillLevel = self:getTotalFood()
		local capacity = self:getFoodCapacity()
		local state = 0
		if capacity > 0 then
			state = fillLevel / capacity
		end

		spec.foodPlane:setState(state)

		if state > 0 and fillTypeIndex ~= nil then
			FillPlaneUtil.assignDefaultMaterials(spec.foodPlane.node)
			FillPlaneUtil.setFillType(spec.foodPlane.node, fillTypeIndex)
			setShaderParameter(spec.foodPlane.node, "isCustomShape", 1, 0, 0, 0, false)
		end
	end
	if spec.dynamicFoodPlane ~= nil then
		FillPlaneUtil.setFillType(spec.dynamicFoodPlane, fillTypeIndex)
	end
end

PlaceableHusbandryFood.updateFillPlanes = Utils.appendedFunction(PlaceableHusbandryFood.updateFillPlanes, loadAnimalFood.updateFillPlanes)