loadAnimals = {}

function loadAnimals:loadMapData(superFunc, xmlFile, missionInfo, baseDirectory)
	self.customEnvironment = missionInfo.customEnvironment
	local mapXML = XMLFile.wrap(xmlFile)
	mapXML:iterate("map.terraLife.animals.animalSubType", function(typeIndex, typeKey)
		local filename = mapXML:getString(typeKey .. "#filename")
		if filename == nil or filename == "" then
			Logging.xmlInfo(mapXML, "TerraLife Plus: No filename for entry..".. typeKey)
		else
			filename = Utils.getFilename(filename, baseDirectory)
			local xmlFileAnimals = XMLFile.load("animals", filename)

			if xmlFileAnimals == nil then
				Logging.xmlInfo(mapXML, "TerraLife Plus: Animal file ".. filename .." does not exist.")
			else
				local num = 1
				local animalSubType = {}
				for i in string.gmatch(filename, "([^/]+)") do
					animalSubType[num] = i
					num = num + 1
				end
				animalSubType = animalSubType[num-2].."/animalData_"..animalSubType[num-1]
				AnimalSystem.currentSubType = animalSubType
				self:loadAnimals(xmlFileAnimals, baseDirectory)
				xmlFileAnimals:delete()
			end
		end
	end)

	mapXML:delete()
	return #self.types > 0
end

AnimalSystem.loadMapData = Utils.overwrittenFunction(AnimalSystem.loadMapData, loadAnimals.loadMapData)

function loadAnimals:loadSubType(superFunc, animalType, subType, xmlFile, subTypeKey, baseDirectory)
    local value = superFunc(self, animalType, subType, xmlFile, subTypeKey, baseDirectory)

    subType.maxAge = xmlFile:getInt(subTypeKey .. "#maxAge", 0)
    subType.maxColostrumProduceAge = xmlFile:getInt(subTypeKey .. "#maxColostrumProduceAge", nil)
    loadAnimals.loadReproductionProbabilities(subType, subTypeKey, xmlFile)
    loadAnimals.loadInsemination(subType, subTypeKey, xmlFile)
    loadAnimals.loadLactation(subType, subTypeKey, xmlFile)
    loadAnimals.filterVisuals(self, subType)

    return value
end

AnimalSystem.loadSubType = Utils.overwrittenFunction(AnimalSystem.loadSubType, loadAnimals.loadSubType)

--animalVariation hinzufügen um Texturen auf ein Tierart zu begrenzen
function loadAnimals:loadAnimalConfig(superFunc, animalType, baseDirectory)
	animalType.animals = {}
	local configFilename = Utils.getFilename(animalType.configFilename, baseDirectory)
	local xmlFile = XMLFile.load("animalsConfig", configFilename)

	if xmlFile == nil then
		return false
	end

	xmlFile:iterate("animalHusbandry.animals.animal", function (_, key)
		local animal = {
			filename = Utils.getFilename(xmlFile:getString(key .. ".assets#filename"), baseDirectory),
			filenamePosed = Utils.getFilename(xmlFile:getString(key .. ".assets#filenamePosed"), baseDirectory),
			variations = {}
		}

		xmlFile:iterate(key .. ".assets.texture", function (_, textureKey)
			local variation = {
				numTilesU = math.max(xmlFile:getInt(textureKey .. "#numTilesU", 1), 1)
			}
			variation.tileUIndex = MathUtil.clamp(xmlFile:getInt(textureKey .. "#tileUIndex", 0), 0, variation.numTilesU - 1)
			variation.numTilesV = math.max(xmlFile:getInt(textureKey .. "#numTilesV", 1), 1)
			variation.tileVIndex = MathUtil.clamp(xmlFile:getInt(textureKey .. "#tileVIndex", 0), 0, variation.numTilesV - 1)
			variation.mirrorV = xmlFile:getBool(textureKey .. "#mirrorV", false)
			variation.multi = xmlFile:getBool(textureKey .. "#multi", true)
			variation.animalVariation = xmlFile:getString(textureKey .. "#animalVariation", nil)

			table.insert(animal.variations, variation)
		end)
		table.insert(animalType.animals, animal)
	end)
	xmlFile:delete()

	return true
end

AnimalSystem.loadAnimalConfig = Utils.overwrittenFunction(AnimalSystem.loadAnimalConfig, loadAnimals.loadAnimalConfig)

------------------------------------------------------- New loadAnimals functions ---------------------------------------------------

function loadAnimals.loadReproductionProbabilities(subType, subTypeKey, xmlFile)
    local reproductionKey = subTypeKey .. ".reproduction"
    subType.malePart = xmlFile:getString(reproductionKey .. "#malePart")

    local reproductionProbabilities = {}
    xmlFile:iterate(reproductionKey .. ".key", function(typeIndex, typeKey)
        local reproductionValue = {
            numOfAnimals = xmlFile:getInt(typeKey .. "#numOfAnimals", 1),
            probability = xmlFile:getInt(typeKey .. "#probability", 100)
        }

        table.insert(reproductionProbabilities, reproductionValue)
    end)

    if #reproductionProbabilities == 0 then
        local reproductionValue = {
            numOfAnimals = 1,
            probability = 100
        }

        table.insert(reproductionProbabilities, reproductionValue)
    end

    subType.reproductionProbabilities = reproductionProbabilities
end

function loadAnimals.loadInsemination(subType, subTypeKey, xmlFile)
    local inseminationKey = subTypeKey .. ".insemination"
    subType.inseminationCost = xmlFile:getInt(inseminationKey .. "#cost", 0)

    local inseminationProbabilities = {}
    xmlFile:iterate(inseminationKey .. ".key", function(typeIndex, typeKey)
        local inseminationValue = {
            monthSinceLastBirth = xmlFile:getInt(typeKey .. "#monthSinceLastBirth"),
            probability = xmlFile:getInt(typeKey .. "#probability")
        }

        table.insert(inseminationProbabilities, inseminationValue)
    end)

    subType.inseminationProbabilities = inseminationProbabilities
end

function loadAnimals.loadLactation(subType, subTypeKey, xmlFile)
    local lactationKey = subTypeKey .. ".lactation"

    local lactation = {}
    xmlFile:iterate(lactationKey .. ".key", function(typeIndex, typeKey)
        local lactationValue = {
            month = xmlFile:getInt(typeKey .. "#month"),
            food = MathUtil.round(xmlFile:getFloat(typeKey .. "#food"), 3),
            milk = MathUtil.round(xmlFile:getFloat(typeKey .. "#milk"), 3)
        }

        table.insert(lactation, lactationValue)
    end)

    subType.lactation = lactation
end

function loadAnimals.filterVisuals(self, subType)
    local visuals = subType.visuals

    for _, visual in ipairs(visuals) do
        local variations = visual.visualAnimal.variations
        local filteredVariations = {}

        for _, variation in ipairs(variations) do
            if variation.animalVariation == nil or variation.animalVariation == subType.name then
                table.insert(filteredVariations, variation)
            end
        end

        visual.visualAnimal.variations = filteredVariations
    end
end