-- load our FillTypes from TLP-Mod

loadFillTypes = {}

function loadFillTypes.loadFillTypes(self, superFunc, xmlFile, baseDirectory, isBaseType, customEnv)
	if type(xmlFile) ~= "table" then
		xmlFile = XMLFile.wrap(xmlFile, FillTypeManager.xmlSchema)
	end
	self.skipTextureArray = true
	--LOAD TLP FillTypes
	if xmlFile:hasProperty("map.terraLife") then
		print("TerraLife Plus: Loading TerraLife Plus Filltypes")
		xmlFile:iterate("map.fillTypes.fillType", function (index, key)
			local name = xmlFile:getValue(key .. "#name")
			local path = Utils.getFilename("$moddir$FS22_TerraLifePlus/fillTypes/"..name..".xml", baseDirectory)
			if fileExists(path) then
				local fillTypeXML = XMLFile.load("fillType", path, FillTypeManager.xmlSchema)
				superFunc(self, fillTypeXML, baseDirectory, false, customEnv)
				--load additionel windrow values
				if fillTypeXML:hasProperty("map.fillTypes.fillType(0).windrow") then
					local fillTypeName = fillTypeXML:getString("map.fillTypes.fillType(0)#name")
					local windrowLiterPerSqm = fillTypeXML:getString("map.fillTypes.fillType(0).windrow#windrowLiterPerSqm")
					if windrowLiterPerSqm ~= nil then
						g_fillTypeManager.fillTypes[g_fillTypeManager:getFillTypeIndexByName(fillTypeName)].windrowLiterPerSqm = windrowLiterPerSqm
					end
				end
				fillTypeXML:delete()
			else
				print("TerraLife Plus Error: XMl-File "..path.." does not exist")
			end
		end)
		self:constructFillTypeTextureArrays()
		loadFillTypes.additionalWindrows = {}
		--TerraLifeFermentationManager can load fillTypes now
		g_terraLifePlusFermentationManager:loadFermentation()

		return true
	else
		--load additional windrows
		if xmlFile:hasProperty("map.additionalWindrows") then
			xmlFile:iterate("map.additionalWindrows.windrow", function (index, key)
				local fillType = {}
				local fillTypeName = xmlFile:getString(key .. "#name")
				fillType.index = g_fillTypeManager:getFillTypeIndexByName(fillTypeName)
				if fillType.index == nil then
					print("TerraLife Plus Error: FillType "..fillTypeName.." does not exist")
				else
					local fruitType = xmlFile:getString(key .. "#fruitType")
					fillType.noChange = xmlFile:getBool(key .. "#noChange", false)
					if loadFillTypes.additionalWindrows[fruitType] == nil then
						loadFillTypes.additionalWindrows[fruitType] = {}
					end
					table.insert(loadFillTypes.additionalWindrows[fruitType], fillType)
				end
			end)
		end

		return superFunc(self, xmlFile, baseDirectory, isBaseType, customEnv)
	end
	xmlFile:delete()
end

FillTypeManager.loadFillTypes = Utils.overwrittenFunction(FillTypeManager.loadFillTypes, loadFillTypes.loadFillTypes)

function loadFillTypes:constructFillTypeTextureArrays(superFunc)
	if self.skipTextureArray == false then
		self:deleteFillTypeTextureArrays()

		local diffuseMapConstr = TextureArrayConstructor.new()
		local normalMapConstr = TextureArrayConstructor.new()
		local specularMapConstr = TextureArrayConstructor.new()
		self.fillTypeTextureArraySize = 0

		for i = 1, #self.fillTypes do
			local fillType = self.fillTypes[i]

			if fillType.diffuseMapFilename ~= nil and fillType.normalMapFilename ~= nil and fillType.specularMapFilename ~= nil then
				diffuseMapConstr:addLayerFilename(fillType.diffuseMapFilename)
				normalMapConstr:addLayerFilename(fillType.normalMapFilename)
				specularMapConstr:addLayerFilename(fillType.specularMapFilename)

				self.fillTypeTextureArraySize = self.fillTypeTextureArraySize + 1
				fillType.textureArrayIndex = self.fillTypeTextureArraySize
			end
		end

		self.fillTypeTextureDiffuseMap = diffuseMapConstr:finalize(true, true, true)
		self.fillTypeTextureNormalMap = normalMapConstr:finalize(true, false, true)
		self.fillTypeTextureSpecularMap = specularMapConstr:finalize(true, false, true)
	end
end

FillTypeManager.constructFillTypeTextureArrays = Utils.overwrittenFunction(FillTypeManager.constructFillTypeTextureArrays, loadFillTypes.constructFillTypeTextureArrays)

-- Note: getFillTypeLiterPerSqm override is no longer necessary due TLMowerExtension.lua, where this code is now integrated.

function loadFillTypes.append_loadMapData(self, xmlFile, missionInfo, baseDirectory)
	self.skipTextureArray = false
	self:constructFillTypeTextureArrays()
end
FillTypeManager.loadMapData = Utils.appendedFunction(FillTypeManager.loadMapData, loadFillTypes.append_loadMapData)