TerraLifeStoreItem = {}

function TerraLifeStoreItem:AddXmlSchema()
	TerraLifeStoreItem.XmlSchema = XMLSchema.new("TerraLife")
	local schema = TerraLifeStoreItem.XmlSchema
	schema:register(XMLValueType.STRING, "terraLife.fillTypes.fillType#filename", "FillTypes Filename", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.storeItems.storeItem(?)#xmlFilename", "Filename", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.brands.brand(?)#name", "BrandName", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.brands.brand(?)#title", "BrandName", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.brands.brand(?)#image", "ImageFilename", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.brands.brand(?)#imageOffset", "Image offset", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.brandColors.color(?)#name", "Name of the Color", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.brandColors.color(?)#value", "Color Value", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.specializations.specialization(?)#name", "Specialization Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.specializations.specialization(?)#className", "Specialization Class Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.vehicleTypes.type(?)#name", "Vehicle Specialization Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.vehicleTypes.type(?)#parent", "Vehicle Specialization Parent Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.vehicleTypes.type(?)#className", "Vehicle Specialization Classname", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.vehicleTypes.type(?)#filename", "Vehicle Specialization Filename", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.vehicleTypes.type(?).specialization#name", "Vehicle Specialization Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.storeCategories.storeCategory(?)#type", "Store Category ", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.storeCategories.storeCategory(?)#name", "Store Category ", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.storeCategories.storeCategory(?)#title", "Store Category ", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.storeCategories.storeCategory(?)#image", "Store Category ", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.storeCategories.storeCategory(?)#orderId", "Store Category ", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.placeableSpecializations.specialization(?)#name", "Specialization Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.placeableSpecializations.specialization(?)#className", "Specialization Class Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.placeableTypes.type(?)#name", "Vehicle Specialization Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.placeableTypes.type(?)#parent", "Vehicle Specialization Parent Name", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.placeableTypes.type(?)#className", "Vehicle Specialization Classname", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.placeableTypes.type(?)#filename", "Vehicle Specialization Filename", nil, false)
	schema:register(XMLValueType.STRING, "terraLife.placeableTypes.type(?).specialization#name", "Vehicle Specialization Name", nil, false)
end

function g_storeManager:loadTerraLife(ModName, ModDirectory, ModXMLPath)
	local xmlFile = XMLFile.load("TempDesc", ModDirectory..ModXMLPath)
	local mod = g_modManager:getModByName(ModName)
	local modEnv = _G[ModName]

	--Add StoreItems
	xmlFile:iterate("terraLife.storeItems.storeItem", function (_, key)
		local storeItemXMLFilename = xmlFile:getString(key .. "#xmlFilename")

		if storeItemXMLFilename ~= nil then
			g_storeManager:addModStoreItem(storeItemXMLFilename, ModDirectory, ModName, true, false, mod.title)
		end
	end)

	--Add Brands
	xmlFile:iterate("terraLife.brands.brand", function (_, key)
		local name = xmlFile:getString(key .. "#name")
		local title = xmlFile:getString(key .. "#title")
		local image = xmlFile:getString(key .. "#image")
		local offset = xmlFile:getFloat(key .. "#imageOffset")

		g_brandManager:addBrand(name, title, image, ModDirectory, true, nil, offset)
	end)

	--Add BrandColors
	xmlFile:iterate("terraLife.brandColors.color", function (_, key)
		g_brandColorManager:loadBrandColorFromXML(xmlFile.handle, key)
	end)

	--Add Specializations
	xmlFile:iterate("terraLife.specializations.specialization", function (_, key)
		local specName = xmlFile:getString(key .. "#name")
		local className = xmlFile:getString(key .. "#className")
		local filename = xmlFile:getString(key .. "#filename")

		if specName ~= nil and className ~= nil and filename ~= nil then
			filename = ModDirectory .. filename
			modEnv.g_specializationManager:addSpecialization(specName, className, filename, ModName)
		end
	end)

	--Add Vehicle-Types
	xmlFile:iterate("terraLife.vehicleTypes.type", function (_, key)
		g_vehicleTypeManager:loadTypeFromXML(xmlFile.handle, key, true, ModDirectory, ModName)
	end)

	--Add placable-Specializations
	xmlFile:iterate("terraLife.placeableSpecializations.specialization", function (_, key)
		local specName = xmlFile:getString(key .. "#name")
		local className = xmlFile:getString(key .. "#className")
		local filename = xmlFile:getString(key .. "#filename")

		if specName ~= nil and className ~= nil and filename ~= nil then
			filename = ModDirectory .. filename
			modEnv.g_placeableSpecializationManager :addSpecialization(specName, className, filename, ModName)
		end
	end)
	
	--Add placable-Types
	xmlFile:iterate("terraLife.placeableTypes.type", function (_, key)
		g_placeableTypeManager:loadTypeFromXML(xmlFile.handle, key, true, ModDirectory, ModName)
	end)

	--Add FillTypes
	xmlFile:iterate("terraLife.fillTypes.fillType", function (_, key)
		local fillTypesFilename = xmlFile:getString(key .. "#filename")
		g_fillTypeManager:addModWithFillTypes(Utils.getFilename(fillTypesFilename, ModDirectory), ModDirectory, ModName)
	end)

	--Add storeCategories
	xmlFile:iterate("terraLife.storeCategories.storeCategory", function (_, key)
		local name = xmlFile:getString(key .. "#name")
		local title = xmlFile:getString(key .. "#title")
		local imageFilename = xmlFile:getString(key .. "#image")
		local type = xmlFile:getString(key .. "#type")
		local orderId = xmlFile:getInt(key .. "#orderId")
		if title ~= nil and title:sub(1, 6) == "$l10n_" then
			title = g_i18n:getText(title:sub(7), ModName)
		end
		local modDir = ModDirectory
		if imageFilename:sub(1, 8) == "$moddir$" then
			local snippedPath = imageFilename:sub(9, -1)
			local modName = snippedPath:split("/")
			local lenght = string.len(modName[1])
			snippedPath = snippedPath:sub(lenght+2, -1)
			modDir = g_modNameToDirectory[modName[1]]
			imageFilename = snippedPath
		end
		g_storeManager:addCategory(name, title, imageFilename, type, modDir, orderId)
	end)
end

TerraLifeStoreItem:AddXmlSchema()