-- This script prevents loading TerraLifePlus if a TerraLifePlus Map was not detected.
-- By: [F/A] Braeven
-- Date: 19.07.2024
-- Version 1.2.0.0
-- Only for use with MaizePlus/TerraLifePlus, permission for other use can one be given through Braeven / Farming Agency

testTerraLifePlus = {}

function testTerraLifePlus:AddXmlSchema()
	testTerraLifePlus.XmlSchema = XMLSchema.new("terraLifePlus")
	local schema = testTerraLifePlus.XmlSchema
	schema:register(XMLValueType.STRING, "map.terraLife#version", "Required TerraLifePlus Version", nil, nil)
	schema:register(XMLValueType.STRING, "map.terraLife#assetsVersion", "Required TerraLifePlus_Assets Version", nil, nil)
	schema:register(XMLValueType.STRING, "map.terraLife#utilsVersion", "Required TerraLifePlus_Utils Version", nil, nil)
end


function testTerraLifePlus.check(TLP, neededVersion)
	local base, major, minor, patch = string.match(TLP, "(%d+)%.(%d+)%.(%d+)%.(%d+)")
	testTerraLifePlus.base = math.floor(base)
	testTerraLifePlus.major = math.floor(major)
	testTerraLifePlus.minor = math.floor(minor)
	testTerraLifePlus.patch = math.floor(patch)

	local base, major, minor, patch = string.match(neededVersion, "(%d+)%.(%d+)%.(%d+)%.(%d+)")
	base = math.floor(base)
	major = math.floor(major)
	minor = math.floor(minor)
	patch = math.floor(patch)
	if testTerraLifePlus.base > base then
		return true
	elseif testTerraLifePlus.base == base then
		if testTerraLifePlus.major > major then
			return true
		elseif testTerraLifePlus.major == major then
			if testTerraLifePlus.minor > minor then
				return true
			elseif testTerraLifePlus.minor == minor then
				if testTerraLifePlus.patch > patch then
					return true
				elseif testTerraLifePlus.patch == patch then
					return true
				end
			end
		end
	end
	return false
end


function testTerraLifePlus:test()
	local mapDirectory = g_mpLoadingScreen.missionInfo.map.baseDirectory
	local CheckAssets = false
	local CheckUtils = false
	local CheckMap = false
	local CheckTLPAssets = false
	local CheckTLPUtils = false
	if mapDirectory == "" then
		--wenn mapDirectory leer ist, handelt es sich um die Basemaps
		return false
	elseif fileExists(mapDirectory .. "dlcDesc.xml") then
		--wenn dlcDesc existiert, handelt es sich um DLC-Map
		return false
	else
		local path = mapDirectory .. "modDesc.xml"
		local MapXmlFile = XMLFile.load("TempDesc", path)
		if MapXmlFile:hasProperty("moddesc.terraLife") then
			local TLP = g_modManager:getModByName("FS22_TerraLifePlus")
			local mapVersion = MapXmlFile:getString("modDesc.terraLife#version")
			
			CheckMap = testTerraLifePlus.check(TLP.version, mapVersion)
			MapXmlFile:delete()
			local assets = g_modManager:getModByName("FS22_TerraLifePlus_Assets")
			local assetsXmlFile = XMLFile.load("AssetsDesc", assets.modDir.."modDesc.xml")
			local assetsVersion = assetsXmlFile:getString("modDesc.terraLife#version")
			CheckAssets = testTerraLifePlus.check(TLP.version, assetsVersion)
			assetsXmlFile:delete()

			local utils = g_modManager:getModByName("FS22_TerraLifePlus_Utils")
			local utilsXmlFile = XMLFile.load("UtilsDesc", utils.modDir.."modDesc.xml")
			local utilsVersion = utilsXmlFile:getString("modDesc.terraLife#version")
			CheckUtils = testTerraLifePlus.check(TLP.version, utilsVersion)
			utilsXmlFile:delete()

			local TLPXmlFile = XMLFile.load("TLPDesc", TLP.modDir.."modDesc.xml")
			local assetsTLPVersion = TLPXmlFile:getString("modDesc.terraLife#assetsVersion")
			local utilsTLPVersion = TLPXmlFile:getString("modDesc.terraLife#utilsVersion")
			CheckTLPAssets = testTerraLifePlus.check(assets.version, assetsTLPVersion)
			CheckTLPUtils = testTerraLifePlus.check(utils.version, utilsTLPVersion)
			TLPXmlFile:delete()

			if CheckMap ~= true then
				testTerraLifePlus.showWarning("FS22_TerraLifePlus", g_mpLoadingScreen.missionInfo.map.customEnvironment, mapVersion)
			elseif CheckAssets ~= true then
				testTerraLifePlus.showWarning("FS22_TerraLifePlus", "FS22_TerraLifePlus_Assets", assetsVersion)
			elseif CheckUtils ~= true then
				testTerraLifePlus.showWarning("FS22_TerraLifePlus", "FS22_TerraLifePlus_Utils", utilsVersion)
			elseif CheckTLPAssets ~= true then
				testTerraLifePlus.showWarning("FS22_TerraLifePlus_Assets", "FS22_TerraLifePlus", assetsTLPVersion)
			elseif CheckTLPUtils ~= true then
				testTerraLifePlus.showWarning("FS22_TerraLifePlus_Utils", "FS22_TerraLifePlus", utilsTLPVersion)
			end
			return true
		else
			MapXmlFile:delete()
			return false
		end
	end
end


function testTerraLifePlus.showWarning(modName, modulName, neededVersion)
	local title = "Outdated Version of "..modName
	local text = "At least one part of TerraLifePlus needs a newer version.\n "..modulName.. " needs at least version " ..neededVersion.. " of "..modName..".\n Please update to the latest one."
	local textyes = "Open Modhub"
	local textno = "Ignore"
	local language = g_languageShort

	local language = g_languageShort
	if language == "de" then
		title = "Veraltete Version von "..modName
		text = "Mindestens ein Teil von TerraLifePlus benötigt eine neuere Version.\n " ..modulName.. " benötigt Version " ..neededVersion.. " von "..modName..".\n Bitte updaten."
		textyes = "Modhub öffnen"
		textno = "Ignorieren"
	end
	
	g_gui:showYesNoDialog({
		title = title,
		text = text,
		dialogType = DialogElement.TYPE_WARNING,
		callback = testTerraLifePlus.done,
		target = self,
		yesText = textyes,
		noText = textno
	})
end


function testTerraLifePlus.done(isYes)
	if isYes == true then
		local link = "mods.php?title=fs2022&searchMod=TerraLifePlus"
		openWebFile(link, "")
	end
end


testTerraLifePlus:AddXmlSchema()

if testTerraLifePlus:test() == true then
  source(g_currentModDirectory .. "scripts/premium.lua")
  source(g_currentModDirectory .. "scripts/MoreTrees.lua")
  source(g_currentModDirectory .. "scripts/addSprayTypes.lua")
  source(g_currentModDirectory .. "scripts/BlockDefaultFillTypes.lua")
  source(g_currentModDirectory .. "scripts/DirectoryRewrite.lua")
  source(g_currentModDirectory .. "scripts/loadAnimals.lua")
  source(g_currentModDirectory .. "scripts/bunkerSiloChanges.lua")
  source(g_currentModDirectory .. "scripts/loadAnimalFood.lua")
  source(g_currentModDirectory .. "scripts/animalDataOverride.lua")
  source(g_currentModDirectory .. "scripts/tedderConverterSwitcher.lua")
  source(g_currentModDirectory .. "scripts/mowerConverterSwitcher.lua")
  source(g_currentModDirectory .. "scripts/l10n_localToGlobal.lua")
  source(g_currentModDirectory .. "scripts/strawBlowerExtension.lua")
  source(g_currentModDirectory .. "scripts/mixerWagonUniversalDisplay.lua")
  source(g_currentModDirectory .. "scripts/registerTerraLifeSpecs.lua")
  source(g_currentModDirectory .. "scripts/addToProductionAndSellingStation.lua")
  source(g_currentModDirectory .. "scripts/green_mowing.lua")
  source(g_currentModDirectory .. "scripts/levelerFix.lua")
  source(g_currentModDirectory .. "scripts/terraLifeBaleExtension.lua")
  source(g_currentModDirectory .. "scripts/terraLifeBalers.lua")
  source(g_currentModDirectory .. "scripts/loadStoreItems.lua")
  source(g_currentModDirectory .. "scripts/loadFillTypes.lua")
  source(g_currentModDirectory .. "scripts/FAShopExtension.lua")
  source(g_currentModDirectory .. "scripts/FAShopRewrite.lua")
  source(g_currentModDirectory .. "scripts/TerraLiveExtension.lua")
  source(g_currentModDirectory .. "scripts/AddRealisticSeeder.lua")		
  source(g_currentModDirectory .. "scripts/AddAnimalFeeding.lua")
  source(g_currentModDirectory .. "scripts/fruitDestructionReloaded.lua")
  source(g_currentModDirectory .. "scripts/AddManuelDischarger.lua")		
  source(g_currentModDirectory .. "scripts/ChangePrecissionFarmingLoadingFile.lua")		
  source(g_currentModDirectory .. "scripts/gui/hud/FillLevelsDisplayExtensions.lua")		
  source(g_currentModDirectory .. "scripts/ChangePeriods.lua")
  source(g_currentModDirectory .. "scripts/CultivatorExtension.lua")
  source(g_currentModDirectory .. "scripts/AnimalDisplayExtension.lua")
  source(g_currentModDirectory .. "scripts/AddLivestockTrailerAnimation.lua")
  source(g_currentModDirectory .. "scripts/animalLimitIncreaser.lua")
  source(g_currentModDirectory .. "scripts/ConstructionDisplayExtension.lua")
  source(g_currentModDirectory .. "scripts/events/InGameNotification.lua")
  source(g_currentModDirectory .. "scripts/fruitPreparerExtension.lua")
  source(g_currentModDirectory .. "scripts/gui/base/GuiExtension.lua")
  source(g_currentModDirectory .. "scripts/gui/dialogs/NumberPickerDialog.lua")
  source(g_currentModDirectory .. "scripts/animals/events/InseminationEvent.lua")
  source(g_currentModDirectory .. "scripts/vehicles/specializations/LivestockTrailerExtension.lua")
  source(g_currentModDirectory .. "scripts/vehicles/specializations/TLFertilizingCultivatorExtension.lua")
  source(g_currentModDirectory .. "scripts/vehicles/specializations/TLFillUnitExtension.lua")
  source(g_currentModDirectory .. "scripts/vehicles/specializations/TLReceivingHopperExtension.lua")
  source(g_currentModDirectory .. "scripts/animals/husbandry/AnimalSystemExtension.lua")
  source(g_currentModDirectory .. "scripts/removeItems.lua")
  source(g_currentModDirectory .. "scripts/realismAddon_animalGrazing_register.lua")
  source(g_currentModDirectory .. "scripts/gui/InGameMenuMapFrameExtension.lua")
  source(g_currentModDirectory .. "scripts/MapInfoLoader.lua")
  source(g_currentModDirectory .. "scripts/Mission00Extension.lua")
  source(g_currentModDirectory .. "scripts/gui/hud/mapHotspots/InfoTextHotspot.lua")
  source(g_currentModDirectory .. "scripts/gui/hud/IngameMapExtension.lua")
  source(g_currentModDirectory .. "scripts/mapFilterExtension.lua")
  source(g_currentModDirectory .. "scripts/testBales.lua")
  source(g_currentModDirectory .. "scripts/milkExtension.lua")
  source(g_currentModDirectory .. "scripts/guiFixes.lua")
  source(g_currentModDirectory .. "scripts/directSowingNoTillReplacement.lua")
  source(g_currentModDirectory .. "scripts/waterTrailerExtension.lua")
  source(g_currentModDirectory .. "scripts/WardRobePlus_helper.lua")
  source(g_currentModDirectory .. "scripts/revamp_check.lua")
  source(g_currentModDirectory .. "scripts/PrecissionFarmingExtension.lua")
  source(g_currentModDirectory .. "scripts/gui/dialogs/EjectColostrumDialog.lua")
  source(g_currentModDirectory .. "scripts/animals/events/EjectColostrumEvent.lua")
  source(g_currentModDirectory .. "scripts/AddSpecialisations.lua")
  source(g_currentModDirectory .. "scripts/field/TLFieldManagerExtension.lua")
  source(g_currentModDirectory .. "scripts/objects/BunkerSiloExtension.lua")
  source(g_currentModDirectory .. "scripts/TerraLifePlusFermentationManager.lua")
  source(g_currentModDirectory .. "scripts/fieldJobs/fieldMissionExtension.lua")
  source(g_currentModDirectory .. "scripts/cutterFix.lua")
  source(g_currentModDirectory .. "scripts/statsFix.lua")
  source(g_currentModDirectory .. "scripts/ep_loader.lua")
  source(g_currentModDirectory .. "scripts/UpdateMowerAreaExtension.lua")
  source(g_currentModDirectory .. "scripts/gui/TLShopConfigScreenExtension.lua")
  source(g_currentModDirectory .. "scripts/player/TLPlayerHUDInfoBoxExtension.lua")
  source(g_currentModDirectory .. "scripts/vehicles/specializations/TLWheelsFix.lua")
  source(g_currentModDirectory .. "scripts/pickupCutterOverride.lua")
  source(g_currentModDirectory .. "scripts/TLExtendedWeedControlExtension.lua")
  source(g_currentModDirectory .. "scripts/TLWeedSpotSprayExtension.lua")
  source(g_currentModDirectory .. "scripts/TLMowerExtension.lua")
  source(g_currentModDirectory .. "scripts/TerraLifePlusDevTools.lua")

  g_storeManager:loadTerraLife(g_currentModName, g_currentModDirectory, "xml/terraLifePlus_storeItems.xml")
  local assets = g_modManager:getModByName("FS22_TerraLifePlus_Assets")
  local utils = g_modManager:getModByName("FS22_TerraLifePlus_Utils")
  g_storeManager:loadTerraLife(g_currentModName, assets.modDir, "xml/terraLifePlus_storeItems.xml")
  g_storeManager:loadTerraLife(g_currentModName, utils.modDir, "xml/terraLifePlus_storeItems.xml")

  g_numberPickerDialog = NumberPickerDialog.new()
  g_gui:loadGui(g_currentModDirectory .. "gui/dialogs/NumberPickerDialog.xml", "NumberPickerDialog", g_numberPickerDialog)
  g_ejectColostrumDialog = EjectColostrumDialog.new()
  g_gui:loadGui(g_currentModDirectory .. "gui/dialogs/EjectColostrumDialog.xml", "EjectColostrumDialog", g_ejectColostrumDialog)
  Logging.info("TerraLifePlus was loaded")
else
  Logging.info("Loading TerraLifePlus was stopped, No TerraLife-Map was detected")
end