-- Changes ModLoading-Order to load PrecisionFarming and TerraLife Plus before everything else
-- Adds Loading Animation
-- Prevents some mods from loading that break TerraLife Plus
-- By [F/A] Braeven - 20.05.2024
-- Version 1.4
-- 09.07.24 - Achimobil - Added Credits part

function MPLoadingScreen.modSortFunc(mod1, mod2)
	if mod1.modName == "FS22_TerraLifePlus" and mod2.isDLC == false then
		return true
	elseif mod2.modName == "FS22_TerraLifePlus" and mod1.isDLC == false then
		return false
	elseif mod1.isDLC == mod2.isDLC then
		return string.lower(mod1.modName) < string.lower(mod2.modName)
	elseif mod1.isDLC then
		return true
	end

	return false
end


loadTerraLifePlus = {}

function loadTerraLifePlus:AddXmlSchema()
	loadTerraLifePlus.XmlSchema = XMLSchema.new("TerraLifePlus")
	local schema = loadTerraLifePlus.XmlSchema
	schema:register(XMLValueType.STRING, "terraLife.incompatible#name", "ModName", nil, false)
end

loadTerraLifePlus:AddXmlSchema()

function loadTerraLifePlus:loadList()
	local mod = g_modManager:getModByName("FS22_TerraLifePlus")
	local xmlFile = XMLFile.load("TempDesc", mod.modDir.."xml/terraLifePlus_modList.xml")
	local list = {}

	xmlFile:iterate("terraLife.incompatible", function (_, key)
		local name = xmlFile:getString(key .. "#name")
		list[name] = true
	end)
	
	xmlFile:delete()
	return list
end


--Load List from TerraLifePlus
local list = loadTerraLifePlus:loadList()

--Inkompatible/doppelte mods deaktiveren
for i = #g_mpLoadingScreen.missionDynamicInfo.mods, 1, -1 do
	local mod = g_mpLoadingScreen.missionDynamicInfo.mods[i]
	if list[mod.modName] ~= nil then
		local xmlFile = XMLFile.load("TempDesc", mod.modDir.."modDesc.xml")
		if xmlFile:hasProperty("moddesc.terraLife") then
			--If the TerraLife Tag was found, the mod was updated to work with TLP
		else
			print("TerraLife Plus: Disabled Incompatible Mod: " .. mod.modName)
			table.remove(g_mpLoadingScreen.missionDynamicInfo.mods, i)
		end
		xmlFile:delete()
	elseif mod.modName == "FS22_precisionFarming" then
		g_mpLoadingScreen.missionDynamicInfo.mods[i].isDLC = true
	end
end

local TLPLogo = {}
TLPLogo.Credits = {
	"Agrar Asse",
	"Giants Software",
	"MrMarcel",
	"Stefan Maurus",
	"SMI-Matteo",
	"Univers-Simu Bruno & Latic",
	"Cinématique Farming DK",
	"MA7 Quentin & Team",
	"Bigfarmer145",
	"EireAgriModding & 4D Modding",
	"BernieSCS",
	"TopAce888",
	"FarmerFiveTom",
	"DSA Kastor & Floowy",
	"AgrarDesign Austria",
	"Landbauer",
	"Creative Mesh",
	"Wopster",
	"RealismusModding",
	"DMI",
	"LeoLeo",
	"Pikaso",
}

function TLPLogo:update(superFunc, dt)
	if self.TLP == nil then
		self.TLP = true
		local xmlGuiFileName = g_modsDirectory.."FS22_TerraLifePlus/gui/guiProfilesLoading.xml"
		g_gui:loadProfiles(xmlGuiFileName)
		local myElement3 = BitmapElement.new()
		myElement3:applyProfile("mpLoadingTLP")
		myElement3:setImageFilename(g_modsDirectory.."FS22_TerraLifePlus/gui/TerraLife_Logo_2.dds")
		self:addElement(myElement3)
		local myElement = AnimationElement.new()
		myElement:applyProfile("mpLoadingAnimationTLP")
		myElement:setImageFilename(g_modsDirectory.."FS22_TerraLifePlus/gui/TerraLife_Logo_1.dds")
		self:addElement(myElement)
		local myElement2 = BitmapElement.new()
		myElement2:applyProfile("mpLoadingTLP")
		myElement2:setImageFilename(g_modsDirectory.."FS22_TerraLifePlus/gui/Plus_logo.dds")
		self:addElement(myElement2)
	end
	
	superFunc(self, dt)
end

MPLoadingScreen.update = Utils.overwrittenFunction(MPLoadingScreen.update, TLPLogo.update)