milkExtension = {}

function milkExtension:updatePallets(superFunc)
	if self.isServer then
		local spec = self.spec_husbandryPallets

		if spec.loadedPallets == nil then
			spec.fillTypeIndexGoat = FillType.GOATMILK
			spec.fillTypeIndexSheep = FillType.SHEEPMILK
			spec.loadedPallets = true

			for fillTypeId, pallet in pairs(spec.palletSpawner:getSupportedFillTypes()) do
				if spec.fillTypeIndex == fillTypeId then
					spec.pallet = pallet
				elseif spec.fillTypeIndexSheep == fillTypeId then
					spec.palletSheep = pallet
				elseif spec.fillTypeIndexGoat == fillTypeId then
					spec.palletGoat = pallet
				end
			end
		end
		if not spec.spawnPending then
			if spec.pendingLiters > spec.pallet.capacity then
				spec.spawnPending = true
				spec.palletSpawner:spawnPallet(self:getOwnerFarmId(), spec.fillTypeIndex,  self.getPalletCallback, self)
			elseif self.spec_husbandryGoatMilk ~= nil and self.spec_husbandryGoatMilk.pendingLiters > spec.palletGoat.capacity then
				spec.spawnPending = true
				spec.palletSpawner:spawnPallet(self:getOwnerFarmId(), spec.fillTypeIndexGoat,  self.getPalletCallback, self)
			elseif self.spec_husbandrySheepMilk ~= nil and self.spec_husbandrySheepMilk.pendingLiters > spec.palletSheep.capacity then
				spec.spawnPending = true
				spec.palletSpawner:spawnPallet(self:getOwnerFarmId(), spec.fillTypeIndexSheep,  self.getPalletCallback, self)
			end
		end
	end
end

PlaceableHusbandryPallets.updatePallets = Utils.overwrittenFunction(PlaceableHusbandryPallets.updatePallets, milkExtension.updatePallets)

--Bugfix
function milkExtension:loadFromXMLFile(superFunc, xmlFile, key)
	local spec = self.spec_husbandryPallets
	spec.pendingLiters = xmlFile:getValue(key .. "#pendingLiters", spec.pendingLiters)

	self:updatePalletInfo()
end

PlaceableHusbandryPallets.loadFromXMLFile = Utils.overwrittenFunction(PlaceableHusbandryPallets.loadFromXMLFile, milkExtension.loadFromXMLFile)


function milkExtension:getPalletCallback(superFunc, pallet, result, fillTypeIndex)
	local spec = self.spec_husbandryPallets
	spec.spawnPending = false
	spec.currentPallet = pallet

	if pallet ~= nil then
		if spec.palletLimitReached then
			spec.palletLimitReached = false

			self:raiseDirtyFlags(spec.dirtyFlag)
		end

		if result == PalletSpawner.RESULT_SUCCESS then
			if fillTypeIndex == FillType.GOATMILK then
				local delta = pallet:addFillUnitFillLevel(self:getOwnerFarmId(), spec.fillUnitIndex, self.spec_husbandryGoatMilk.pendingLiters, fillTypeIndex, ToolType.UNDEFINED)
				self.spec_husbandryGoatMilk.pendingLiters = math.max(self.spec_husbandryGoatMilk.pendingLiters - delta, 0)
				self:raiseDirtyFlags(self.spec_husbandryGoatMilk.dirtyFlag)
			elseif fillTypeIndex == FillType.SHEEPMILK then
				local delta = pallet:addFillUnitFillLevel(self:getOwnerFarmId(), spec.fillUnitIndex, self.spec_husbandrySheepMilk.pendingLiters, fillTypeIndex, ToolType.UNDEFINED)
				self.spec_husbandrySheepMilk.pendingLiters = math.max(self.spec_husbandrySheepMilk.pendingLiters - delta, 0)
				self:raiseDirtyFlags(self.spec_husbandrySheepMilk.dirtyFlag)
			else
				pallet:emptyAllFillUnits(true)
				spec.pallets[pallet] = true
				local delta = pallet:addFillUnitFillLevel(self:getOwnerFarmId(), spec.fillUnitIndex, spec.pendingLiters, fillTypeIndex, ToolType.UNDEFINED)
				spec.pendingLiters = math.max(spec.pendingLiters - delta, 0)
			end
		end

		self:updatePallets()
	elseif result == PalletSpawner.RESULT_NO_SPACE then
		self:showSpawnerBlockedWarning()
	elseif result == PalletSpawner.PALLET_LIMITED_REACHED and not spec.palletLimitReached then
		spec.palletLimitReached = true

		self:raiseDirtyFlags(spec.dirtyFlag)
	end

	self:updatePalletInfo()
end

PlaceableHusbandryPallets.getPalletCallback = Utils.overwrittenFunction(PlaceableHusbandryPallets.getPalletCallback, milkExtension.getPalletCallback)


--Bugfix2
function PlaceableHusbandryPallets:getConditionInfos(superFunc)
	local spec = self.spec_husbandryPallets
	local infos = superFunc(self)
	local info = {}
	local fillType = g_fillTypeManager:getFillTypeByIndex(spec.fillTypeIndex)
	info.title = fillType.title
	info.value = spec.pendingLiters
	local capacity = 3000
	info.valueText = g_i18n:formatNumber(info.value, 0) .. " / " .. g_i18n:formatVolume(capacity, 0);
	local ratio = info.value / capacity

	info.ratio = MathUtil.clamp(ratio, 0, 1)
	info.invertedBar = true

	table.insert(infos, info)

	return infos
end

PlaceableHusbandryPallets.loadFromXMLFile = Utils.overwrittenFunction(PlaceableHusbandryPallets.loadFromXMLFile, milkExtension.loadFromXMLFile)


function milkExtension:onReadStream(streamId, connection)
	local spec = self.spec_husbandryPallets
	spec.pendingLiters = streamReadFloat32(streamId)
end

PlaceableHusbandryPallets.onReadStream = Utils.appendedFunction(PlaceableHusbandryPallets.onReadStream, milkExtension.onReadStream)


function milkExtension:onWriteStream(streamId, connection)
	local spec = self.spec_husbandryPallets

	streamWriteFloat32(streamId, spec.pendingLiters)
end

PlaceableHusbandryPallets.onWriteStream = Utils.appendedFunction(PlaceableHusbandryPallets.onWriteStream, milkExtension.onWriteStream)


function milkExtension:onReadUpdateStream(streamId, timestamp, connection)
	if connection:getIsServer() and streamReadBool(streamId) then
		local spec = self.spec_husbandryPallets
		spec.pendingLiters = streamReadFloat32(streamId)
	end
end

PlaceableHusbandryPallets.onReadUpdateStream = Utils.appendedFunction(PlaceableHusbandryPallets.onReadUpdateStream, milkExtension.onReadUpdateStream)


function milkExtension:onWriteUpdateStream(streamId, connection, dirtyMask)
	if not connection:getIsServer() then
		local spec = self.spec_husbandryPallets

		if streamWriteBool(streamId, bitAND(dirtyMask, spec.dirtyFlag) ~= 0) then
			streamWriteFloat32(streamId, spec.pendingLiters)
		end
	end
end

PlaceableHusbandryPallets.onWriteUpdateStream = Utils.appendedFunction(PlaceableHusbandryPallets.onWriteUpdateStream, milkExtension.onWriteUpdateStream)