-- Simple Display to show Mixer Wagon contents when outside of mixerwagon vehicle, shows contents, % values, possible fillTypes and fillLevel 
-- by modelleicher (Farming Agency)

mixerWagonUniversalDisplay = {}

-- default distance set to 5m for now 
mixerWagonUniversalDisplay.distance = 5
mixerWagonUniversalDisplay.isActive = true

function mixerWagonUniversalDisplay:loadMap()
    self.inputsActive = false;
    self.lastControlledVehicle = g_currentMission.controlledVehicle
end

function mixerWagonUniversalDisplay:toggleMixerDisplay()
    mixerWagonUniversalDisplay.isActive = not mixerWagonUniversalDisplay.isActive
    g_inputBinding:setActionEventText(self.eventId, g_i18n:getText("TL_action_TOGGLE_MIXER_DISPLAY")..": "..g_i18n:getText("TL_actionState_TOGGLE_MIXER_DISPLAY_"..tostring(mixerWagonUniversalDisplay.isActive)))
end

function mixerWagonUniversalDisplay:mixerDisplayDistance()
    mixerWagonUniversalDisplay.distance = mixerWagonUniversalDisplay.distance + 5
    if mixerWagonUniversalDisplay.distance > 70 then
        mixerWagonUniversalDisplay.distance = 5
    end
    g_inputBinding:setActionEventText(self.eventIdDistance, g_i18n:getText("TL_action_MIXER_DISPLAY_DISTANCE")..": "..tostring(mixerWagonUniversalDisplay.distance)) 
end

function mixerWagonUniversalDisplay:draw()
	-- get distance to mixerWagons and nearest
	local nearest = mixerWagonUniversalDisplay.distance
	local nearestMixer = nil

	for _, vehicle in pairs(g_currentMission.vehicles) do
		if vehicle.spec_mixerWagon and vehicle.rootNode ~= nil then -- for some reason in MP rootNode can be nil while vehicle already exists with spec_mixerWagon so.. 
			local cameraRoot = getCamera()
			if g_currentMission.controlledVehicle then
				cameraRoot = g_currentMission.controlledVehicle.rootNode
			end
			local aX, aY, aZ = getWorldTranslation(vehicle.rootNode)
			local cX, cY, cZ = getWorldTranslation(cameraRoot);
			local dist = MathUtil.vector3Length(aX-cX, aY-cY, aZ-cZ);

			local isControlled = (vehicle == g_currentMission.controlledVehicle) or (vehicle.spec_attachable ~= nil and vehicle.spec_attachable.attacherVehicle == g_currentMission.controlledVehicle)
			if dist < mixerWagonUniversalDisplay.distance and dist <= nearest and not isControlled then
				nearest = dist
				nearestMixer = vehicle
			end
		end
	end

	if nearestMixer ~= nil then
		if not self.inputsActive or g_currentMission.controlledVehicle ~= self.lastControlledVehicle then
			local _ , eventId = g_inputBinding:registerActionEvent(InputAction.TOGGLE_MIXER_DISPLAY, mixerWagonUniversalDisplay, mixerWagonUniversalDisplay.toggleMixerDisplay, false, true, false, true);
			g_inputBinding:setActionEventText(eventId, g_i18n:getText("TL_action_TOGGLE_MIXER_DISPLAY")..": "..g_i18n:getText("TL_actionState_TOGGLE_MIXER_DISPLAY_"..tostring(mixerWagonUniversalDisplay.isActive)))   
			self.eventId = eventId

			local _ , eventId = g_inputBinding:registerActionEvent(InputAction.MIXER_DISPLAY_DISTANCE, mixerWagonUniversalDisplay, mixerWagonUniversalDisplay.mixerDisplayDistance, false, true, false, true);
			g_inputBinding:setActionEventText(eventId, g_i18n:getText("TL_action_MIXER_DISPLAY_DISTANCE")..": "..tostring(mixerWagonUniversalDisplay.distance))          
			self.eventIdDistance = eventId 
			self.inputsActive = true
		end

		local spec = nearestMixer.spec_mixerWagon
		-- only mixerWagons affected by M+ switcher 
		if spec.currentRecipe ~= nil and mixerWagonUniversalDisplay.isActive then
			local currentRecipe = g_currentMission.animalFoodSystem.recipes[spec.currentRecipe]
			local scale = 1
			local lowerY = 0.05
			local yPerLine = 0.019 * scale 
			local storeItem = g_storeManager:getItemByXMLFilename(nearestMixer.configFileName)

			-- reset text values
			setTextColor(1, 1, 1, 0.6)
			setTextBold(false) 

			local fillLevel = nearestMixer:getFillUnitFillLevel(spec.fillUnitIndex)
			local capacity = nearestMixer:getFillUnitCapacity(spec.fillUnitIndex)

			-- cycle ingredients
			local bonuslines = 0
			for i = #spec.mixerWagonFillTypes, 1, -1 do
				local ingredient = spec.mixerWagonFillTypes[i]
				local index = #spec.mixerWagonFillTypes - i + 1
				local text = ""
				local text2 = ""
				for fillTypeIndex, _ in pairs(ingredient.fillTypes) do
					local title = g_fillTypeManager.indexToFillType[fillTypeIndex].title
					text = text.." "..title
					if string.len(text) > 90 then
						text2 = text
						text = ""
						bonuslines = bonuslines + 1
					end
				end

				--Ingredient Title
				setTextAlignment(RenderText.ALIGN_LEFT)
				renderText(0.25, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, tostring(ingredient.title))

				--fillevel, aktuelle %, min %, max %
				setTextAlignment(RenderText.ALIGN_RIGHT)
				if fillLevel > 0 then
					renderText(0.375, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, tostring(math.floor(ingredient.fillLevel)).."l")
					if math.floor((ingredient.fillLevel / fillLevel) * 100) > ingredient.maxPercentage * 100 then
						setTextColor(0.9157, 0.1420, 0.0002, 1)
					end
					if math.floor((ingredient.fillLevel / fillLevel) * 100) >= ingredient.minPercentage * 100 and math.floor((ingredient.fillLevel / fillLevel) * 100) <= ingredient.maxPercentage * 100 and math.floor((ingredient.fillLevel / fillLevel) * 100) > 0 then
						setTextColor(0.0203, 0.1441, 0.0704, 1)
					end
					if math.floor((ingredient.fillLevel / fillLevel) * 100) < ingredient.minPercentage * 100 then
						setTextColor(1, 0, 0, 1)
					end
					renderText(0.41, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, "("..tostring(math.floor((ingredient.fillLevel / fillLevel) * 100)).."%)")
					setTextColor(1, 1, 1, 1)
				else
					renderText(0.375, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, "0l")
					renderText(0.41, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, "(0%)")
				end
				renderText(0.44, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, tostring(ingredient.minPercentage * 100).."%")
				renderText(0.48, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, tostring(ingredient.maxPercentage * 100).."%")

				--Zweite Zeile der FillTypes erstellen wenn nötig
				setTextAlignment(RenderText.ALIGN_LEFT)
				if text2 ~= "" then
					renderText(0.50, lowerY + (yPerLine * (index + bonuslines-1)), 0.017 * scale, text)
					renderText(0.50, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, text2)
				else
					renderText(0.50, lowerY + (yPerLine * (index + bonuslines)), 0.017 * scale, text)
				end
			end
				
			-- number of lines needed: (number of ingredients + 1 line as header and 1 line for fillVolume total)
			local linesNeeded = #currentRecipe.ingredients + 1 + bonuslines
			-- render header line 

			-- render foot line
			local footText = g_i18n:getText("TL_mixerDisplay_recipe").." "..g_i18n:getText("TL_recipe_"..tostring(spec.outputFillType)).."           "..g_i18n:getText("TL_mixerDisplay_currentFillLevel").." "..tostring(math.floor(fillLevel)).."l / "..tostring(capacity).."l      "..tostring(storeItem.name)
			setTextBold(true)
			setTextAlignment(RenderText.ALIGN_LEFT)
			renderText(0.25, lowerY, 0.019 * scale, footText)
			setTextBold(false) 

			--render Header
			setTextBold(true)
			renderText(0.25, lowerY + (yPerLine * (linesNeeded)), 0.018 * scale, g_i18n:getText("TL_mixerDisplay_ingredient"))
			setTextAlignment(RenderText.ALIGN_RIGHT)
			renderText(0.385, lowerY + (yPerLine * (linesNeeded)), 0.018 * scale, g_i18n:getText("TL_mixerDisplay_currentPercent"))
			renderText(0.44, lowerY + (yPerLine * (linesNeeded)), 0.018 * scale, g_i18n:getText("TL_mixerDisplay_minPercent"))
			renderText(0.48, lowerY + (yPerLine * (linesNeeded)), 0.018 * scale, g_i18n:getText("TL_mixerDisplay_maxPercent"))
			setTextAlignment(RenderText.ALIGN_LEFT)
			renderText(0.50, lowerY + (yPerLine * (linesNeeded)), 0.018 * scale, g_i18n:getText("TL_mixerDisplay_fillTypes"))
			setTextBold(false)  
		end
	else
		self.inputsActive = false
		g_inputBinding:removeActionEvent(self.eventId)
		g_inputBinding:removeActionEvent(self.eventIdDistance)
	end

	self.lastControlledVehicle = g_currentMission.controlledVehicle
end


function mixerWagonUniversalDisplay.onPostLoad(self, superFunc, savegame)

    local returnValues = superFunc(self, savegame)

    if savegame ~= nil then
        local spec = self.spec_mixerWagon

        local displayState = getXMLBool(savegame.xmlFile.handle, savegame.key..".mixerWagon#universalDisplayState")
        if displayState ~= nil then
            mixerWagonUniversalDisplay.isActive = displayState
        end

        local distance = getXMLInt(savegame.xmlFile.handle, savegame.key..".mixerWagon#universalDisplayDistance")
        if distance ~= nil then
            mixerWagonUniversalDisplay.distance = distance
        end

    end    

    return returnValues
end
MixerWagon.onPostLoad = Utils.overwrittenFunction(MixerWagon.onPostLoad, mixerWagonUniversalDisplay.onPostLoad)


function mixerWagonUniversalDisplay.saveToXMLFile(self, superFunc, xmlFile, key, usedModNames)

    local returnValues = superFunc(self, xmlFile, key, usedModNames)

	local spec = self.spec_mixerWagon
    if mixerWagonUniversalDisplay.isActive ~= nil then
        setXMLBool(xmlFile.handle, key.."#universalDisplayState", mixerWagonUniversalDisplay.isActive)
    end
    if mixerWagonUniversalDisplay.distance ~= nil then
        setXMLInt(xmlFile.handle, key.."#universalDisplayDistance", mixerWagonUniversalDisplay.distance)
    end

    return returnValues 
end
MixerWagon.saveToXMLFile = Utils.overwrittenFunction(MixerWagon.saveToXMLFile, mixerWagonUniversalDisplay.saveToXMLFile)



function mixerWagonUniversalDisplay:drawUI(superFunc, leftPosX, rightPosX, posY)
	setTextColor(unpack(self.uiTextColor))
	setTextBold(true)
	setTextAlignment(RenderText.ALIGN_LEFT)

	local text = string.format("%s (%s)", g_i18n:getText("info_mixingRatio"), self.vehicle:getFullName())

	renderText(leftPosX, posY + self.displayHeight - self.uiTextSize * 1.5, self.uiTextSize, text)
	setTextBold(false)

	local totalFillLevel = 0

	if self.vehicle:getFillUnitFillLevel(self.mixerWagon.fillUnitIndex) > 0 then
		for i, mixerWagonFillType in ipairs(self.mixerWagon.mixerWagonFillTypes) do
			totalFillLevel = totalFillLevel + mixerWagonFillType.fillLevel
			self.fillTypeStatus[i].fillLevel = mixerWagonFillType.fillLevel
		end
	end

	local barY = posY + #self.fillTypeStatus * self.helpHeightPerFruit + self.helpHeightPerFruit * 0.3

	for _, fillTypeDisplay in ipairs(self.fillTypeStatus) do
		barY = barY - self.helpHeightPerFruit
		local percentage = 0

		if self.vehicle:getFillUnitFillLevel(self.mixerWagon.fillUnitIndex) > 0 then
			percentage = fillTypeDisplay.fillLevel / totalFillLevel
		end

		--resetten, orange ist normale Farbe vom Balken
		fillTypeDisplay.statusBar:setColor(0.9157, 0.1420, 0.0002, 1)
		fillTypeDisplay.statusBar.overlayBackground:setColor(unpack(MixerWagonHUDExtension.COLOR.STATUS_BAR_BG))

		if percentage > fillTypeDisplay.maxPercentage then
			--nichts machen
		elseif percentage < fillTypeDisplay.minPercentage then
			--mindestMenge nicht erreicht
			--grüner balken, roter BG
			fillTypeDisplay.statusBar:setColor(unpack(fillTypeDisplay.statusBarColor))
			fillTypeDisplay.statusBar.overlayBackground:setColor(unpack(fillTypeDisplay.statusBarColor2))
		elseif fillTypeDisplay.minPercentage <= percentage and percentage <= fillTypeDisplay.maxPercentage then
			--grün, alles in Ordnung
			fillTypeDisplay.statusBar:setColor(unpack(fillTypeDisplay.statusBarColor))
		end

		fillTypeDisplay.statusBar:setPosition(rightPosX - fillTypeDisplay.statusBar.width, barY + (self.helpHeightPerFruit - fillTypeDisplay.statusBar.height) * 0.5)
		fillTypeDisplay.statusBar:setValue(percentage)
		fillTypeDisplay.statusBar:render()

		if fillTypeDisplay.overlay ~= nil then
			fillTypeDisplay.overlay:setPosition(leftPosX, barY + (self.helpHeightPerFruit - fillTypeDisplay.overlay.height) * 0.5)
			fillTypeDisplay.overlay:render()
		end

		setTextAlignment(RenderText.ALIGN_RIGHT)
		renderText(leftPosX + self.fillLevelTextOffsetX, barY + self.fillLevelTextOffsetY, self.uiTextSize, string.format("%1.1f%%", percentage * 100))
		setTextAlignment(RenderText.ALIGN_LEFT)

		local y = fillTypeDisplay.statusBar.y + fillTypeDisplay.statusBar.height * 0.5
		local maxLeft = fillTypeDisplay.statusBar.x + fillTypeDisplay.statusBar.width
		local xLeft = math.min(fillTypeDisplay.statusBar.x + fillTypeDisplay.statusBar.width * fillTypeDisplay.minPercentage, maxLeft)
		local xRight = math.min(fillTypeDisplay.statusBar.x + fillTypeDisplay.statusBar.width * fillTypeDisplay.maxPercentage, maxLeft)

		self.fillRangeMarkerOverlay:setInvertX(false)
		self.fillRangeMarkerOverlay:setPosition(xLeft, y)
		self.fillRangeMarkerOverlay:render()
		self.fillRangeMarkerOverlay:setInvertX(true)
		self.fillRangeMarkerOverlay:setPosition(xRight - self.fillRangeMarkerOverlay.width, y)
		self.fillRangeMarkerOverlay:render()
	end

	return posY
end
MixerWagonHUDExtension.draw = Utils.overwrittenFunction(MixerWagonHUDExtension.draw, mixerWagonUniversalDisplay.drawUI)

addModEventListener(mixerWagonUniversalDisplay)