BunkerSiloExtension = {
    DAYS_UNTIL_ROTTEN = 1
}

local BunkerSiloExtension_mt = Class(BunkerSiloExtension)

function BunkerSiloExtension.new(self, superFunc, isServer, isClient, customMt)
    local self = superFunc(self, isServer, isClient, customMt)
	g_messageCenter:subscribe(MessageType.WEEK_CHANGED, self.onWeekChanged, self)
    self.rottenFillType = FillType.COMPOST
    self.weeksSinceFermented = 0
    return self
end

BunkerSilo.new = Utils.overwrittenFunction(BunkerSilo.new, BunkerSiloExtension.new)

function BunkerSiloExtension.registerSavegameXMLPaths(schema, superFunc, basePath)
    superFunc(schema, basePath)
	schema:register(XMLValueType.INT, basePath .. "#weeksSinceFermented", "Month since silo is fermented", 0)
end

BunkerSilo.registerSavegameXMLPaths = Utils.overwrittenFunction(BunkerSilo.registerSavegameXMLPaths, BunkerSiloExtension.registerSavegameXMLPaths)

function BunkerSiloExtension:readStream(superFunc, streamId, connection)
	superFunc(self, streamId, connection)
	self.weeksSinceFermented = streamReadUInt8(streamId)
end

BunkerSilo.readStream = Utils.overwrittenFunction(BunkerSilo.readStream, BunkerSiloExtension.readStream)

function BunkerSiloExtension:writeStream(superFunc, streamId, connection)
	superFunc(self, streamId, connection)
	streamWriteUInt8(streamId, self.weeksSinceFermented)
end

BunkerSilo.writeStream = Utils.overwrittenFunction(BunkerSilo.writeStream, BunkerSiloExtension.writeStream)

function BunkerSiloExtension:saveToXMLFile(superFunc, xmlFile, key, usedModNames)
	superFunc(self, xmlFile, key, usedModNames)
	xmlFile:setValue(key .. "#weeksSinceFermented", self.weeksSinceFermented)
end

BunkerSilo.saveToXMLFile = Utils.overwrittenFunction(BunkerSilo.saveToXMLFile, BunkerSiloExtension.saveToXMLFile)

function BunkerSiloExtension:loadFromXMLFile(superFunc, xmlFile, key)
	superFunc(self, xmlFile, key)
	self.weeksSinceFermented = xmlFile:getValue(key .. "#weeksSinceFermented", 0)

	return true
end

BunkerSilo.loadFromXMLFile = Utils.overwrittenFunction(BunkerSilo.loadFromXMLFile, BunkerSiloExtension.loadFromXMLFile)

function BunkerSiloExtension:setState(state, showNotification)
    if state == BunkerSilo.STATE_FERMENTED then
        self.weeksSinceFermented = 0
    end
end

BunkerSilo.setState = Utils.appendedFunction(BunkerSilo.setState, BunkerSiloExtension.setState)

function BunkerSiloExtension:openSilo(superFunc, px, py, pz)
    if self:isBunkerSiloRotten() then
        self:spawnRottenFillTypeOnOpen(px, py, pz)
    else
        superFunc(self, px, py, pz)
    end
end

BunkerSilo.openSilo = Utils.overwrittenFunction(BunkerSilo.openSilo, BunkerSiloExtension.openSilo)

function BunkerSiloExtension:onChangedFillLevelCallback(superFunc, vehicle, fillDelta, fillType, x, y, z)
    if fillDelta >= 0 then
		return
	end

    if self:isBunkerSiloRotten() then
        self:spawnRottenFillTypeOnFillLevelCallback(vehicle, fillDelta, fillType, x, y, z)
    else
        superFunc(self, vehicle, fillDelta, fillType, x, y, z)
    end
end

BunkerSilo.onChangedFillLevelCallback = Utils.overwrittenFunction(BunkerSilo.onChangedFillLevelCallback, BunkerSiloExtension.onChangedFillLevelCallback)

function BunkerSiloExtension:onHourChanged()
	-- Override because change to weeks. Original makes fermentation of 1/24 per hour divided by the daysPerPeriod.
	-- So planned fermentation is 24 hours or one month. 
	if self.state == BunkerSilo.STATE_CLOSED and self.isServer then
		self.fermentingPercent = math.min(self.fermentingPercent + (1/36/4) / g_currentMission.environment.daysPerPeriod, 1)

		self:raiseDirtyFlags(self.bunkerSiloDirtyFlag)

		if self.fermentingPercent >= 0.998 then
			self:setState(BunkerSilo.STATE_FERMENTED, true)
		end
	end
end
BunkerSilo.onHourChanged = Utils.overwrittenFunction(BunkerSilo.onHourChanged, BunkerSiloExtension.onHourChanged)

--------------------------------------------------- New BunkerSilo functions ---------------------------------------------------

function BunkerSilo:isBunkerSiloRotten()
	local fermentation = g_terraLifePlusFermentationManager.fermentation[self.outputFillType]

    if fermentation == nil then
        return false
    else
        return self.weeksSinceFermented >= tonumber(fermentation.time)
    end
end

function BunkerSilo:spawnRottenFillTypeOnOpen(px, py, pz)
    self:setState(BunkerSilo.STATE_DRAIN, true)

    self.bunkerSiloArea.offsetFront = self:getBunkerAreaOffset(true, 0, self.fermentingFillType)
    self.bunkerSiloArea.offsetBack = self:getBunkerAreaOffset(false, 0, self.fermentingFillType)
    local openAtFront = self:getIsCloserToFront(px, py, pz)

    if openAtFront and not self.isOpenedAtFront then
        self:switchFillTypeAtOffsetFromTo(true, self.bunkerSiloArea.offsetFront, self.openingLength, self.fermentingFillType, self.rottenFillType)

        self.isOpenedAtFront = true

        self:raiseDirtyFlags(self.bunkerSiloDirtyFlag)
    elseif not self.isOpenedAtBack then
        self:switchFillTypeAtOffsetFromTo(false, self.bunkerSiloArea.offsetBack, self.openingLength, self.fermentingFillType, self.rottenFillType)

        self.isOpenedAtBack = true

        self:raiseDirtyFlags(self.bunkerSiloDirtyFlag)
    end
end

function BunkerSilo:spawnRottenFillTypeOnFillLevelCallback(vehicle, fillDelta, fillType, x, y, z)
    local area = self.bunkerSiloArea

    if x == nil or y == nil or z == nil then
        x, y, z = getWorldTranslation(vehicle.components[1].node)
    end

    local closerToFront = self:getIsCloserToFront(x, y, z)
    local length = self.openingLength

    if closerToFront then
        if self.isOpenedAtFront then
            local p1 = MathUtil.getProjectOnLineParameter(x, z, area.sx, area.sz, area.dhx_norm, area.dhz_norm)

            if p1 > area.offsetFront - length then
                local offset = self:getBunkerAreaOffset(true, area.offsetFront, self.fermentingFillType)
                local targetOffset = math.max(p1, offset) + length
                self:switchFillTypeAtOffsetFromTo(false, area.offsetFront, targetOffset - area.offsetFront, self.fermentingFillType, self.rottenFillType)
                area.offsetFront = targetOffset
            end
        end
    elseif self.isOpenedAtBack then
        local p1 = MathUtil.getProjectOnLineParameter(x, z, area.hx, area.hz, -area.dhx_norm, -area.dhz_norm)

        if p1 > area.offsetBack - length then
            local offset = self:getBunkerAreaOffset(false, area.offsetBack, self.fermentingFillType)
            local targetOffset = math.max(p1, offset) + length
            self:switchFillTypeAtOffsetFromTo(false, area.offsetBack, targetOffset - area.offsetBack, self.fermentingFillType, self.rottenFillType)
            area.offsetBack = targetOffset
        end
    end
end

function BunkerSilo:onWeekChanged()
    self.weeksSinceFermented = self.weeksSinceFermented or 0
    self.weeksSinceFermented = self.weeksSinceFermented + 1

    local fermentation = g_terraLifePlusFermentationManager.fermentation[self.outputFillType]

    if fermentation ~= nil and self.weeksSinceFermented == tonumber(fermentation.time) then
        self:makeAllFermentedFillTypesToRotten()
    end
end

function BunkerSilo:makeAllFermentedFillTypesToRotten()
	local area = self.bunkerSiloArea
	local length = MathUtil.vector3Length(area.dhx, area.dhy, area.dhz)
    self:switchFillTypeAtOffsetFromTo(true, 0, length, self.outputFillType, self.rottenFillType)
end

function BunkerSilo:switchFillTypeAtOffsetFromTo(switchAtFront, offset, length, fillType, newFillType)
	local a0x, a0z, a1x, a1z, a2x, a2z = nil
	local area = self.bunkerSiloArea

	if switchAtFront then
		a0z = area.sz + offset * area.dhz_norm
		a0x = area.sx + offset * area.dhx_norm
		a1z = a0z + area.dwz
		a1x = a0x + area.dwx
		a2z = area.sz + (offset + length) * area.dhz_norm
		a2x = area.sx + (offset + length) * area.dhx_norm
	else
		a0z = area.hz - offset * area.dhz_norm
		a0x = area.hx - offset * area.dhx_norm
		a1z = a0z + area.dwz
		a1x = a0x + area.dwx
		a2z = area.hz - (offset + length) * area.dhz_norm
		a2x = area.hx - (offset + length) * area.dhx_norm
	end

	DensityMapHeightUtil.changeFillTypeAtArea(a0x, a0z, a1x, a1z, a2x, a2z, fillType, newFillType)
end