--[[
Copyright (C) braeven, 2022-2023

Author: braeven / Achimobil
Date: 01.06.2024
Version: 1.3.0.0 for TLP

Contact/Help/Tutorials:
discord.gg/gHmnFZAypk

Changelog:
1.3.0.0 Version Working with TLP

Important:
Kopiere diese Datei in deine Produktionen um einen Revamp-Versionscheck ausführen zu können. Diese Datei ist außerdem nötig um Filltypes bei bestehenden Sellingstations anmelden zu können und StoreItems An-/Abzumelden.

Copy this File into your Productions to add a Revamp-Versionscheck. This File is also needed to add Filltypes into existing Sellingstations and registering/unregistering StoreItems.
]]

RevampChecker = {}


--Production Revamp: Function to compare the needed with the avaible version of Revamp
function RevampChecker.check(revampVersion, neededVersion)
	local base, major, minor, patch = string.match(revampVersion, "(%d+)%.(%d+)%.(%d+)%.(%d+)")
	RevampChecker.base = math.floor(base)
	RevampChecker.major = math.floor(major)
	RevampChecker.minor = math.floor(minor)
	RevampChecker.patch = math.floor(patch)

	local base, major, minor, patch = string.match(neededVersion, "(%d+)%.(%d+)%.(%d+)%.(%d+)")
	base = math.floor(base)
	major = math.floor(major)
	minor = math.floor(minor)
	patch = math.floor(patch)
	if RevampChecker.base > base then
		return true
	elseif RevampChecker.base == base then
		if RevampChecker.major > major then
			return true
		elseif RevampChecker.major == major then
			if RevampChecker.minor > minor then
				return true
			elseif RevampChecker.minor == minor then
				if RevampChecker.patch > patch then
					return true
				elseif RevampChecker.patch == patch then
					return true
				end
			end
		end
	end
	return false
end


--Production Revamp: Callback function for the warning message. Opens the modhub page if needed.
function RevampChecker.done(isYes)
	if isYes == true then
		local link = "mods.php?title=fs2022&searchMod=revamp"
		openWebFile(link, "")
	end
end


--Production Revamp: Warning function - Will generate the warning message for 2 languages
function RevampChecker.warning(revampVersion, neededVersion, modName)
	local title = "Wrong Version of Production Revamp"
	local text = "At least one mod needs a newer version.\n Mod " ..modName.. " needs at least version " ..neededVersion.. " of Production Revamp.\n The installed version is " ..revampVersion.. ", please update to the latest one."
	local textyes = "Open Modhub"
	local textno = "Ignore"
	local language = g_languageShort
	if language == "de" then
		title = "Veraltete Version von Production Revamp"
		text = "Mindestens ein Mod benötigt eine neuere Version.\n Der Mod " ..modName.. " benötigt Version " ..neededVersion.. " von Production Revamp.\n Die vorhandene Version ist " ..revampVersion.. ", bitte updaten."
		textyes = "Modhub öffnen"
		textno = "Ignorieren"
	end

	g_gui:showYesNoDialog({
		title = title,
		text = text,
		dialogType = DialogElement.TYPE_WARNING,
		callback = RevampChecker.done,
		target = self,
		yesText = textyes,
		noText = textno
	})
end


--Production Revamp: Checking the Version of Revamp
function RevampChecker.test()
	local mods = g_modManager:getActiveMods(FS22_A_ProductionRevamp)
	local revampversion = ""

	for index, activemod in pairs(mods) do
		if activemod.title == "Production Revamp" then
			revampversion = activemod.version
		end
	end

	local modName = RevampChecker.currentModName
	local mod = g_modManager:getModByName(modName)

	if mod == nil and g_iconGenerator then
		-- im Icon generator passiert so kein fehler
		return
	end

	local xmlFile = XMLFile.load("TempDesc", mod.modFile)

	if xmlFile ~= nil and xmlFile ~= 0 then
		local version = xmlFile:getString("modDesc.revamp#minVersion")

		if revampversion == "" and version ~= nil then
			print("Production Revamp: Mod " ..modName.. " needs Revamp Version: " ..version.. ", Revamp not was found.")
		elseif version ~= nil then
			print("Production Revamp: Testing Mod " ..modName.. " for Version: " ..version.. ", Revamp Version " ..revampversion.. " was found.")
			if not RevampChecker.check(revampversion, version) then
				RevampChecker.warning(revampversion, version, modName)
			end
		end
	end

	xmlFile:delete()
end

RevampChecker.currentModName = g_currentModName
Mission00.load = Utils.appendedFunction(Mission00.load, RevampChecker.test)