ManuelDischarger = {}

function ManuelDischarger.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(FillUnit, specializations)
end

function ManuelDischarger.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", ManuelDischarger)
	SpecializationUtil.registerEventListener(vehicleType, "onDelete", ManuelDischarger)
end

function ManuelDischarger.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "manuelDischargeTriggerCallback", ManuelDischarger.manuelDischargeTriggerCallback)
end

function ManuelDischarger.initSpecialization()
	local schema = Vehicle.xmlSchema
    local baseKey = "vehicle.manuelDischarger"

	schema:register(XMLValueType.BOOL, baseKey.."#isStowable", "Is object stowable", false)
	schema:register(XMLValueType.STRING, baseKey.."#dischargeObjectFileXml", "Discharge object")
	schema:register(XMLValueType.STRING, baseKey.."#triggerNode", "Discharge trigger node")
	schema:register(XMLValueType.STRING, baseKey.."#spawnNode", "Spawn node for objects")
end

function ManuelDischarger:onLoad(savegame)
	self.spec_manuelDischarger = {}
	local spec = self.spec_manuelDischarger

	local baseKey = "vehicle.manuelDischarger"
	if self.xmlFile:hasProperty(baseKey) then
		spec.isStowable = self.xmlFile:getValue(baseKey.."#isStowable", false)
		
		local dischargeObjectFileXml = self.xmlFile:getValue(baseKey.."#dischargeObjectFileXml")
		if dischargeObjectFileXml ~=nil then
			dischargeObjectFileXml = Utils.getFilename(dischargeObjectFileXml, self.baseDirectory)
			if fileExists(dischargeObjectFileXml) then
			 spec.dischargeObjectFileXml = dischargeObjectFileXml
			end
		end
		local spawnNodePath = self.xmlFile:getValue(baseKey.."#spawnNode")
		spec.spawnNode = I3DUtil.indexToObject(self.components, spawnNodePath, self.i3dMappings)

		local triggerNodeXmlPath = self.xmlFile:getValue(baseKey.."#triggerNode")
		spec.manuelDischargeTrigger = I3DUtil.indexToObject(self.components, triggerNodeXmlPath, self.i3dMappings)

		if spec.manuelDischargeTrigger ~= nil and spec.dischargeObjectFileXml ~= nil then
				spec.manuelDischargeTriggerId = addTrigger(spec.manuelDischargeTrigger, "manuelDischargeTriggerCallback", self)
		end
	end
end

function ManuelDischarger:onDelete()
	local spec = self.spec_manuelDischarger
	if spec.manuelDischargeTrigger ~= nil then
		removeTrigger(spec.manuelDischargeTrigger)
	end
end

function ManuelDischarger.addDischargeActionEvent(self)
    local spec = self.spec_manuelDischarger

    if spec.dischargeObjectFileXml == nil then
        return
    end

    self:raiseActive()
    spec.actionEvents = {}
    self:clearActionEventsTable(spec.actionEvents)
    local _ , eventId = self:addActionEvent(spec.actionEvents, InputAction.MANUELDISCHARGER_DISCHARGE, self, ManuelDischarger.discharge, false, true, false, true)
    g_currentMission.player.manuelDischargerDischargeEventId = eventId
end

function ManuelDischarger.addStoreActionEventIfNeeded()
    local player = g_currentMission.player
    if player == nil or not player.isCarryingObject or player.manuelDischargerStoreEventId ~= nil or player.manuelDischargerPalletsInRange == nil or #player.manuelDischargerPalletsInRange == 0 then
        return
    end

    local _, eventId = g_inputBinding:registerActionEvent(InputAction.MANUELDISCHARGER_STORE, ManuelDischarger, ManuelDischarger.storeCallback, false, true, false, true)

    if eventId == nil then
        return
    end

    g_inputBinding:setActionEventTextVisibility(eventId, true)
    player.manuelDischargerStoreEventId = eventId
end

function ManuelDischarger.removeDischargeActionEventIfNeeded()
    local player = g_currentMission.player

    if player.manuelDischargerDischargeEventId ~= nil then
        g_inputBinding:removeActionEvent(player.manuelDischargerDischargeEventId)
        player.manuelDischargerDischargeEventId = nil
    end
end

function ManuelDischarger.removeStoreActionEventIfNeeded()
    local player = g_currentMission.player

    if (player.manuelDischargerPalletsInRange ~= nil and #player.manuelDischargerPalletsInRange == 0) or not player.isCarryingObject then
        if player.manuelDischargerStoreEventId == nil then
            return
        end

        g_inputBinding:removeActionEvent(player.manuelDischargerStoreEventId)
        player.manuelDischargerStoreEventId = nil
    end
end

function ManuelDischarger:manuelDischargeTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
    local player = g_currentMission.player

    if onEnter and player ~= nil and otherId == player.rootNode then
        local currentPallets = player.manuelDischargerPalletsInRange or {}
        table.insert(currentPallets, self)
        player.manuelDischargerPalletsInRange = currentPallets
        ManuelDischarger.addStoreActionEventIfNeeded()
    elseif onLeave and player ~= nil and otherId == player.rootNode then
        ManuelDischarger.removeFromArray(player.manuelDischargerPalletsInRange, self)
		ManuelDischarger.removeStoreActionEventIfNeeded()
    end
end

function ManuelDischarger.canStoreObject(object)
    local storeObject = g_currentMission:getNodeObject(object)
    if storeObject ~= nil and storeObject.spec_manuelDischarger ~= nil then
        return storeObject.spec_manuelDischarger.isStowable
    else
        return false
    end
end

function ManuelDischarger.storeCallback()
    if g_currentMission:getIsServer() then
        ManuelDischarger.storeObject(g_currentMission.player, g_currentMission.player.manuelDischargerPalletsInRange)
    else
        g_client:getServerConnection():sendEvent(ManuelDischargerStoreObjectEvent.new(g_currentMission.player.manuelDischargerPalletsInRange))
    end
end

function ManuelDischarger.storeObject(player, palletsInRange)
    local pickedUpObject = g_currentMission:getNodeObject(player.pickedUpObject)

    if pickedUpObject == nil then
        Logging.warning("Could not find pickedUpObject")
        return
    end

    if not ManuelDischarger.canStoreObject(player.pickedUpObject) then
        ManuelDischarger.showManuelDischargeWarningObjectCanNotStore(player, true)
        return
    end

    local pickedUpObjectFillType = pickedUpObject.spec_fillUnit.fillUnits[1]
    local pickedUpObjectFillLevel = pickedUpObjectFillType.fillLevel

    local triggerPallet = nil
    local foundPalletButNotEnoughSpace = false
    for _, pallet in pairs(palletsInRange) do
        if pallet ~= nil and pallet.spec_fillUnit.fillUnits[1].fillType == pickedUpObjectFillType.fillType then
            local freeSpace = pallet.spec_fillUnit:getFillUnitFreeCapacity(1)

            if freeSpace >= pickedUpObjectFillLevel then
                triggerPallet = pallet
                break
            else
                foundPalletButNotEnoughSpace = true
            end
        end
    end

    if triggerPallet ~= nil then
        local fillTypeIndex = triggerPallet.spec_fillUnit.fillUnits[1].fillType
        triggerPallet:addFillUnitFillLevel(triggerPallet.spec_fillUnit:getOwnerFarmId(), 1, pickedUpObjectFillLevel, fillTypeIndex, ToolType.UNDEFINED, nil)
        player:pickUpObject(false)
        pickedUpObject:addFillUnitFillLevel(pickedUpObject.spec_fillUnit:getOwnerFarmId(), 1, -math.huge, fillTypeIndex, ToolType.UNDEFINED, nil)
    elseif foundPalletButNotEnoughSpace then
        ManuelDischarger.showManuelDischargeWarningNotEnoughSpace(player, true)
    else
        local fillType = g_fillTypeManager:getFillTypeByIndex(pickedUpObjectFillType.fillType)
        ManuelDischarger.showManuelDischargeWarningWrongFillType(player, fillType.title, true)
    end
end

function ManuelDischarger:discharge()
    ManuelDischarger.spawnObject(self, g_currentMission.player)
end

function ManuelDischarger.spawnObject(pallet, player)
    if pallet == nil then
        Logging.warning("Pallet is not available")
        return
    end

    if pallet.isServer then
        local spec = pallet.spec_manuelDischarger

        local fillTypeIndex = pallet.spec_fillUnit.fillUnits[1].fillType
        local posX, posY, posZ = getWorldTranslation(spec.spawnNode)

        local locationData = {
            x = posX,
            y = posY,
            z = posZ
        }

        local asyncCallbackFunction = function(_, vehicle, vehicleLoadState, arguments)
            if vehicleLoadState == VehicleLoadingUtil.VEHICLE_LOAD_OK then
                local dischargeObjectCapacity = vehicle.spec_fillUnit:getFillUnitCapacity(1)
                local palletLevel = pallet.spec_fillUnit:getFillUnitFillLevel(1)
                if dischargeObjectCapacity > palletLevel then
                    vehicle.spec_fillUnit.fillUnits[1].fillLevel = palletLevel
                    pallet:addFillUnitFillLevel(pallet.spec_fillUnit:getOwnerFarmId(), 1, -palletLevel, fillTypeIndex, ToolType.UNDEFINED, nil)
                else
                    vehicle.spec_fillUnit.fillUnits[1].fillLevel = dischargeObjectCapacity
                    pallet:addFillUnitFillLevel(pallet.spec_fillUnit:getOwnerFarmId(), 1, -dischargeObjectCapacity, fillTypeIndex, ToolType.UNDEFINED, nil)
                end

                if player == nil then
                    Logging.warning("Player not found")
                    return
                end

                player.lastFoundObject = vehicle.rootNode
                player.isObjectInRange = true
                player:pickUpObject(true)
                player.hudUpdater:setCurrentRaycastTarget(vehicle.rootNode)
            end
        end

				local newPalletxml = XMLFile.load("newPallet", spec.dischargeObjectFileXml)
				local palletDesign = pallet.configurations.design

				if palletDesign ~= nil then
					newPalletxml:iterate("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration", function (palletIndex, curPath)
						newPalletxml:iterate(curPath..".fillUnits.fillUnit", function (_, UnitPath)
							local startFillType = newPalletxml:getString(UnitPath .. "#startFillType")
							if startFillType~=nil then
								if g_fillTypeManager:getFillTypeIndexByName(startFillType) == fillTypeIndex then
									palletDesign = palletIndex
								end
							end
						end)
					end)
				end

				newPalletxml:delete()

        VehicleLoadingUtil.loadVehicle(spec.dischargeObjectFileXml, locationData, true, 0, Vehicle.PROPERTY_STATE_OWNED, pallet.spec_fillUnit:getOwnerFarmId(), { fillUnit = palletDesign, design = palletDesign }, nil, asyncCallbackFunction, nil, nil, true, false)
    elseif pallet.isClient then
        g_client:getServerConnection():sendEvent(ManuelDischargerCreateObjectEvent.new(pallet))
    end
end

function ManuelDischarger:pickUpObjectRaycastCallback(superfunc, hitObjectId, x, y, z, distance)
	local returnValue = superfunc(self, hitObjectId, x, y, z, distance)

	if self.isCarryingObject then
		return returnValue
	end

	local hitObject = g_currentMission:getNodeObject(hitObjectId)

	-- Remove discharge action event if the hit object is not a manual discharger pallet
	-- Such pallets are identified by having the manual discharger spec, but not being stowable themselves (that would be a single bag)
	if hitObject == nil or hitObject.spec_manuelDischarger == nil or hitObject.spec_manuelDischarger.isStowable then
		ManuelDischarger.removeDischargeActionEventIfNeeded()
	end

	if hitObject == nil then
		local isTree = getHasClassId(hitObjectId, ClassIds.MESH_SPLIT_SHAPE)
		local _, animalId = getAnimalFromCollisionNode(hitObjectId)
		if isTree or animalId > 0 then
			return false -- Stop searching so we don't get info on an object behind the one we're looking at, especially with super strength
		end

		return true -- keep searching for visible things
	else
		-- A valid object was found
		if hitObject.spec_manuelDischarger ~= nil then
			-- it is our custom pallet => allow discharging
			ManuelDischarger.addDischargeActionEvent(hitObject)
		end
		return returnValue -- let base game decide whether or not the search should continue
	end

end

Player.pickUpObjectRaycastCallback = Utils.overwrittenFunction(Player.pickUpObjectRaycastCallback, ManuelDischarger.pickUpObjectRaycastCallback)

function ManuelDischarger:update(dt)
    if self.isCarryingObject then
        ManuelDischarger.addStoreActionEventIfNeeded()
        ManuelDischarger.removeDischargeActionEventIfNeeded()
    else
        ManuelDischarger.removeStoreActionEventIfNeeded()
    end
end

Player.update = Utils.appendedFunction(Player.update, ManuelDischarger.update)

-- Helper methods

function ManuelDischarger.showManuelDischargeWarningNotEnoughSpace(player, sendEvent)
    if player == g_currentMission.player then
        g_currentMission:showBlinkingWarning(g_i18n:getText("TL_manuelDischargerWarningTextNotEnoughSpace"), 3000)
    else
        if sendEvent and g_server ~= nil then
            g_server:broadcastEvent(ManuelDischargerShowWarningEvent.new(player, g_i18n:getText("TL_manuelDischargerWarningTextNotEnoughSpace")), false)
        end
    end
end

function ManuelDischarger.showManuelDischargeWarningWrongFillType(player, fillTypeName, sendEvent)
    local text = string.format("%s: %s", g_i18n:getText("TL_manuelDischargerWarningTextWrongFillType"), fillTypeName)
    if player == g_currentMission.player then
        g_currentMission:showBlinkingWarning(text, 3000)
    else
        if sendEvent and g_server ~= nil then
            g_server:broadcastEvent(ManuelDischargerShowWarningEvent.new(player, text), false)
        end
    end
end

function ManuelDischarger.showManuelDischargeWarningObjectCanNotStore(player, sendEvent)
    if player == g_currentMission.player then
        g_currentMission:showBlinkingWarning(g_i18n:getText("TL_manuelDischargerWarningTextCanNotStoreObject"), 3000)
    else
        if sendEvent and g_server ~= nil then
            g_server:broadcastEvent(ManuelDischargerShowWarningEvent.new(player, g_i18n:getText("TL_manuelDischargerWarningTextCanNotStoreObject")), false)
        end
    end
end

function ManuelDischarger.removeFromArray(array, value)
    for index, element in pairs(array) do
        if element == value then
            table.remove(array, index)
        end
    end
end