SowingMachineExtension = {};
SowingMachineExtension.debug = true;			
			
function SowingMachineExtension:onLoad(superFunc, savegame)
	superFunc(self, savegame);

	-- einfügen aller früchte der fruitcategory weeder, wenn grass drin ist
	local fruitTypeDescGrass = g_fruitTypeManager:getFruitTypeByName("GRASS");
	local fruitTypeDescPotato = g_fruitTypeManager:getFruitTypeByName("POTATO");
	local fruitTypeDescCotton = g_fruitTypeManager:getFruitTypeByName("COTTON");
	local fruitTypeDescStarchPotato = g_fruitTypeManager:getFruitTypeByName("STARCHPOTATO");
	local fillTypeIndexStarchPotato = g_fillTypeManager:getFillTypeIndexByName("STARCHPOTATO");
	local alreadyAdded = {}
	local hasPotato = false;
	local hasStarchPotato = false;
	for i, fruitTypeIndex in pairs(self.spec_sowingMachine.seeds) do
		alreadyAdded[fruitTypeIndex] = true;
		if fruitTypeIndex == fruitTypeDescCotton.index then
			table.remove(self.spec_sowingMachine.seeds, i)
		end
		if fruitTypeIndex == fruitTypeDescPotato.index then
			hasPotato = true;
		end
		if fruitTypeIndex == fruitTypeDescStarchPotato.index then
			hasStarchPotato = true;
		end
	end
	
	--Kverneland Optima RS
	local fruitTypeNames = self.xmlFile:getValue("vehicle.sowingMachine.seedFruitTypes")
	if fruitTypeNames ~= nil then
		if self.xmlFile:getValue("vehicle.storeData.name") == "Optima RS" then
			local planterFruits = g_fruitTypeManager:getFruitTypesByCategoryNames("planter")
			for _, fruitIndex in pairs(planterFruits) do
				if alreadyAdded[fruitIndex] == nil then
					table.insert(self.spec_sowingMachine.seeds, fruitIndex)
					alreadyAdded[fruitIndex] = true;
				end
			end
		end
	end

	local fillUnitIndex = self:getSowingMachineFillUnitIndex();

	-- STARCHPOTATO einfügen, wenn noch nicht drin, rest geht dann automatisch mit dem SEED
	if hasPotato and not hasStarchPotato then
		table.insert(self.spec_sowingMachine.seeds, fruitTypeDescStarchPotato.index)
		
		-- needs also to be added to the fillunit, because otherwise not fillable with harvested starch potatos
		self.spec_fillUnit.fillUnits[fillUnitIndex].supportedFillTypes[fillTypeIndexStarchPotato] = true;
	end
	
	if alreadyAdded[fruitTypeDescGrass.index] ~= nil then
		local fruitTypesWeederIndexes = g_fruitTypeManager:getFruitTypesByCategoryNames("WEEDER", "Warning: 'WEEDER' not found.");
		for _, fruitTypeWeederIndex in pairs(fruitTypesWeederIndexes) do
			if alreadyAdded[fruitTypeWeederIndex] == nil then
				table.insert(self.spec_sowingMachine.seeds, fruitTypeWeederIndex)
				alreadyAdded[fruitTypeWeederIndex] = true;
			end
		end
	end
	
	local spec = self.spec_fillUnit;
	
	-- fürs umschalten der auswählbaren Früchte hier liste machen
	self.spec_sowingMachine.seedFillTypeIndexToFruitTypeIndex = {};
	self.spec_sowingMachine.seedFruitFillTypeIndexToSeedFillTypeIndex = {};
	
	for _, fruitTypeIndex in pairs(self.spec_sowingMachine.seeds) do
		local fruitName = g_fruitTypeManager:getFruitTypeNameByIndex(fruitTypeIndex)
		local seedFillTypeName = "SEEDS_" .. fruitName;
		local seedFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(seedFillTypeName);
		if seedFillTypeIndex ~= nil then
			spec.fillUnits[fillUnitIndex].supportedFillTypes[seedFillTypeIndex] = true;
			self.spec_sowingMachine.seedFillTypeIndexToFruitTypeIndex[seedFillTypeIndex] = fruitTypeIndex;
			local fruitFillTypeIndex = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(fruitTypeIndex);
			self.spec_sowingMachine.seedFruitFillTypeIndexToSeedFillTypeIndex[fruitFillTypeIndex] = seedFillTypeIndex;
		elseif SowingMachineExtension.debug then
			Logging.warning("TerraLife Plus Error: Missing special seed Type '%s'", seedFillTypeName)
		end
	end
	
	self.spec_sowingMachine.originalSeeds = self.spec_sowingMachine.seeds;
end
SowingMachine.onLoad = Utils.overwrittenFunction(SowingMachine.onLoad, SowingMachineExtension.onLoad)

function SowingMachineExtension:getFillUnitAllowsFillType(superFunc2, superFunc, fillUnitIndex, fillType)
	if superFunc(self, fillUnitIndex, fillType) then
		return true
	end

	-- implementierung von sowingMachine entfernt, da diese immer SEEDS zulässt und damit das SEEDS_ was drin ist überschreiben würde

	return false
end
SowingMachine.getFillUnitAllowsFillType = Utils.overwrittenFunction(SowingMachine.getFillUnitAllowsFillType, SowingMachineExtension.getFillUnitAllowsFillType)

function SowingMachineExtension:addFillUnitFillLevel(superFunc2, superFunc, farmId, fillUnitIndex, fillLevelDelta, fillType, toolType, fillInfo)
	-- TerraLiveExtension.DebugText("SowingMachineExtension.addFillUnitFillLevel(%s, %s, %s, %s, %s, %s, %s, %s)", superFunc2, superFunc, farmId, fillUnitIndex, fillLevelDelta, fillType, toolType, fillInfo)
	local spec = self.spec_sowingMachine
		
	-- bei FERTILIZER das ganze ignorieren
	if fillType ~= FillType.FERTILIZER then
		if fillUnitIndex == spec.fillUnitIndex then

			local fruitType = spec.seeds[spec.currentSeed]

			if fruitType ~= nil then
				local seedsFillType = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(fruitType)
				-- TerraLiveExtension.DebugText("SowingMachineExtension:addFillUnitFillLevel fillType: %s, fruitType: %s, seedsFillType: %s", fillType, fruitType, seedsFillType)

				if seedsFillType ~= nil and self:getFillUnitSupportsFillType(fillUnitIndex, seedsFillType) then
					self:setFillUnitForcedMaterialFillType(fillUnitIndex, seedsFillType)
				end
			end
		end
	end

	local result = superFunc(self, farmId, fillUnitIndex, fillLevelDelta, fillType, toolType, fillInfo)
	
	-- festlegen des seeds, wenn notwendig und noch nicht geschehen
	-- TerraLiveExtension.DebugText("FillType.SEEDS %s, fillType %s, spec.lastUsedFillType %s)", FillType.SEEDS, fillType, spec.lastUsedFillType)
	-- bei FERTILIZER das ganze ignorieren
	if fillType ~= FillType.FERTILIZER then
		if fillType == FillType.SEEDS then
			if spec.lastUsedFillType ~= fillType then
				-- wiederherstellen alles wählbar
				spec.seeds = spec.originalSeeds;
				spec.lastUsedFillType = fillType;
				self:setSeedIndex(1);
				self:updateActionEvents();
			end
		else
			if spec.lastUsedFillType ~= fillType then
				-- TerraLiveExtension.DebugText("lastUsedFillType %s", g_fillTypeManager:getFillTypeNameByIndex(spec.lastUsedFillType))
				-- TerraLiveExtension.DebugText("fillType %s", g_fillTypeManager:getFillTypeNameByIndex(fillType))
				-- einstellen auf nur das was das seed hat
				local fruitTypeIndex = spec.seedFillTypeIndexToFruitTypeIndex[fillType];
				if fruitTypeIndex ~= nil then
					spec.seeds = {};
					table.insert(spec.seeds, fruitTypeIndex)
					self:setSeedIndex(1);
					self:updateActionEvents();
				end
				spec.lastUsedFillType = fillType;
			end
		end
	end
	
	return result;
end
SowingMachine.addFillUnitFillLevel = Utils.overwrittenFunction(SowingMachine.addFillUnitFillLevel, SowingMachineExtension.addFillUnitFillLevel)

function SowingMachineExtension:onEndWorkAreaProcessing(superFunc, dt, hasProcessed)
	local spec = self.spec_sowingMachine
-- print(string.format("SowingMachineExtension:onEndWorkAreaProcessing(%s, %s, %s)", superFunc, dt, hasProcessed))
	if self.isServer then
		local stats = g_farmManager:getFarmById(self:getLastTouchedFarmlandFarmId()).stats

		if spec.workAreaParameters.lastChangedArea > 0 then
			local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(spec.workAreaParameters.seedsFruitType)
			local lastHa = MathUtil.areaToHa(spec.workAreaParameters.lastChangedArea, g_currentMission:getFruitPixelsToSqm())
			local usage = fruitDesc.seedUsagePerSqm * lastHa * 10000
			local ha = MathUtil.areaToHa(spec.workAreaParameters.lastStatsArea, g_currentMission:getFruitPixelsToSqm())
			local damage = self:getVehicleDamage()

			if damage > 0 then
				usage = usage * (1 + damage * SowingMachine.DAMAGED_USAGE_INCREASE)
			end
			
			-- erhöhter verbrauch wenn standard saatgut benutzt wird
			if spec.lastUsedFillType == FillType.SEEDS then
				usage = usage * 3;
			end

			stats:updateStats("seedUsage", usage)
			stats:updateStats("sownHectares", ha)
			self:updateLastWorkedArea(spec.workAreaParameters.lastStatsArea)

			if not self:getIsAIActive() or not g_currentMission.missionInfo.helperBuySeeds then
				local vehicle = spec.workAreaParameters.seedsVehicle
				local fillUnitIndex = spec.workAreaParameters.seedsVehicleFillUnitIndex
				local unloadInfoIndex = spec.workAreaParameters.seedsVehicleUnloadInfoIndex
				local fillType = vehicle:getFillUnitFillType(fillUnitIndex)
				local unloadInfo = nil

				if vehicle.getFillVolumeUnloadInfo ~= nil then
					unloadInfo = vehicle:getFillVolumeUnloadInfo(unloadInfoIndex)
				end

				vehicle:addFillUnitFillLevel(self:getOwnerFarmId(), fillUnitIndex, -usage, fillType, ToolType.UNDEFINED, unloadInfo)
			else
				local price = usage * g_currentMission.economyManager:getCostPerLiter(spec.lastUsedFillType, false) * 1.5

				stats:updateStats("expenses", price)
				g_currentMission:addMoney(-price, self:getOwnerFarmId(), MoneyType.PURCHASE_SEEDS)
			end
		end

		self:updateLastWorkedArea(0)

		if spec.isWorking then
			stats:updateStats("sownTime", dt / 60000)
		end
	end

	if self.isClient then
		if spec.isWorking then
			if not spec.isWorkSamplePlaying then
				g_soundManager:playSample(spec.samples.work)

				spec.isWorkSamplePlaying = true
			end
		elseif spec.isWorkSamplePlaying then
			g_soundManager:stopSample(spec.samples.work)

			spec.isWorkSamplePlaying = false
		end
	end
end
SowingMachine.onEndWorkAreaProcessing = Utils.overwrittenFunction(SowingMachine.onEndWorkAreaProcessing, SowingMachineExtension.onEndWorkAreaProcessing)

function SowingMachineExtension:onStateChange(superFunc, state, data)
	-- überschrieben, damit beim einfüllen von saatgut in den externen tank in der sähmaschine das saatgut ausgewählt wird.
	
	-- TerraLiveExtension.DebugText("SowingMachineExtension:onStateChange(%s, %s, %s)", superFunc, state, data)
	-- TerraLiveExtension.DebugTable("Vehicle", Vehicle);
	-- TerraLiveExtension.DebugTable("self.spec_sowingMachine", self.spec_sowingMachine)
	if state == Vehicle.STATE_CHANGE_FILLTYPE_CHANGE  or state == Vehicle.STATE_CHANGE_DETACH or Vehicle.STATE_CHANGE_FILLTYPE_CHANGE then
		-- print("STATE_CHANGE_FILLTYPE_CHANGE")
		local spec = self.spec_sowingMachine
		
		-- suchen welcher fillType im attachment ist
		local fillTypeInImplement = nil;
		local childVehicles = self.rootVehicle:getChildVehicles()
		for _, childVehicle in pairs(childVehicles) do
			if childVehicle.typeName == "fillableImplement" or childVehicle.typeName == "pdlc_horschAgrovation.fillableImplementExtended" or childVehicle.typeName == "pdlc_premiumExpansion.fillableImplementExtended" then
				local fillUnitIndex = childVehicle:getFirstValidFillUnitToFill(FillType.SEEDS, true)
				-- TerraLiveExtension.DebugText("FillType.SEEDS: %s", FillType.SEEDS)
				-- TerraLiveExtension.DebugText("fillUnitIndex: %s", fillUnitIndex)
				-- TerraLiveExtension.DebugTable("childVehicle.spec_fillUnit.fillUnits", childVehicle.spec_fillUnit.fillUnits)
				if fillUnitIndex ~= nil then
					local fillType = g_fillTypeManager:getFillTypeByIndex(childVehicle.spec_fillUnit.fillUnits[fillUnitIndex].fillType)
					if fillType ~= nil then
						fillTypeInImplement = fillType.index;
						-- childVehicle:setFillUnitFillTypeToDisplay(fillUnitIndex, fillType.index)
					end
				end
			end
		end
		
		-- hier jetzt umstellen in der sämaschine
		-- TerraLiveExtension.DebugText("fillTypeInImplement: %s", fillTypeInImplement)
		if fillTypeInImplement ~= nil then
			-- festlegen des seeds, wenn notwendig und noch nicht geschehen
			if fillTypeInImplement == FillType.SEEDS then
				-- TerraLiveExtension.DebugText("fillTypeInImplement == FillType.SEEDS")
				if spec.lastUsedFillType ~= fillTypeInImplement then
					-- wiederherstellen alles wählbar
					spec.seeds = spec.originalSeeds;
					spec.lastUsedFillType = fillTypeInImplement;
					self:setSeedIndex(1);
					self:updateActionEvents();
				end
			else
				-- TerraLiveExtension.DebugText("fillTypeInImplement ~= FillType.SEEDS")
				-- TerraLiveExtension.DebugText("spec.lastUsedFillType: %s", spec.lastUsedFillType)
				-- if spec.lastUsedFillType ~= fillTypeInImplement then
					-- einstellen auf nur das was das seed hat
					local fruitTypeIndex = spec.seedFillTypeIndexToFruitTypeIndex[fillTypeInImplement];
					-- TerraLiveExtension.DebugText("fruitTypeIndex: %s", fruitTypeIndex)
					if fruitTypeIndex ~= nil then
						spec.seeds = {};
						table.insert(spec.seeds, fruitTypeIndex)
						self:setSeedIndex(1);
						self:updateActionEvents();
						spec.lastUsedFillType = fillTypeInImplement;
					end
					
					--FillType nur ändern, wenn es SEED ist
					if fillTypeInImplement == FillType.SEEDS then
						spec.lastUsedFillType = fillTypeInImplement;
					end
				-- end
			end
			
			spec.fillTypeSources = {}
			
			spec.fillTypeSources[fillTypeInImplement] = {}
			local root = self.rootVehicle

			FillUnit.addFillTypeSources(spec.fillTypeSources, root, self, {
				fillTypeInImplement
			})

			-- local fillType = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(spec.seeds[spec.currentSeed])

			-- self:setFillTypeSourceDisplayFillType(fillTypeInImplement)
		end
		
	end
end
SowingMachine.onStateChange = Utils.overwrittenFunction(SowingMachine.onStateChange, SowingMachineExtension.onStateChange)

function SowingMachineExtension:onStartWorkAreaProcessing(superFunc, dt)
	local spec = self.spec_sowingMachine
	spec.isWorking = false
	spec.isProcessing = false
	local seedsFruitType = spec.seeds[spec.currentSeed]
	local dx, _, dz = localDirectionToWorld(spec.directionNode, 0, 0, 1)
	local angleRad = MathUtil.getYRotationFromDirection(dx, dz)
	local desc = g_fruitTypeManager:getFruitTypeByIndex(seedsFruitType)

	if desc ~= nil and desc.directionSnapAngle ~= 0 then
		angleRad = math.floor(angleRad / desc.directionSnapAngle + 0.5) * desc.directionSnapAngle
	end

	local angle = FSDensityMapUtil.convertToDensityMapAngle(angleRad, g_currentMission.fieldGroundSystem:getGroundAngleMaxValue())
	local seedsVehicle, seedsVehicleFillUnitIndex, seedsVehicleUnloadInfoIndex = nil

	if self:getFillUnitFillLevel(spec.fillUnitIndex) > 0 then
		seedsVehicle = self
		seedsVehicleFillUnitIndex = spec.fillUnitIndex
		seedsVehicleUnloadInfoIndex = spec.unloadInfoIndex
	elseif spec.fillTypeSources[spec.lastUsedFillType] ~= nil then -- überschrieben da nicht nur SEED sein kann
		for _, src in ipairs(spec.fillTypeSources[spec.lastUsedFillType]) do
			local vehicle = src.vehicle

			if vehicle:getFillUnitFillLevel(src.fillUnitIndex) > 0 and vehicle:getFillUnitFillType(src.fillUnitIndex) == spec.lastUsedFillType then
				seedsVehicle = vehicle
				seedsVehicleFillUnitIndex = src.fillUnitIndex

				break
			end
		end
	end

	if seedsVehicle ~= nil and seedsVehicle ~= self then
		local fillType = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(seedsFruitType)

		seedsVehicle:setFillUnitFillTypeToDisplay(seedsVehicleFillUnitIndex, fillType)
	end

	local isTurnedOn = self:getIsTurnedOn()
	local canFruitBePlanted = false

	if desc.terrainDataPlaneId ~= nil then
		canFruitBePlanted = true
	end

	if spec.showWrongFruitForMissionWarning then
		spec.showWrongFruitForMissionWarning = false
	end

	local isPlantingSeason = true

	if not self:getCanPlantOutsideSeason() then
		isPlantingSeason = g_currentMission.growthSystem:canFruitBePlanted(seedsFruitType)
	end

	local seedVehicleChanged = seedsVehicle ~= spec.workAreaParameters.seedsVehicle or seedsVehicleFillUnitIndex ~= spec.workAreaParameters.seedsVehicleFillUnitIndex
	spec.showFruitCanNotBePlantedWarning = not canFruitBePlanted
	spec.showWrongPlantingTimeWarning = not isPlantingSeason and (isTurnedOn or not spec.needsActivation and self:getIsLowered())
	spec.workAreaParameters.isActive = not spec.needsActivation or isTurnedOn
	spec.workAreaParameters.canFruitBePlanted = canFruitBePlanted and isPlantingSeason
	spec.workAreaParameters.seedsFruitType = seedsFruitType
	spec.workAreaParameters.fieldGroundType = spec.fieldGroundType
	spec.workAreaParameters.angle = angle
	spec.workAreaParameters.seedsVehicle = seedsVehicle
	spec.workAreaParameters.seedsVehicleFillUnitIndex = seedsVehicleFillUnitIndex
	spec.workAreaParameters.seedsVehicleUnloadInfoIndex = seedsVehicleUnloadInfoIndex
	spec.workAreaParameters.lastTotalArea = 0
	spec.workAreaParameters.lastChangedArea = 0
	spec.workAreaParameters.lastStatsArea = 0

	if seedVehicleChanged then
		SowingMachine.updateAiParameters(self)
	end
end
SowingMachine.onStartWorkAreaProcessing = Utils.overwrittenFunction(SowingMachine.onStartWorkAreaProcessing, SowingMachineExtension.onStartWorkAreaProcessing)


function SowingMachineExtension:setFillTypeSourceDisplayFillType(superFunc, fillType)
	-- TerraLiveExtension.DebugText("SowingMachineExtension:setFillTypeSourceDisplayFillType(%s, %s)", superFunc, fillType)
	local spec = self.spec_sowingMachine
	-- TerraLiveExtension.DebugTable("spec.fillTypeSources", spec.fillTypeSources);
	-- TerraLiveExtension.DebugTable("spec.seedFruitFillTypeIndexToSeedFillTypeIndex", spec.seedFruitFillTypeIndexToSeedFillTypeIndex);
	if spec.seedFruitFillTypeIndexToSeedFillTypeIndex ~= nil and spec.seedFruitFillTypeIndexToSeedFillTypeIndex[fillType] ~= nil then
		local seedFillTypeIndex = spec.seedFruitFillTypeIndexToSeedFillTypeIndex[fillType];
		
		if spec.fillTypeSources[seedFillTypeIndex] ~= nil then
			-- TerraLiveExtension.DebugText("spec.fillTypeSources[fillType] ~= nil")
			for _, src in ipairs(spec.fillTypeSources[seedFillTypeIndex]) do
				local vehicle = src.vehicle
				local fillLevel = vehicle:getFillUnitFillLevel(src.fillUnitIndex)
				-- TerraLiveExtension.DebugText("vehicle:getFillUnitFillType(src.fillUnitIndex) - %s", vehicle:getFillUnitFillType(src.fillUnitIndex))
				if fillLevel > 0 and vehicle:getFillUnitFillType(src.fillUnitIndex) == seedFillTypeIndex then
					vehicle:setFillUnitFillTypeToDisplay(src.fillUnitIndex, fillType)

					break
				elseif fillLevel == 0 then
					local fillTypes = vehicle:getFillUnitSupportedFillTypes(src.fillUnitIndex)
					local numFillTypes = 0

					for fillTypeIndex, state in pairs(fillTypes) do
						if state then
							numFillTypes = numFillTypes + 1
						end
					end

					if numFillTypes == 1 and fillTypes[fillType] == true then
						vehicle:setFillUnitFillTypeToDisplay(src.fillUnitIndex, fillType)

						break
					end
				end
			end
		end
	end
end
SowingMachine.setFillTypeSourceDisplayFillType = Utils.overwrittenFunction(SowingMachine.setFillTypeSourceDisplayFillType, SowingMachineExtension.setFillTypeSourceDisplayFillType)


function SowingMachineExtension.loadSpecValueSeedFillTypes(xmlFile, customEnvironment, baseDir)
	local categories = Utils.getNoNil(xmlFile:getValue("vehicle.storeData.specs.seedFruitTypeCategories"), xmlFile:getValue("vehicle.sowingMachine.seedFruitTypeCategories"))
	local names = Utils.getNoNil(xmlFile:getValue("vehicle.storeData.specs.seedFruitTypes"), xmlFile:getValue("vehicle.sowingMachine.seedFruitTypes"))

	--Names rebuilden um Cotton zu entfernen
	if names ~= nil then
		--Kverneland Optima RS
		if xmlFile:getValue("vehicle.storeData.name") == "Optima RS" then
			names = ""
			local planterFruits = g_fruitTypeManager:getFruitTypesByCategoryNames("planter")
			for _, fruitIndex in pairs(planterFruits) do
				if names == "" then
					names = g_fruitTypeManager:getFruitTypeNameByIndex(fruitIndex)
				else
					names = names .." ".. g_fruitTypeManager:getFruitTypeNameByIndex(fruitIndex)
				end
			end
		else
			local split = string.split(names, " ")
			names = ""
			local added = {}
			local grass = false
			local potato = false
			
			for _, fruit in pairs(split) do
				if fruit:upper() ~= "COTTON" then
					if fruit:upper() == "GRASS" then
						grass = true
					elseif fruit:upper() == "POTATO" then
						potato = true
					end
					added[fruit] = true
					if names == "" then
						names = fruit
					else
						names = names .." ".. fruit
					end
				end
			end

			if grass == true then
				local fruitTypesWeederIndexes = g_fruitTypeManager:getFruitTypesByCategoryNames("WEEDER")
				for _, fruitTypeWeederIndex in pairs(fruitTypesWeederIndexes) do
					if added[fruitTypeWeederIndex] == nil then
						if names == "" then
							names = g_fruitTypeManager:getFruitTypeNameByIndex(fruitTypeWeederIndex)
						else
							names = names .." ".. g_fruitTypeManager:getFruitTypeNameByIndex(fruitTypeWeederIndex)
						end
						added[fruitTypeWeederIndex] = true
					end
				end
			end
			if potato == true then
				names = names.." STARCHPOTATO"
			end
		end
	end
	
	return {
		categories = categories,
		names = names
	}
end

SowingMachine.loadSpecValueSeedFillTypes = Utils.overwrittenFunction(SowingMachine.loadSpecValueSeedFillTypes, SowingMachineExtension.loadSpecValueSeedFillTypes)