--[[TerraLife Bale Extension
Allows more Fun with Bales :D
]]

terraLifeBaleExtension = {}

--Zusätzliche XML-Schema
BaleManager.baleXMLSchema:register(XMLValueType.STRING, "bale.fillTypes.fillType(?).fermenting#outputFillTypeUnwrapped", "Output fill type after fermenting")
BaleManager.baleXMLSchema:register(XMLValueType.FLOAT, "bale.fillTypes.fillType(?).fermenting#timeUnwrapped", "Fermenting time in ingame days which represent months", 1)

--Beim Ende der Fermentaion FillType wechseln abhängig ob der Ballen gewickelt ist/ein outputFillTypeUnwrapped vorhanden ist
function terraLifeBaleExtension:onFermentationEnd(superFunc)
	if self.isServer and self.isFermenting then
		local newTime = 0
		local fillType = self.fillType
		local fillTypeInfo = self:getFillTypeInfo(self.fillType)

		if fillTypeInfo ~= nil and fillTypeInfo.fermenting ~= nil then
			if fillTypeInfo.fermenting.outputFillTypeUnwrappedIndex ~= 0 and self.wrappingState == 0 then
				self:setFillType(fillTypeInfo.fermenting.outputFillTypeUnwrappedIndex)
				fillType = fillTypeInfo.fermenting.outputFillTypeUnwrappedIndex
			else
				if fillTypeInfo.fermenting.outputFillTypeIndex == FillType.COMPOSTRAW and self.isMissionBale == true then
				else
					self:setFillType(fillTypeInfo.fermenting.outputFillTypeIndex)
					fillType = fillTypeInfo.fermenting.outputFillTypeIndex
				end
			end
		end

		self.isFermenting = false

		--neue Fermentierung starten, falls vorhanden
		local newFillTypeInfo = self:getFillTypeInfo(fillType)
		if newFillTypeInfo.fermenting ~= nil then
			if newFillTypeInfo.fermenting.outputFillTypeIndex ~= nil and self.wrappingState ~= 0 then
				if newFillTypeInfo.fermenting.outputFillTypeIndex == FillType.COMPOSTRAW and self.isMissionBale == true then
				else
					self.isFermenting = true
					newTime = newFillTypeInfo.fermenting.time * 86400000
				end
			elseif newFillTypeInfo.fermenting.outputFillTypeUnwrappedIndex ~= nil and self.wrappingState == 0 then
				if newFillTypeInfo.fermenting.outputFillTypeUnwrappedIndex == FillType.COMPOSTRAW and self.isMissionBale == true then
				else
					self.isFermenting = true
					newTime = newFillTypeInfo.fermenting.timeUnwrapped * 86400000
				end
			end
		end

		self:raiseDirtyFlags(self.fermentingDirtyFlag)
		return newTime
	end
end

Bale.onFermentationEnd = Utils.overwrittenFunction(Bale.onFermentationEnd, terraLifeBaleExtension.onFermentationEnd)

--Zusätzliche Parameter Laden und Hinterlegen
function terraLifeBaleExtension.loadFillTypesFromXML(fillTypes, superFunc, xmlFile, baseDirectory)
	if terraLifeBaleExtension.fermentation == nil then
		terraLifeBaleExtension.fermentation = g_terraLifePlusFermentationManager.fermentation
	end
	local TLP = false
	if xmlFile:hasProperty("bale.terraLifePlus") then
		TLP = true
	end
	xmlFile:iterate("bale.fillTypes.fillType", function (index, key)
		local fillTypeName = xmlFile:getValue(key .. "#name")
		local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeName)

		if fillTypeIndex ~= nil then
			local fillTypeInfo = {
				fillTypeIndex = fillTypeIndex,
				capacity = xmlFile:getValue(key .. "#capacity", 1000),
				mass = xmlFile:getValue(key .. "#mass", 500) / 1000,
				forceAcceleration = xmlFile:getValue(key .. "#forceAcceleration"),
				supportsWrapping = xmlFile:getValue(key .. "#supportsWrapping", false)
			}
			local diffuseFilename = xmlFile:getValue(key .. ".diffuse#filename")

			if diffuseFilename ~= nil then
				fillTypeInfo.diffuseFilename = Utils.getFilename(diffuseFilename, baseDirectory)
			end

			local normalFilename = xmlFile:getValue(key .. ".normal#filename")

			if normalFilename ~= nil then
				fillTypeInfo.normalFilename = Utils.getFilename(normalFilename, baseDirectory)
			end

			local specularFilename = xmlFile:getValue(key .. ".specular#filename")

			if specularFilename ~= nil then
				fillTypeInfo.specularFilename = Utils.getFilename(specularFilename, baseDirectory)
			end

			local alphaFilename = xmlFile:getValue(key .. ".alpha#filename")

			if alphaFilename ~= nil then
				fillTypeInfo.alphaFilename = Utils.getFilename(alphaFilename, baseDirectory)
			end

			local outputFillTypeName = xmlFile:getValue(key .. ".fermenting#outputFillType")
			local outputFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(outputFillTypeName)
			local outputFillTypeUnwrappedName = xmlFile:getValue(key .. ".fermenting#outputFillTypeUnwrapped")
			local outputFillTypeUnwrappedIndex = g_fillTypeManager:getFillTypeIndexByName(outputFillTypeUnwrappedName)
			local timeUnwrapped = xmlFile:getValue(key .. ".fermenting#timeUnwrapped", 0)
			local requiresWrapping = xmlFile:getValue(key .. ".fermenting#requiresWrapping", true)
			local time = xmlFile:getValue(key .. ".fermenting#time", 0)

			if TLP == true then
				local fermantation = terraLifeBaleExtension.fermentation[fillTypeIndex]
				if fermantation ~= nil then
					outputFillTypeIndex = fermantation.outputFillTypeIndex
					outputFillTypeUnwrappedIndex = fermantation.outputFillTypeUnwrappedIndex
					timeUnwrapped = fermantation.timeUnwrapped
					requiresWrapping = fermantation.requiresWrapping
					time = fermantation.time
				end
			end

			if outputFillTypeIndex ~= nil then
				if outputFillTypeUnwrappedIndex == nil then
					outputFillTypeUnwrappedIndex = 0
				end

				fillTypeInfo.fermenting = {
					outputFillTypeIndex = outputFillTypeIndex,
					outputFillTypeUnwrappedIndex = outputFillTypeUnwrappedIndex,
					requiresWrapping = requiresWrapping,
					timeUnwrapped = timeUnwrapped,
					time = time
				}
			end

			table.insert(fillTypes, fillTypeInfo)
		end
	end)
end

Bale.loadFillTypesFromXML = Utils.overwrittenFunction(Bale.loadFillTypesFromXML, terraLifeBaleExtension.loadFillTypesFromXML)


--Fermentation starten wenn outputFillTypeUnwrappedIndex vorhanden ist
function terraLifeBaleExtension:setFillType(superFunc, fillTypeIndex, fillBale)
	Bale.setFillTypeTextures(self.nodeId, self.fillTypes, fillTypeIndex)

	self.fillType = fillTypeIndex
	self.supportsWrapping = false

	local fillTypeInfo = self:getFillTypeInfo(self.fillType)

	if fillTypeInfo ~= nil then
		self.supportsWrapping = fillTypeInfo.supportsWrapping

		setMass(self.nodeId, fillTypeInfo.mass)

		self.defaultMass = fillTypeInfo.mass

		if self.isServer then
			if fillTypeInfo.forceAcceleration ~= nil then
				self:setMountableObjectAttributes(nil, fillTypeInfo.forceAcceleration, self.dynamicMountForceLimitScale, self.dynamicMountSingleAxisFreeY, self.dynamicMountSingleAxisFreeX)
			end

			if fillTypeInfo.fermenting ~= nil and not fillTypeInfo.fermenting.requiresWrapping then
				if self.isFermenting then
					g_baleManager:removeFermentation(self)
				end
				local maxTime = fillTypeInfo.fermenting.time * 86400000

				if (self.wrappingState == 0 and fillTypeInfo.fermenting.timeUnwrapped ~= 0) or maxTime <= 1 then
					--Unverpackte Ballen verrotten und fertig vergorene gehen ins verrotten
					maxTime = fillTypeInfo.fermenting.timeUnwrapped * 86400000
				end

				g_baleManager:registerFermentation(self, 0, maxTime)

				self.isFermenting = true

				self:raiseDirtyFlags(self.fermentingDirtyFlag)
			elseif fillTypeInfo.fermenting ~= nil and fillTypeInfo.fermenting.outputFillTypeUnwrappedIndex ~= 0 and self.wrappingState == 0 then
				if self.isFermenting then
					g_baleManager:removeFermentation(self)
				end

				local maxTime = fillTypeInfo.fermenting.timeUnwrapped * 86400000

				g_baleManager:registerFermentation(self, 0, maxTime)

				self.isFermenting = true

				self:raiseDirtyFlags(self.fermentingDirtyFlag)
			end
		end

		if fillBale == true then
			self:setFillLevel(fillTypeInfo.capacity)
		end
	end

	Bale.updateVisualMeshVisibility(self.meshes, self.fillType, self.wrappingState ~= 0)

	if self.isServer then
		self:raiseDirtyFlags(self.fillTypeDirtyFlag)
	end
end

Bale.setFillType = Utils.overwrittenFunction(Bale.setFillType, terraLifeBaleExtension.setFillType)



--Wenn ein Ballen gewickelt wird, Fermentation resetten und neue Werte laden
function terraLifeBaleExtension:setWrappingState(superFunc, wrappingState, updateFermentation)
	if self.isServer and self.wrappingState ~= wrappingState then
		self:raiseDirtyFlags(self.wrapStateDirtyFlag)
	end

	--Wenn ein Ballen ausgepackt wurde, Fermentation updaten zu Kompost
	if self.isServer and wrappingState == 0 then
		local fillTypeInfo = self:getFillTypeInfo(self.fillType)

		if fillTypeInfo ~= nil and fillTypeInfo.fermenting ~= nil then
			if fillTypeInfo.fermenting.timeUnwrapped ~= nil and fillTypeInfo.fermenting.timeUnwrapped ~= 0 then
				if self.isFermenting then
					g_baleManager:removeFermentation(self)
					self.isFermenting = false
				end
				local maxTime = fillTypeInfo.fermenting.timeUnwrapped * 86400000

				g_baleManager:registerFermentation(self, 0, maxTime)

				self.isFermenting = true

				self:raiseDirtyFlags(self.fermentingDirtyFlag)
			end
		end
	end

	self.wrappingState = wrappingState

	--Wickler Animation
	for i = 1, #self.meshes do
		local meshData = self.meshes[i]
		local visibility = (meshData.supportsWrapping or wrappingState == 0) and meshData.fillTypeVisibility

		setVisibility(meshData.node, visibility)

		if visibility then
			setShaderParameter(meshData.node, "wrappingState", self.wrappingState, 0, 0, 0, false)
		end
	end

	--Ballen wurde gewickelt
	if self.isServer and updateFermentation ~= false and self.wrappingState >= 1 then
		if self.isFermenting then
			g_baleManager:removeFermentation(self)
			self.isFermenting = false
		end
		local fillTypeInfo = self:getFillTypeInfo(self.fillType)

		if fillTypeInfo ~= nil and fillTypeInfo.fermenting ~= nil and fillTypeInfo.fermenting.requiresWrapping and not self.isFermenting then
			local maxTime = fillTypeInfo.fermenting.time * 86400000

			if self.isMissionBale then
				maxTime = 0
			end

			g_baleManager:registerFermentation(self, 0, maxTime)

			self.isFermenting = true

			self:raiseDirtyFlags(self.fermentingDirtyFlag)
		end
	end

	if wrappingState > 0 then
		g_currentMission.activatableObjectsSystem:addActivatable(self.activatable)
	else
		g_currentMission.activatableObjectsSystem:removeActivatable(self.activatable)
	end
end

Bale.setWrappingState = Utils.overwrittenFunction(Bale.setWrappingState, terraLifeBaleExtension.setWrappingState)


--Folie lässt sich entfernen, wenn diese am verrotten sind
function terraLifeBaleExtension:getCanBeOpened(superFunc)
	if self.wrappingState <= 0 then
		return false
	end

	if self.isFermenting then
		local fillTypeInfo = self:getFillTypeInfo(self.fillType)
		if fillTypeInfo.fermenting.outputFillTypeIndex == FillType.COMPOSTRAW then
			return true
		end
		return false
	end

	if self.dynamicMountType ~= MountableObject.MOUNT_TYPE_NONE then
		return false
	end

	return true
end

Bale.getCanBeOpened = Utils.overwrittenFunction(Bale.getCanBeOpened, terraLifeBaleExtension.getCanBeOpened)


--Anzeige auf verrotten wechseln
function terraLifeBaleExtension:showInfo(superFunc, box)
	local fillType = self:getFillType()
	local fillLevel = self:getFillLevel()
	local fillTypeDesc = g_fillTypeManager:getFillTypeByIndex(fillType)

	box:addLine(fillTypeDesc.title, g_i18n:formatVolume(fillLevel, 0))

	if self:getIsFermenting() then
		local fillTypeInfo = self:getFillTypeInfo(self.fillType)
		if fillTypeInfo.fermenting.outputFillTypeIndex == FillType.COMPOSTRAW then
			box:addLine(g_i18n:getText("TL_bale_rotting"), string.format("%d%%", self:getFermentingPercentage() * 100))
		else
			if self.wrappingState == 1 then
				box:addLine(g_i18n:getText("info_fermenting"), string.format("%d%%", self:getFermentingPercentage() * 100))
			else
				--Ausgepackt verrottet
				box:addLine(g_i18n:getText("TL_bale_rotting"), string.format("%d%%", self:getFermentingPercentage() * 100))
			end
		end
	end

	box:addLine(g_i18n:getText("infohud_mass"), g_i18n:formatMass(self:getMass()))
end

Bale.showInfo = Utils.overwrittenFunction(Bale.showInfo, terraLifeBaleExtension.showInfo)


--Update erweitert um Funktion, mit der eine neue Fermentation ausgelöst werden kann
function terraLifeBaleExtension:update(superFunc, dt)
	-- TerraLiveExtension.DebugTable("self.fermentations", self.fermentations)
	if g_server ~= nil then
		local numFermentations = #self.fermentations
		if numFermentations > 0 then
			local timeScale = g_currentMission:getEffectiveTimeScale()
			-- TerraLiveExtension.DebugText("dt: %s timeScale: %s", dt, timeScale)
			
			for i = numFermentations, 1, -1 do
				local fermentation = self.fermentations[i]
				fermentation.time = fermentation.time + dt * timeScale

				if fermentation.maxTime <= fermentation.time then
					local newTime = fermentation.bale:onFermentationEnd()
					if newTime == nil then
						table.remove(self.fermentations, i)
					elseif newTime > 0 then
						fermentation.maxTime = newTime
						fermentation.time = 0
					else
						table.remove(self.fermentations, i)
					end
				end


				local percentage = 0
				if fermentation.time ~= 0 and fermentation.maxTime ~= 0 then
					percentage = fermentation.time / fermentation.maxTime
				end
				
				-- TerraLiveExtension.DebugText("percentage: %s", percentage)
				-- TerraLiveExtension.DebugTable("percentage", fermentation)
				if math.floor(percentage * 100) ~= math.floor(fermentation.percentageSend * 100) then
					fermentation.bale:onFermentationUpdate(percentage)

					fermentation.percentageSend = percentage
				end
			end
		end
	end
end

BaleManager.update = Utils.overwrittenFunction(BaleManager.update, terraLifeBaleExtension.update)


function terraLifeBaleExtension:registerFermentation(superFunc, bale, currentTime, maxTime)
	-- maxTime = maxTime * g_currentMission.missionInfo.economicDifficulty
	local fermentation = {
		bale = bale,
		time = currentTime,
		percentageSend = 0,
		maxTime = maxTime
	}

	table.insert(self.fermentations, fermentation)
end
BaleManager.registerFermentation = Utils.overwrittenFunction(BaleManager.registerFermentation, terraLifeBaleExtension.registerFermentation)


--Überschrieben, damit Ballen nicht mehrfach zum Fermentieren eingetragen werden und dadurch Stufen überspringen beim SaveGame laden
function terraLifeBaleExtension:applyBaleAttributes(superFunc, attributes)
	self:setOwnerFarmId(attributes.farmId or AccessHandler.EVERYONE)
	self:setFillLevel(attributes.fillLevel or self.fillLevel)

	if attributes.fillTypeName ~= nil then
		local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(attributes.fillTypeName)

		self:setFillType(fillTypeIndex or self.fillType)
	elseif attributes.fillType ~= nil then
		self:setFillType(attributes.fillType)
	end

	self:setWrapTextures(attributes.wrapDiffuse, attributes.wrapNormal)
	self:setWrappingState(attributes.wrappingState, false)
	self:setColor(unpack(attributes.wrappingColor))

	self.baleValueScale = attributes.baleValueScale or self.baleValueScale
	self.isMissionBale = Utils.getNoNil(attributes.isMissionBale, self.isMissionBale)
	
	-- TerraLiveExtension.DebugTable("attributes", attributes)
	
	-- mission bales are on load not "isFermenting", so must be checked here for loading the correct time
	if self.isServer and (attributes.isFermenting or attributes.isMissionBale) then
		local fillTypeInfo = self:getFillTypeInfo(self.fillType)

		if fillTypeInfo ~= nil and fillTypeInfo.fermenting ~= nil then
			-- TerraLiveExtension.DebugTable("fermenting", fillTypeInfo.fermenting)
			local maxTime = fillTypeInfo.fermenting.time * 86400000

			if attributes.wrappingState == 0 and fillTypeInfo.fermenting.timeUnwrapped ~= nil and fillTypeInfo.fermenting.timeUnwrapped ~= 0 then
				maxTime = fillTypeInfo.fermenting.timeUnwrapped * 86400000
			end

			if self.isMissionBale and not attributes.isFermenting then
				maxTime = 0
			end

			--mehrfaches Eintragen zum Fermentieren verhindern
			if self.isFermenting == true then
				g_baleManager:removeFermentation(self)
			end
			-- TerraLiveExtension.DebugText("fermentationTime: %s maxTime: %s", attributes.fermentationTime, maxTime)
			g_baleManager:registerFermentation(self, attributes.fermentationTime or 0, maxTime)

			self.isFermenting = true

			self:raiseDirtyFlags(self.fermentingDirtyFlag)
		end
	end
	
	-- TerraLiveExtension.DebugTable("g_baleManager.fermentations", g_baleManager.fermentations)
	return true
end

Bale.applyBaleAttributes = Utils.overwrittenFunction(Bale.applyBaleAttributes, terraLifeBaleExtension.applyBaleAttributes)

function terraLifeBaleExtension:getCanBeOpenedInline()
	for _, bale in ipairs(self.bales) do
		if bale.wrappingState < 1 or bale:getIsFermenting() then
			local fillTypeInfo = bale:getFillTypeInfo(bale.fillType)
			if fillTypeInfo.fermenting.outputFillTypeIndex == FillType.COMPOSTRAW then
				return true
			else
				return false
			end
		end
	end

	return true
end

InlineBale.getCanBeOpened = Utils.overwrittenFunction(InlineBale.getCanBeOpenedInline, terraLifeBaleExtension.getCanBeOpenedInline)