--[[
Adds TerraLife Bales to Stationary Balers
by: [F/A]Braeven
date: 05.11.2023
Version 1.5

Changelog
Version 1.0 - Initial Release
Version 1.1 - Added Support for Silage Additives for BALERSTATIONARY
Version 1.2 - Added Custom Wrap Colors for all BALERS
Version 1.3 - Added Support for Silage Additive for all Balers, Forage Harvesters and ForageWagons
Version 1.4 - Changed to use FillTypeCategories
Version 1.5 - Added some QOL Stuff
]]

terraLifeBalers = {}
terraLifeBalers.balesLoaded = false
terraLifeBalers.overWriteBalesBalerStationary = ""
terraLifeBalers.overWriteAdditiveBalerStationary = ""
terraLifeBalers.overWriteAdditiveBaler = ""
terraLifeBalers.overWriteAdditiveForageWagon = ""
terraLifeBalers.overWriteAdditiveForageHarvesters= ""

terraLifeBalers.additionalWrappingColors = "SHARED_YELLOW1 DEUTZ_GREEN5 SHARED_SILVER SHARED_GREY SHARED_GREYLIGHT SHARED_GREYDARK SHARED_BLACKONYX SHARED_BLACKJET"
terraLifeBalers.additionalWrappingColorNames = "$l10n_ui_colorYellow $l10n_ui_colorGreenLight $l10n_ui_colorSilver $l10n_ui_colorGreyLight $l10n_ui_colorGrey $l10n_ui_colorGreyDark $l10n_ui_colorBlackOnyx $l10n_ui_colorBlackJet"

function terraLifeBalers.loadBaleCategories()
	terraLifeBalers.balesLoaded = true
	
	local function getFillTypeNames(Categorie)
		local fillTypes = g_fillTypeManager:getFillTypesByCategoryNames(Categorie)
		local fillTypeList = ""
		for _, fillType in pairs(fillTypes) do
			local fillTypeName = g_fillTypeManager:getFillTypeByIndex(fillType)
			if fillTypeList == "" then
				fillTypeList = fillTypeName.name
			else
				fillTypeList = fillTypeList.." "..fillTypeName.name
			end
		end
		return fillTypeList
	end

	terraLifeBalers.overWriteBalesBalerStationary = getFillTypeNames("TL_STATIONARYBALER")
	terraLifeBalers.overWriteAdditiveBalerStationary = getFillTypeNames("TL_STATIONARYBALER_USE_ADDITIVE")
	terraLifeBalers.overWriteAdditiveBaler = getFillTypeNames("TL_BALER_USE_ADDITIVE")
	terraLifeBalers.overWriteAdditiveForageWagon = getFillTypeNames("TL_FORAGEWAGON_USE_ADDITIVE")
	terraLifeBalers.overWriteAdditiveForageHarvesters = getFillTypeNames("TL_FORAGEHARVESTER_USE_ADDITIVE")
end


function terraLifeBalers.loadFillUnitFromXML(self, superFunc, xmlFile, key, entry, index)
	if terraLifeBalers.balesLoaded == false then
		terraLifeBalers.loadBaleCategories()
	end
	local test = string.upper(xmlFile:getValue("vehicle#type", ""))
	if test == "BALERSTATIONARY" then
		local fillTypeNames = xmlFile:getValue(key .. "#fillTypes")
		if fillTypeNames ~= nil then
			--Nur Überschreiben, wenn die richtigen FillTypes vorher drin waren
			if string.upper(fillTypeNames) == "STRAW DRYGRASS_WINDROW GRASS_WINDROW CHAFF SUGARBEET_CUT FORAGE" then
				xmlFile:setValue(key .. "#fillTypes", terraLifeBalers.overWriteBalesBalerStationary)
			end
		end
	end

	local returnValue = superFunc(self, xmlFile, key, entry, index)

	return returnValue
end

FillUnit.loadFillUnitFromXML = Utils.overwrittenFunction(FillUnit.loadFillUnitFromXML, terraLifeBalers.loadFillUnitFromXML)



function terraLifeBalers.loadSpecValueFillTypes(xmlFile, superFunc, customEnvironment, baseDir)
	if terraLifeBalers.balesLoaded == false then
		terraLifeBalers.loadBaleCategories()
	end
	local rootName = xmlFile:getRootName()
	local test = string.upper(xmlFile:getValue("vehicle#type", ""))
	if test ~= nil then
		test = string.upper(test)
		if test == "BALERSTATIONARY" then
			local i = 0

			while true do
				local key = string.format(rootName .. ".fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d)", i)
				if not xmlFile:hasProperty(key) then
					break
				end

				local j = 0

				while true do
					local unitKey = string.format(key .. ".fillUnits.fillUnit(%d)", j)
					if not xmlFile:hasProperty(unitKey) then
						break
					end

					local showInShop = xmlFile:getValue(unitKey .. "#showInShop")
					local capacity = xmlFile:getValue(unitKey .. "#capacity")

					if (showInShop == nil or showInShop) and (capacity == nil or capacity > 0) then
						local currentFillTypes = xmlFile:getValue(unitKey .. "#fillTypes")
						if currentFillTypes ~= nil then
							--Nur Überschreiben, wenn die richtigen FillTypes vorher drin waren
							if string.upper(currentFillTypes) == "STRAW DRYGRASS_WINDROW GRASS_WINDROW CHAFF SUGARBEET_CUT FORAGE" then
								xmlFile:setValue(unitKey .. "#fillTypes", terraLifeBalers.overWriteBalesBalerStationary)
							end
						end
					end

					j = j + 1
				end

				i = i + 1
			end
		end
	end

	return superFunc(xmlFile, customEnvironment, baseDir)
end

FillUnit.loadSpecValueFillTypes = Utils.overwrittenFunction(FillUnit.loadSpecValueFillTypes, terraLifeBalers.loadSpecValueFillTypes)



function terraLifeBalers.onLoadBaler(self, superFunc, savegame)
	if terraLifeBalers.balesLoaded == false then
		terraLifeBalers.loadBaleCategories()
	end
	local rootName = self.xmlFile:getRootName()
	local test = string.upper(self.xmlFile:getValue("vehicle#type", ""))
	if test ~= nil then
		test = string.upper(test)
		if test == "BALERSTATIONARY" then
			local currentFillTypes = self.xmlFile:getValue("vehicle.baler.additives#fillTypes")
			if currentFillTypes ~= nil then
				--Nur Überschreiben, wenn die richtigen FillTypes vorher drin waren
				if string.upper(currentFillTypes) == "GRASS_WINDROW CHAFF" then
					self.xmlFile:setValue("vehicle.baler.additives#fillTypes", terraLifeBalers.overWriteAdditiveBalerStationary)
				end
			end
		elseif test == "BALERWRAPPEREXTENDED" or test == "BALER" or test == "BALERWRAPPER" then
			if self.xmlFile:hasProperty("vehicle.baler.additives") then
				local currentFillTypes = self.xmlFile:getValue("vehicle.baler.additives#fillTypes")
				if currentFillTypes ~= nil then
					--Nur Überschreiben, wenn die richtigen FillTypes vorher drin waren
					if string.upper(currentFillTypes) == "GRASS_WINDROW" then
						self.xmlFile:setValue("vehicle.baler.additives#fillTypes", terraLifeBalers.overWriteAdditiveBaler)
					end
				end
			end
		end
	end

	return superFunc(self, savegame)
end

Baler.onLoad = Utils.overwrittenFunction(Baler.onLoad, terraLifeBalers.onLoadBaler)



function terraLifeBalers.onLoadForageWagon(self, superFunc, savegame)
	if terraLifeBalers.balesLoaded == false then
		terraLifeBalers.loadBaleCategories()
	end
	local rootName = self.xmlFile:getRootName()
	local test = string.upper(self.xmlFile:getValue("vehicle#type", ""))
	if test ~= nil then
		if string.upper(test) == "FORAGEWAGON" then
			if self.xmlFile:hasProperty("vehicle.forageWagon.additives") then
				local currentFillTypes = self.xmlFile:getValue("vehicle.forageWagon.additives#fillTypes")
				if currentFillTypes ~= nil then
					--Nur Überschreiben, wenn die richtigen FillTypes vorher drin waren
					if string.upper(currentFillTypes) == "GRASS_WINDROW" then
						self.xmlFile:setValue("vehicle.forageWagon.additives#fillTypes", terraLifeBalers.overWriteAdditiveForageWagon)
					end
				end
			end
		end
	end

	return superFunc(self, savegame)
end

ForageWagon.onLoad = Utils.overwrittenFunction(ForageWagon.onLoad, terraLifeBalers.onLoadForageWagon)


function terraLifeBalers.loadCombineSetup(self, superFunc, xmlFile, baseKey, entry)
	if terraLifeBalers.balesLoaded == false then
		terraLifeBalers.loadBaleCategories()
	end
	local rootName = xmlFile:getRootName()
	local test = string.upper(xmlFile:getValue("vehicle#type", ""))
	if test ~= nil then
		if string.upper(test) == "COMBINEDRIVABLE" then
			if xmlFile:hasProperty("vehicle.combine.additives") then
				local currentFillTypes = xmlFile:getValue("vehicle.combine.additives#fillTypes")
				if currentFillTypes ~= nil then
					--Nur Überschreiben, wenn die richtigen FillTypes vorher drin waren
					if string.upper(currentFillTypes) == "CHAFF GRASS_WINDROW" then
						xmlFile:setValue("vehicle.combine.additives#fillTypes", terraLifeBalers.overWriteAdditiveForageHarvesters)
					end
				end
			end
		end
	end

	return superFunc(self, xmlFile, baseKey, entry)
end

Combine.loadCombineSetup = Utils.overwrittenFunction(Combine.loadCombineSetup, terraLifeBalers.loadCombineSetup)



function terraLifeBalers.getConfigurationsFromXML(xmlFile, superFunc, key, baseDir, customEnvironment, isMod, storeItem)
	if xmlFile:hasProperty("vehicle.wrappingColorConfigurations") then
		local newWraps = string.split(terraLifeBalers.additionalWrappingColors, " ")
		local newWrapNames = string.split(terraLifeBalers.additionalWrappingColorNames, " ")
	
		local wrapNumber = -1
			xmlFile:iterate("vehicle.wrappingColorConfigurations.wrappingColorConfiguration", function (index, wrappingKey)
			wrapNumber = wrapNumber +1
		end)

		for i = wrapNumber+1, wrapNumber + #newWraps do
				xmlFile:setValue("vehicle.wrappingColorConfigurations.wrappingColorConfiguration("..i..")#color", newWraps[i - wrapNumber])
				xmlFile:setValue("vehicle.wrappingColorConfigurations.wrappingColorConfiguration("..i..")#name", newWrapNames[i - wrapNumber])
				xmlFile:setValue("vehicle.wrappingColorConfigurations.wrappingColorConfiguration("..i..")#price", 100)
		end
	end
	return superFunc(xmlFile, key, baseDir, customEnvironment, isMod, storeItem)
end

StoreItemUtil.getConfigurationsFromXML = Utils.overwrittenFunction(StoreItemUtil.getConfigurationsFromXML, terraLifeBalers.getConfigurationsFromXML)



function terraLifeBalers:onLoadBaleLoader(savegame, superFunc)
	local spec = self.spec_baleLoader
	local filename = self.xmlFile:getValue("vehicle.baleLoader.balePacker#packedFilename")
	if filename == "$data/objects/squarebales/packedSquareBale120.xml" then
		spec.balePacker.filename =  Utils.getFilename("$moddir$FS22_TerraLifePlus_Assets/bales/squarebale120/packedSquareBale120.xml", self.baseDirectory)
	end
end
BaleLoader.onLoad = Utils.appendedFunction(BaleLoader.onLoad, terraLifeBalers.onLoadBaleLoader)



function terraLifeBalers:replacePendingBale(superFunc, spawnNode, color)
	if self.pendingBale ~= nil then
		if self.replacementBaleFilename == "data/objects/squarebales/squarebale180.xml" then
			self.replacementBaleFilename = Utils.getFilename("$moddir$FS22_TerraLifePlus_Assets/bales/squarebale180/squarebale180.xml", self.baseDirectory)
		end
	end
	return superFunc(self, spawnNode, color)
end

InlineBale.replacePendingBale = Utils.overwrittenFunction(InlineBale.replacePendingBale, terraLifeBalers.replacePendingBale)