LivestockTrailerAnimation = {}

local modDirectory = g_currentModDirectory

IDLE_SOUND_GROUP = "yellSoundGroup"
WALK_SOUND_GROUP = "walkSoundGroup"

FA_IDLE1 = "FA_idleLong01"
FA_IDLE2 = "FA_idleLong02"
FA_IDLE3 = "FA_idleLong03"

FA_WALK_FRONT = "FA_walkFront01"
FA_WALK_BACK = "FA_walkBack01"

MAX_SPEED_LIMIT = 6
MIN_TROT_SPEED = 5

function LivestockTrailerAnimation.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(LivestockTrailer, specializations)
end

function LivestockTrailerAnimation.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "startIdleAnimation", LivestockTrailerAnimation.startIdleAnimation)
	SpecializationUtil.registerFunction(vehicleType, "startWalkAnimation", LivestockTrailerAnimation.startWalkAnimation)
	SpecializationUtil.registerFunction(vehicleType, "updateWalkAnimation", LivestockTrailerAnimation.updateWalkAnimation)
	SpecializationUtil.registerFunction(vehicleType, "selectIdleAnimation", LivestockTrailerAnimation.selectIdleAnimation)
	SpecializationUtil.registerFunction(vehicleType, "stopTimers", LivestockTrailerAnimation.stopTimers)
	SpecializationUtil.registerFunction(vehicleType, "loadSounds", LivestockTrailerAnimation.loadSounds)
	SpecializationUtil.registerFunction(vehicleType, "replaceFilename", LivestockTrailerAnimation.replaceFilename)
	SpecializationUtil.registerFunction(vehicleType, "startSound", LivestockTrailerAnimation.startSound)
	SpecializationUtil.registerFunction(vehicleType, "stopAllSounds", LivestockTrailerAnimation.stopAllSounds)
	SpecializationUtil.registerFunction(vehicleType, "removeAllSounds", LivestockTrailerAnimation.removeAllSounds)
	SpecializationUtil.registerFunction(vehicleType, "setRandomUV", LivestockTrailerAnimation.setRandomUV)
end

function LivestockTrailerAnimation.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", LivestockTrailerAnimation)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdateTick", LivestockTrailerAnimation)
end

function LivestockTrailerAnimation.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "onAnimalLoaded", LivestockTrailerAnimation.onAnimalLoaded)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "doCheckSpeedLimit", LivestockTrailerAnimation.doCheckSpeedLimit)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "updateAnimals", LivestockTrailerAnimation.updateAnimals)
end

function LivestockTrailerAnimation.initSpecialization()
	local schema = Vehicle.xmlSchema
    local baseKey = "vehicle.animations"

	schema:register(XMLValueType.BOOL, baseKey.."#shouldAnimalsWalkWhileDriving", "Should animals walk while driving", false)
end

function LivestockTrailerAnimation:onLoad(savegame)
	self.spec_livestockTrailerAnimation = {}
	local spec = self.spec_livestockTrailerAnimation
    spec.animalNodes = {}
    spec.sounds = {}
    spec.currentPlayedSounds = {}

    local baseKey = "vehicle.animations"

    spec.shouldAnimalsWalkWhileDriving = self.xmlFile:getValue(baseKey.."#shouldAnimalsWalkWhileDriving", false)
    spec.animalsAreWalking = false
    spec.animationTimer = {}

    if spec.shouldAnimalsWalkWhileDriving then
        self.speedLimit = MAX_SPEED_LIMIT
    end
end

function LivestockTrailerAnimation:updateAnimals(superFunc)
    self.spec_livestockTrailerAnimation.animalNodes = {}
    self:stopTimers()
    self:stopAllSounds()
    self:removeAllSounds()
    superFunc(self)
end

function LivestockTrailerAnimation:onAnimalLoaded(superFunc, i3dNode, failedReason, args)
    superFunc(self, i3dNode, failedReason, args)

    if string.find(args.slot.filename, "dataS") then
        return
    end

	if i3dNode == 0 then
        return
	end

    local components = {}
    I3DUtil.loadI3DComponents(i3dNode, components)
    local skeletonNode = I3DUtil.indexToObject(components, "0")

    self:setRandomUV(i3dNode, args)

    table.addElement(self.spec_livestockTrailerAnimation.animalNodes, skeletonNode)
    self:loadSounds(args.visual, skeletonNode)
		local animalType = self:getCurrentAnimalType()
		--Our Chicken don't have the new Animations
		if animalType.name ~= "CHICKEN" then
			self:startIdleAnimation(skeletonNode)
		end
end

function LivestockTrailerAnimation:setRandomUV(i3dNode, args)
    local randomIndex = math.random(1, #args.visual.visualAnimal.variations)
    local variation = args.visual.visualAnimal.variations[randomIndex]
    local tileU = variation.tileUIndex / variation.numTilesU
    local tileV = variation.tileVIndex / variation.numTilesV

    I3DUtil.setShaderParameterRec(i3dNode, "atlasInvSizeAndOffsetUV", 1 / variation.numTilesU, 1 / variation.numTilesV, tileU, tileV, false)
    I3DUtil.setShaderParameterRec(i3dNode, "RDT", nil, 0, nil, nil, false)
end

function LivestockTrailerAnimation:startIdleAnimation(skeletonNode)
    self:startSound(IDLE_SOUND_GROUP, skeletonNode)
    local characterSet = getAnimCharacterSet(skeletonNode)

    disableAnimTrack(characterSet, 0)
    setAnimTrackTime(characterSet, 0, 0, true)
    clearAnimTrackClip(characterSet, 0)

    local idleName = self:selectIdleAnimation()
    local clipIndex1 = getAnimClipIndex(characterSet, idleName)

    if clipIndex1 == nil then
        Logging.warning("Clip "..idleName.." not exist for current animal type")

        local timer = Timer.createOneshot(1, function()
            if self.spec_livestockTrailerAnimation.animalsAreWalking == false then
                self:startIdleAnimation(skeletonNode)
            end
        end)

        table.addElement(self.spec_livestockTrailerAnimation.animationTimer, timer)
        return
    end

    assignAnimTrackClip(characterSet, 0, clipIndex1)
    setAnimTrackLoopState(characterSet, 0, true)
    setAnimTrackTime(characterSet, 0, 0, true)
    setAnimTrackSpeedScale(characterSet, 0, 1)
    setAnimTrackBlendWeight(characterSet, 0, 1)
    enableAnimTrack(characterSet, 0)

    local clipDuration = getAnimClipDuration(characterSet, clipIndex1)
    local timer = Timer.createOneshot(clipDuration, function()
        if self.spec_livestockTrailerAnimation.animalsAreWalking == false then
            self:startIdleAnimation(skeletonNode)
        end
    end)

    table.addElement(self.spec_livestockTrailerAnimation.animationTimer, timer)
end

function LivestockTrailerAnimation:selectIdleAnimation()
    local index = math.random(1,3)

    if index == 1 then
        return FA_IDLE1
    elseif index == 2 then
        return FA_IDLE2
    else
        return FA_IDLE3
    end
end

function LivestockTrailerAnimation:doCheckSpeedLimit(superFunc)
    if self.spec_livestockTrailerAnimation.shouldAnimalsWalkWhileDriving and #self.spec_livestockTrailerAnimation.animalNodes > 0 then
        return true
    else
        return superFunc(self)
    end
end

function LivestockTrailerAnimation:onUpdateTick(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    if #self.spec_livestockTrailerAnimation.animalNodes == 0 then
        return
    end

    local currentSpeed = self:getLastSpeed()

    if currentSpeed < 1 and self.spec_livestockTrailerAnimation.animalsAreWalking == true then
        self.spec_livestockTrailerAnimation.animalsAreWalking = false
        self.spec_livestockTrailerAnimation.lastSpeed = 0

        for index, skeletonNode in pairs(self.spec_livestockTrailerAnimation.animalNodes) do
            self:startIdleAnimation(skeletonNode)
        end
    elseif currentSpeed >= 1 and self.spec_livestockTrailerAnimation.animalsAreWalking == false and self.spec_livestockTrailerAnimation.shouldAnimalsWalkWhileDriving then
        self:stopTimers()

        for index, skeletonNode in pairs(self.spec_livestockTrailerAnimation.animalNodes) do
            local driveForward = self.lastSignedSpeedReal > 0
            self:startWalkAnimation(skeletonNode, currentSpeed, driveForward)
        end

        self.spec_livestockTrailerAnimation.animalsAreWalking = true
        self.spec_livestockTrailerAnimation.lastSpeed = currentSpeed
    elseif currentSpeed >= 1 and self.spec_livestockTrailerAnimation.animalsAreWalking == true and self.spec_livestockTrailerAnimation.shouldAnimalsWalkWhileDriving then
        for index, skeletonNode in pairs(self.spec_livestockTrailerAnimation.animalNodes) do
            self:updateWalkAnimation(skeletonNode, currentSpeed)
        end
    end
end

function LivestockTrailerAnimation:startWalkAnimation(skeletonNode, currentSpeed, driveForward)
    self:startSound(WALK_SOUND_GROUP, skeletonNode)
    local characterSet = getAnimCharacterSet(skeletonNode)

    local walkClip = FA_WALK_FRONT

    if driveForward == false then
        walkClip = FA_WALK_BACK
    end

    setAnimTrackTime(characterSet, 0, 0, true)
    clearAnimTrackClip(characterSet, 0)

    local clipWalk = getAnimClipIndex(characterSet, walkClip)

    if clipWalk == nil then
        Logging.warning("Walk clip "..walkClip.." not exist for current animal type")
        return
    end

    assignAnimTrackClip(characterSet, 0, clipWalk)
    setAnimTrackLoopState(characterSet, 0, true)
    setAnimTrackTime(characterSet, 0, 0, true)
    setAnimTrackSpeedScale(characterSet, 0, 0)
    setAnimTrackBlendWeight(characterSet, 0, 1)
    enableAnimTrack(characterSet, 0)
end

function LivestockTrailerAnimation:updateWalkAnimation(skeletonNode, currentSpeed)
    local characterSet = getAnimCharacterSet(skeletonNode)

    local minSpeed = 0
    local maxSpeed = MAX_SPEED_LIMIT
    local range = maxSpeed - minSpeed

    local factor = (currentSpeed - minSpeed) / range
    local scale = (factor * range) / maxSpeed

    setAnimTrackSpeedScale(characterSet, 0, scale)
end

function LivestockTrailerAnimation:stopTimers()
    for index, timer in pairs(self.spec_livestockTrailerAnimation.animationTimer) do
        timer:delete()
    end

    self.spec_livestockTrailerAnimation.animationTimer = {}
end

function LivestockTrailerAnimation:loadSounds(visual, skeletonNode)
    local spec = self.spec_livestockTrailerAnimation

    local animalType = self:getCurrentAnimalType()
    local configFilename = Utils.getFilename(animalType.configFilename, modDirectory)
	  local xmlFile = XMLFile.load("animalsConfig", configFilename)

    local sampleName = "sample"
    local soundKey = "animalHusbandry.sound.soundGroup"
    local audioGroup = AudioGroup.ENVIRONMENT

    self:replaceFilename(xmlFile, soundKey, sampleName, animalType)

    local supportedSounds = {}
    for key, value in pairs(visual.audio) do
        supportedSounds[value] = key
    end


    local sounds = {}

    xmlFile:iterate(soundKey, function (_, key)
        if supportedSounds[xmlFile:getString(key.."#name")] ~= nil then
            sounds[supportedSounds[xmlFile:getString(key.."#name")]] = g_soundManager:loadSamplesFromXML(xmlFile, key, sampleName, modDirectory, skeletonNode, 1, audioGroup, self.i3dMappings, self)
        end
    end)

    spec.sounds[skeletonNode] = sounds

    xmlFile:delete()
end

function LivestockTrailerAnimation:replaceFilename(xmlFile, soundKey, sampleName, animalType)
	animalType = animalType.name:lower()
	if animalType == "haycow" then
		animalType = "cow"
	end
	xmlFile:iterate(soundKey, function (_, key)
		xmlFile:iterate(key.."."..sampleName, function (_, soundGroupKey)
			if xmlFile:getString(soundGroupKey.."#file") == nil then
				local filename = xmlFile:getString(soundGroupKey.."#filename")
				if filename:sub(1, 5) ~= "$data" then
					filename = "$moddir$FS22_TerraLifePlus_Assets/animals/" .. animalType .. "/" .. filename
				end
				xmlFile:setString(soundGroupKey.."#file", filename)
			end
		end)
	end)
end

function LivestockTrailerAnimation:startSound(soundName, skeletonNode)
    local spec = self.spec_livestockTrailerAnimation

    if spec.currentPlayedSounds[skeletonNode] ~= nil then
        g_soundManager:stopSample(spec.currentPlayedSounds[skeletonNode])
        spec.currentPlayedSounds[skeletonNode] = nil
    end

    local sounds = spec.sounds[skeletonNode]
    local soundGroup = sounds[soundName]
    local randomSoundIndex = math.random(1, #soundGroup)
    local sound = soundGroup[randomSoundIndex]

    spec.currentPlayedSounds[skeletonNode] = sound
    g_soundManager:playSample(sound)
end

function LivestockTrailerAnimation:stopAllSounds()
    local spec = self.spec_livestockTrailerAnimation
    for key, value in pairs(spec.currentPlayedSounds) do
        g_soundManager:stopSample(value)
    end
    spec.currentPlayedSounds = {}
end

function LivestockTrailerAnimation:removeAllSounds()
    local spec = self.spec_livestockTrailerAnimation
    for _, animal in pairs(spec.sounds) do
        for _, soundGroupValue in pairs(animal) do
            for _, sampleValue in pairs(soundGroupValue) do
                g_soundManager:deleteSample(sampleValue)
            end
        end
    end
    spec.sounds = {}
end