LivestockTrailerExtension = {
	AnimalsNeedToAdd = {
		haycow = "COW",
		goat = "SHEEP"
	}
}

function LivestockTrailerExtension.initSpecialization(self, superFunc)
	g_storeManager:addSpecType("numAnimalsHaycow", "shopListAttributeIconHaycow", LivestockTrailer.loadSpecValueNumberAnimalsHaycow, LivestockTrailer.getSpecValueNumberAnimalsHaycow, "vehicle")
	g_storeManager:addSpecType("numAnimalsGoat", "shopListAttributeIconGoat", LivestockTrailer.loadSpecValueNumberAnimalsGoat, LivestockTrailer.getSpecValueNumberAnimalsGoat, "vehicle")
	g_storeManager:addSpecType("numAnimalsChicken", "shopListAttributeIconChicken", LivestockTrailer.loadSpecValueNumberAnimalsChicken, LivestockTrailer.getSpecValueNumberAnimalsChicken, "vehicle")
	superFunc(self)
end

LivestockTrailer.initSpecialization = Utils.overwrittenFunction(LivestockTrailer.initSpecialization, LivestockTrailerExtension.initSpecialization)

function LivestockTrailerExtension.registerFunctions(vehicleType, superFunc)
	superFunc(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "setMonthSinceBirthIfNeeded", LivestockTrailer.setMonthSinceBirthIfNeeded)
end

LivestockTrailer.registerFunctions = Utils.overwrittenFunction(LivestockTrailer.registerFunctions, LivestockTrailerExtension.registerFunctions)

function LivestockTrailerExtension:onLoad(superFunc, savegame)
	LivestockTrailerExtension.addNewAnimalTypes(self.xmlFile, self)
	return superFunc(self, savegame)
end

LivestockTrailer.onLoad = Utils.overwrittenFunction(LivestockTrailer.onLoad, LivestockTrailerExtension.onLoad)

function LivestockTrailerExtension:addAnimals(superFunc, subTypeIndex, numAnimals, age)
   local cluster = g_currentMission.animalSystem:createClusterFromSubTypeIndex(subTypeIndex)

	if cluster:getSupportsMerging() then
		cluster.numAnimals = numAnimals
		cluster.age = age
        self:setMonthSinceBirthIfNeeded(subTypeIndex, cluster)

		self:addCluster(cluster)
	else
		for i = 1, numAnimals do
			if i > 1 then
				cluster = g_currentMission.animalSystem:createClusterFromSubTypeIndex(subTypeIndex)
			end

			cluster.numAnimals = 1
			cluster.age = age
            self:setMonthSinceBirthIfNeeded(subTypeIndex, cluster)

			self:addCluster(cluster)
		end
	end
end

LivestockTrailer.addAnimals = Utils.overwrittenFunction(LivestockTrailer.addAnimals, LivestockTrailerExtension.addAnimals)

function LivestockTrailerExtension.addNewAnimalTypes(xmlFile, vehicle)
	local supportedTypes = {}

	xmlFile:iterate("vehicle.livestockTrailer.animal", function (index, key)
		supportedTypes[string.upper(xmlFile:getString(key .. "#type"))] = true
	end)

	for key, value in pairs(LivestockTrailerExtension.AnimalsNeedToAdd) do
		if supportedTypes[value] and supportedTypes[key] == nil then
			LivestockTrailerExtension.addAnimalType(value, key, xmlFile, vehicle)
		end
	end
end

function LivestockTrailerExtension.addAnimalType(existingType, newType, xmlFile, vehicle)
	if g_currentMission.animalSystem:getTypeIndexByName(newType) ~= nil then
		local place = {
			type = newType
		}

		local index = 0

		xmlFile:iterate("vehicle.livestockTrailer.animal", function (_, key)
			index = index + 1
			if string.upper(xmlFile:getString(key .. "#type")) == existingType then
				if vehicle ~= nil then
					place.node = xmlFile:getValue(key .. "#node", nil, vehicle.components, vehicle.i3dMappings)
				end
				place.numSlots = xmlFile:getValue(key .. "#numSlots")
			end
		end)

		xmlFile:setValue("vehicle.livestockTrailer.animal("..index..")#type", place.type)
		if place.node ~= nil then
				xmlFile:setValue("vehicle.livestockTrailer.animal("..index..")#node", place.node)
		end
		xmlFile:setValue("vehicle.livestockTrailer.animal("..index..")#numSlots", place.numSlots)
	end
end

function LivestockTrailerExtension.loadSpecValueNumberAnimals(xmlFile, superFunc, customEnvironment, baseDir, animalTypeName)
    LivestockTrailerExtension.addNewAnimalTypes(xmlFile, nil)
	local maxNumAnimals = superFunc(xmlFile, customEnvironment, baseDir, animalTypeName)
	return maxNumAnimals
end

LivestockTrailer.loadSpecValueNumberAnimals = Utils.overwrittenFunction(LivestockTrailer.loadSpecValueNumberAnimals, LivestockTrailerExtension.loadSpecValueNumberAnimals)

--------------------------------------------------- New LivestockTrailer functions ---------------------------------------------------

function LivestockTrailer:setMonthSinceBirthIfNeeded(subTypeIndex, cluster)
    local subType = g_currentMission.animalSystem:getSubTypeByIndex(subTypeIndex)
    local neededAge = subType.reproductionMinAgeMonth + subType.reproductionDurationMonth
    if subType.supportsReproduction and neededAge <= cluster:getAge() and cluster.monthSinceLastBirth == nil then
        cluster.monthSinceLastBirth = cluster:getAge() - neededAge
    end
end

function LivestockTrailer.loadSpecValueNumberAnimalsHaycow(xmlFile, customEnvironment, baseDir)
	return LivestockTrailer.loadSpecValueNumberAnimals(xmlFile, customEnvironment, baseDir, "haycow")
end

function LivestockTrailer.loadSpecValueNumberAnimalsGoat(xmlFile, customEnvironment, baseDir)
	return LivestockTrailer.loadSpecValueNumberAnimals(xmlFile, customEnvironment, baseDir, "goat")
end

function LivestockTrailer.loadSpecValueNumberAnimalsChicken(xmlFile, customEnvironment, baseDir)
	return LivestockTrailer.loadSpecValueNumberAnimals(xmlFile, customEnvironment, baseDir, "chicken")
end

function LivestockTrailer.getSpecValueNumberAnimalsHaycow(storeItem, realItem)
	return LivestockTrailer.getSpecValueNumberAnimals(storeItem, realItem, "numAnimalsHaycow")
end

function LivestockTrailer.getSpecValueNumberAnimalsGoat(storeItem, realItem)
	return LivestockTrailer.getSpecValueNumberAnimals(storeItem, realItem, "numAnimalsGoat")
end

function LivestockTrailer.getSpecValueNumberAnimalsChicken(storeItem, realItem)
	return LivestockTrailer.getSpecValueNumberAnimals(storeItem, realItem, "numAnimalsChicken")
end