NoTillFertilizingSowingMachine = {
	prerequisitesPresent = function (specializations)
		return SpecializationUtil.hasSpecialization(SowingMachine, specializations) and SpecializationUtil.hasSpecialization(Sprayer, specializations) --and SpecializationUtil.hasSpecialization(FertilizingSowingMachine, specializations)
	end,
	initSpecialization = function ()
	end
}

function NoTillFertilizingSowingMachine.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "processSowingMachineArea", NoTillFertilizingSowingMachine.processSowingMachineArea)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getUseSprayerAIRequirements", NoTillFertilizingSowingMachine.getUseSprayerAIRequirements)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getAreEffectsVisible", NoTillFertilizingSowingMachine.getAreEffectsVisible)
end

function NoTillFertilizingSowingMachine.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", NoTillFertilizingSowingMachine)
end


function NoTillFertilizingSowingMachine:onLoad(savegame)
	self.spec_sprayer.needsToBeFilledToTurnOn = false
	self.spec_sprayer.useSpeedLimit = false
end

function NoTillFertilizingSowingMachine:processSowingMachineArea(superFunc, workArea, dt)
	-- TerraLiveExtension.DebugText("NoTillFertilizingSowingMachine:processSowingMachineArea(%s, %s, %s)", superFunc, workArea, dt)
	-- local spec = self.spec_fertilizingSowingMachine
	local specSowingMachine = self.spec_sowingMachine
	local specSpray = self.spec_sprayer
	local sprayerParams = specSpray.workAreaParameters
	local sowingParams = specSowingMachine.workAreaParameters
	local changedArea, totalArea = nil
	self.spec_sowingMachine.isWorking = self:getLastSpeed() > 0.5

	if not sowingParams.isActive then
		return 0, 0
	end

	if (not self:getIsAIActive() or not g_currentMission.missionInfo.helperBuySeeds) and sowingParams.seedsVehicle == nil then
		if self:getIsAIActive() then
			local rootVehicle = self.rootVehicle

			rootVehicle:stopCurrentAIJob(AIMessageErrorOutOfFill.new())
		end

		return 0, 0
	end

	if (specSpray.isSlurryTanker and g_currentMission.missionInfo.helperSlurrySource == 1 or specSpray.isManureSpreader and g_currentMission.missionInfo.helperManureSource == 1 or specSpray.isFertilizerSprayer and not g_currentMission.missionInfo.helperBuyFertilizer) and self:getIsAIActive() then
		if sprayerParams.sprayFillType == nil or sprayerParams.sprayFillType == FillType.UNKNOWN then
			if sprayerParams.lastAIHasSprayed ~= nil then
				local rootVehicle = self.rootVehicle

				rootVehicle:stopCurrentAIJob(AIMessageErrorOutOfFill.new())

				sprayerParams.lastAIHasSprayed = nil
			end
		else
			sprayerParams.lastAIHasSprayed = true
		end
	end

	if not sowingParams.canFruitBePlanted then
		return 0, 0
	end

	local sprayTypeIndex = SprayType.FERTILIZER

	if sprayerParams.sprayFillLevel <= 0 or true and not self:getIsTurnedOn() then
	-- if sprayerParams.sprayFillLevel <= 0 then
		sprayTypeIndex = nil
	end

	local startX, _, startZ = getWorldTranslation(workArea.start)
	local widthX, _, widthZ = getWorldTranslation(workArea.width)
	local heightX, _, heightZ = getWorldTranslation(workArea.height)

	-- for PF this call is needed additional here
	if self.preProcessExtUnderRootFertilizerArea ~= nil and sprayTypeIndex ~= nil then
		self:preProcessExtUnderRootFertilizerArea(workArea, dt)
	end
		
	changedArea, totalArea = FSDensityMapUtil.updateNoTillSowingArea(sowingParams.seedsFruitType, startX, startZ, widthX, widthZ, heightX, heightZ, sowingParams.fieldGroundType, sowingParams.angle, nil, sprayTypeIndex)
	-- TerraLiveExtension.DebugText("changedArea, totalArea =  %s, %s", changedArea, totalArea)
	
	self.spec_sowingMachine.isProcessing = self.spec_sowingMachine.isWorking
	if sprayTypeIndex ~= nil then
		local sprayAmount = specSpray.doubledAmountIsActive and 2 or 1
		-- TerraLiveExtension.DebugText("FSDensityMapUtil.updateSprayArea(%s, %s, %s, %s, %s, %s, %s, %s)", startX, startZ, widthX, widthZ, heightX, heightZ, sprayTypeIndex, sprayAmount)
		local sprayChangedArea, sprayTotalArea = FSDensityMapUtil.updateSprayArea(startX, startZ, widthX, widthZ, heightX, heightZ, sprayTypeIndex, sprayAmount)
		-- TerraLiveExtension.DebugText("sprayChangedArea, sprayTotalArea =  %s, %s", sprayChangedArea, sprayTotalArea)
		sprayerParams.lastChangedArea = sprayerParams.lastChangedArea + sprayChangedArea
		sprayerParams.lastTotalArea = sprayerParams.lastTotalArea + sprayTotalArea
		sprayerParams.lastStatsArea = 0
		sprayerParams.isActive = true
		sprayerParams.lastSprayTime = g_time
		local stats = g_currentMission:farmStats(self:getLastTouchedFarmlandFarmId())
		local ha = MathUtil.areaToHa(sprayerParams.lastChangedArea, g_currentMission:getFruitPixelsToSqm())

		stats:updateStats("sprayedHectares", ha)
		stats:updateStats("sprayedTime", dt / 60000)
		stats:updateStats("sprayUsage", sprayerParams.usage)
	end

	sowingParams.lastChangedArea = sowingParams.lastChangedArea + changedArea
	sowingParams.lastStatsArea = sowingParams.lastStatsArea + changedArea
	sowingParams.lastTotalArea = sowingParams.lastTotalArea + totalArea

	FSDensityMapUtil.eraseTireTrack(startX, startZ, widthX, widthZ, heightX, heightZ)
	self:updateMissionSowingWarning(startX, startZ)

	-- for PF this call is needed additional here
	if self.postProcessExtUnderRootFertilizerArea ~= nil and sprayTypeIndex ~= nil then
		self:postProcessExtUnderRootFertilizerArea(workArea, dt, changedArea, totalArea)
	end
	
	-- making NoTill PF Bonus ready
-- TerraLiveExtension.DebugText("changedArea: %s - %s", changedArea, self.getPFStatisticInfo);
	if changedArea > 0 and self.getPFStatisticInfo ~= nil then
		local _, _, farmlandId = self:getPFStatisticInfo()
		if farmlandId ~= nil then
			FS22_precisionFarming.g_precisionFarming.environmentalScore.scoreObjects.EnvironmentalScoreTillage:addWorkedArea(farmlandId, changedArea, 3)
		end
	end
		
	return changedArea, totalArea
end

function NoTillFertilizingSowingMachine:getUseSprayerAIRequirements(superFunc)
	return false
end

function NoTillFertilizingSowingMachine:getAreEffectsVisible(superFunc)
	return superFunc(self) and self:getFillUnitFillType(self:getSprayerFillUnitIndex()) ~= FillType.UNKNOWN
end