
TLFertilizingCultivatorExtension = {}

--[[
	Extend FertilizingCultivator to use other spraytype as FERTILIZER and to use the ExtendedSprayer:processSprayerArea instead of the FSDensityMapUtil.updateSprayArea which is capable of lime
--]]
function TLFertilizingCultivatorExtension:processCultivatorArea(superFunc2, superFunc, workArea, dt)
	-- TerraLiveExtension.DebugText("TLFertilizingCultivatorExtension:processCultivatorArea(%s, %s, %s, %s)", superFunc2, superFunc, workArea, dt)
	
	local spec = self.spec_fertilizingCultivator
	local specCultivator = self.spec_cultivator
	local specSpray = self.spec_sprayer
	local xs, _, zs = getWorldTranslation(workArea.start)
	local xw, _, zw = getWorldTranslation(workArea.width)
	local xh, _, zh = getWorldTranslation(workArea.height)
	local cultivatorParams = specCultivator.workAreaParameters
	local sprayerParams = specSpray.workAreaParameters

	if (specSpray.isSlurryTanker and g_currentMission.missionInfo.helperSlurrySource == 1 or specSpray.isManureSpreader and g_currentMission.missionInfo.helperManureSource == 1 or specSpray.isFertilizerSprayer and not g_currentMission.missionInfo.helperBuyFertilizer) and self:getIsAIActive() then
		if sprayerParams.sprayFillType == nil or sprayerParams.sprayFillType == FillType.UNKNOWN then
			if sprayerParams.lastAIHasSprayed ~= nil then
				local rootVehicle = self.rootVehicle

				rootVehicle:stopCurrentAIJob(AIMessageErrorOutOfFill.new())

				sprayerParams.lastAIHasSprayed = nil
			end
		else
			sprayerParams.lastAIHasSprayed = true
		end
	end

	local sprayTypeIndex = SprayType.FERTILIZER
	-- nur wenn gekalkt wird, egal womit, dann nicht FERTILIZER verwenden
	if self.getCurrentSprayerMode ~= nil then
		local isLiming, isFertilizing = self:getCurrentSprayerMode();
		if isLiming then
			sprayTypeIndex = g_sprayTypeManager:getSprayTypeIndexByFillTypeIndex(sprayerParams.sprayFillType)
		end
	end

	if sprayerParams.sprayFillLevel <= 0 or spec.needsSetIsTurnedOn and not self:getIsTurnedOn() then
		sprayTypeIndex = nil
	end
	
	-- TerraLiveExtension.DebugText("sprayTypeIndex: %s: SprayType.FERTILIZER: %s", sprayTypeIndex, SprayType.FERTILIZER)
	
	local cultivatorChangedArea = 0
	local cultivatorTotalArea = 0

	if specCultivator.isEnabled then
		if specCultivator.useDeepMode then
			cultivatorChangedArea, cultivatorTotalArea = FSDensityMapUtil.updateCultivatorArea(xs, zs, xw, zw, xh, zh, not cultivatorParams.limitToField, cultivatorParams.limitFruitDestructionToField, cultivatorParams.angle, sprayTypeIndex)
			cultivatorChangedArea = cultivatorChangedArea + FSDensityMapUtil.updateVineCultivatorArea(xs, zs, xw, zw, xh, zh)
		else
			cultivatorChangedArea, cultivatorTotalArea = FSDensityMapUtil.updateDiscHarrowArea(xs, zs, xw, zw, xh, zh, not cultivatorParams.limitToField, cultivatorParams.limitFruitDestructionToField, cultivatorParams.angle, sprayTypeIndex)
			cultivatorChangedArea = cultivatorChangedArea + FSDensityMapUtil.updateVineCultivatorArea(xs, zs, xw, zw, xh, zh)
		end

		cultivatorParams.lastChangedArea = cultivatorParams.lastChangedArea + cultivatorChangedArea
		cultivatorParams.lastTotalArea = cultivatorParams.lastTotalArea + cultivatorTotalArea
		cultivatorParams.lastStatsArea = cultivatorParams.lastStatsArea + cultivatorChangedArea
	end

	if specCultivator.isSubsoiler then
		FSDensityMapUtil.updateSubsoilerArea(xs, zs, xw, zw, xh, zh)
	end

	FSDensityMapUtil.eraseTireTrack(xs, zs, xw, zw, xh, zh)

	if sprayTypeIndex == SprayType.FERTILIZER then
		local sprayAmount = specSpray.doubledAmountIsActive and 2 or 1
		local sprayChangedArea, sprayTotalArea = FSDensityMapUtil.updateSprayArea(xs, zs, xw, zw, xh, zh, sprayTypeIndex, sprayAmount)
		-- TerraLiveExtension.DebugText("FERTILIZER sprayChangedArea: %s", sprayChangedArea)
		sprayerParams.lastChangedArea = sprayerParams.lastChangedArea + sprayChangedArea
		sprayerParams.lastTotalArea = sprayerParams.lastTotalArea + sprayTotalArea
		sprayerParams.lastStatsArea = 0
		sprayerParams.isActive = true
	elseif sprayTypeIndex ~= nil then
		-- wenn es nicht fertilizer ist, dann processSprayerArea benutzen, damit gekalkt wird
		local sprayAmount = specSpray.doubledAmountIsActive and 2 or 1
		local sprayChangedArea, sprayTotalArea = self:processSprayerArea(workArea, dt)
		-- TerraLiveExtension.DebugText("OTHER sprayChangedArea: %s", sprayChangedArea)
		sprayerParams.lastChangedArea = sprayerParams.lastChangedArea + sprayChangedArea
		sprayerParams.lastTotalArea = sprayerParams.lastTotalArea + sprayTotalArea
		sprayerParams.lastStatsArea = 0
		sprayerParams.isActive = true
	end

	specCultivator.isWorking = self:getLastSpeed() > 0.5

	return cultivatorChangedArea, cultivatorTotalArea
end
FertilizingCultivator.processCultivatorArea = Utils.overwrittenFunction(FertilizingCultivator.processCultivatorArea, TLFertilizingCultivatorExtension.processCultivatorArea)