--[[
There are two types of field ground values:
A: The FieldGroundType enum which has fixed values 0-14 and gets extended by TLP
B: The field ground value defined in terraLifePlus_fieldGround.xml, which has a different order

wheel.densityType contains the current field ground value (type B) and is then:
1. compared to spec.tireTrackGroundGrassValue (type B)                          <-- OK
2. compared to spec.tireTrackGroundGrassCutValue (type B)                       <-- OK
3. used as an index for accessing Wheels.MAX_SINK which has keys of type A      <-- Not OK
4. compared to FieldGroundType.PLOWED (type A, same function as above)          <-- Not OK
5. compared to the field ground value for FieldGroundType.GRASS (type B)        <-- OK

=> Least invasive fix is to change the value temporarily to type A during the execution of the method for cases 3 and 4
]]

TLWheelsFix = {}

local fieldTypeValues = nil
local function getFieldTypeValue(fieldGroundValue)
	if not g_currentMission or not g_currentMission.fieldGroundSystem then
		return fieldGroundValue
	end
	if fieldTypeValues == nil then
		fieldTypeValues = {}
		-- Cache the reverse fieldGroundTypeValue mapping
		for groundType, fieldGroundValue in ipairs(g_currentMission.fieldGroundSystem.fieldGroundTypeValue) do
			fieldTypeValues[fieldGroundValue] = groundType
		end
	end

	return fieldTypeValues[fieldGroundValue] or fieldGroundValue
end

function TLWheelsFix:updateWheelSink(superFunc, wheel, dt, groundWetness)
	local originalValue = wheel.densityType
	wheel.densityType = getFieldTypeValue(originalValue)
	superFunc(self, wheel, dt, groundWetness)
	wheel.densityType = originalValue
end
Wheels.updateWheelSink = Utils.overwrittenFunction(Wheels.updateWheelSink, TLWheelsFix.updateWheelSink)